// BMPSite.cpp: implementation of the CBMPSite class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BMPSite.h"
#include "StringToken.h"
#include "./swmm5/odesolve.h"
#include <math.h>
#include "Global.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static  BMP_D* theBUFFERSTRIP; // BUFFERSTRIP to which getDdDt() is applied
static  BMP_E* theAREABMP;     // AREABMP to which getDdDt() is applied
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBMPSite::CBMPSite()
{
	m_strID = "";
	m_strName = "";
	m_strType = "";
	m_bChecked = false;
	m_lfBMPUnit = 1.0;
	m_nBMPClass = 0;
	m_lfDDarea = 0.0;
	m_lfAccDArea = 0.0;
	m_lfPerArea = 0.0;
	m_lfImpArea = 0.0;
	m_lfCost = 0.0;
	m_lfSurfaceArea = 0.0;
	m_lfExcavatnVol = 0.0;
	m_lfSurfStorVol = 0.0;
	m_lfSoilStorVol = 0.0;
	m_lfUdrnStorVol	= 0.0;
	m_nInterEvent = 0;
	m_lfThreshFlow = 0.0;
	m_lfLowFlowLimit = 0.0;
	m_lfHighFlowLimit = 0.0;
	m_lfSiteDArea = 0.0;
	m_pDecay = NULL;
	m_pK = NULL;
	m_pCstar = NULL;
	m_pConc = NULL;
	m_pUndRemoval = NULL;
	m_pSiteProp = NULL;
	m_preLU = NULL;
	m_pAquifer = NULL;
	m_RAConc = NULL;
	m_fileOut = NULL;
	m_nQualNum = 0;
	m_nTSNum = 0;
	m_strCostFile = "";
	m_nBreakPoints = 0;
	m_lfBreakPtID = 0.0;
	m_TradeOff = NULL;
	m_pDataMixLU = NULL;
	m_pDataPreLU = NULL;
	m_tmStart = COleDateTime(1890,1,1,0,0,0);

	m_sediment.m_lfBEDWID = 0.0;
	m_sediment.m_lfBEDDEP = 0.0;
	m_sediment.m_lfBEDPOR = 0.0;
	m_sediment.m_lfSAND_FRAC = 0.0;
	m_sediment.m_lfSILT_FRAC = 0.0;
	m_sediment.m_lfCLAY_FRAC = 0.0;

	m_sand.m_lfD = 0.0;
	m_sand.m_lfW = 0.0;
	m_sand.m_lfRHO = 0.0;
	m_sand.m_lfKSAND = 0.0;
	m_sand.m_lfEXPSND = 0.0;

	m_silt.m_lfD = 0.0;
	m_silt.m_lfW = 0.0;
	m_silt.m_lfRHO = 0.0;
	m_silt.m_lfTAUCD = 0.0;
	m_silt.m_lfTAUCS = 0.0;
	m_silt.m_lfM = 0.0;

	m_clay.m_lfD = 0.0;
	m_clay.m_lfW = 0.0;
	m_clay.m_lfRHO = 0.0;
	m_clay.m_lfTAUCD = 0.0;
	m_clay.m_lfTAUCS = 0.0;
	m_clay.m_lfM = 0.0;

	m_nInfiltMethod = 0;
	m_nPolRotMethod = 1;
	m_nPolRemMethod = 0;
	m_nInfil_Index = 0;
	m_bUndSwitch = false;
	m_lfSoilDepth = 0.0;
	m_lfPorosity = 0.0;
	m_lfFCapacity = 0.3;
	m_lfWPoint = 0.15;
	m_lfInfilt = 0.0;
	m_lfUndDepth = 0.0;
	m_lfUndVoid = 0.0;
	m_lfUndInfilt = 0.0;
	m_holtanParam.m_lfVegA = 0.0;
//	m_holtanParam.m_lfFInfilt = 0.0;
	for (int i=0; i<12; i++)
		m_holtanParam.m_lfGrowth[i] = 0.0;

	m_costParam.m_lfLinearCost = 0.0;			
	m_costParam.m_lfAreaCost = 0.0;            				
	m_costParam.m_lfTotalVolumeCost = 0.0;     				
	m_costParam.m_lfMediaVolumeCost = 0.0;     				
	m_costParam.m_lfUnderDrainVolumeCost = 0.0;				
	m_costParam.m_lfConstantCost = 0.0;        				
	m_costParam.m_lfPercentCost = 0.0;         
	m_costParam.m_lfLengthExp = 0.0;         
	m_costParam.m_lfAreaExp = 0.0;         
	m_costParam.m_lfTotalVolExp = 0.0;         
	m_costParam.m_lfMediaVolExp = 0.0;         
	m_costParam.m_lfUDVolExp = 0.0;         
	
	//Ftable
	m_nFtableFlag = 0;
	m_pFtable = NULL;

	//Flow Duration Curve
	m_nStartBin = 0;			// starting bin that meet the low flow condition
	m_nEndBin = 0;				// ending bin that meet the high flow condition
	m_lfMinFlow = 1e12;			// minimum flow value for the entire simulation (cfs)
	m_lfMaxFlow = 0.;			// maximum flow value for the entire simulation (cfs)
	m_strPreDevFile = "";		// predeveloped file path
	m_nNumBins = 0;				// number of bins
	m_nPreDevFlag = 0;			// Predeveloped flag (1-internal predeveloped timeseries,
								//					  2-external predeveloped timeseries) 
	m_pPreDevBinID = NULL;		// array of bin IDs for the predeveloped timeseries
	m_pPreDevFlow = NULL;		// array of predeveloped flow (cfs)
	m_pPreDevBinCFS = NULL;		// array of predeveloped average flow by bins (cfs)

	m_pBinID = NULL;			// array of bin IDs for the simulated timeseries
	m_pFlow = NULL;				// array of simulated flow timeseries (cfs)
	m_pBinCFS = NULL;			// array of simulated average flow by bins (cfs)

	fp_FDC = NULL;				// Flow Duration Curve output file
}

CBMPSite::CBMPSite(const CString& strID, const CString& strName, const CString& strType, int bmpClass)
{
	m_strID = strID;
	m_strName = strName;
	m_strType = strType;
	m_bChecked = false;
	m_lfBMPUnit = 1.0;
	m_nBMPClass = bmpClass;
	m_lfDDarea = 0.0;
	m_lfAccDArea = 0.0;
	m_lfPerArea = 0.0;
	m_lfImpArea = 0.0;
	m_lfCost = 0.0;		

	m_lfSurfaceArea = 0.0;
	m_lfExcavatnVol = 0.0;
	m_lfSurfStorVol = 0.0;
	m_lfSoilStorVol = 0.0;
	m_lfUdrnStorVol	= 0.0;
	m_nInterEvent = 0;
	m_lfThreshFlow = 0.0;
	m_lfLowFlowLimit = 0.0;
	m_lfHighFlowLimit = 0.0;
	m_lfSiteDArea = 0.0;
	m_pDecay = NULL;
	m_pK = NULL;
	m_pCstar = NULL;
	m_pConc = NULL;
	m_pUndRemoval = NULL;

	//Ftable
	m_nFtableFlag = 0;
	m_pFtable = NULL;

	//Flow Duration Curve
	m_nStartBin = 0;			// starting bin that meet the low flow condition
	m_nEndBin = 0;				// ending bin that meet the high flow condition
	m_lfMinFlow = 1e12;			// minimum flow value for the entire simulation (cfs)
	m_lfMaxFlow = 0.;			// maximum flow value for the entire simulation (cfs)
	m_strPreDevFile = "";		// predeveloped file path
	m_nNumBins = 0;				// number of bins
	m_nPreDevFlag = 0;			// Predeveloped flag (1-internal predeveloped timeseries,
								//					  2-external predeveloped timeseries) 
	m_pPreDevBinID = NULL;		// array of bin IDs for the predeveloped timeseries
	m_pPreDevFlow = NULL;		// array of predeveloped flow timeseries (cfs)
	m_pPreDevBinCFS = NULL;		// array of predeveloped average flow by bins (cfs)

	m_pBinID = NULL;			// array of bin IDs for the simulated timeseries
	m_pFlow = NULL;				// array of simulated flow timeseries (cfs)
	m_pBinCFS = NULL;			// array of simulated average flow by bins (cfs)
	
	fp_FDC = NULL;				// Flow Duration Curve output file

	switch (m_nBMPClass)
	{
		case CLASS_A:
		{
			BMP_A* pBmpA = new BMP_A;
			pBmpA->m_lfBasinWidth = 0.0;
			pBmpA->m_lfBasinLength = 0.0;
			pBmpA->m_lfOrificeHeight = 0.0;
			pBmpA->m_lfOrificeDiameter = 0.0;
			pBmpA->m_lfOrificeCoeff = 1.0;
			pBmpA->m_nExitType = 0;
			pBmpA->m_nWeirType = -1;
			pBmpA->m_lfWeirHeight = 0.0;
			pBmpA->m_lfWeirWidth = 0.0;
			pBmpA->m_lfWeirAngle = 0.0;
			pBmpA->m_lfPETmultiplier = 1.0;
			pBmpA->m_nORelease = -1;
			pBmpA->m_nPeople = -1;
			pBmpA->m_nDays = -1;
			pBmpA->m_lfWatDep_i = 0.;
			pBmpA->m_lfTheta_i = 0.;
			for(int i=0; i<24; i++)
				pBmpA->m_lfRelease[i] = 0.0;

			// green-ampt parameters
			pBmpA->m_pGAInfil.F = 0.0;
			pBmpA->m_pGAInfil.FU = 0.0;
			pBmpA->m_pGAInfil.FUmax = 0.0;
			pBmpA->m_pGAInfil.IMD = 0.0;
			pBmpA->m_pGAInfil.IMDmax = 0.0;
			pBmpA->m_pGAInfil.Ks = 0.0;
			pBmpA->m_pGAInfil.L = 0.0;
			pBmpA->m_pGAInfil.S = 0.0;
			pBmpA->m_pGAInfil.Sat = FALSE;
			pBmpA->m_pGAInfil.T = 0.0;

			// horton parameters
			pBmpA->m_pHortonInfil.decay = 0.0;
			pBmpA->m_pHortonInfil.f0 = 0.0;
			pBmpA->m_pHortonInfil.Fmax = 0.0;
			pBmpA->m_pHortonInfil.fmin = 0.0;
			pBmpA->m_pHortonInfil.regen = 0.0;
			pBmpA->m_pHortonInfil.tp = 0.0;

			//pump controls
			pBmpA->m_pPumpControl.m_nPumpFlag = 0;
			pBmpA->m_pPumpControl.m_bIsPumping = false;
			pBmpA->m_pPumpControl.m_strPCurveID = "";
			pBmpA->m_pPumpControl.m_lfDepth_ON = -1;	
			pBmpA->m_pPumpControl.m_lfDepth_OFF = 0;	
			pBmpA->m_pPumpCurve = NULL;

			m_pSiteProp = (void*) pBmpA;
			break;
		}
		case CLASS_B:
		{
			BMP_B* pBmpB = new BMP_B;
			pBmpB->m_lfBasinWidth = 0.0;
			pBmpB->m_lfBasinLength = 0.0;
			pBmpB->m_lfMaximumDepth = 0.0;
			pBmpB->m_lfSideSlope1 = 0.0;
			pBmpB->m_lfSideSlope2 = 0.0;
			pBmpB->m_lfSideSlope3 = 0.0;
			pBmpB->m_lfManning = 0.0;
			pBmpB->m_lfPETmultiplier = 1.0;
			pBmpB->m_lfWatDep_i = 0.;
			pBmpB->m_lfTheta_i = 0.;

			// green-ampt parameters
			pBmpB->m_pGAInfil.F = 0.0;
			pBmpB->m_pGAInfil.FU = 0.0;
			pBmpB->m_pGAInfil.FUmax = 0.0;
			pBmpB->m_pGAInfil.IMD = 0.0;
			pBmpB->m_pGAInfil.IMDmax = 0.0;
			pBmpB->m_pGAInfil.Ks = 0.0;
			pBmpB->m_pGAInfil.L = 0.0;
			pBmpB->m_pGAInfil.S = 0.0;
			pBmpB->m_pGAInfil.Sat = FALSE;
			pBmpB->m_pGAInfil.T = 0.0;

			// horton parameters
			pBmpB->m_pHortonInfil.decay = 0.0;
			pBmpB->m_pHortonInfil.f0 = 0.0;
			pBmpB->m_pHortonInfil.Fmax = 0.0;
			pBmpB->m_pHortonInfil.fmin = 0.0;
			pBmpB->m_pHortonInfil.regen = 0.0;
			pBmpB->m_pHortonInfil.tp = 0.0;

			m_pSiteProp = (void*) pBmpB;
			break;
		}
		case CLASS_C:	// conduit (01-2005)
		{
			BMP_C* pBmpC = new BMP_C;
			pBmpC->m_strCondType = "DUMMY";
			pBmpC->m_strCondName = "";
			pBmpC->m_strID = "";
			pBmpC->m_nIndex = -1;
	
			// --- initialize link properties
			pBmpC->m_pTLink.ID = "";				    // link ID
			pBmpC->m_pTLink.type = CONDUIT;				// link type code
			pBmpC->m_pTLink.subIndex = -1;			    // index of link's sub-category
			pBmpC->m_pTLink.rptFlag = FALSE;		    // reporting flag
			pBmpC->m_pTLink.node1 = 0;				    // start node index
			pBmpC->m_pTLink.node2 = 0;				    // end node index
			pBmpC->m_pTLink.z1 = 0.;				    // upstrm invert ht. above node invert (ft)
			pBmpC->m_pTLink.z2 = 0.;				    // downstrm invert ht. above node invert (ft)
			pBmpC->m_pTLink.q0 = 0.;				    // initial flow (cfs)
			pBmpC->m_pTLink.qLimit = 0.;			    // constraint on max. flow (cfs)
			pBmpC->m_pTLink.cLossInlet = 0.;		    // inlet loss coeff.
			pBmpC->m_pTLink.cLossOutlet = 0.;		    // outlet loss coeff.
			pBmpC->m_pTLink.cLossAvg = 0.;			    // avg. loss coeff.
			pBmpC->m_pTLink.hasFlapGate = FALSE;	    // true if flap gate present
			pBmpC->m_pTLink.oldFlow = 0.;			    // previous flow rate (cfs)
			pBmpC->m_pTLink.newFlow = 0.;			    // current flow rate (cfs)
			pBmpC->m_pTLink.oldDepth = 0.;			    // previous flow depth (ft)
			pBmpC->m_pTLink.newDepth = 0.;			    // current flow depth (ft)
			pBmpC->m_pTLink.oldVolume = 0.;				// previous flow volume (ft3)
			pBmpC->m_pTLink.newVolume = 0.;				// current flow volume (ft3)
			pBmpC->m_pTLink.qFull = 0.;					// flow when full (cfs)
			pBmpC->m_pTLink.setting = 0.;			    // control setting
			pBmpC->m_pTLink.froude = 0.;			    // Froude number
			pBmpC->m_pTLink.oldQual = NULL;				// previous quality state
			pBmpC->m_pTLink.newQual = NULL;				// current quality state
			pBmpC->m_pTLink.flowClass = -1;				// flow classification
			pBmpC->m_pTLink.dqdh = 0.;				    // change in flow w.r.t. head (ft2/sec)
			pBmpC->m_pTLink.direction = -1;				// flow direction flag
			pBmpC->m_pTLink.isClosed = FALSE;		    // flap gate closed flag
			pBmpC->m_pTLink.xsect.type = -1;		    // type code of cross section shape
			pBmpC->m_pTLink.xsect.transect = -1;	    // index of transect (if applicable)
			pBmpC->m_pTLink.xsect.yFull = 0.;		    // depth when full (ft)
			pBmpC->m_pTLink.xsect.wMax = 0.;		    // width at widest point (ft)
			pBmpC->m_pTLink.xsect.aFull = 0.;		    // area when full (ft2)
			pBmpC->m_pTLink.xsect.rFull = 0.;		    // hyd. radius when full (ft)
			pBmpC->m_pTLink.xsect.sFull = 0.;		    // section factor when full (ft^4/3)
			pBmpC->m_pTLink.xsect.sMax = 0.;		    // section factor at max. flow (ft^4/3)
			pBmpC->m_pTLink.xsect.yBot = 0.;		    // depth of bottom section
			pBmpC->m_pTLink.xsect.aBot = 0.;		    // area of bottom section
			pBmpC->m_pTLink.xsect.sBot = 0.;		    // slope of bottom section
			pBmpC->m_pTLink.xsect.rBot = 0.;			// radius of bottom section
			
			// --- initialize conduit properties
			pBmpC->m_pTConduit.length = 0.;				// conduit length (ft)
			pBmpC->m_pTConduit.roughness = 0.;		    // Manning's n
			pBmpC->m_pTConduit.barrels = 1;				// number of barrels
			pBmpC->m_pTConduit.modLength = 0.;		    // modified conduit length (ft)
			pBmpC->m_pTConduit.roughFactor = 0.;	    // roughness factor for DW routing
			pBmpC->m_pTConduit.slope = 0.;			    // slope
			pBmpC->m_pTConduit.beta = 0.;	    		// discharge factor
			pBmpC->m_pTConduit.qMax = 0.;	    		// max. flow (cfs)
			pBmpC->m_pTConduit.a1 = 0.;		    	    // upstream areas (ft2)
			pBmpC->m_pTConduit.a2 = 0.;		    	    // downstream areas (ft2)
			pBmpC->m_pTConduit.q1 = 0.;		    	    // upstream flows per barrel (cfs)
			pBmpC->m_pTConduit.q2 = 0.;					// downstream flows per barrel (cfs)
			pBmpC->m_pTConduit.q1Old = 0.;			    // previous values of q1 & q2 (cfs)
			pBmpC->m_pTConduit.q2Old = 0.;			    // previous values of q1 & q2 (cfs)
			pBmpC->m_pTConduit.superCritical = FALSE;	// super-critical flow flag
			pBmpC->m_pTConduit.hasLosses = FALSE;	    // local losses flag
			
			// --- initialize transect properties
			pBmpC->m_pTTransect.ID = "-999";            // section ID
			pBmpC->m_pTTransect.yFull = 0.;             // depth when full (ft)
			pBmpC->m_pTTransect.aFull = 0.;             // area when full (ft2)
			pBmpC->m_pTTransect.rFull = 0.;             // hyd. radius when full (ft)
			pBmpC->m_pTTransect.wMax = 0.;              // width at widest point (ft)
			pBmpC->m_pTTransect.sMax = 0.;              // section factor at max. flow (ft^4/3)
			pBmpC->m_pTTransect.aMax = 0.;              // area at max. flow (ft2)
			pBmpC->m_pTTransect.roughness = 0.;         // Manning's n
			for (int i=0; i<N_TRANSECT_TBL; i++)
			{
				pBmpC->m_pTTransect.areaTbl[i] = 0.0;	// table of area v. depth
				pBmpC->m_pTTransect.hradTbl[i] = 0.0;	// table of hyd. radius v. depth
				pBmpC->m_pTTransect.widthTbl[i] = 0.0;	// table of top width v. depth
			}
			pBmpC->m_pTTransect.nTbl = N_TRANSECT_TBL;	// size of geometry tablesnTbl = N_TRANSECT_TBL;

			m_pSiteProp = (void*) pBmpC;
			break;
		}
		case CLASS_D:	
		{
			//bufferstrip
			BMP_D* pBmpD = new BMP_D;
			pBmpD->m_strID = "";			
			pBmpD->m_nPolRemMethod = 0.0;
			pBmpD->m_lfdStore = 0.0;
			pBmpD->m_lfN = 0.01;
			pBmpD->m_lfalpha = 0.0;				
			pBmpD->m_lfinflow = 0.0;			
			pBmpD->m_lfrunoff = 0.0;			
			pBmpD->m_lfdepth = 0.0;	
			pBmpD->m_lflosses = 0.0;
			pBmpD->m_lfBufferWidth = 0.0;
			pBmpD->m_lfFlowLength = 0.0;
			pBmpD->m_lfPETmultiplier = 1.0;
			pBmpD->m_lfWatDep_i = 0.;
			pBmpD->m_lfTheta_i = 0.;
			pBmpD->m_lfOverlandSlope = 0.001;

			// green-ampt parameters
			pBmpD->m_pGAInfil.F = 0.0;
			pBmpD->m_pGAInfil.FU = 0.0;
			pBmpD->m_pGAInfil.FUmax = 0.0;
			pBmpD->m_pGAInfil.IMD = 0.0;
			pBmpD->m_pGAInfil.IMDmax = 0.0;
			pBmpD->m_pGAInfil.Ks = 0.0;
			pBmpD->m_pGAInfil.L = 0.0;
			pBmpD->m_pGAInfil.S = 0.0;
			pBmpD->m_pGAInfil.Sat = FALSE;
			pBmpD->m_pGAInfil.T = 0.0;

			// horton parameters
			pBmpD->m_pHortonInfil.decay = 0.0;
			pBmpD->m_pHortonInfil.f0 = 0.0;
			pBmpD->m_pHortonInfil.Fmax = 0.0;
			pBmpD->m_pHortonInfil.fmin = 0.0;
			pBmpD->m_pHortonInfil.regen = 0.0;
			pBmpD->m_pHortonInfil.tp = 0.0;

			m_pSiteProp = (void*) pBmpD;
			break;
		}
		case CLASS_E:
		{
			BMP_E* pBmpE = new BMP_E;
			pBmpE->m_strID = "";			
			pBmpE->m_nPolRemMethod = 0.0;
			pBmpE->m_lfdStore = 0.0;
			pBmpE->m_lfN = 0.01;
			pBmpE->m_lfSatInfil = 0.0;
			pBmpE->m_lfsArea = 0.0;
			pBmpE->m_lfalpha = 0.0;				
			pBmpE->m_lfinflow = 0.0;			
			pBmpE->m_lfrunoff = 0.0;			
			pBmpE->m_lfdepth = 0.0;	
			pBmpE->m_lflosses = 0.0;
			pBmpE->m_lfDCIA = 0.0;
			pBmpE->m_lfFlowLength = 0.0;
			pBmpE->m_lfOverlandSlope = 0.001;
			m_pSiteProp = (void*) pBmpE;
			break;
		}
		default:
			m_pSiteProp = NULL;
			break;
	}

	m_preLU = NULL;
	m_pAquifer = NULL;
	m_RAConc = NULL;
	m_fileOut = NULL;
	m_nQualNum = 0;
	m_nTSNum = 0;
	m_strCostFile = "";
	m_nBreakPoints = 0;
	m_lfBreakPtID = 0.0;
	m_TradeOff = NULL;
	m_pDataMixLU = NULL;
	m_pDataPreLU = NULL;
	m_tmStart = COleDateTime(1890,1,1,0,0,0);

	m_sediment.m_lfBEDWID = 0.0;
	m_sediment.m_lfBEDDEP = 0.0;
	m_sediment.m_lfBEDPOR = 0.0;
	m_sediment.m_lfSAND_FRAC = 0.0;
	m_sediment.m_lfSILT_FRAC = 0.0;
	m_sediment.m_lfCLAY_FRAC = 0.0;

	m_sand.m_lfD = 0.0;
	m_sand.m_lfW = 0.0;
	m_sand.m_lfRHO = 0.0;
	m_sand.m_lfKSAND = 0.0;
	m_sand.m_lfEXPSND = 0.0;

	m_silt.m_lfD = 0.0;
	m_silt.m_lfW = 0.0;
	m_silt.m_lfRHO = 0.0;
	m_silt.m_lfTAUCD = 0.0;
	m_silt.m_lfTAUCS = 0.0;
	m_silt.m_lfM = 0.0;

	m_clay.m_lfD = 0.0;
	m_clay.m_lfW = 0.0;
	m_clay.m_lfRHO = 0.0;
	m_clay.m_lfTAUCD = 0.0;
	m_clay.m_lfTAUCS = 0.0;
	m_clay.m_lfM = 0.0;

	m_nInfiltMethod = 0;
	m_nPolRotMethod = 1;
	m_nPolRemMethod = 0;
	m_nInfil_Index = 0;
	m_bUndSwitch = false;
	m_lfSoilDepth = 0.0;
	m_lfPorosity = 0.0;
	m_lfFCapacity = 0.3;
	m_lfWPoint = 0.15;
	m_lfInfilt = 0.0;
	m_lfUndDepth = 0.0;
	m_lfUndVoid = 0.0;
	m_lfUndInfilt = 0.0;
	m_holtanParam.m_lfVegA = 0.0;
//	m_holtanParam.m_lfFInfilt = 0.0;
	for (int i=0; i<12; i++)
		m_holtanParam.m_lfGrowth[i] = 0.0;

	m_costParam.m_lfLinearCost = 0.0;			
	m_costParam.m_lfAreaCost = 0.0;            				
	m_costParam.m_lfTotalVolumeCost = 0.0;     				
	m_costParam.m_lfMediaVolumeCost = 0.0;     				
	m_costParam.m_lfUnderDrainVolumeCost = 0.0;				
	m_costParam.m_lfConstantCost = 0.0;        				
	m_costParam.m_lfPercentCost = 0.0;         
	m_costParam.m_lfLengthExp = 0.0;         
	m_costParam.m_lfAreaExp = 0.0;         
	m_costParam.m_lfTotalVolExp = 0.0;         
	m_costParam.m_lfMediaVolExp = 0.0;         
	m_costParam.m_lfUDVolExp = 0.0;         
}

CBMPSite::~CBMPSite()
{
	if (m_pSiteProp != NULL)
	{
		switch (m_nBMPClass)
		{
			case CLASS_A:
				{
					delete (BMP_A*)m_pSiteProp;
					break;
				}
			case CLASS_B:
				{
					delete (BMP_B*)m_pSiteProp;
					break;
				}
			case CLASS_C: 
				{
					BMP_C* pBmpC = (BMP_C*)m_pSiteProp;
					if (pBmpC->m_pTLink.newQual != NULL)
						delete []pBmpC->m_pTLink.newQual;
					if (pBmpC->m_pTLink.oldQual != NULL)
						delete []pBmpC->m_pTLink.oldQual;
					delete (BMP_C*)m_pSiteProp;
					break;
				}
			case CLASS_D:
				{
					delete (BMP_D*)m_pSiteProp;
					break;
				}
			case CLASS_E:
				{
					delete (BMP_E*)m_pSiteProp;
					break;
				}
			default:
				break;
		}
	}
	
	if (m_TradeOff != NULL)
		delete []m_TradeOff;
	if (m_pDecay != NULL)
		delete []m_pDecay;
	if (m_pK != NULL)
		delete []m_pK;
	if (m_pCstar != NULL)
		delete []m_pCstar;
	if (m_pConc != NULL)
		delete []m_pConc;
	if (m_pUndRemoval != NULL)
		delete []m_pUndRemoval;
	if (m_RAConc != NULL)
		delete []m_RAConc;
	if (m_pDataMixLU != NULL)
		delete []m_pDataMixLU;
	if (m_pDataPreLU != NULL)
		delete []m_pDataPreLU;

	//Flow Duration Curve
	if (m_pPreDevBinID != NULL)
		delete []m_pPreDevBinID;
	if (m_pPreDevFlow != NULL)
		delete []m_pPreDevFlow;
	if (m_pPreDevBinCFS != NULL)
		delete []m_pPreDevBinCFS;
	if (m_pBinID != NULL)
		delete []m_pBinID;
	if (m_pFlow != NULL)
		delete []m_pFlow;
	if (m_pBinCFS != NULL)
		delete []m_pBinCFS;

	m_siteluList.RemoveAll();
	m_sitepsList.RemoveAll();

	while (!qFlow.empty())
		qFlow.pop();
	while (!m_dsbmpsiteList.IsEmpty())
		delete (DS_BMPSITE*) m_dsbmpsiteList.RemoveTail();
	while (!m_usbmpsiteList.IsEmpty())
		delete (US_BMPSITE*) m_usbmpsiteList.RemoveTail();
	while (!m_adjustList.IsEmpty())
		delete (ADJUSTABLE_PARAM*) m_adjustList.RemoveTail();
	while (!m_factorList.IsEmpty())
		delete (EVALUATION_FACTOR*) m_factorList.RemoveTail();
}

double* CBMPSite::GetVariablePointer(CString strVarName)
{
	if (m_nBMPClass == CLASS_A)
	{
		BMP_A* pBmpA = (BMP_A*) m_pSiteProp;
		if (strVarName.CompareNoCase("Width") == 0)
			return &pBmpA->m_lfBasinWidth;
		else if (strVarName.CompareNoCase("Length") == 0)
			return &pBmpA->m_lfBasinLength;
		else if (strVarName.CompareNoCase("OrificeH") == 0)
			return &pBmpA->m_lfOrificeHeight;
		else if (strVarName.CompareNoCase("OrificeD") == 0)
			return &pBmpA->m_lfOrificeDiameter;
		else if (strVarName.CompareNoCase("WeirH") == 0)
			return &pBmpA->m_lfWeirHeight;
		else if (strVarName.CompareNoCase("SDepth") == 0)
			return &m_lfSoilDepth;
		else if (strVarName.CompareNoCase("CECurve") == 0)
			return &m_lfBreakPtID;
		else if (strVarName.CompareNoCase("NUMUNIT") == 0)
			return &m_lfBMPUnit;
		else
			return NULL;
	}
	else if (m_nBMPClass == CLASS_B)
	{
		BMP_B* pBmpB = (BMP_B*) m_pSiteProp;
		if (strVarName.CompareNoCase("Width") == 0)
			return &pBmpB->m_lfBasinWidth;
		else if (strVarName.CompareNoCase("Length") == 0)
			return &pBmpB->m_lfBasinLength;
		else if (strVarName.CompareNoCase("MaxDepth") == 0)
			return &pBmpB->m_lfMaximumDepth;
		else if (strVarName.CompareNoCase("SDepth") == 0)
			return &m_lfSoilDepth;
		else if (strVarName.CompareNoCase("CECurve") == 0)
			return &m_lfBreakPtID;
		else if (strVarName.CompareNoCase("NUMUNIT") == 0)
			return &m_lfBMPUnit;
		else
			return NULL;
	}
	else if (m_nBMPClass == CLASS_D)
	{
		BMP_D* pBmpD = (BMP_D*) m_pSiteProp;
		if (strVarName.CompareNoCase("Length") == 0)
			return &pBmpD->m_lfFlowLength;
		else
			return NULL;
	}
	else if (m_nBMPClass == CLASS_E)
	{
		BMP_E* pBmpE = (BMP_E*) m_pSiteProp;
		if (strVarName.CompareNoCase("DCIA") == 0)
			return &pBmpE->m_lfDCIA;
		else
			return NULL;
	}
	else
	{
		if (strVarName.CompareNoCase("CECurve") == 0)
			return &m_lfBreakPtID;
	}
	
	return NULL;
}

double CBMPSite::GetBMPArea()
{
	if (m_nBMPClass == CLASS_A)
	{
		BMP_A* pBmpA = (BMP_A*) m_pSiteProp;
		return pBmpA->m_lfBasinWidth*pBmpA->m_lfBasinLength;
	}
	else if (m_nBMPClass == CLASS_B)
	{
		BMP_B* pBmpB = (BMP_B*) m_pSiteProp;
		//calculate the top width (ft)
		double top_width = (pBmpB->m_lfMaximumDepth/pBmpB->m_lfSideSlope1 
			+ pBmpB->m_lfMaximumDepth/pBmpB->m_lfSideSlope1 + pBmpB->m_lfBasinWidth); 
		//return pBmpB->m_lfBasinWidth*pBmpB->m_lfBasinLength;
		return top_width*pBmpB->m_lfBasinLength;
	}
	else if (m_nBMPClass == CLASS_D)
	{
		BMP_D* pBmpD = (BMP_D*) m_pSiteProp;
		return pBmpD->m_lfBufferWidth*pBmpD->m_lfFlowLength;
	}
	else if (m_nBMPClass == CLASS_E)
	{
		BMP_E* pBmpE = (BMP_E*) m_pSiteProp;
		return pBmpE->m_lfsArea;
	}
	return NULL;
}

// load the time series data
bool CBMPSite::LoadWatershedTSData(long deltm, CString BMPSiteID, COleDateTime startDate, COleDateTime endDate,
								   CString strFileName, double* multiplier, int landfg)
{
	int i, j;
	char strLine[MAXLINE];
	CString str;

	FILE *fpin = NULL;
	// open the file for reading
	fpin = fopen (strFileName, "rt");
	if(fpin == NULL)
		return false;

	// skip first two lines
	i = 2;
	while(i-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	//read the reporting timestep in seconds
	fgets(strLine, MAXLINE, fpin);
	str = strLine;
	CStringToken strToken1(str);
	double delts = atof((LPCSTR)strToken1.NextToken());	//sec/ivl

	//read the number of constituents
	fgets(strLine, MAXLINE, fpin);
	str = strLine;
	CStringToken strToken2(str);
	m_nQualNum = atoi((LPCSTR)strToken2.NextToken());	// including flow

	// skip constituents names
	i = m_nQualNum;
	while(i-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	//read the number of nodes
	fgets(strLine, MAXLINE, fpin);
	str = strLine;
	CStringToken strToken(str);
	int nNodes = atoi((LPCSTR)strToken.NextToken());
	CString strID;

	// check if the BMPSITE exist in the file
	i = nNodes;
	while(i-- > 0 && !feof(fpin))
	{
		fgets(strLine, MAXLINE, fpin);
		str = strLine;
		CStringToken strToken(str);
		strID = strToken.NextToken(); // node ID
		if (strID.CompareNoCase(BMPSiteID) == 0)
			break;
	}

	if (strID.CompareNoCase(BMPSiteID) != 0)
		return true;

	// skip lines until find the key word "node"
	CString strFind = "node";
	while (!feof(fpin))
	{
		fgets(strLine, MAXLINE, fpin);
		str = strLine;
		str.MakeLower();
		if(str.Find(strFind) != -1)
			break;
	}

	// count the time series numbers
	COleDateTimeSpan tsSpan = endDate - startDate;
	//m_nTSNum = (long)tsSpan.GetTotalHours() + 24;
	m_nTSNum = (long)tsSpan.GetTotalMinutes()/deltm + 1;
	
    // read first data line for starting date of the time series data
	long nStart = ftell (fpin);
	fgets(strLine, MAXLINE, fpin);
    fseek(fpin, nStart, SEEK_SET);

	int year, month, day, hour, minute, sec;
	str = strLine;
	CStringToken strToken3(str);
	strToken3.NextToken(); // skip the node number
	str = strToken3.NextToken();
	year = atoi((LPCSTR)str);

	str = strToken3.NextToken();
	month = atoi((LPCSTR)str);

	str = strToken3.NextToken();
	day = atoi((LPCSTR)str);
	
	str = strToken3.NextToken();
	hour = atoi((LPCSTR)str);

	str = strToken3.NextToken();
	minute = atoi((LPCSTR)str);

	str = strToken3.NextToken();
	sec = atoi((LPCSTR)str);

	m_tmStart = COleDateTime(year, month, day, 0, 0, 0);
	tsSpan = startDate - m_tmStart;

	// calculate how many lines we need to skip
	//long nSkipLineNum = (long)tsSpan.GetTotalHours();
	long nSkipLineNum = (long)tsSpan.GetTotalMinutes()/deltm;

    // skip all lines the time stamp is before the specified start date
	while (nSkipLineNum-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	double* pData = NULL;
	if (landfg == 0)
	{
		if(m_pDataPreLU != NULL)
			delete []m_pDataPreLU;
		m_pDataPreLU = new double[m_nTSNum*m_nQualNum];
		pData = m_pDataPreLU;
	}
	else
	{
		if(m_pDataMixLU != NULL)
			delete []m_pDataMixLU;
		m_pDataMixLU = new double[m_nTSNum*m_nQualNum];
		pData = m_pDataMixLU;
	}
	
    // read the data
	i = 0;

    while (!feof(fpin))
    {
		// read one line
		fgets(strLine, MAXLINE, fpin);

		// get the data
		str = strLine;
		CStringToken strToken(str);
		strID = strToken.NextToken(); // node ID
		if (strID.CompareNoCase(BMPSiteID) != 0)
			continue;
		strToken.NextToken(); // year
		strToken.NextToken(); // month
		strToken.NextToken(); // day
		strToken.NextToken(); // hour
		strToken.NextToken(); // minute
		strToken.NextToken(); // second
		double flow = 0.0;

		for(j=0; j<m_nQualNum; j++)
		{
			str = strToken.NextToken();
			if (j == 0)	// flow (cfs)
			{
				flow = atof((LPCSTR)str) * delts;	//ft3/sec * sec/ivl
				*(pData++) = flow / INACRE2CF;		// in-acre/ivl 
			}
			else // conc (mg/l)
			{
				*(pData++) = atof((LPCSTR)str) / LBpCFT2MGpL * flow;	//lb/ivl
			}
		}

		i++;
		if (i == m_nTSNum)
			break;
	}

	fclose(fpin);
	return (i == m_nTSNum);
}
// read the tradeoff curve solution file
bool CBMPSite::ReadTradeOffCurveCosts()
{
	int j, k;
	char strLine[MAXLINE];
	CString str;

	if (m_strCostFile == "")
		return true;

	FILE *fpin = NULL;
	// open the file for reading
	fpin = fopen (m_strCostFile, "rt");
	if(fpin == NULL)
		return false;

	// skip first three line
	j = 3;
	while(j-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);
	
	// read the data
	k = 1;
	while (!feof(fpin))
	{
		// read one line
		fgets(strLine, MAXLINE, fpin);

		// get the data
		str = strLine;
		CStringToken strToken1(str);
		int nBreak = atoi((LPCSTR)strToken1.NextToken());		// Target Break
		int nSolution = atoi((LPCSTR)strToken1.NextToken());	// Solution #
		double lfCost = atof((LPCSTR)strToken1.NextToken());	// total cost ($)

		if (nBreak == k && nSolution == 1)
		{
			m_TradeOff[k+2].m_lfCost = lfCost;	//0,1,2 for init, pre, and postdev conditions
			k++;
			if (k > m_nBreakPoints)
				break;
		}
	}

	fclose(fpin);

	return true;
}
    
// load the time series data
bool CBMPSite::LoadTradeOffCurveData(long deltm, int BPindex,COleDateTime startDate,COleDateTime endDate)
{
	if (m_TradeOff == NULL)
		return true;

	int i = BPindex, j, k;
	FILE *fpin = NULL;
	char strLine[MAXLINE];
	CString str;

	// open the file for reading
	fpin = fopen (m_TradeOff[i].m_strBrPtFile, "rt");
	if(fpin == NULL)
		return false;

	// find the flag "date/time"
	while (!feof(fpin))
	{
		fgets(strLine, MAXLINE, fpin);
		str = strLine;
		str.MakeLower();

		// skip no line once found "date/time"
		if(str.Find("date/time") != -1)
		{
			j = 0;
			while(j-- > 0 && !feof(fpin))
				fgets(strLine, MAXLINE, fpin);
			break;
		}
	}
	
	// count the time series numbers
	COleDateTimeSpan tsSpan = endDate - startDate;
	//m_TradeOff[i].m_nTSNum = (long)tsSpan.GetTotalHours() + 24;
	m_TradeOff[i].m_nTSNum = (long)tsSpan.GetTotalMinutes()/deltm + 1;

	double delts = deltm * 60.;	//sec/ivl

	// read first data line for starting date of the time series data
	long nStart = ftell (fpin);
	fgets(strLine, MAXLINE, fpin);
	fseek(fpin, nStart, SEEK_SET);

	int year, month, day, hour, minute;
	str = strLine;
	CStringToken strToken(str);
	strToken.NextToken(); // skip the dummy number
	str = strToken.NextToken();
	year = atoi((LPCSTR)str);

	str = strToken.NextToken();
	month = atoi((LPCSTR)str);

	str = strToken.NextToken();
	day = atoi((LPCSTR)str);
	
	str = strToken.NextToken();
	hour = atoi((LPCSTR)str);

	str = strToken.NextToken();
	minute = atoi((LPCSTR)str);

	if(hour == 24)																	
	{                          
		hour = 0;                  
		m_TradeOff[i].m_tmStart = COleDateTime(year, month, day, hour, minute, 0) + COleDateTimeSpan(1,0,0,0);
	}
	else
	{
		m_TradeOff[i].m_tmStart = COleDateTime(year, month, day, hour, minute, 0);
	}

	tsSpan = startDate - m_TradeOff[i].m_tmStart;

	// calculate how many lines we need to skip
	//long nSkipLineNum = (long)tsSpan.GetTotalHours();
	long nSkipLineNum = (long)tsSpan.GetTotalMinutes()/deltm;

	// skip all lines the time stamp is before the specified start date
	while (nSkipLineNum-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	if(m_TradeOff[i].m_pDataBrPt != NULL)
		delete []m_TradeOff[i].m_pDataBrPt;
	m_TradeOff[i].m_pDataBrPt = new double[m_TradeOff[i].m_nTSNum*m_TradeOff[i].m_nQualNum];

	// read the data
	k = 0;
	double *pData = m_TradeOff[i].m_pDataBrPt;
	
	while (!feof(fpin))
	{
		// read one line
		fgets(strLine, MAXLINE, fpin);

		// get the data
		str = strLine;
		CStringToken strToken1(str);
		strToken1.NextToken(); // serial number
		strToken1.NextToken(); // year
		strToken1.NextToken(); // month
		strToken1.NextToken(); // day
		strToken1.NextToken(); // hour
		strToken1.NextToken(); // minute
		strToken1.NextToken(); // volume
		strToken1.NextToken(); // stage     
		strToken1.NextToken(); // inflow_t  
		strToken1.NextToken(); // outflow_w 
		strToken1.NextToken(); // outflow_o 
		strToken1.NextToken(); // outflow_ud
		strToken1.NextToken(); // outflow_ut
		*(pData++) = atof((LPCSTR)strToken1.NextToken())*delts/INACRE2CF;//cfs to in-ac/ivl
		strToken1.NextToken(); // infiltration 
		strToken1.NextToken(); // percolation 
		strToken1.NextToken(); // evapotranspiration 
		strToken1.NextToken(); // seepage 

		for(j=1; j<m_TradeOff[i].m_nQualNum; j++)
		{
			strToken1.NextToken(); // inflow mass
			strToken1.NextToken(); // mass outflow from weir
			strToken1.NextToken(); // mass outflow from orifice
			strToken1.NextToken(); // mass outflow from underdrain
			strToken1.NextToken(); // mass outflow from untreated
			*(pData++) = atof((LPCSTR)strToken1.NextToken());// lb
			strToken1.NextToken(); // outflow concentration 
		}

		k++;
		if (k == m_TradeOff[i].m_nTSNum)
			break;
	}

	fclose(fpin);

	return true;
}
    
bool CBMPSite::UnLoadTradeOffCurveData(int nBrPtIndex)
{

	if (m_TradeOff == NULL)
		return true;

	if (m_nBreakPoints > 0)
	{
		if(m_TradeOff[nBrPtIndex].m_pDataBrPt != NULL)
		{
			delete []m_TradeOff[nBrPtIndex].m_pDataBrPt;
			m_TradeOff[nBrPtIndex].m_pDataBrPt = NULL;
		}
	}

	return true;
}

void CBMPSite::getSubareaRunoff2(float inflow, float precip, float evap, float tStep, float* infil, float* depth, float* runoff)
//
//  Input:   inflow = runoff over subarea (ft/sec)
//			 precip = rainfall + snowmelt over subarea (ft/sec)
//           evap = evaporation (ft/sec)
//           tStep = time step (sec/ivl)
//           infil = infiltration rate (ft/sec)
//           depth = depth of surface runoff (ft)
//           runoff = runoff for the current time step (ft/sec)
//  Output:  none
//  Purpose: computes runoff & losses from a subarea over the current time step.
//
{
    float  tRunoff = 0.0;           // time over which runoff occurs (sec)
    float  oldRunoff = 0.0;         // runoff from previous time period
    float  availMoisture = 0.0;     // sum of precipitation & ponded water (ft)
    float  xDepth = 0.0;            // ponded depth above dep. storage (ft)
    float  surfEvap = 0.0;          // evap. used for surface water (ft/sec)
    float  subsurfEvap = 0.0;       // evap. available for subsurface water
    float  losses = 0.0;			// evap. + infil. loss rate (ft/sec)

	float dStore = 0.0;	//depression storage (ft)
	float alpha = 0.0;	//overland flow factor
	float fArea = 0.0;	//fraction of subarea
	float N = 0.0;		//Manning's roughness coeff
	float infilt = 0.0;	//infiltration rate (ft/sec)

	if (m_nBMPClass == CLASS_D)
	{
		BMP_D* pBMP = (BMP_D*) m_pSiteProp;
		dStore = pBMP->m_lfdStore; 
		alpha = pBMP->m_lfalpha;
		fArea = 1.0;
		N = pBMP->m_lfN;
		infilt = *infil;
	}
	else if (m_nBMPClass == CLASS_E)
	{
		BMP_E* pBMP = (BMP_E*) m_pSiteProp;
		dStore = pBMP->m_lfdStore; 
		alpha = pBMP->m_lfalpha;
		fArea = (100.0-(float)pBMP->m_lfDCIA)/100.0;// fraction of disconnected impervious area (0-1)
		N = pBMP->m_lfN;
		infilt = pBMP->m_lfSatInfil; 
	}

    // --- assume runoff occurs over entire time step
    tRunoff = tStep;

    // --- initialize runoff & losses
    oldRunoff = *runoff;
    *runoff = 0.0;

    // --- no runoff if no area
    if ( fArea == 0.0 ) return;

    inflow += precip;
    availMoisture = inflow + *depth / tStep;
    surfEvap = MIN(availMoisture, evap);
    subsurfEvap = evap - surfEvap; 

    // --- compute infiltration loss rate for pervious subarea
    *infil = infilt;
    if ( *infil > availMoisture - surfEvap )
    {
        *infil = MAX(0.0, availMoisture - surfEvap);
    }

    // --- if losses exceed available moisture then there's no ponded water
    losses = surfEvap + *infil;
    if ( losses >= availMoisture )
    {
        losses = availMoisture;
        *depth = 0.0;

		if (m_nBMPClass == CLASS_D)
		{
			BMP_D* pBMP = (BMP_D*) m_pSiteProp;
			pBMP->m_lflosses = losses;
		}
		else if (m_nBMPClass == CLASS_E)
		{
			BMP_E* pBMP = (BMP_E*) m_pSiteProp;
			pBMP->m_lflosses = losses;
		}
    }

    // --- otherwise update depth of ponded water
    //     and time over which runoff occurs
    else 
	{
		if (m_nBMPClass == CLASS_D)
		{
			BMP_D* pBMP = (BMP_D*) m_pSiteProp;
			pBMP->m_lflosses = losses;
			pBMP->m_lfinflow = inflow; 
		}
		else if (m_nBMPClass == CLASS_E)
		{
			BMP_E* pBMP = (BMP_E*) m_pSiteProp;
			pBMP->m_lflosses = losses;
			pBMP->m_lfinflow = inflow; 
		}

		updatePondedDepth2(depth, &tRunoff);
	}

    // --- compute runoff based on updated ponded depth
    xDepth = *depth - dStore;
    if ( xDepth > MIN_EXCESS_DEPTH )
    {
        // --- case where nonlinear routing is used
        if ( N > 0.0 )
        {
			float mexp = 5./3.; // exponent in Manning Eq.
            *runoff = alpha * pow(xDepth, mexp);
        }

        // --- case where no routing is used (Mannings N = 0)
        else
        {
            *runoff = xDepth / tRunoff;
            *depth = dStore;
        }
    }
    else 
	{
		*runoff = 0.0;
	}
}

void CBMPSite::updatePondedDepth2(float* depth, float* dt)
//
//  Input:   depth = depth of surface runoff (ft)
//           dt = time step (sec)
//  Output:  dt = time ponded depth is above depression storage (sec)
//  Purpose: computes new ponded depth over subarea after current time step.
//
{
    float ix = 0.0;	// excess inflow to subarea (ft/sec)
    float dx = 0.0; // depth above depression storage (ft)
    float tx = *dt; // time over which dx > 0 (sec)

	float dStore = 0.0;	//depression storage (ft)
	float inflow = 0.0; //inflow rate (ft/sec)
	float losses = 0.0; //evap. + infil. loss rate (ft/sec)
	float alpha = 0.0;	//overland flow factor
	
	theBUFFERSTRIP = NULL;
	theAREABMP = NULL;

	if (m_nBMPClass == CLASS_D)
	{
		BMP_D* pBMP = (BMP_D*) m_pSiteProp;
		dStore = pBMP->m_lfdStore;
		inflow = pBMP->m_lfinflow;
		losses = pBMP->m_lflosses;
		alpha = pBMP->m_lfalpha;
		theBUFFERSTRIP = pBMP;
	}
	else if (m_nBMPClass == CLASS_E)
	{
		BMP_E* pBMP = (BMP_E*) m_pSiteProp;
		dStore = pBMP->m_lfdStore;
		inflow = pBMP->m_lfinflow;
		losses = pBMP->m_lflosses;
		alpha = pBMP->m_lfalpha;
		theAREABMP = pBMP;
	}
    // --- excess inflow = total inflow - losses
    ix = inflow - losses;

    // --- see if not enough inflow to fill depression storage (dStore)
    if ( *depth + ix*tx <= dStore )
    {
        *depth += ix * tx;
    }

    // --- otherwise use the ODE solver to integrate flow depth
    else
    {
        // --- if depth < dStore then fill up dStore & reduce time step
        dx = dStore - *depth;
        if ( dx > 0.0 && ix > 0.0 )
        {
            tx -= dx / ix;
            *depth = dStore;
        }

        // --- now integrate depth over remaining time step tx
        if ( alpha > 0.0 && tx > 0.0)
        {
			float odetol = 0.0001; // acceptable error for ODE solver
			if (m_nBMPClass == CLASS_D)
	            odesolve_integrate(depth, 1, 0, tx, odetol, tx, getDdDt2);
			else if (m_nBMPClass == CLASS_E)
	            odesolve_integrate(depth, 1, 0, tx, odetol, tx, getDdDt3);
        }
        else
        {
            if ( tx < 0.0 ) tx = 0.0;
            *depth += ix * tx;
        }
    }

    // --- do not allow ponded depth to go negative
    if ( *depth < 0.0 ) *depth = 0.0;

    // --- replace original time step with time ponded depth
    //     is above depression storage
    *dt = tx;
}

bool CBMPSite::ReadPreDevFile(int landtimestep,COleDateTime startDate,COleDateTime endDate)
{
	long ii,jj,nTSNum;
	bool checkread = false;

	POSITION pos; 
	pos = m_factorList.GetHeadPosition();
	while (pos != NULL)
	{
		EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) m_factorList.GetNext(pos);
		if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == -4)
		{
			checkread = true;
			break;
		}
	}

	if(checkread == false)
		return true;

	if(m_strPreDevFile.GetLength() == 0)
		return false;

	FILE *fpin = NULL;
	char strLine[MAXLINE];
	
	// open the file for reading
	fpin = fopen (m_strPreDevFile, "rt");
	if(fpin == NULL)
	{
		AfxMessageBox("Cannot open file " + m_strPreDevFile + " for reading");
		return false;
	}

	// count the total simulation intervals
	COleDateTimeSpan tsSpan = endDate - startDate;
	nTSNum = long(tsSpan.GetTotalMinutes()/landtimestep + 0.5);
	
	// skip the first header line
	fgets(strLine, MAXLINE, fpin);

    // read first data line for start date of the timeseries
	long nStart = ftell (fpin);
	fgets(strLine, MAXLINE, fpin);
    fseek(fpin, nStart, SEEK_SET);

	int year, month, day, hour, minute;
	CString str = strLine;
	CStringToken strToken1(str);
	strToken1.NextToken(); // skip the first value
	
	str = strToken1.NextToken();
	year = atoi((LPCSTR)str);

	str = strToken1.NextToken();
	month = atoi((LPCSTR)str);

	str = strToken1.NextToken();
	day = atoi((LPCSTR)str);
	
	str = strToken1.NextToken();
	hour = atoi((LPCSTR)str);
	
	str = strToken1.NextToken();
	minute = atoi((LPCSTR)str);
	
	COleDateTime tmStart = COleDateTime(year, month, day, hour, minute, 0);
	tsSpan = startDate - tmStart;

	// calculate how many lines need to skip
	long nSkipLineNum = long(tsSpan.GetTotalMinutes()/landtimestep + 0.5);

    // skip all lines the time stamp is before the specified start date
	while (nSkipLineNum-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	// allocate memory
	if(m_pPreDevFlow!= NULL)	
		delete[]m_pPreDevFlow;
	m_pPreDevFlow = new double[nTSNum];

	//initialize
	m_lfMinFlow = 1e12;
	m_lfMaxFlow = 0.;
	for (ii=0; ii<nTSNum; ii++)
		m_pPreDevFlow[ii] = 0.;
	
	// read the data
	ii = 0;
	while (fgets(strLine, MAXLINE, fpin) != NULL)
	{
		CString str(strLine);
		CStringToken strToken2(str);
		//skip the first dummy number
		strToken2.NextToken();
		
		//read the date
		int nYear = atoi(LPCSTR(strToken2.NextToken()));
		int nMonth = atoi(LPCSTR(strToken2.NextToken()));
		int nDay = atoi(LPCSTR(strToken2.NextToken()));
		int nHour = atoi(LPCSTR(strToken2.NextToken()));
		int nMinute = atoi(LPCSTR(strToken2.NextToken()));
		//read the flow (cfs)
		m_pPreDevFlow[ii] = atof(LPCSTR(strToken2.NextToken()));

		if(m_pPreDevFlow[ii] < m_lfMinFlow)
			m_lfMinFlow = m_pPreDevFlow[ii];
		if(m_pPreDevFlow[ii] > m_lfMaxFlow)
			m_lfMaxFlow = m_pPreDevFlow[ii];

		ii++;
		if (ii >= nTSNum)
			break;
	}

	//close the predeveloped timeseries file
	fclose(fpin);

	return true;
}

bool CBMPSite::ProcessPreDevData(int landtimestep,COleDateTime startDate,COleDateTime endDate, CString strOutputDir)
{
	CString strValue = "", strCurrent = "";
	bool checkread = false;
	int i,j,nIndex,nPrevIndex,nNextIndex,nCounter,nCounter2;
	int nZeroFlowCount,nZeroFlowCount2,nNormFlowBin,nBinRecords;
	long ii,lTSNum;
	double lfBinRecords = 0.,lfweightedPrevFlow = 0.,lfweightedNextFlow = 0.;
	double lowflowlimit = 0;	//cfs
	double highflowlimit = 0;	//cfs

	POSITION pos; 
	pos = m_factorList.GetHeadPosition();
	while (pos != NULL)
	{
		EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) m_factorList.GetNext(pos);
		if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == -4)
		{
			lowflowlimit = ef->m_lfLowFlowLimit;
			highflowlimit = ef->m_lfHighFlowLimit;
			checkread = true;
			break;
		}
	}

	if(checkread == false)
		return true;

	if(m_pPreDevFlow == NULL)
		return false;

	//open the FDC output file
	if (!Open_FDC_OutputFiles("PreDev", strOutputDir))	// open file for evaluation factor
		return false;

	//count the total flow timeseries values
	COleDateTimeSpan tsSpan = endDate - startDate;
	lTSNum = long(tsSpan.GetTotalMinutes()/landtimestep + 0.5);
	
	//calculate the number of records in one bin
	lfBinRecords = (double)lTSNum / m_nNumBins;

	//assign memory and initialize
	int* nBinFlowCounts = NULL;
	double* lfBinFlowSum = NULL;
	if(m_pPreDevBinCFS!= NULL)	
		delete[]m_pPreDevBinCFS;
	
	nBinFlowCounts = new int[m_nNumBins];
	lfBinFlowSum = new double[m_nNumBins];
	m_pPreDevBinCFS = new double[m_nNumBins];
	
	for(i=0; i<m_nNumBins; i++)
	{
		nBinFlowCounts[i] = 0;
		lfBinFlowSum[i] = 0.;
		m_pPreDevBinCFS[i] = 0.;
	}

	//normalize the flow values to the number of bins
	nZeroFlowCount = 0;
	for (ii=0; ii<lTSNum; ii++)
	{
		//count the zero flow values
		if(m_pPreDevFlow[ii] == 0)
			nZeroFlowCount++;

		nNormFlowBin = int(m_pPreDevFlow[ii]*m_nNumBins/m_lfMaxFlow + 0.5);
		if(nNormFlowBin == 0)
			nNormFlowBin = 1;

		nBinFlowCounts[nNormFlowBin-1]++;
		lfBinFlowSum[nNormFlowBin-1] += m_pPreDevFlow[ii];//cfs
	}

	nZeroFlowCount2 = nZeroFlowCount;

	//write to the FDC output file		
	strCurrent.Format("%.5lf\t%.5lf\n", lowflowlimit, highflowlimit);
	strValue += strCurrent;
	//minimum flow value
	strCurrent.Format("%s\t%.5lf\n", m_strID, m_lfMinFlow);
	strValue += strCurrent;

	//calculate the average flow in each bin
	nCounter = 0;
	nCounter2 = 0;
	nIndex = 0;
	for (i=0; i<m_nNumBins; i++)
	{
		//skip the bin if there is no flow value
		if(nBinFlowCounts[i] == 0)
			continue;

		//calculate the previous index of a bin with flow value
		nPrevIndex = i;
		for (j=i; j>0; j--)
		{
			if(nBinFlowCounts[j-1] > 0)
			{
				nPrevIndex = j-1;
				break;
			}
		}

		//calculate the next index of a bin with flow value
		nNextIndex = i;
		for (j=i; j<m_nNumBins-1; j++)
		{
			if(nBinFlowCounts[j+1] > 0)
			{
				nNextIndex = j+1;
				break;
			}
		}

		if(nPrevIndex == i)
			// it is the first bin with flow values
			lfweightedPrevFlow = m_lfMinFlow;
		else if(nPrevIndex == 0)
			//calculate the weighted average flow with the previous bin (ignore zeros)
			lfweightedPrevFlow = (lfBinFlowSum[nPrevIndex]+lfBinFlowSum[i])/(nBinFlowCounts[nPrevIndex]-nZeroFlowCount+nBinFlowCounts[i]);
		else
			//calculate the weighted average flow with the previous bin
			lfweightedPrevFlow = (lfBinFlowSum[nPrevIndex]+lfBinFlowSum[i])/(nBinFlowCounts[nPrevIndex]+nBinFlowCounts[i]);

		if(nNextIndex == i)
			// it is the last bin with flow values
			lfweightedNextFlow = m_lfMaxFlow;
		else if(i == 0)
			//calculate the weighted average flow with the next bin (ignore zeros)
			lfweightedNextFlow = (lfBinFlowSum[i]+lfBinFlowSum[nNextIndex])/(nBinFlowCounts[i]-nZeroFlowCount+nBinFlowCounts[nNextIndex]);
		else
			//calculate the weighted average flow with the next bin
			lfweightedNextFlow = (lfBinFlowSum[i]+lfBinFlowSum[nNextIndex])/(nBinFlowCounts[i]+nBinFlowCounts[nNextIndex]);

		//add flow values to each bin
		for (j=0; j<nBinFlowCounts[i]; j++)
		{
			nBinRecords = int(lfBinRecords*(nIndex+1) + 0.5);
			if(nCounter2 >= nBinRecords && nIndex < (m_nNumBins-1))
			{
				//calculate the average flow in each bin (cfs)
				m_pPreDevBinCFS[nIndex] /= nCounter;

				//write to the output file
				strCurrent.Format("%s\t%.5lf\n", m_strID, m_pPreDevBinCFS[nIndex]);
				strValue += strCurrent;

				nCounter = 0;
				nIndex++;
			}

			//m_pPreDevBinCFS[nIndex] += lfBinFlowSum[i]/nBinFlowCounts[i];//cfs
			if(nZeroFlowCount2 > 0)
			{
				//m_pPreDevBinCFS[nIndex] += 0.;
				nZeroFlowCount2--;
			}
			else if(i==0)
			{
				m_pPreDevBinCFS[nIndex] += (lfweightedPrevFlow+(lfweightedNextFlow-lfweightedPrevFlow)*(j+1-nZeroFlowCount)/(nBinFlowCounts[i]-nZeroFlowCount));//cfs
			}
			else
			{
				m_pPreDevBinCFS[nIndex] += (lfweightedPrevFlow+(lfweightedNextFlow-lfweightedPrevFlow)*(j+1)/nBinFlowCounts[i]);//cfs
			}

			nCounter++;
			nCounter2++;
		}
	}

	//calculate the average flow for the last bin (cfs)
	m_pPreDevBinCFS[nIndex] /= nCounter;

	//write to the FDC output file		
	strCurrent.Format("%s\t%.5lf\n", m_strID, m_pPreDevBinCFS[nIndex]);
	strValue += strCurrent;
	//maximum flow value
	strCurrent.Format("%s\t%.5lf\n", m_strID, m_lfMaxFlow);
	strValue += strCurrent;

	fputs(LPCSTR(strValue), fp_FDC);

	//find the cutoff bin for min/max flow limit
	for (i=0; i<m_nNumBins; i++)
	{
		m_nStartBin = 0;
		if(m_pPreDevBinCFS[i] < lowflowlimit)
			m_nStartBin = i+1;
		if(highflowlimit >= m_pPreDevBinCFS[i])
			m_nEndBin = i+1;
	}

	//release the memory
	if(nBinFlowCounts != NULL)
		delete []nBinFlowCounts;
	if(lfBinFlowSum != NULL)
		delete []lfBinFlowSum;
	if(m_pPreDevFlow != NULL)
	{
		delete []m_pPreDevFlow;
		m_pPreDevFlow = NULL;
	}

	//close FDC file
	Close_FDC_OutputFiles();

	return true;
}

bool CBMPSite::ProcessSimRunData(CString strRunMode,int landtimestep,COleDateTime startDate,COleDateTime endDate,CString strOutputDir)
{
	CString strValue = "", strCurrent = "";
	bool checkread = false;
	int i,j,nIndex,nPrevIndex,nNextIndex,nCounter,nCounter2;
	int nZeroFlowCount,nZeroFlowCount2,nNormFlowBin,nBinRecords;
	double lfBinRecords = 0.,lfweightedPrevFlow = 0.,lfweightedNextFlow = 0.;
	long ii,lTSNum;

	POSITION pos; 
	pos = m_factorList.GetHeadPosition();
	while (pos != NULL)
	{
		EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) m_factorList.GetNext(pos);
		if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == -4)
		{
			checkread = true;
			break;
		}
	}

	if(checkread == false)
		return true;

	if(m_pFlow == NULL)
		return false;

	//open the FDC output file
	if(strRunMode.GetLength() > 0)
	{
		if (!Open_FDC_OutputFiles(strRunMode, strOutputDir))	// open file for evaluation factor
			return false;
	}

	//count the total flow timeseries values
	COleDateTimeSpan tsSpan = endDate - startDate;
	lTSNum = long(tsSpan.GetTotalMinutes()/landtimestep + 0.5);
	
	//calculate the number of records in one bin
	lfBinRecords = (double)lTSNum / m_nNumBins;

	//assign memory and initialize
	int* nBinFlowCounts = NULL;
	double* lfBinFlowSum = NULL;
	if(m_pBinCFS!= NULL)	
		delete[]m_pBinCFS;
	
	nBinFlowCounts = new int[m_nNumBins];
	lfBinFlowSum = new double[m_nNumBins];
	m_pBinCFS = new double[m_nNumBins];
	
	for(i=0; i<m_nNumBins; i++)
	{
		nBinFlowCounts[i] = 0;
		lfBinFlowSum[i] = 0.;
		m_pBinCFS[i] = 0.;
	}

	//normalize the flow values to the number of bins
	nZeroFlowCount = 0;
	for (ii=0; ii<lTSNum; ii++)
	{
		//count the zero flow values
		if(m_pFlow[ii] == 0)
			nZeroFlowCount++;

		nNormFlowBin = int(m_pFlow[ii]*m_nNumBins/m_lfMaxFlow + 0.5);
		if(nNormFlowBin == 0)
			nNormFlowBin = 1;

		nBinFlowCounts[nNormFlowBin-1]++;
		lfBinFlowSum[nNormFlowBin-1] += m_pFlow[ii];//cfs
	}

	nZeroFlowCount2 = nZeroFlowCount;

	//write minimum flow value to the FDC output file		
	strCurrent.Format("%s\t%.5lf\n", m_strID, m_lfMinFlow);
	strValue += strCurrent;

	//calculate the average flow in each bin
	nCounter = 0;
	nCounter2 = 0;
	nIndex = 0;
	for (i=0; i<m_nNumBins; i++)
	{
		//skip the bin if there is no flow value
		if(nBinFlowCounts[i] == 0)
			continue;

		//calculate the previous index of a bin with flow value
		nPrevIndex = i;
		for (j=i; j>0; j--)
		{
			if(nBinFlowCounts[j-1] > 0)
			{
				nPrevIndex = j-1;
				break;
			}
		}

		//calculate the next index of a bin with flow value
		nNextIndex = i;
		for (j=i; j<m_nNumBins-1; j++)
		{
			if(nBinFlowCounts[j+1] > 0)
			{
				nNextIndex = j+1;
				break;
			}
		}

		if(nPrevIndex == i)
			// it is the first bin with flow values
			lfweightedPrevFlow = m_lfMinFlow;
		else if(nPrevIndex == 0)
			//calculate the weighted average flow with the previous bin (ignore zeros)
			lfweightedPrevFlow = (lfBinFlowSum[nPrevIndex]+lfBinFlowSum[i])/(nBinFlowCounts[nPrevIndex]-nZeroFlowCount+nBinFlowCounts[i]);
		else
			//calculate the weighted average flow with the previous bin
			lfweightedPrevFlow = (lfBinFlowSum[nPrevIndex]+lfBinFlowSum[i])/(nBinFlowCounts[nPrevIndex]+nBinFlowCounts[i]);

		if(nNextIndex == i)
			// it is the last bin with flow values
			lfweightedNextFlow = m_lfMaxFlow;
		else if(i == 0)
			//calculate the weighted average flow with the next bin (ignore zeros)
			lfweightedNextFlow = (lfBinFlowSum[i]+lfBinFlowSum[nNextIndex])/(nBinFlowCounts[i]-nZeroFlowCount+nBinFlowCounts[nNextIndex]);
		else
			//calculate the weighted average flow with the next bin
			lfweightedNextFlow = (lfBinFlowSum[i]+lfBinFlowSum[nNextIndex])/(nBinFlowCounts[i]+nBinFlowCounts[nNextIndex]);

		//add flow values to each bin
		for (j=0; j<nBinFlowCounts[i]; j++)
		{
			nBinRecords = int(lfBinRecords*(nIndex+1) + 0.5);
			if(nCounter2 >= nBinRecords && nIndex < (m_nNumBins-1))
			{
				//calculate the average flow in each bin (cfs)
				m_pBinCFS[nIndex] /= nCounter;
				
				if(strRunMode.GetLength() > 0)
				{
					//write to the output file
					strCurrent.Format("%s\t%.5lf\n", m_strID, m_pBinCFS[nIndex]);
					strValue += strCurrent;
				}

				nCounter = 0;
				nIndex++;
			}

			//m_pBinCFS[nIndex] += lfBinFlowSum[i]/nBinFlowCounts[i];//cfs
			if(nZeroFlowCount2 > 0)
			{
				//m_pBinCFS[nIndex] += 0.;
				nZeroFlowCount2--;
			}
			else if(i==0)
			{
				m_pBinCFS[nIndex] += (lfweightedPrevFlow+(lfweightedNextFlow-lfweightedPrevFlow)*(j+1-nZeroFlowCount)/(nBinFlowCounts[i]-nZeroFlowCount));//cfs
			}
			else
			{
				m_pBinCFS[nIndex] += (lfweightedPrevFlow+(lfweightedNextFlow-lfweightedPrevFlow)*(j+1)/nBinFlowCounts[i]);//cfs
			}

			nCounter++;
			nCounter2++;
		}
	}

	//calculate the average flow for the last bin (cfs)
	m_pBinCFS[nIndex] /= nCounter;

	if(strRunMode.GetLength() > 0)
	{
		//write to the FDC output file		
		strCurrent.Format("%s\t%.5lf\n", m_strID, m_pBinCFS[nIndex]);
		strValue += strCurrent;
		//maximum flow value
		strCurrent.Format("%s\t%.5lf\n", m_strID, m_lfMaxFlow);
		strValue += strCurrent;
		fputs(LPCSTR(strValue), fp_FDC);
	}

	//release the memory
	if(nBinFlowCounts != NULL)
		delete []nBinFlowCounts;
	if(lfBinFlowSum != NULL)
		delete []lfBinFlowSum;
	if(m_pFlow != NULL)
	{
		delete []m_pFlow;
		m_pFlow = NULL;
	}

	if(strRunMode.GetLength() > 0)
	{
		//close FDC file
		Close_FDC_OutputFiles();
	}

	return true;
}

bool CBMPSite::Open_FDC_OutputFiles(CString strRunMode, CString strOutputDir)
{
	CString strFilePath;
	strFilePath.Format("%s_FDC.out", strRunMode);
	strFilePath = strOutputDir + strFilePath;
	fp_FDC = fopen(LPCSTR(strFilePath), "wt");
	if(fp_FDC == NULL)
	{
		AfxMessageBox("Cannot open file " + strFilePath + " for writing.");
		return false;
	}

	Write_FDC_FileHeader(strRunMode);

	return true;
}

void CBMPSite::Write_FDC_FileHeader(CString strRunMode)
{
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_FDC);
	fputs("TT\n",fp_FDC);
	fputs("TT SUSTAIN: System for Urban Stormwater Treatment and Analysis INtegration\n",fp_FDC);
	fputs("TT\n",fp_FDC);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_FDC);
	fputs("TT BMP Site Assessment Results for Flow Duration Curve\n",fp_FDC);
	fprintf(fp_FDC, "TT %s\n",SUSTAIN_VERSION);
	fputs("TT\n",fp_FDC);
	fputs("TT Designed and maintained by:\n",fp_FDC);
	fputs("TT     Tetra Tech, Inc.\n",fp_FDC);
	fputs("TT     10306 Eaton Place, Suite 340\n",fp_FDC);
	fputs("TT     Fairfax, VA 22030\n",fp_FDC);
	fputs("TT     (703) 385-6000\n",fp_FDC);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_FDC);
	fputs("TT	\n", fp_FDC);
	SYSTEMTIME tm;
	GetLocalTime(&tm);
	CString str;
	str.Format("TT This output file was created at %02d:%02d:%02d%s on %02d/%02d/%04d\n",(tm.wHour>12)?tm.wHour-12:tm.wHour,tm.wMinute,tm.wSecond,(tm.wHour>=12)?"pm":"am",tm.wMonth,tm.wDay,tm.wYear);
	fputs(LPCSTR(str),fp_FDC);
	fputs("TT    \n", fp_FDC);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_FDC);
	fputs("TT    \n", fp_FDC);

	if(strRunMode.CompareNoCase("PreDev") == 0)
	{
		fprintf(fp_FDC, "TT LowFlow       Low flow limit (cfs) for predeveloped condition at the assessment point\n");
		fprintf(fp_FDC, "TT HighFlow      High flow limit (cfs) for predeveloped condition at the assessment point\n");
		fputs("TT    \n", fp_FDC);
	}

	fprintf(fp_FDC, "TT BMPSiteID     BMP site identifier in card 715 and is an assessment point\n");
	fprintf(fp_FDC, "TT Flow          Sorted flow rate (cfs) at the assessment point\n");
	fputs("TT    \n", fp_FDC);
	fflush(fp_FDC);
}				

bool CBMPSite::Close_FDC_OutputFiles()
{
	if (fp_FDC != NULL)
	{
		fclose(fp_FDC);
		fp_FDC = NULL;
	}

	return true;
}

void  getDdDt2(float t, float* d, float* dddt)
//
//  Input:   t = current time (not used)
//           d = stored depth (ft)
//  Output   dddt = derivative of d with respect to time
//  Purpose: evaluates derivative of stored depth w.r.t. time
//           for the BUFFERSTRIP whose runoff is being computed.
//
{
	float dStore = 0.0;	//depression storage (ft)
	float inflow = 0.0;	//inflow rate (ft/sec)
	float losses = 0.0;	//evap. + infil. loss rate (ft/sec)
	float alpha = 0.0;	//overland flow factor
	
	if (theBUFFERSTRIP != NULL)
	{
		dStore = theBUFFERSTRIP->m_lfdStore;	//depression storage (ft)
		inflow = theBUFFERSTRIP->m_lfinflow;	//inflow rate (ft/sec)
		losses = theBUFFERSTRIP->m_lflosses;	//evap. + infil. loss rate (ft/sec)
		alpha = theBUFFERSTRIP->m_lfalpha;		//overland flow factor
	}

    float ix = inflow - losses; //ft/sec
    float rx = *d - dStore; //ft
    if ( rx < 0.0 )
    {
        rx = 0.0;
    }
    else
    {
		float mexp = 5./3.; // exponent in Manning Eq.
        rx = alpha * pow(rx, mexp);
    }
    *dddt = ix - rx;
}

void  getDdDt3(float t, float* d, float* dddt)
//
//  Input:   t = current time (not used)
//           d = stored depth (ft)
//  Output   dddt = derivative of d with respect to time
//  Purpose: evaluates derivative of stored depth w.r.t. time
//           for the AREABMP whose runoff is being computed.
//
{
	float dStore = 0.0;	//depression storage (ft)
	float inflow = 0.0;	//inflow rate (ft/sec)
	float losses = 0.0;	//evap. + infil. loss rate (ft/sec)
	float alpha = 0.0;	//overland flow factor
	
	if (theAREABMP != NULL)
	{
		dStore = theAREABMP->m_lfdStore;	//depression storage (ft)
		inflow = theAREABMP->m_lfinflow;	//inflow rate (ft/sec)
		losses = theAREABMP->m_lflosses;	//evap. + infil. loss rate (ft/sec)
		alpha = theAREABMP->m_lfalpha;		//overland flow factor
	}

    float ix = inflow - losses; //ft/sec
    float rx = *d - dStore; //ft
    if ( rx < 0.0 )
    {
        rx = 0.0;
    }
    else
    {
		float mexp = 5./3.; // exponent in Manning Eq.
        rx = alpha * pow(rx, mexp);
    }
    *dddt = ix - rx;
}
