// Ftable.cpp : implementation file
//

#include "stdafx.h"
#include "Ftable.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFtable

CFtable::CFtable()
{
	m_strFtableID = "";			// Unique Ftable identifier
	m_nRecords = 0;				// Number of records in table
	m_lfLength = 0.;			// Flow length (ft)
	m_lfSlope = 0.;				// Slope of flow length (ft/ft)
	m_lfDepth = NULL;			// array of depth (ft)
	m_lfArea = NULL;			// array of surface area (ft2)
	m_lfVolume = NULL;			// array of storage volume (ft3)
	m_lfWetPerimeter = NULL;	// array of wetted perimeter (ft)
	m_lfWeirOutflow = NULL;		// array of weir outflow rate (cfs)
	m_lfOrificeOutflow = NULL;	// array of orifice outflow rate (cfs)
}

CFtable::CFtable(CString strFtableID,int nRecords,double lfLength,double lfSlope,
				 double* lfDepth,double* lfArea,double* lfVolume,double* lfWetPerimeter,
				 double* lfWeirOutflow,double* lfOrificeOutflow)
{
	m_strFtableID = strFtableID;		
	m_nRecords = nRecords;			
	m_lfLength = lfLength;			
	m_lfSlope = lfSlope;			
	m_lfDepth = new double[nRecords];
	m_lfArea = new double[nRecords];
	m_lfVolume = new double[nRecords];
	m_lfWetPerimeter = new double[nRecords];
	m_lfWeirOutflow = new double[nRecords];
	m_lfOrificeOutflow = new double[nRecords];
	for (int i=0; i<nRecords; i++)
	{
		m_lfDepth[i] = lfDepth[i];
		m_lfArea[i] = lfArea[i];
		m_lfVolume[i] = lfVolume[i];
		m_lfWetPerimeter[i] = lfWetPerimeter[i];
		m_lfWeirOutflow[i] = lfWeirOutflow[i];
		m_lfOrificeOutflow[i] = lfOrificeOutflow[i];
	}
}

CFtable::~CFtable()
{
	if(m_lfDepth != NULL)
		delete []m_lfDepth;
	if(m_lfArea != NULL)
		delete []m_lfArea;
	if(m_lfVolume != NULL)
		delete []m_lfVolume;
	if(m_lfWetPerimeter != NULL)
		delete []m_lfWetPerimeter;
	if(m_lfWeirOutflow != NULL)
		delete []m_lfWeirOutflow;
	if(m_lfOrificeOutflow != NULL)
		delete []m_lfOrificeOutflow;
}

double CFtable::table_interpolate(double x,double x1,double y1,double x2,double y2)
//
//  Input:   x = x value being interpolated
//           x1, x2 = x values on either side of x
//           y1, y2 = y values corrresponding to x1 and x2, respectively
//  Output:  returns the y value corresponding to x
//  Purpose: interpolates a y value for a given x value.
//
{
    double dx = x2 - x1;
    if ( fabs(dx) < 1.0e-20 ) return (y1 + y2) / 2.;
    return y1 + (x - x1) * (y2 - y1) / dx;
}

double CFtable::Depth_lookup(double x,double* y)
//
//  Input:   x = an x-value
//           y = an array of y-value 
//  Output:  returns a y-value
//  Purpose: retrieves the y-value corresponding to an x-value in a table,
//           using interploation if necessary.
//
//  NOTE: if x is below the first table entry, then the first y-value is
//        returned; if x is above the last entry, then the last y-value is
//        returned.
//
//	Note: Check the array "y" is not NULL before calling this function.
{
    double x1,y1,x2,y2;

	x1 = m_lfDepth[0];
	y1 = y[0];

    if ( x <= x1 ) return y1;

	for (int i=1; i<m_nRecords; i++)
    {
		x2 = m_lfDepth[i];
		y2 = y[i];
        
		if ( x <= x2 ) 
			return table_interpolate(x, x1, y1, x2, y2);
        
		x1 = x2;
        y1 = y2;
    }
    return y1;
}

double CFtable::Volume_lookup(double x,double* y)
//
//  Input:   x = an x-value
//           y = an array of y-value 
//  Output:  returns a y-value
//  Purpose: retrieves the y-value corresponding to an x-value in a table,
//           using interploation if necessary.
//
//  NOTE: if x is below the first table entry, then the first y-value is
//        returned; if x is above the last entry, then the last y-value is
//        returned.
//
//	Note: Check the array "y" is not NULL before calling this function.
{
    double x1,y1,x2,y2;

	x1 = m_lfVolume[0];
	y1 = y[0];

    if ( x <= x1 ) return y1;

	for (int i=1; i<m_nRecords; i++)
    {
		x2 = m_lfVolume[i];
		y2 = y[i];
        
		if ( x <= x2 ) 
			return table_interpolate(x, x1, y1, x2, y2);
        
		x1 = x2;
        y1 = y2;
    }
    return y1;
}