Attribute VB_Name = "ModuleFile"
'******************************************************************************
'   Application: SUSTAIN - Best Management Practice Decision Support System
'   Company:     Tetra Tech, Inc
'   File Name:   ModuleFile
'   Purpose:     This module contains all functions and utilities to write the input
'                text file.
'
'   Designer:    Leslie Shoemaker, Ting Dai, Khalid Alvi, Jenny Zhen, John Riverson, Sabu Paul
'   Developer:   Haihong Yang, Sabu Paul, Mira Chokshi
'   History:     Created:
'                Modified: 08/19/2004 - Mira Chokshi
'
'******************************************************************************

Option Explicit
Option Base 0

Public gHasInFileError As Boolean

'Define all private variables
Private fso As Scripting.FileSystemObject
Private pFile As TextStream
Private pInputFilePath As String
Private StrCard715BMPTypes As String
Private StrCard720PointSources As String 'New point source card
Private StrCard725BMPClassA As String
Private StrCard730BMPClassA As String
Private StrCard735BMPClassB As String
Private StrCard740BMPSoilIndex As String
Private StrCard745BMPGrowthIndex As String
Private StrCard747BMPClassABF As String
Private StrCard750ConduitDimensions As String
Private StrCard755ConduitCrossSections As String
Private StrCard760IrregularCrossSections As String
Private StrCard761BMPClassF As String
Private StrCard762BMPClassE As String
'Private StrCard763BMPClassF As String
Private StrCard1190ConduitLosses As String
'Private StrCard1200VFSParameters As String ' VFS is handled differently - June 18, 2007
'Private StrCard1210VFSParameters As String
Private StrCard765DecayFactors As String
Private StrCard770PercentRemoval As String
Private StrCard810AdjustParameter As String
Private StrCard805BMPCost As String
Private StrCard800OptimizationControls As String
Private StrCard815Assess As String
Private pBMPDetailDict As Scripting.Dictionary
Private pBasinToBMPDict As Scripting.Dictionary
Private pMaxLandTypeGroupID As Integer
Public pInputFileName As String
Public strTimeStepLine As String
Public gStrStartDate As String
Public gStrEndDate As String
'Public pInputFolder As String
Public pOutputFolder As String
Public pPredevelopedLanduse As String
Public pLanduseSimulationOption As Integer
Public pSWMMLanduseOutflowFile As String
Public pSWMMPreDevOutflowFile As String

Private StrCard766KFactors As String
Private StrCard767CValues As String

Private StrCard775Sediment As String
Private StrCard780SandTransport As String
Private StrCard785SiltTransport As String
Private StrCard786ClayTransport As String

Private StrCard901_VFS_Dim As String
Private StrCard902_VFS_SegDetails As String
Private StrCard903_VFS_SoilProps As String
Private StrCard904_VFS_Buf_Sed As String
Private StrCard905_VFS_Sed_Filt As String
Private StrCard906_VFS_Sed_Fracion As String
Private StrCard907_VFS_FO_Adsorbed As String
Private StrCard908_VFS_FO_Dissolved As String
Private StrCard909_VFS_TC_Adsorbed As String
Private StrCard910_VFS_TC_Dissolved As String

Private strETOptions As String
Private strMonETCoeffs As String
    
Private pTotalPollutantCount As Integer

Private gBMPDrainAreaDict As Scripting.Dictionary
Private StrCard790LandTypeRouting As String
'Const c_sModuleFileName As String = "D:\SUSTAIN\VBProject\ModuleFile.bas"

Private gBmpTypeClassDict As Scripting.Dictionary
Private bInputFileWritingError As Boolean



    
'******************************************************************************
'Subroutine: WriteInputTextFile
'Author:     Mira Chokshi
'Purpose:    Main subroutine to write the output file. Asks the user about
'            the input file name. Calls subroutines to write each card.
'******************************************************************************
Public Function WriteInputTextFile() As Boolean
On Error GoTo ShowError
    
''    BMP Type    BMP Type ID Class
''BioRetentionBasin   BIORETENTION   A
''WetPond WETPOND A
''Cistern CISTERN A
''DryPond DRYPOND A
''InfiltrationTrench  INFILTRATIONTRENCH  A
''GreenRoof   GREENROOF   A
''PorousPavement  POROUSPAVEMENT  A
''RainBarrel  RAINBARREL  A
''VegetatedSwale SWALE   B
''VFS BUFFERSTRIP D
''Conduit CONDUIT C
''Regulator   REGULATOR   X
''VirtualOutlet   VIRTUALOUTLET   X


    bInputFileWritingError = False
    Set gBmpTypeClassDict = New Scripting.Dictionary
    gBmpTypeClassDict.Item("BioRetentionBasin") = "BIORETENTION"
    gBmpTypeClassDict.Item("WetPond") = "WETPOND"
    gBmpTypeClassDict.Item("Cistern") = "CISTERN"
    gBmpTypeClassDict.Item("DryPond") = "DRYPOND"
    gBmpTypeClassDict.Item("InfiltrationTrench") = "INFILTRATIONTRENCH"
    gBmpTypeClassDict.Item("GreenRoof") = "GREENROOF"
    gBmpTypeClassDict.Item("PorousPavement") = "POROUSPAVEMENT"
    gBmpTypeClassDict.Item("RainBarrel") = "RAINBARREL"
    gBmpTypeClassDict.Item("VegetatedSwale") = "SWALE"
    gBmpTypeClassDict.Item("VFS") = "BUFFERSTRIP"
    gBmpTypeClassDict.Item("Conduit") = "CONDUIT"
    gBmpTypeClassDict.Item("Regulator") = "REGULATOR"
    gBmpTypeClassDict.Item("VirtualOutlet") = "VIRTUALOUTLET"
    gBmpTypeClassDict.Item("Junction") = "JUNCTION"
    gBmpTypeClassDict.Item("BufferStrip") = "BUFFERSTRIP"
    gBmpTypeClassDict.Item("DisconnectedImp") = "AREABMP"
    
    
    
    gHasInFileError = False
    
    If (CheckInputFileDataRequirements = False) Then
        WriteInputTextFile = False
        Exit Function
    End If
    
    If gExternalSimulation Then
        If (CheckTimeSeriesFile = False) Then
            WriteInputTextFile = False
            Exit Function
        End If
    End If
    
    pInputFileName = ""
    FrmSimulationPeriod.Frame1.Visible = False
    FrmSimulationPeriod.Height = 4320
    FrmSimulationPeriod.Show vbModal
        
    If (pInputFileName = "") And (Not gHasInFileError) Then WriteInputTextFile = False: Exit Function
    
    If gHasInFileError Then Err.Raise 5001, "Error in setting simulation options"
    
'    If (pInputFileName = "") Then
'        WriteInputTextFile = False
'        Exit Function
'    End If
    Set pBMPDetailDict = CreateObject("Scripting.Dictionary")
    Set fso = CreateObject("Scripting.FileSystemObject")
    
     'Clear memory
     StrCard715BMPTypes = ""
     StrCard720PointSources = ""
     StrCard725BMPClassA = ""
     StrCard730BMPClassA = ""
     StrCard735BMPClassB = ""
     StrCard740BMPSoilIndex = ""
     StrCard747BMPClassABF = ""
     StrCard745BMPGrowthIndex = ""
     StrCard750ConduitDimensions = ""
     StrCard755ConduitCrossSections = ""
     StrCard760IrregularCrossSections = ""
     StrCard762BMPClassE = ""
     StrCard761BMPClassF = ""

     StrCard1190ConduitLosses = ""
     StrCard765DecayFactors = ""
     StrCard770PercentRemoval = ""
     StrCard810AdjustParameter = ""
     StrCard805BMPCost = ""
     StrCard800OptimizationControls = ""
     StrCard815Assess = ""
     
     StrCard766KFactors = ""
     StrCard767CValues = ""
     
     StrCard775Sediment = ""
     StrCard780SandTransport = ""
     StrCard785SiltTransport = ""
     StrCard786ClayTransport = ""

     StrCard901_VFS_Dim = ""
     StrCard902_VFS_SegDetails = ""
     StrCard903_VFS_SoilProps = ""
     StrCard904_VFS_Buf_Sed = ""
     StrCard905_VFS_Sed_Filt = ""
     StrCard906_VFS_Sed_Fracion = ""
     StrCard907_VFS_FO_Adsorbed = ""
     StrCard908_VFS_FO_Dissolved = ""
     StrCard909_VFS_TC_Adsorbed = ""
     StrCard910_VFS_TC_Dissolved = ""
    
    strETOptions = ""
    strMonETCoeffs = ""
    
    Dim totalNumCards As Integer
    totalNumCards = 26
    
    Dim pVFSFLayer As IFeatureLayer
    Set pVFSFLayer = GetInputFeatureLayer("VFS")
    If Not pVFSFLayer Is Nothing Then totalNumCards = totalNumCards + 11
    
    
    
        
    Set pFile = fso.OpenTextFile(pInputFileName, ForWriting, True, TristateUseDefault)
    'Call subroutines to write card data to input file
    
    Dim pStatusBar As esriSystem.IStatusBar
    Set pStatusBar = gApplication.StatusBar
    pStatusBar.Visible = True
    
    Dim pStepProgressor As IStepProgressor
    Set pStepProgressor = pStatusBar.ProgressBar
    

    pStepProgressor.Show
    pStepProgressor.MinRange = 1
    pStepProgressor.MaxRange = 43
    pStepProgressor.Position = 1
    
    pStepProgressor.Step
    
    WriteCard700
    
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 700"
    
    pStepProgressor.Step
    
    WriteCard705
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 705"
    
    pStepProgressor.Step
    WriteCard710
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 710"
    
    Call GetLandTypeRouting
    If gHasInFileError Then Err.Raise 5001, "Error in calculating land use distribution"
    
    pStepProgressor.Step
    If gExternalSimulation Then
        WriteCard712
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 712"
        WriteCard713
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 713"
    End If
    
    
    
    pStepProgressor.Step
    WriteCard714
    
    pStepProgressor.Step
    WriteCard715
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 715"
    
    pStepProgressor.Step
    WriteCard720
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 720"
    
    pStepProgressor.Step
    WriteCard721
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 721"
    
    pStepProgressor.Step
    WriteCard722
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 722"
    
    pStepProgressor.Step
    WriteCard723
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 723"
    
    
    pStepProgressor.Step
    WriteCard725
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 725"
    
    pStepProgressor.Step
    WriteCard730
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 730"
    
    pStepProgressor.Step
    WriteCard735
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 735"
    
    pStepProgressor.Step
    
    WriteCard740
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 740"
    
    pStepProgressor.Step
    WriteCard745
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 745"
    
    pStepProgressor.Step
    
    WriteCard747
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 747"
    
    pStepProgressor.Step
    WriteCard750
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 750"
    
    pStepProgressor.Step
    
    WriteCard755
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 755"
    
    pStepProgressor.Step
    WriteCard760
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 760"
    
    pStepProgressor.Step
    WriteCard761
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 761"
    
    pStepProgressor.Step
    WriteCard762
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 762"
    
    
    pStepProgressor.Step
    WriteCard765
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 765"
    
    pStepProgressor.Step
    
    WriteCard766
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 766"
    
    pStepProgressor.Step
    
    WriteCard767
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 767"
    
    pStepProgressor.Step
    
    WriteCard770
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 770"
    
    pStepProgressor.Step
    
    WriteCard775 ' New sediment card
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 775"
    
    pStepProgressor.Step
    
    WriteCard780
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 780"
    
    pStepProgressor.Step
    
    WriteCard785
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 785"
    
    pStepProgressor.Step
    
    WriteCard786
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 786"
    
    pStepProgressor.Step
    
    WriteCard790
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 790"
    
    pStepProgressor.Step
    
    WriteCard795
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 795"
    
    pStepProgressor.Step
    
    WriteCard800
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 800"
    
    pStepProgressor.Step
    
    WriteCard805
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 805"
    
    pStepProgressor.Step
    
    WriteCard810
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 810"
    
    pStepProgressor.Step
    
    WriteCard814
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 814"
    pStepProgressor.Step
    
    WriteCard815
    If gHasInFileError Then Err.Raise 5001, "Error in writing card 815"
    
    'Write the VFSMOD parameters if there is any filter strip in the project
    
    'If filter strip is present in the study area then write the following cards
    If StrCard901_VFS_Dim <> "" Then
        
        pStepProgressor.Step
        
        WriteCard900
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 900"
        
        pStepProgressor.Step
        
        WriteCard901
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 901"
        
        pStepProgressor.Step
        
        WriteCard902
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 902"
        
        pStepProgressor.Step
        
        WriteCard903
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 903"
        
        pStepProgressor.Step
        
        WriteCard904
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 904"
        
        pStepProgressor.Step
        
        WriteCard905
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 905"
        
        pStepProgressor.Step
        
        WriteCard906
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 906"
        
        pStepProgressor.Step
        
        WriteCard907
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 907"
        
        pStepProgressor.Step
        
        WriteCard908
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 908"
        
        pStepProgressor.Step
        
        WriteCard909
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 909"
        
        pStepProgressor.Step
        WriteCard910
        If gHasInFileError Then Err.Raise 5001, "Error in writing card 910"
    End If
    
    pStatusBar.Message(0) = "Ready"
    pStepProgressor.Hide

    'Close the input file
    pFile.Close
    If Not bInputFileWritingError Then
        MsgBox "Input file created successfully", vbInformation
    End If
    Set pFile = Nothing
    Set fso = Nothing
   
    WriteInputTextFile = True
    Exit Function
ShowError:
    WriteInputTextFile = False
    MsgBox "WriteInputTextFile: " & Err.description
End Function


'******************************************************************************
'Subroutine: CheckInputFileDataRequirements
'Author:     Mira Chokshi
'Purpose:    This subroutine checks for data required for writing input file.
'******************************************************************************
Public Function CheckInputFileDataRequirements() As Boolean
On Error GoTo ShowError

    '*** Check for BMP layer to get the number of BMPs
    Dim pBMPFeatureLayer As IFeatureLayer
    Set pBMPFeatureLayer = GetInputFeatureLayer("BMPs")
    If (pBMPFeatureLayer Is Nothing) Then
        CheckInputFileDataRequirements = False
        MsgBox "BMPs feature layer not found."
        Exit Function
    End If
    Dim pBMPFeatureClass As IFeatureClass
    Set pBMPFeatureClass = pBMPFeatureLayer.FeatureClass
    
    Dim lNumBMPs
    lNumBMPs = pBMPFeatureClass.FeatureCount(Nothing)
    
    Dim pVFSFeatureLayer As IFeatureLayer
    Set pVFSFeatureLayer = GetInputFeatureLayer("VFS")
    If Not (pVFSFeatureLayer Is Nothing) Then
        Dim pVFSFeatureClass As IFeatureClass
        Set pVFSFeatureClass = pVFSFeatureLayer.FeatureClass
        lNumBMPs = lNumBMPs + pVFSFeatureClass.FeatureCount(Nothing)
    End If
    
    '*** Check for Basin to BMP routing connection
    Dim pWatershedFLayer As IFeatureLayer
    Set pWatershedFLayer = GetInputFeatureLayer("Watershed")
    If (pWatershedFLayer Is Nothing) Then
        CheckInputFileDataRequirements = False
    End If
    Dim pWatershedFClass As IFeatureClass
    Set pWatershedFClass = pWatershedFLayer.FeatureClass
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    pQueryFilter.WhereClause = "BMPID = 0"
    Dim pFeatureCursor As IFeatureCursor
    Set pFeatureCursor = pWatershedFClass.Search(pQueryFilter, True)
    Dim pFeature As IFeature
    Set pFeature = pFeatureCursor.NextFeature
    Dim iID As Long
    iID = pFeatureCursor.FindField("ID")
    Dim strMessage As String
    strMessage = ""
    Do While Not (pFeature Is Nothing)
        If (strMessage <> "") Then
            strMessage = strMessage & ","
        End If
        strMessage = strMessage & pFeature.value(iID)
        Set pFeature = pFeatureCursor.NextFeature
    Loop
    
    If (strMessage <> "") Then
        MsgBox "Watershed(s) " & strMessage & " are not routed to BMPs. Please use Basin-BMP routing tool to define the network.", vbExclamation
        CheckInputFileDataRequirements = False
        GoTo CleanUp
    End If
    
    
    '*** Check for BMP to BMP network connection
    Dim pBMPNetwork As iTable
    Set pBMPNetwork = GetInputDataTable("BMPNetwork")
    Dim pCursor As ICursor
    Dim pRow As iRow
    Dim pCollection As Collection
    Set pCollection = New Collection
    pQueryFilter.WhereClause = "DSID = 0"
    Set pCursor = pBMPNetwork.Search(pQueryFilter, True)
    Set pRow = pCursor.NextRow
    Dim iTypeDescFld As Long
    iTypeDescFld = pBMPNetwork.FindField("TypeDesc")
    Dim iIDFld As Long
    iIDFld = pBMPNetwork.FindField("ID")
    'Loop over all records with DSID = 0, i.e. no downstream defined
    Do While Not (pRow Is Nothing)
        pCollection.add pRow.value(iIDFld)
        Set pRow = pCursor.NextRow
    Loop
    'Loop over this collection to check which ID's don't have upstream
    Dim pID As Integer
    Dim pCount As Long
    Dim pCount2 As Long
    
    Dim i As Integer
    strMessage = ""
    Dim pQueryFilter2 As IQueryFilter
    Set pQueryFilter2 = New QueryFilter
    For i = 1 To pCollection.Count
        pID = pCollection.Item(i)
        pQueryFilter.WhereClause = "DSID = " & pID
        pCount = pBMPNetwork.RowCount(pQueryFilter)
        pQueryFilter2.WhereClause = "ID = " & pID
        pCount2 = pBMPNetwork.RowCount(pQueryFilter2)
        If (pCount = 0) And (pCount2 = 1) Then
            If (strMessage <> "") Then
                strMessage = strMessage & ","
            End If
            strMessage = strMessage & pID
        End If
    Next
    If lNumBMPs > 1 Then
        If (strMessage <> "") Then
            MsgBox "BMPs " & strMessage & " are not connected to other BMPs. Please use BMP network tool to define this network.", vbExclamation
            CheckInputFileDataRequirements = False
            GoTo CleanUp
        End If
    End If
    
    'Check the BMP with splitter on and Regulator

    Dim pDicBMPofMultiOutlet As Dictionary
    Set pDicBMPofMultiOutlet = New Dictionary

    Set pCursor = pBMPNetwork.Search(Nothing, True)
    Set pRow = pCursor.NextRow
    Do While Not (pRow Is Nothing)
        pID = pRow.value(iIDFld)
        pQueryFilter.WhereClause = "ID = " & pID
        If pBMPNetwork.RowCount(pQueryFilter) > 1 Then
            If Not pDicBMPofMultiOutlet.Exists(pID) Then
                pDicBMPofMultiOutlet.add pID, pID
            End If
        End If
        Set pRow = pCursor.NextRow
    Loop
    
    Dim o As Variant
    For Each o In pDicBMPofMultiOutlet
        pID = CLng(o)
        pQueryFilter.WhereClause = "DSID = " & pID
        pCount = pBMPNetwork.RowCount(pQueryFilter)
        pQueryFilter2.WhereClause = "ID = " & pID & " AND DSID > 0"
        pCount2 = pBMPNetwork.RowCount(pQueryFilter2)
        If (pCount = 0) And (pCount2 = 0) Then
            If (strMessage <> "") Then
                strMessage = strMessage & ","
            End If
            strMessage = strMessage & pID
        End If
        
    Next
    If lNumBMPs > 1 Then
        If (strMessage <> "") Then
            MsgBox "BMPs " & strMessage & " are not connected to other BMPs. Please use BMP network tool to define this network.", vbExclamation
            CheckInputFileDataRequirements = False
            GoTo CleanUp
        End If
    End If
   
    For Each o In pDicBMPofMultiOutlet
        pID = CLng(o)
        pQueryFilter.WhereClause = "ID = " & pID & " AND DSID > 0"
        pCount = pBMPNetwork.RowCount(pQueryFilter)
        If (pCount > 0) Then
            pQueryFilter2.WhereClause = "ID = " & pID & " AND DSID = 0"
            Set pCursor = pBMPNetwork.Search(pQueryFilter2, False)
            Set pRow = pCursor.NextRow
            Do While Not (pRow Is Nothing)
                If (strMessage <> "") Then
                    strMessage = strMessage & ","
                End If
                strMessage = strMessage & pRow.value(iTypeDescFld)
                Set pRow = pCursor.NextRow
            Loop
            If lNumBMPs > 1 Then
                If (strMessage <> "") Then
                    'MsgBox "Please add connection " & strMessage & " for BMP " & pID & ".", vbExclamation
                    MsgBox "Define the routing for " & strMessage & " outlet for BMP " & pID & ".", vbExclamation
                    CheckInputFileDataRequirements = False
                    GoTo CleanUp
                End If
            End If

        End If
    Next

'    Dim pTable As iTable
'    Set pTable = GetInputDataTable("BMPDetail")
'    If (pTable Is Nothing) Then
'        MsgBox "BMPDetail table not found.", vbExclamation
'        CheckInputFileDataRequirements = False
'        GoTo CleanUp
'    End If
    'All checks successful, return TRUE
    CheckInputFileDataRequirements = True
    GoTo CleanUp
ShowError:
    MsgBox "CheckInputFileDataRequirements: " & Err.description
CleanUp:

End Function



'******************************************************************************
'Subroutine: WriteCard700
'Author:     Sabu Paul, Mira Chokshi
'Purpose:    Write INPUT/OUTPUT FILE DIRECTORIES
'            Get the start and end date of simulation and simulation time-step.
'Modified:   Mira Chokshi modified to update the simulation parameters from
'            dialog box
'******************************************************************************

Public Sub WriteCard700()
On Error GoTo ShowError
    Dim fso As Scripting.FileSystemObject
    Set fso = CreateObject("Scripting.FileSystemObject")

    If (Right(pOutputFolder, 1) <> "\") Then
        pOutputFolder = pOutputFolder & "\"
    End If
    'pInputFolder = pOutputFolder & "In\"
    
    If Not (fso.FolderExists(pOutputFolder)) Then
        fso.CreateFolder pOutputFolder
    End If

    Dim dtmDate As Date
    dtmDate = DateValue(Now)
    Dim dtmTime As Date
    dtmTime = TimeValue(Now)

    pFile.WriteLine ("c-----------------------------------------------------------------------------------------")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c SUSTAIN: System for Urban Stormwater Treatment and Analysis INtegration")
    pFile.WriteLine ("c Version 1.2 - May 2013")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c Designed and maintained by:")
    pFile.WriteLine ("c     Tetra Tech, Inc.")
    pFile.WriteLine ("c     10306 Eaton Place, Suite 340")
    pFile.WriteLine ("c     Fairfax, VA 22030")
    pFile.WriteLine ("c     (703) 385-6000")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c NOTE: The line starting with the letter c and followed by space or - is a comment line.")
    pFile.WriteLine ("c       There must be no comment line in between the data lines")
    pFile.WriteLine ("c       The input text field must be a continuous string without any space in between the characters")
    pFile.WriteLine ("c")

    pFile.WriteLine ("c This input file was created at " & dtmTime & " on " & dtmDate)
    pFile.WriteLine ("c")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")


    pFile.WriteLine ("c700 Model Controls")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c LINE1 = Land simulation control (0-external,1-internal),")
    pFile.WriteLine ("c         Land output directory (containing land output timeseries),")
    pFile.WriteLine ("c         external land timeseries data must be in this order;  flow (in./timestep), groundwater recharge (in./timestep), pollutant 1 (lb/acre/timestep), pollutant2, ...")
    pFile.WriteLine ("c         Mixed land use output file name (for internal control),")
    pFile.WriteLine ("c         PreDeveloped land use output file name (for internal control)")
    pFile.WriteLine ("c LINE2 = Start date of simulation (Year Month Day)")
    pFile.WriteLine ("c LINE3 = End date of simulation (Year Month Day)")
    pFile.WriteLine ("c LINE4 = Land Timeseries timestep (Min),")
    pFile.WriteLine ("c         BMP simulation timestep (Min),")
    pFile.WriteLine ("c         CRRAT = The ratio of max velocity to mean velocity under typical flow conditions (value of 1.0 or greater)")
    pFile.WriteLine ("c         Model output control (0-the same timestep as land time series; 1-hourly),")
    pFile.WriteLine ("c         Model output directory")
    pFile.WriteLine ("c LINE5 = ET Flag (0-constant monthly ET,1-daily ET from the timeseries,2-calculate daily ET from the daily temperature data),")
    pFile.WriteLine ("c         Climate time series file path (required if ET flag is 1 or 2),")
    pFile.WriteLine ("c         Latitude (Decimal degrees) required if ET flag is 2")
    pFile.WriteLine ("c LINE6 = Monthly ET rate (in/day) if ET flag is 0   OR")
    pFile.WriteLine ("c         Monthly pan coefficient (multiplier to ET value) if ET flag is 1   OR")
    pFile.WriteLine ("c         Monthly variable coefficient to calculate ET values")
    pFile.WriteLine ("c")
    If pLanduseSimulationOption = 0 Then
        'pFile.WriteLine ("0" & vbTab & pInputFolder)
        pFile.WriteLine ("0" & vbTab & pOutputFolder)
    Else
        'pFile.WriteLine ("1" & vbTab & pInputFolder & vbTab & pSWMMLanduseOutflowFile & vbTab & pSWMMPreDevOutflowFile)
        'pFile.WriteLine ("1" & vbTab & pInputFolder & vbTab & Replace(pSWMMLanduseOutflowFile, """", "") & vbTab & Replace(pSWMMPreDevOutflowFile, """", ""))
        pFile.WriteLine ("1" & vbTab & pOutputFolder & vbTab & Replace(pSWMMLanduseOutflowFile, """", "") & vbTab & Replace(pSWMMPreDevOutflowFile, """", ""))
    
    End If
    
    pFile.WriteLine (gStrStartDate)
    pFile.WriteLine (gStrEndDate)
    pFile.WriteLine (strTimeStepLine)
      
    Call SetETOptionString
    
    pFile.WriteLine (strETOptions)
    pFile.WriteLine (strMonETCoeffs)
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 700:", Err.description
End Sub

Private Sub SetETOptionString()
On Error GoTo ErrorHandler
    ReadLayerTagDictionaryToSRCFile

    Dim etOption As Integer
    etOption = gLayerNameDictionary.Item("ETOPTION")
    
    Dim months
    months = Array("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
    Dim i As Integer
    For i = 0 To 11
        strMonETCoeffs = strMonETCoeffs & gLayerNameDictionary.Item("MonET" & months(i)) & vbTab
    Next
        
    If etOption = 0 Then
        strETOptions = etOption
    ElseIf etOption = 1 Then
        strETOptions = etOption & vbTab & gLayerNameDictionary.Item("TSFILE")
    Else
        strETOptions = etOption & vbTab & gLayerNameDictionary.Item("TSFILE") & vbTab & gLayerNameDictionary.Item("LATITUDE")
    End If
    Exit Sub
    
ErrorHandler:
    MsgBox "Error in SetETOptionString: " & Err.description
End Sub
'******************************************************************************
'Subroutine: WriteCard1000
'Author:     Mira Chokshi
'Purpose:    Write INPUT/OUTPUT FILE DIRECTORIES
'            Get the start and end date of simulation and simulation time-step.
'Modified:   Mira Chokshi modified to update the simulation parameters from
'            dialog box
'******************************************************************************
''Private Sub WriteCard1000()
''On Error GoTo ShowError
''
''    Dim fso As Scripting.FileSystemObject
''    Set fso = CreateObject("Scripting.FileSystemObject")
''    'pInputFolder = pInputFilePath & "In\"
''    If (Right(pOutputFolder, 1) <> "\") Then
''        pOutputFolder = pOutputFolder & "\"
''    End If
''    pInputFolder = pOutputFolder & "In\"
''    'pOutputFolder = pInputFilePath & "Out\"
''    If Not (fso.FolderExists(pOutputFolder)) Then
''        fso.CreateFolder pOutputFolder
''    End If
''    If Not (fso.FolderExists(pInputFolder)) Then
''        fso.CreateFolder pInputFolder
''    End If
''
''    pFile.WriteLine ("c1000 INPUT/OUTPUT FILE DIRECTORIES")
''    pFile.WriteLine ("c LINE1 = Input Directory")
''    pFile.WriteLine ("c LINE2 = Output Directory")
''    pFile.WriteLine ("c LINE3 = Start Date of Simulation(Year Month Day)")
''    pFile.WriteLine ("c LINE4 = End Date of Simulation(Year Month Day)")
''    pFile.WriteLine ("c LINE5 = Land Simulation (0-External, 1-Internal) BMP Simulation timestep(Min) Output Simulation timestep(0-Daily,1-Hourly) ")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    'Modified for new input file format -- Sabu Paul ; Aug 24, 2004
''    pFile.WriteLine ("1" & vbTab & pInputFolder) 'to avoid confusion between data lines and comment lines
''    pFile.WriteLine ("2" & vbTab & pOutputFolder)
''    pFile.WriteLine (gStrStartDate)
''    pFile.WriteLine (strEndDate)
''    pFile.WriteLine (strTimeStepLine)
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c")
''    Exit Sub
''ShowError:
''    MsgBox "Card 1000:", err.description
''End Sub


'******************************************************************************
'Subroutine: WriteCard705
'Author:     Sabu Paul
'Purpose:    Write pollutant identification card
'
'Modified:
'******************************************************************************
Private Sub WriteCard705()
On Error GoTo ShowError
    pFile.WriteLine ("c705 Pollutant Definition")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c POLLUT_ID   = Unique pollutant identifier (Sequence number same as in land output time series)")
    pFile.WriteLine ("c POLLUT_NAME = Unique pollutant name")
    pFile.WriteLine ("c MULTIPLIER  = Multiplying factor used to convert the pollutant load to lbs (external control)")
    pFile.WriteLine ("c SED_FLAG    = The sediment flag (0-not sediment,1-sand,2-silt,3-clay,4-total sediment)")
    pFile.WriteLine ("c               if = 4 SEDIMENT will be splitted into sand, silt,and clay based on the fractions defined in card 710.")
    pFile.WriteLine ("c SED_QUAL     = The sediment-associated pollutant flag (0-no, 1-yes)")
    pFile.WriteLine ("c               if = 1 then SEDIMENT is required in the pollutant list")
    pFile.WriteLine ("c SAND_QFRAC  = The sediment-associated qual-fraction on sand (0-1), only required if SED_QUAL = 1")
    pFile.WriteLine ("c SILT_QFRAC  = The sediment-associated qual-fraction on silt (0-1), only required if SED_QUAL = 1")
    pFile.WriteLine ("c CLAY_QFRAC  = The sediment-associated qual-fraction on clay (0-1), only required if SED_QUAL = 1")

    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  POLLUT_ID    POLLUT_NAME    MULTIPLIER     SED_FLAG    SED_QUAL     SAND_QFRAC     SILT_QFRAC     CLAY_QFRAC")
    pFile.WriteLine ("c POLLUT_ID" & vbTab & "POLLUT_NAME" & vbTab & "MULTIPLIER" & vbTab & "SED_FLAG" & vbTab & "SED_QUAL" & vbTab & "SAND_QFRAC" & vbTab & "SILT_QFRAC" & vbTab & "CLAY_QFRAC")
    'Add the pollutant lines.
    pFile.Write GetPollutantDetails
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 705:", Err.description
End Sub

'******************************************************************************
'Subroutine: WriteCard1010
'Author:     Mira Chokshi
'Purpose:    Write LAND DEFINITION -- LEGEND
'******************************************************************************
''Private Sub WriteCard1010()
''    '** Write this information only if external land use types are selected.
''    If (pLanduseSimulationOption = 0) Then
''        pFile.WriteLine ("c1010 land use DEFINITION")
''        pFile.WriteLine ("c")
''        pFile.WriteLine ("c LANDTYPE       = Unique land use definition identifier")
''        pFile.WriteLine ("c LANDNAME       = land use name")
''        pFile.WriteLine ("c IMPERVIOUS     = Distinguishes pervious/impervious land unit")
''        pFile.WriteLine ("c TIMESERIESFILE = File name containing input timeseries")
''        pFile.WriteLine ("c")
''        pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''        pFile.WriteLine ("c  LANDTYPE       LANDNAME   IMPERVIOUS     TIMESERIESFILE ")
''        pFile.Write (GetLandTypeDetails)
''        pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    End If
''End Sub

Private Sub WriteCard710()
On Error GoTo ShowError
    '** Write this information only if external land use types are selected.
    If (pLanduseSimulationOption = 0) Then
        pFile.WriteLine ("c710 LAND USE DEFINITION (required if land simulation control is external)")
        pFile.WriteLine ("c")
        pFile.WriteLine ("c LANDTYPE       = Unique land use definition identifier")
        pFile.WriteLine ("c LANDNAME       = land use name")
        pFile.WriteLine ("c IMPERVIOUS     = Distinguishes pervious/impervious land unit (0-pervious; 1-impervious)")
        pFile.WriteLine ("c TIMESERIESFILE = File name containing input timeseries")
        pFile.WriteLine ("c SAND_FRAC      = The fraction of total sediment from the land which is sand (0-1)")
        pFile.WriteLine ("c SILT_FRAC      = The fraction of total sediment from the land which is silt (0-1)")
        pFile.WriteLine ("c CLAY_FRAC      = The fraction of total sediment from the land which is clay (0-1)")
        pFile.WriteLine ("c")
        'pFile.WriteLine ("c  LANDTYPE       LANDNAME              IMPERVIOUS      TIMESERIESFILE     SAND_FRAC       SILT_FRAC       CLAY_FRAC")
        pFile.WriteLine ("c LANDTYPE" & vbTab & "LANDNAME" & vbTab & "IMPERVIOUS" & vbTab & "TIMESERIESFILE" & vbTab & "SAND_FRAC" & vbTab & "SILT_FRAC" & vbTab & "CLAY_FRAC")
        pFile.Write (GetLandTypeDetails)
        pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    End If
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub



'******************************************************************************
'Subroutine: WriteCard1020
'Author:     Mira Chokshi
'Purpose:    Write SWMM land use & predeveloped land use outflow file paths.
'******************************************************************************
'No need of this card - June 14, 2007
''Private Sub WriteCard1020()
''    '** Write this information only if internal swmm simulation is selected.
''    If (pLanduseSimulationOption = 1) Then
''        pFile.WriteLine ("c1020 SWMM LAND OUTFLOWS")
''        pFile.WriteLine ("c")
''        pFile.WriteLine ("c LAND_OUTFLOW = Land simulation output file")
''        pFile.WriteLine ("c PREDEV_OUTFLOW  = Pre-developed land simulation output file")
''        pFile.WriteLine ("c")
''        pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''        pFile.WriteLine ("c  LAND_OUTFLOW       PREDEV_OUTFLOW")
''        pFile.WriteLine (pSWMMLanduseOutflowFile & vbTab & pSWMMPreDevOutflowFile)
''        pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    End If
''End Sub




Private Sub WriteCard712()
On Error GoTo ShowError
    pFile.WriteLine ("c712 Aquifer INFORMATION")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c AquiferID          = Unique Aquifer identifier")
    pFile.WriteLine ("c AquiferNAME        = Aquifer name")
    pFile.WriteLine ("c Initial Storage    = Initial Storage (ac-ft)")
    pFile.WriteLine ("c RecessionCoef      = Recession Coefficient (1/hr)")
    pFile.WriteLine ("c SeepageCoef        = Seepage Coefficient (1/hr)")
    
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c AquiferID         AquiferNAME       InitialStorage      RecessionCoef       SeepageCoef")
    pFile.WriteLine ("c AquiferID" & vbTab & "AquiferNAME" & vbTab & "InitialStorage" & vbTab & "RecessionCoef" & vbTab & "SeepageCoef")

    Dim pTable As iTable
    Dim pTableName As String
    
    pTableName = "Aquifers"
    
    Set pTable = GetInputDataTable(pTableName)
    If Not (pTable Is Nothing) Then
    
        '** Write this information only if external land use types are selected.
        pFile.Write GetAquifersDetails
    
    End If
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard713()
On Error GoTo ShowError

    pFile.WriteLine ("c713 Aquifer Pollutant Background Concentration")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c AquiferID          = Unique Aquifer identifier as in c712")
    pFile.WriteLine ("c Ci        = Background concentration for pollutant i (mg/l)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from card 705)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c AQUIFER_ID  QUALC1  QUALC2 ... QUALCN")
    pFile.WriteLine ("c AQUIFER_ID" & vbTab & "QUALC1" & vbTab & "QUALC2" & vbTab & "..." & vbTab & "QUALCN")
    Dim pTable As iTable
    Dim pTableName As String
    
    pTableName = "Aquifers"
    
    Set pTable = GetInputDataTable(pTableName)
    If Not (pTable Is Nothing) Then
    
        '** Write this information only if external land use types are selected.
        pFile.Write GetAquiferParameters
    
    End If
    
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
Private Sub WriteCard714()
On Error GoTo ShowError


    pFile.WriteLine ("c714 Ftable for BMP Class A, B, and C")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c FTABLE_ID                = Unique Ftable identifier (continuous string)")
    pFile.WriteLine ("c FLOW_LENGTH              = Flow length (ft)")
    pFile.WriteLine ("c BED_SLOPE                = Longitudinal bed slope (ft/ft)")
    pFile.WriteLine ("c NUM_RECORD               = Number of layers in the Ftable")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c DEPTH                    = Water depth (ft)")
    pFile.WriteLine ("c SURFACE_AREA             = Water surface area at the given depth (acre)")
    pFile.WriteLine ("c VOLUME                   = Storage volume at the given depth (ac-ft)")
    pFile.WriteLine ("c FLOW_WEIR                = Overflow or weir outflow rate at the given depth (cfs)")
    pFile.WriteLine ("c FLOW_ORIFICE             = Channel flow or orifice outflow rate at the given depth (cfs)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c FTABLE_ID   FLOW_LENGTH   BED_SLOPE   NUM_RECORD")
    'pFile.WriteLine ("c DEPTH   SURFACE_AREA   VOLUME   FLOW_WEIR   FLOW_ORIFICE")
    
    pFile.WriteLine ("c FTABLE_ID" & vbTab & "FLOW_LENGTH" & vbTab & "BED_SLOPE" & vbTab & "NUM_RECORD")
    pFile.WriteLine ("c DEPTH" & vbTab & "SURFACE_AREA" & vbTab & "VOLUME" & vbTab & "FLOW_WEIR" & vbTab & "FLOW_ORIFICE")
    
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'******************************************************************************
'Subroutine: WriteCard1100
'Author:     Mira Chokshi
'Purpose:    Write BMP DEFINITION -- LEGEND
'******************************************************************************
Private Sub WriteCard715()
On Error GoTo ShowError
    Call CreatePollutantList
    pTotalPollutantCount = UBound(gPollutants) + 1 'Get total pollutants
    'Get total BMP,Conduits count
    Dim pBMPFeatureLayer As IFeatureLayer
    Set pBMPFeatureLayer = GetInputFeatureLayer("BMPs")
    If (pBMPFeatureLayer Is Nothing) Then
        MsgBox "BMPs feature layer not found."
        Exit Sub
    End If
    Dim pConduitFeatureLayer As IFeatureLayer
    Set pConduitFeatureLayer = GetInputFeatureLayer("Conduits")
    Dim pVFSFeatureLayer As IFeatureLayer
    Set pVFSFeatureLayer = GetInputFeatureLayer("VFS")

    '** Sum up total BMPs
    Dim pTotalBMPCount As Integer
    pTotalBMPCount = pBMPFeatureLayer.FeatureClass.FeatureCount(Nothing)
    If Not (pConduitFeatureLayer Is Nothing) Then
        pTotalBMPCount = pTotalBMPCount + pConduitFeatureLayer.FeatureClass.FeatureCount(Nothing)
    End If
    '** ADD VFS to total bmps + conduits
    If (Not pVFSFeatureLayer Is Nothing) Then
        pTotalBMPCount = pTotalBMPCount + pVFSFeatureLayer.FeatureClass.FeatureCount(Nothing)
    End If
    
    Set pBMPFeatureLayer = Nothing
    Set pConduitFeatureLayer = Nothing
    'Loop through each bmp sequentially now, modified 04/05/2005, mira chokshi.
    Dim iBMP As Integer
    For iBMP = 1 To pTotalBMPCount
        If Not bInputFileWritingError Then
            BMPCardsDetail (iBMP)
            ConduitCardsDetails (iBMP)
            'VFSCardsDetail (iBMP)
            VFSCardsDetail_New (iBMP)
        End If
    Next
    'Write header information
    pFile.WriteLine ("c715 BMP SITE INFORMATION")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE         = Unique BMP site identifier")
    pFile.WriteLine ("c BMPNAME         = BMP template name or site name")
    pFile.WriteLine ("c BMPTYPE         = Unique BMP Types (must use the exact same keyword)")
    pFile.WriteLine ("c                  (BIORETENTION,WETPOND,CISTERN,DRYPOND,INFILTRATIONTRENCH,GREENROOF,POROUSPAVEMENT,RAINBARREL,REGULATOR,SWALE,CONDUIT,BUFFERSTRIP,AREABMP)")

    'pFile.WriteLine ("c BMPCLASS        = Distinguishes BMP Class (A, B, ..., X)")
    'pFile.WriteLine ("c BMPTYPE         = Distinguishes BMP Types (Bioretention, Rainbarrel, etc.)")
    pFile.WriteLine ("c DArea           = Total Drainage Area in acre")
    pFile.WriteLine ("c NUMUNIT = Number of BMP structures")
    pFile.WriteLine ("c DDAREA  = Design drainage area of the BMP structure (acre)")
    pFile.WriteLine ("c PreLUType       = Predevelopment land use type (for external land simulation option)")
    pFile.WriteLine ("c AquiferID       = Unique Aquifer ID, 0 --- no aquifer (for external land simulation option)")
    'pFile.WriteLine ("c FtableFLG         = Ftable flag, 0 = no Ftable (for BMP Class A, B, and C)")
    pFile.WriteLine ("c FtableFLG       = Ftable flag, 0 = no, 1 = yes (for BMP Class A, B, and C)")
    pFile.WriteLine ("c FTABLE_ID       = Unique Ftable identifier (continuous string) as in card 714")
    
    pFile.WriteLine ("c")

    'pFile.WriteLine ("c BMPSITE         BMPNAME       BMPTYPE      DArea       NUMUNIT     DDAREA      PreLUType    AquiferID    FtableFLG     FTABLE_ID")
    pFile.WriteLine ("c BMPSITE" & vbTab & "BMPNAME" & vbTab & "BMPTYPE" & vbTab & "DArea" & vbTab & "NUMUNIT" & vbTab & "DDAREA" & vbTab & "PreLUType" & vbTab & "AquiferID" & vbTab & "FtableFLG" & vbTab & "FTABLE_ID")
    
    pFile.Write StrCard715BMPTypes
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard720()
On Error GoTo ShowError
    Dim pTable As iTable
    Set pTable = GetInputDataTable("ExternalTS")
    
    pFile.WriteLine ("c720 Point Source Definition")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c point source timeseries data must be in this order;  flow (in./timestep), pollutant 1 (lb/acre/timestep), pollutant2, ...")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c POINTSOURCE    = Unique point source identifier")
    pFile.WriteLine ("c DESCRIPTION    = Point source description (a continuous string without any space)")
    pFile.WriteLine ("c BMPSITE        = BMP site identifier in card 715")
    pFile.WriteLine ("c MULTIPLIER     = Multiplier applied to the timeseries file (flow and pollutants). It will be in addition to the pollutant multiplier in card 705")
    pFile.WriteLine ("c TIMESERIESFILE = File name containing input timeseries")
    pFile.WriteLine ("c SAND_FRAC      = The fraction of total sediment which is sand (0-1)")
    pFile.WriteLine ("c SILT_FRAC      = The fraction of total sediment which is silt (0-1)")
    pFile.WriteLine ("c CLAY_FRAC      = The fraction of total sediment which is clay (0-1)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  POINTSOURCE  DESCRIPTION  BMPSITE   MULTIPLIER     TIMESERIESFILE   SAND_FRAC       SILT_FRAC       CLAY_FRAC")
    pFile.WriteLine ("c POINTSOURCE" & vbTab & "DESCRIPTION" & vbTab & "BMPSITE" & vbTab & "MULTIPLIER" & vbTab & "TIMESERIESFILE" & vbTab & "SAND_FRAC" & vbTab & "SILT_FRAC" & vbTab & "CLAY_FRAC")
    If Not pTable Is Nothing Then
        pFile.Write GetPointSourceDetails
    End If
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard721()
On Error GoTo ShowError
    
    pFile.WriteLine ("c721 Tier-1 Watershed Outlets Definition")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE        = BMP site (watershed outlet) identifier in card 715")
    pFile.WriteLine ("c NUMBREAKS      = Number of break points on the cost-effectiveness curve")
    pFile.WriteLine ("c CECurveFile    = CECurve_Solutions file for the project cost (sorted cost value) of each break point")
    pFile.WriteLine ("c")

    'pFile.WriteLine ("c TRADEOFFCURVEFILE = CECurve_Solutions file for the project cost of each break point")
    'pFile.WriteLine ("c BMPSITE     NUMBREAKS     TRADEOFFCURVEFILE")
    pFile.WriteLine ("c BMPSITE" & vbTab & "NUMBREAKS" & vbTab & "CECurveFile")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard722()
On Error GoTo ShowError
    
    pFile.WriteLine ("c722 Tier-1 Watershed Timeseries Definition")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE        = BMP site (watershed outlet) identifier in card 721")
    pFile.WriteLine ("c BREAKPOINTID   = Unique break point id on cost-effectiveness curve")
    pFile.WriteLine ("c                  (0 for initial, -1 for PreDev, and -2 for PostDev condition)")
    pFile.WriteLine ("c MULTIPLIER     = Multiplier applied to the timeseries file")
    pFile.WriteLine ("c TIMESERIESFILE = Timeseries output file corresponding to the breakpoint id")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  BMPSITE    BREAKPOINTID    MULTIPLIER    TIMESERIESFILE")
    pFile.WriteLine ("c BMPSITE" & vbTab & "BREAKPOINTID" & vbTab & "MULTIPLIER" & vbTab & "TIMESERIESFILE")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1110
'''Author:     Mira Chokshi
'''Purpose:    Write CLASS A DIMENSION GROUPS
'''Modified:   Jenny Z. required additional information added in card 1110.
'''            Added RELTP, PEOPLE, DDAYS - Mira Chokshi: modified on 08/20/04
'''******************************************************************************
''Private Sub WriteCard1110()
''    pFile.WriteLine ("c1110 CLASS A BMP Site DIMENSION GROUPS")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE = Class A BMP dimension group identifier in card 715")
''    pFile.WriteLine ("c WIDTH   = Basin bottom width (ft) / no of units used for rain barrel or cistern")
''    pFile.WriteLine ("c LENGTH  = Basin bottom length (ft) / diameter (ft) for rain barrel or cistern")
''    pFile.WriteLine ("c OHEIGHT = Orifice Height (ft)")
''    pFile.WriteLine ("c DIAM    = Orifice Diameter (in)")
''    pFile.WriteLine ("c EXTP    = Exit Type   (1 for C=1,2 for C=0.61, 3 for C=0.61, 4 for C=0.5)")
''    pFile.WriteLine ("c RELTP   = Release Type   (1-Cistern, 2-Rain barrel, 3-others)")
''    pFile.WriteLine ("c PEOPLE  = Number of persons (Cistern Option)")
''    pFile.WriteLine ("c DDAYS   = Number of dry days (Rain Barrel Option)")
''    pFile.WriteLine ("c WEIRTP  = Weir Type   (1-Rectangular,2-Triangular)")
''    pFile.WriteLine ("c WEIRH   = Weir Height (ft)")
''    pFile.WriteLine ("c WEIRW   = (weir type 1) Weir width  (ft)")
''    pFile.WriteLine ("c THETA   = (weir type 2) Weir angle  (degrees)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  WIDTH  LENGTH  OHEIGHT    DIAM     EXITYPE   RELEASETYPE   PEOPLE    DDAYS    WEIRTYPE    WEIRH    WEIRW   THETA")
''    pFile.Write (StrCard1110BMPClassA)
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''
''End Sub

Private Sub WriteCard723()
On Error GoTo ShowError

    'Write header information
    
    pFile.WriteLine ("c723 Pump Curve (applies if PUMP_FLG is ON in card 725)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c PUMP_CURVE = The unique name of pump curve (continuous string without space)")
    pFile.WriteLine ("c NUM_RECORD = Number of points on the curve ")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c DEPTH = Depth (ft)")
    pFile.WriteLine ("c FLOW = Pumping flow rate (cfs)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c PUMP_CURVE   NUM_RECORD")
    'pFile.WriteLine ("c DEPTH   FLOW")
    pFile.WriteLine ("c PUMP_CURVE" & vbTab & "NUM_RECORD")
    pFile.WriteLine ("c DEPTH" & vbTab & "FLOW")
    Dim pCursorFindNewID As ICursor
    Dim pRowFindNewID As iRow
    

    '* Load the list box with pollutant names
    Dim pPumpCurveTable As iTable
    Set pPumpCurveTable = GetInputDataTable("PumpCurve")
    Dim pDictionary As Scripting.Dictionary
    Set pDictionary = New Dictionary

    If Not (pPumpCurveTable Is Nothing) Then
        Dim iPropName As Long
        iPropName = pPumpCurveTable.FindField("PropName")
        Dim iPropValue As Long
        iPropValue = pPumpCurveTable.FindField("PropValue")
        Dim pQueryFilter As IQueryFilter
        Set pQueryFilter = New QueryFilter
        Dim pCursor As ICursor
        Dim pRow As iRow

        Dim pID As Long
        Set pCursorFindNewID = pPumpCurveTable.Search(Nothing, False)
        Set pRowFindNewID = pCursorFindNewID.NextRow
        
        Dim strPumpCurveName As String
        Dim strPumpDepth As String
        Dim strPumpFlowrate As String
    
        Do Until pRowFindNewID Is Nothing
            pID = pRowFindNewID.value(pPumpCurveTable.FindField("ID"))
            If Not pDictionary.Exists(pID) Then
            
                pDictionary.add pID, pID
                
                'find the pump curve name
                pQueryFilter.WhereClause = "ID = " & pID & " AND PropName = 'Name'"
                Set pCursor = pPumpCurveTable.Search(pQueryFilter, True)
                Set pRow = pCursor.NextRow
                strPumpCurveName = pRow.value(iPropValue)
                
                'find the pump curve Depth
                pQueryFilter.WhereClause = "ID = " & pID & " AND PropName = 'Depth'"
                Set pCursor = pPumpCurveTable.Search(pQueryFilter, True)
                Set pRow = pCursor.NextRow
                strPumpDepth = pRow.value(iPropValue)
                
                'find the pump curve Flowrate
                pQueryFilter.WhereClause = "ID = " & pID & " AND PropName = 'FlowRate'"
                Set pCursor = pPumpCurveTable.Search(pQueryFilter, True)
                Set pRow = pCursor.NextRow
                strPumpFlowrate = pRow.value(iPropValue)
                
                
                Dim pValues(0 To 1), iCnt As Integer
                pValues(0) = Split(strPumpDepth, ";")
                pValues(1) = Split(strPumpFlowrate, ";")
                pFile.WriteLine (strPumpCurveName & vbTab & CStr(UBound(pValues(0)) + 1))
                For iCnt = 0 To UBound(pValues(0))
                    pFile.WriteLine (pValues(0)(iCnt) & vbTab & pValues(1)(iCnt))
                Next
            End If
            Set pRowFindNewID = pCursorFindNewID.NextRow
        Loop
        

    End If
    
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'******************************************************************************
'Subroutine: WriteCard725
'Author:     Mira Chokshi
'Purpose:    Write CLASS A DIMENSION GROUPS
'Modified:   Jenny Z. required additional information added in card 1110.
'            Added RELTP, PEOPLE, DDAYS - Mira Chokshi: modified on 08/20/04
'******************************************************************************
Private Sub WriteCard725()
On Error GoTo ShowError
    pFile.WriteLine ("c725 CLASS-A BMP Site Parameters (required if BMPSITE is CLASS-A in card 715)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = Class A BMP dimension group identifier in card 715")
    pFile.WriteLine ("c WIDTH   = Basin bottom width (ft)")
    pFile.WriteLine ("c LENGTH  = Basin bottom length (ft) / diameter (ft) for rain barrel or cistern")
    pFile.WriteLine ("c OHEIGHT = Orifice Height (ft)")
    pFile.WriteLine ("c DIAM    = Orifice Diameter (in)")
    pFile.WriteLine ("c EXTP    = Exit Type   (1 for C=1,2 for C=0.61, 3 for C=0.61, 4 for C=0.5)")
    pFile.WriteLine ("c RELTP   = Release Type   (1-Cistern, 2-Rain barrel, 3-others)")
    pFile.WriteLine ("c PEOPLE  = Number of persons (Cistern Option)")
    pFile.WriteLine ("c DDAYS   = Number of dry days (Rain Barrel Option)")
    pFile.WriteLine ("c WEIRTP  = Weir Type   (1-Rectangular,2-Triangular)")
    pFile.WriteLine ("c WEIRH   = Weir Height (ft)")
    pFile.WriteLine ("c WEIRW   = (weir type 1) Weir width  (ft)")
    pFile.WriteLine ("c THETA   = (weir type 2) Weir angle  (degrees)")
    pFile.WriteLine ("c ET_MULT = multiplier to PET")
    pFile.WriteLine ("c PUMP_FLG = pump option (0-OFF, 1-ON)")
    pFile.WriteLine ("c DEPTH_ON = water Depth (ft) at which the pump is started")
    pFile.WriteLine ("c DEPTH_OFF = water Depth (ft) at which the pump is stopped")
    pFile.WriteLine ("c PUMP_CURVE = The unique name of pump curve (continuous string without space)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  WIDTH  LENGTH  OHEIGHT    DIAM     EXITYPE   RELEASETYPE   PEOPLE    DDAYS    WEIRTYPE    WEIRH    WEIRW   THETA   ET_MULT    PUMP_FLG   DEPTH_ON   DEPTH_OFF  PUMP_CURVE")
    pFile.WriteLine ("c BMPSITE" & vbTab & "WIDTH" & vbTab & "LENGTH" & vbTab & "OHEIGHT" & vbTab & "DIAM" & vbTab & "EXITYPE" & vbTab & "RELEASETYPE" & vbTab & "PEOPLE" & vbTab & "DDAYS" & vbTab & "WEIRTYPE" & vbTab & "WEIRH" & vbTab & "WEIRW" & vbTab & "THETA" & vbTab & "ET_MULT" & vbTab & "PUMP_FLG" & vbTab & "DEPTH_ON" & vbTab & "DEPTH_OFF" & vbTab & "PUMP_CURVE")
    pFile.Write (StrCard725BMPClassA)
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1120
'''Author:     Mira Chokshi
'''Purpose:    Write CLASS A Cistern Control Water Release Curve
'''Modified:   Mira Chokshi modified on 08/20/04 to add hourly water release
'''            for cistern type of BMP.
'''******************************************************************************
''Private Sub WriteCard1120()
''    pFile.WriteLine ("c1120 CLASS A Cistern Control Water Release Curve")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE = Class A BMP dimension group identifier in card 715")
''    pFile.WriteLine ("c Flow    = Hourly water release per capita from the Cistern Control (ft3/hr/capita)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  FLOW")
''    pFile.Write (StrCard1120BMPClassA)
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard730
'Author:     Mira Chokshi
'Purpose:    Write CLASS A Cistern Control Water Release Curve
'Modified:   Mira Chokshi modified on 08/20/04 to add hourly water release
'            for cistern type of BMP.
'******************************************************************************
Private Sub WriteCard730()
On Error GoTo ShowError
    pFile.WriteLine ("c730  Cistern Control Water Release Curve (applies if release type is cistern in card 720)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = Class A BMP dimension group identifier in card 715")
    pFile.WriteLine ("c Flow    = Hourly water release per capita from the Cistern Control (ft3/hr/capita)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  FLOW")
    pFile.WriteLine ("c BMPSITE" & vbTab & "FLOW")
    pFile.Write (StrCard730BMPClassA)
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
''
'''******************************************************************************
'''Subroutine: WriteCard1130
'''Author:     Mira Chokshi
'''Purpose:    Write CLASS B DIMENSION GROUPS
'''******************************************************************************
''Private Sub WriteCard1130()
''    pFile.WriteLine ("c1130 CLASS B BMP Site DIMENSION GROUPS")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE = BMP Site identifier in card 715")
''    pFile.WriteLine ("c WIDTH    = basin bottom width (ft)")
''    pFile.WriteLine ("c LENGTH   = basin bottom Length (ft)")
''    pFile.WriteLine ("c MAXDEPTH = Maximum depth of channel (ft)")
''    pFile.WriteLine ("c SLOPE1   = Side slope 1 (ft/ft)")
''    pFile.WriteLine ("c SLOPE2   = Side slope 2 (ft/ft)   (1-4)")
''    pFile.WriteLine ("c SLOPE3   = Side slope 3 (ft/ft)")
''    pFile.WriteLine ("c MANN_N = Manning  's roughness coefficient")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  WIDTH  LENGTH  MAXDEPTH   SLOPE1   SLOPE2   SLOPE3    MANN_N")
''    pFile.Write StrCard1130BMPClassB
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard735
'Author:     Mira Chokshi
'Purpose:    Write CLASS B DIMENSION GROUPS
'******************************************************************************
Private Sub WriteCard735()
On Error GoTo ShowError
    pFile.WriteLine ("c735 CLASS B BMP Site DIMENSION GROUPS")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = BMP Site identifier in card 715")
    pFile.WriteLine ("c WIDTH    = basin bottom width (ft)")
    pFile.WriteLine ("c LENGTH   = basin bottom Length (ft)")
    pFile.WriteLine ("c MAXDEPTH = Maximum depth of channel (ft)")
    pFile.WriteLine ("c SLOPE1   = Side slope 1 (ft/ft)")
    pFile.WriteLine ("c SLOPE2   = Side slope 2 (ft/ft)   (1-4)")
    pFile.WriteLine ("c SLOPE3   = Side slope 3 (ft/ft)")
    pFile.WriteLine ("c MANN_N = Manning  's roughness coefficient")
    pFile.WriteLine ("c ET_MULT  = multiplier to PET")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  WIDTH  LENGTH  MAXDEPTH   SLOPE1   SLOPE2   SLOPE3    MANN_N   ET_MULT")
    pFile.WriteLine ("c BMPSITE" & vbTab & "WIDTH" & vbTab & "LENGTH" & vbTab & "MAXDEPTH" & vbTab & "SLOPE1" & vbTab & "SLOPE2" & vbTab & "SLOPE3" & vbTab & "MANN_N" & vbTab & "ET_MULT")
    pFile.Write StrCard735BMPClassB
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub


'''******************************************************************************
'''Subroutine: WriteCard1140
'''Author:     Mira Chokshi
'''Purpose:    Write BOTTOM SOIL/VEGITATION CHARACTERISTICS FOR HOLTAN EQUATION AND UNDERDRAIN STRUCTURE
'''******************************************************************************
''Private Sub WriteCard1140()
''    pFile.WriteLine ("c1140 BMP Site BOTTOM SOIL/VEGITATION CHARACTERISTICS FOR HOLTAN EQUATION AND UNDERDRAIN STRUCTURE")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c HOLTAN EQUATION:    F = GI * AVEG * (Computed Available Soil Storage)^1.4 + FINFILT")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in c715")
''    pFile.WriteLine ("c SDEPTH    =  Soil Depth (ft)")
''    pFile.WriteLine ("c POROSITY  =  Soil Porosity (0-1)")
''    pFile.WriteLine ("c AVEG      =  Vegetative Parameter A (0.1-1.0) (Empirical)")
''    pFile.WriteLine ("c FINFILT   =  Soil layer infiltration rate (in/hr)")
''    pFile.WriteLine ("c UNDSWITCH =  Consider underdrain (1), Do not consider underdrain (0)")
''    pFile.WriteLine ("c UNDDEPTH  =  Depth of storage media below underdrain")
''    pFile.WriteLine ("c UNDVOID   =  Fraction of underdrain storage depth that is void space (0-1)")
''    pFile.WriteLine ("c UNDINFILT =  Background infiltration rate, below underdrain (in/hr)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE   SDEPTH   POROSITY   AVEG   FINFILT   UNDSWITCH   UNDDEPTH   UNDVOID   UNDINFILT")
''    pFile.Write StrCard1140BMPSoilIndex
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub
'******************************************************************************
'Subroutine: WriteCard740
'Author:     Mira Chokshi
'Purpose:    Write BOTTOM SOIL/VEGITATION CHARACTERISTICS FOR HOLTAN EQUATION AND UNDERDRAIN STRUCTURE
'******************************************************************************
Private Sub WriteCard740()
On Error GoTo ShowError
    'pFile.WriteLine ("c740 BMP Site BOTTOM SOIL/VEGITATION CHARACTERISTICS FOR HOLTAN EQUATION AND UNDERDRAIN STRUCTURE")
    'pFile.WriteLine ("c")
    'pFile.WriteLine ("c HOLTAN EQUATION:    F = GI * AVEG * (Computed Available Soil Storage)^1.4 + FINFILT")
    'pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in c715")
    'pFile.WriteLine ("c INFILTM   =  Infiltration Method (0-Holtan, 1-Green Ampt)")
    ''pFile.WriteLine ("c POLROTM   =  Pollutant Routing Method (0-Plug Flow,1-Completely mixed, >1-number of CSTRs in series)")
    'pFile.WriteLine ("c POLROTM   =  Pollutant Routing Method (1-Completely mixed, >1-number of CSTRs in series)")
    'pFile.WriteLine ("c POLREMM   =  Pollutant Removal Method (0-1st order decay, 1-kadlec and knight method )")
    'pFile.WriteLine ("c SDEPTH    =  Soil Depth (ft)")
    'pFile.WriteLine ("c POROSITY  =  Soil Porosity (0-1)")
    'pFile.WriteLine ("c FCAPACITY =  Soil Field Capacity (ft/ft)")
    'pFile.WriteLine ("c WPOINT    =  Soil Wilting Point (ft/ft)")
    'pFile.WriteLine ("c AVEG      =  Vegetative Parameter A (0.1-1.0) (Empirical)")
    'pFile.WriteLine ("c FINFILT   =  Soil layer infiltration rate (in/hr)")
    'pFile.WriteLine ("c UNDSWITCH =  Consider underdrain (1), Do not consider underdrain (0)")
    'pFile.WriteLine ("c UNDDEPTH  =  Depth of storage media below underdrain")
    'pFile.WriteLine ("c UNDVOID   =  Fraction of underdrain storage depth that is void space (0-1)")
    'pFile.WriteLine ("c UNDINFILT =  Background infiltration rate, below underdrain (in/hr)")
    'pFile.WriteLine ("c SUCTION   =  Average value of soil capillary suction along the wetting front, value must be greater than zero (in)")
    'pFile.WriteLine ("c HYDCON    =  Soil saturated hydraulic conductivity, value must be greater than zero (in/hr)")
    'pFile.WriteLine ("c IMDMAX    =  Difference between soil porosity and initial moisture content, value must be greater than or equal to zero (a fraction)")
    'pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   INFILTM   POLROTM POLREMM SDEPTH   POROSITY   FCAPACITY   WPOINT  AVEG   FINFILT   UNDSWITCH   UNDDEPTH   UNDVOID   UNDINFILT   SUCTION HYDCON  IMDMAX")
    'pFile.Write StrCard740BMPSoilIndex
    'pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'modified by ji Oct 2010
    pFile.WriteLine ("c740 BMP Site BOTTOM SOIL/VEGITATION CHARACTERISTICS")
    'pFile.WriteLine ("c")
    'pFile.WriteLine ("c HOLTAN EQUATION:    F = GI * AVEG * (Computed Available Soil Storage)^1.4 + FINFILT")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in c715")
    pFile.WriteLine ("c INFILTM   =  Infiltration Method (0-Green Ampt, 1-Horton, 2-Holtan)")
    pFile.WriteLine ("c POLROTM   =  Pollutant Routing Method (1-Completely mixed, >1-number of CSTRs in series)")
    pFile.WriteLine ("c POLREMM   =  Pollutant Removal Method (0-1st order decay, 1-kadlec and knight method )")
    pFile.WriteLine ("c SDEPTH    =  Soil Depth (ft)")
    pFile.WriteLine ("c POROSITY  =  Soil Porosity (0-1)")
    pFile.WriteLine ("c FCAPACITY =  Soil Field Capacity (ft/ft)")
    pFile.WriteLine ("c WPOINT    =  Soil Wilting Point (ft/ft)")
    pFile.WriteLine ("c AVEG      =  Vegetative Parameter A (0.1-1.0) (Empirical), only required for Holtan infiltration method")
    pFile.WriteLine ("c FINFILT   =  Soil layer infiltration rate (in/hr)")
    pFile.WriteLine ("c UNDSWITCH =  Consider underdrain (1), Do not consider underdrain (0)")
    pFile.WriteLine ("c UNDDEPTH  =  Depth of storage media below underdrain (ft)")
    pFile.WriteLine ("c UNDVOID   =  Fraction of underdrain storage depth that is void space (0-1)")
    pFile.WriteLine ("c UNDINFILT =  Background infiltration rate, below underdrain (in/hr)")
    pFile.WriteLine ("c SUCTION   =  Average value of soil capillary suction along the wetting front, value must be greater than zero (in), only required for Green-Ampt infiltration method")
    'pFile.WriteLine ("c HYDCON    =  Soil saturated hydraulic conductivity, value must be greater than zero (in/hr)")
    pFile.WriteLine ("c IMDMAX    =  Difference between soil porosity and initial moisture content, value must be greater than or equal to zero (a fraction), only required for Green-Ampt infiltration method")
    
    pFile.WriteLine ("c MAXINFILT =  Maximum rate on the Horton infiltration curve (in/hr), only required for Horton infiltration method")
    pFile.WriteLine ("c DECAYCONS =  Decay constant for the Horton infiltration curve (1/hr), only required for Horton infiltration method")
    pFile.WriteLine ("c DRYTIME   =  Time for a fully staurated soil to completely dry (day), only required for Horton infiltration method")
    pFile.WriteLine ("c MAXVOLUME =  Maximum infiltration volume possible (in), only required for Horton infiltration method")

    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   INFILTM   POLROTM POLREMM SDEPTH   POROSITY   FCAPACITY   WPOINT  AVEG   FINFILT   UNDSWITCH   UNDDEPTH   UNDVOID   UNDINFILT   SUCTION   IMDMAX   MAXINFILT   DECAYCONS   DRYTIME   MAXVOLUME")
    pFile.WriteLine ("c BMPSITE" & vbTab & "INFILTM" & vbTab & "POLROTM" & vbTab & "POLREMM" & vbTab & "SDEPTH" & vbTab & "POROSITY" & vbTab & "FCAPACITY" & vbTab & "WPOINT" & vbTab & "AVEG" & vbTab & "FINFILT" & vbTab & "UNDSWITCH" & vbTab & "UNDDEPTH" & vbTab & "UNDVOID" & vbTab & "UNDINFILT" & vbTab & "SUCTION" & vbTab & "IMDMAX" & vbTab & "MAXINFILT" & vbTab & "DECAYCONS" & vbTab & "DRYTIME" & vbTab & "MAXVOLUME")
    pFile.Write StrCard740BMPSoilIndex
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    
    
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1150
'''Author:     Mira Chokshi
'''Purpose:    Write HOLTAN GROWTH INDEX
'''******************************************************************************
''Private Sub WriteCard1150()
''    pFile.WriteLine ("c1150 BMP Site HOLTAN GROWTH INDEX")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c HOLTAN EQUATION:    F = GI * AVEG * (Computed Available Soil Storage)^1.4 + FINFILT")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in card 715")
''    pFile.WriteLine ("c GIi       =  12 monthly values for GI in HOLTAN equation")
''    pFile.WriteLine ("c              Where i = jan, feb, mar ... dec")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c  BMPSITE   jan   feb   mar   apr    may    jun    jul    aug    sep    oct    nov    dec")
''    pFile.Write StrCard1150BMPGrowthIndex
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard745
'Author:     Mira Chokshi
'Purpose:    Write HOLTAN GROWTH INDEX
'******************************************************************************
Private Sub WriteCard745()
On Error GoTo ShowError
    pFile.WriteLine ("c745 BMP Site HOLTAN GROWTH INDEX")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c HOLTAN EQUATION:    F = GI * AVEG * (Computed Available Soil Storage)^1.4 + FINFILT")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in card 715")
    pFile.WriteLine ("c GIi       =  12 monthly values for GI in HOLTAN equation")
    pFile.WriteLine ("c              Where i = jan, feb, mar ... dec")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  BMPSITE   jan   feb   mar   apr    may    jun    jul    aug    sep    oct    nov    dec")
    pFile.WriteLine ("c BMPSITE" & vbTab & "jan" & vbTab & "feb" & vbTab & "mar" & vbTab & "apr" & vbTab & "may" & vbTab & "jun" & vbTab & "jul" & vbTab & "aug" & vbTab & "sep" & vbTab & "oct" & vbTab & "nov" & vbTab & "dec")
    pFile.Write StrCard745BMPGrowthIndex
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard747()
On Error GoTo ShowError
    pFile.WriteLine ("c747 BMP Site Initial Moisture Content")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = BMP Site identifier in card 715")
    pFile.WriteLine ("c WATDEP_i = initial surface water depth (ft)")
    pFile.WriteLine ("c THETA_i  = initial soil moisture (ft/ft)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  WATDEP_i  THETA_i")
    pFile.WriteLine ("c BMPSITE" & vbTab & "WATDEP_i" & vbTab & "THETA_i")
    pFile.Write StrCard747BMPClassABF
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

''
'''******************************************************************************
'''Subroutine: WriteCard1160
'''Author:     Mira Chokshi
'''Purpose:    Write Conduit Dimension Groups
'''******************************************************************************
''Private Sub WriteCard1160()
''    'Call conduit cards details to get Class C- conduit parameters
''    pFile.WriteLine ("c1160 Class C Conduit Dimension Groups")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c NAME = BMP site identifier in card 715")
''    pFile.WriteLine ("c INLET_NODE = BMP Id at the entrance of the conduit")
''    pFile.WriteLine ("c OUTLET_NODE = BMP Id at the exit of the conduit")
''    pFile.WriteLine ("c LENGTH = Conduit length")
''    pFile.WriteLine ("c MANNING_N = Manning's roughness coefficient")
''    pFile.WriteLine ("c INLET_IEL = Invert Elevation at the entrance of the conduit")
''    pFile.WriteLine ("c OUTLET_IEL = Invert Elevation at the exit of the conduit")
''    pFile.WriteLine ("c INIT_FLOW = Initial flow in the conduit (cfs)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c  NAME   INLET_NODE   OUTLET_NODE  LENGTH   MANNING_N    INLET_IEL    OUTLET_IEL    INIT_FLOW")
''    pFile.Write StrCard1160ConduitDimensions
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub



'******************************************************************************
'Subroutine: WriteCard750
'Author:     Mira Chokshi
'Purpose:    Write Conduit Dimension Groups
'******************************************************************************
Private Sub WriteCard750()
On Error GoTo ShowError
    'Call conduit cards details to get Class C- conduit parameters
    pFile.WriteLine ("c750 Class-C Conduit Parameters (required if BMPSITE is CLASS-C in card 715)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = BMP site identifier in card 715")
    pFile.WriteLine ("c INLET_NODE = BMP Id at the entrance of the conduit")
    pFile.WriteLine ("c OUTLET_NODE = BMP Id at the exit of the conduit")
    pFile.WriteLine ("c LENGTH = Conduit length (ft)")
    pFile.WriteLine ("c MANNING_N = Manning's roughness coefficient")
    pFile.WriteLine ("c INLET_IEL = Invert Elevation at the entrance of the conduit (ft)")
    pFile.WriteLine ("c OUTLET_IEL = Invert Elevation at the exit of the conduit (ft)")
    pFile.WriteLine ("c INIT_FLOW = Initial flow in the conduit (cfs)")
    pFile.WriteLine ("c INLET_HL    = Head loss coefficient at the entrance of the conduit")
    pFile.WriteLine ("c OUTLET_HL   = Head loss coefficient at the exit of the conduit")
    pFile.WriteLine ("c AVERAGE_HL  = Head loss coefficient along the length of the conduit")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  BMPSITE   INLET_NODE   OUTLET_NODE  LENGTH   MANNING_N    INLET_IEL    OUTLET_IEL    INIT_FLOW   INLET_HL    OUTLET_HL   AVERAGE_HL")
    pFile.WriteLine ("c BMPSITE" & vbTab & "INLET_NODE" & vbTab & "OUTLET_NODE" & vbTab & "LENGTH" & vbTab & "MANNING_N" & vbTab & "INLET_IEL" & vbTab & "OUTLET_IEL" & vbTab & "INIT_FLOW" & vbTab & "INLET_HL" & vbTab & "OUTLET_HL" & vbTab & "AVERAGE_HL")
    pFile.Write StrCard750ConduitDimensions
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub



'''******************************************************************************
'''Subroutine: WriteCard1170
'''Author:     Mira Chokshi
'''Purpose:    Write Conduit Cross-Section
'''******************************************************************************
''Private Sub WriteCard1170()
''    pFile.WriteLine ("c1170 Class C Conduit Cross Sections")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c LINK = BMP site identifier in card 715")
''    pFile.WriteLine ("c TYPE = Conduit Type (rectangular, circular...)")
''    pFile.WriteLine ("c GEOM1 = Geometric cross-sectional property of the conduit")
''    pFile.WriteLine ("c GEOM2 = Geometric cross-sectional property of the conduit")
''    pFile.WriteLine ("c GEOM3 = Geometric cross-sectional property of the conduit")
''    pFile.WriteLine ("c GEOM4 = Geometric cross-sectional property of the conduit")
''    pFile.WriteLine ("c BARRELS = Number of Barrels in the conduit")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c  LINK   TYPE   GEOM1   GEOM2   GEOM3    GEOM4    BARRELS")
''    pFile.Write StrCard1170ConduitCrossSections
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub
'******************************************************************************
'Subroutine: WriteCard755
'Author:     Mira Chokshi
'Purpose:    Write Conduit Cross-Section
'******************************************************************************
Private Sub WriteCard755()
On Error GoTo ShowError
    pFile.WriteLine ("c755 Class C Conduit Cross Sections")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c LINK = BMP site identifier in card 715")
    pFile.WriteLine ("c TYPE = Conduit Type (rectangular, circular...)")
    pFile.WriteLine ("c GEOM1 = Geometric cross-sectional property of the conduit")
    pFile.WriteLine ("c GEOM2 = Geometric cross-sectional property of the conduit")
    pFile.WriteLine ("c GEOM3 = Geometric cross-sectional property of the conduit")
    pFile.WriteLine ("c GEOM4 = Geometric cross-sectional property of the conduit")
    pFile.WriteLine ("c BARRELS = Number of Barrels in the conduit")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c  LINK   TYPE   GEOM1   GEOM2   GEOM3    GEOM4    BARRELS")
    pFile.WriteLine ("c LINK" & vbTab & "TYPE" & vbTab & "GEOM1" & vbTab & "GEOM2" & vbTab & "GEOM3" & vbTab & "GEOM4" & vbTab & "BARRELS")
    pFile.Write StrCard755ConduitCrossSections
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'******************************************************************************
Public Sub WriteCard760()
    'Call SetConduitTransectDict
    GetIrregularCSParameters
    
    pFile.WriteLine ("c760 Irregular Cross Sections")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c Format of transect data follows:")
    'pFile.WriteLine ("c NC  nLeft  nRight  nChannel")
    'pFile.WriteLine ("c X1  name  nSta  xLeftBank  xRightBank  0  0  0  xFactor  yFactor")
    'pFile.WriteLine ("c GR  Elevation  Station  ...")
    pFile.WriteLine ("c NC" & vbTab & "nLeft" & vbTab & "nRight" & vbTab & "nChannel")
    pFile.WriteLine ("c X1" & vbTab & "name" & vbTab & "nSta" & vbTab & "xLeftBank" & vbTab & "xRightBank" & vbTab & "0" & vbTab & "0" & vbTab & "0" & vbTab & "xFactor" & vbTab & "yFactor")
    pFile.WriteLine ("c GR" & vbTab & "Elevation" & vbTab & "Station" & vbTab & "...")
    pFile.WriteLine ("c")
    pFile.Write StrCard760IrregularCrossSections
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
End Sub


Private Sub WriteCard762()
On Error GoTo ShowError
    pFile.WriteLine ("c762 Area BMP Parameters (required if BMPTYPE is AREABMP in card 715)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE      = BMP site identifier in card 715")
    pFile.WriteLine ("c Area         = BMP area (ft2)")
    pFile.WriteLine ("c FLength      = flow length (ft) note: area width = area / flow length")
    pFile.WriteLine ("c DStorage     = Surface depression storage (in)")
    pFile.WriteLine ("c SLOPE        = Overland slope (ft / ft)")
    pFile.WriteLine ("c MANNING_N    = Overland Manning's roughness coefficient")
    pFile.WriteLine ("c SAT_INFILT   = Saturated infiltration rate (in/hr)")
    pFile.WriteLine ("c POLREMM      =  Pollutant Removal Method (0-1st order decay, 1-kadlec and knight method)")
    pFile.WriteLine ("c DCIA         = Percentage of Directly Connected Impervious Area (0-100)")
    pFile.WriteLine ("c TOTAL_IMP_DA = Total Impervious Drainage Area (acre)")
    'pFile.WriteLine ("c BMPSITE   Area    FLength D   SLOPE   MANNING_N   SAT_INFILT  POLREMM DCIA  TOTAL_IMP_DA")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE" & vbTab & "Area" & vbTab & "FLength" & vbTab & "DStorage" & vbTab & "SLOPE" & vbTab & "MANNING_N" & vbTab & "SAT_INFILT" & vbTab & "POLREMM" & vbTab & "DCIA" & vbTab & "TOTAL_IMP_DA")
    pFile.Write StrCard762BMPClassE
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard761()
On Error GoTo ShowError

    pFile.WriteLine ("c761 BufferStrip BMP Parameters (required if BMPTYPE is BUFFERSTRIP in card 715)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   = BMP site identifier in card 715")
    pFile.WriteLine ("c Width     = BMP width (ft)")
    pFile.WriteLine ("c FLength   = flow length (ft)")
    pFile.WriteLine ("c DStorage  = Surface depression storage (in)")
    pFile.WriteLine ("c SLOPE     = Overland slope (ft / ft)")
    pFile.WriteLine ("c MANNING_N = Overland Manning's roughness coefficient")
    pFile.WriteLine ("c POLREMM   =  Pollutant Removal Method (0-1st order decay, 1-kadlec and knight method)")
    pFile.WriteLine ("c ET_MULT   = multiplier to PET")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   Width    FLength    D   SLOPE   MANNING_N   POLREMM   ET_MULT")
    pFile.WriteLine ("c BMPSITE" & vbTab & "Width" & vbTab & "FLength" & vbTab & "DStorage" & vbTab & "SLOPE" & vbTab & "MANNING_N" & vbTab & "POLREMM" & vbTab & "ET_MULT")
    pFile.Write StrCard761BMPClassF
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'''******************************************************************************
'''Subroutine: WriteCard1190
'''Author:     Mira Chokshi
'''Purpose:    Write Conduit Dimension Groups
'''******************************************************************************
''Private Sub WriteCard1190()
''    'Call conduit cards details to get Class C- conduit parameters
''    pFile.WriteLine ("c1190 Class C Conduit Dimension Groups")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c NAME = BMP site identifier in card 715")
''    pFile.WriteLine ("c INLET = Head loss coefficient at the entrance of the conduit")
''    pFile.WriteLine ("c OUTLET = Head loss coefficient at the exit of the conduit")
''    pFile.WriteLine ("c AVERAGE = Head loss coefficient along the length of the conduit")
''    pFile.WriteLine ("c FLAP_GATE = Flag Gate present to stop conduit backflow")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c  NAME   INLET   OUTLET  AVERAGE   FLAP_GATE")
''    pFile.Write StrCard1190ConduitLosses
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'**************************************
'Commented the following two subs - VFS is handled differently - June 18, 2007
'**************************************

'******************************************************************************
'Subroutine: WriteCard1200
'Author:     Mira Chokshi
'Purpose:    Write Conduit Dimension Groups
'******************************************************************************
''Private Sub WriteCard1200()
''    'Call conduit cards details to get Class D - Buffer Strip parameters
''    pFile.WriteLine ("c1200 CLASS D (Buffer Strip) Site GROUPS")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c   BMPSITE = Class D BMP dimension group identifier in card 715")
''    pFile.WriteLine ("c   Width  = Buffer strip width along the stream (ft)")
''    pFile.WriteLine ("c   INFILT1 =")
''    pFile.WriteLine ("c   INFILT2 =")
''    pFile.WriteLine ("c   INFILT3 =")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  WIDTH  INFILT1  INFILT2  INFILT3")
''    pFile.Write StrCard1200VFSParameters
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub


'******************************************************************************
'Subroutine: WriteCard1210
'Author:     Mira Chokshi
'Purpose:    Write Conduit Dimension Groups
'******************************************************************************
''Private Sub WriteCard1210()
''    'Call conduit cards details to get Class D - Buffer Strip parameters
''    pFile.WriteLine ("c1210 CLASS D (Buffer Strip) Site Segments")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE = Class D BMP dimension group identifier in card 715")
''    pFile.WriteLine ("c SEGMENT   = Segment ID")
''    pFile.WriteLine ("c LENGTH    = Buffer strip length along the flow direction (perpendicular to stream) for each segment (ft)")
''    pFile.WriteLine ("c SLOPE     = Slope (vertical drop / longitudinal distance)")
''    pFile.WriteLine ("c Manning 's n   =")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  SEGMENT  LENGTH  SLOPE   MANNING'S")
''    pFile.Write StrCard1210VFSParameters
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'''******************************************************************************
'''Subroutine: WriteCard1300
'''Author:     Mira Chokshi
'''Purpose:    Write LAND ROUTING NETWORK
'''******************************************************************************
''Private Sub WriteCard1300()
''    pFile.WriteLine ("c1300 LAND ROUTING NETWORK")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c UniqueID   = Identifies an instance of LANDTYPE in SCHEMATIC")
''    pFile.WriteLine ("c LANDTYPE   = Corresponds to LANDTYPE in c710")
''    pFile.WriteLine ("c AREA       = Area of LANDTYPE in ACRES")
''    pFile.WriteLine ("c DS         = UNIQUE ID of DS BMP (0 - no BMP, add to end)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c UniqueID    LANDTYPE     AREA      DS")
''    pFile.Write GetLandTypeRouting
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard790
'Author:     Mira Chokshi
'Purpose:    Write LAND ROUTING NETWORK
'******************************************************************************
Private Sub WriteCard790()
On Error GoTo ShowError
    pFile.WriteLine ("c790 LAND TO BMP ROUTING NETWORK (required for external land simulation control in card 700)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c UniqueID   = Identifies an instance of LANDTYPE in SCHEMATIC")
    pFile.WriteLine ("c LANDTYPE   = Corresponds to LANDTYPE in c710")
    pFile.WriteLine ("c AREA       = Area of LANDTYPE in ACRES")
    pFile.WriteLine ("c DS         = UNIQUE ID of DS BMP (0 - no BMP, add to end)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c UniqueID    LANDTYPE     AREA      DS")
    pFile.WriteLine ("c UniqueID" & vbTab & "LANDTYPE" & vbTab & "AREA" & vbTab & "DS")
    pFile.Write StrCard790LandTypeRouting 'GetLandTypeRouting
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1310
'''Author:     Mira Chokshi
'''Purpose:    Write BMP ROUTING NETWORK
'''Modified:   Mira Chokshi modified on 08/19/04. The new changes allow easy
'''            implementation of a splitter. Outlet type = "1" indicates no splitter
'''******************************************************************************
''Private Sub WriteCard1310()
''    pFile.WriteLine ("c1310 BMP Site ROUTING NETWORK")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in card 715")
''    pFile.WriteLine ("c OUTLET_TYPE   =  Outlet type (1-total, 2-weir, 3-orifice or channel, 4-underdrain)")
''    pFile.WriteLine ("c DS            =  Downstrem BMP site identifier in card 715 (0 - no BMP, add to end)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE      OUTLET_TYPE      DS")
''    pFile.Write GetBMPNetworkRouting
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard795
'Author:     Mira Chokshi
'Purpose:    Write BMP ROUTING NETWORK
'Modified:   Mira Chokshi modified on 08/19/04. The new changes allow easy
'            implementation of a splitter. Outlet type = "1" indicates no splitter
'******************************************************************************
Private Sub WriteCard795()
On Error GoTo ShowError
    pFile.WriteLine ("c795 BMP Site ROUTING NETWORK")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   =  BMPSITE identifier in card 715")
    pFile.WriteLine ("c OUTLET_TYPE   =  Outlet type (1-total, 2-weir, 3-orifice or channel, 4-underdrain)")
    pFile.WriteLine ("c DS            =  Downstrem BMP site identifier in card 715 (0 - no BMP, add to end)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE      OUTLET_TYPE      DS")
    pFile.WriteLine ("c BMPSITE" & vbTab & "OUTLET_TYPE" & vbTab & "DS")
    pFile.Write GetBMPNetworkRouting
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'''******************************************************************************
'''Subroutine: WriteCard1320
'''Author:     Mira Chokshi
'''Purpose:    Write BMP Pollutant Decay/Loss rates
'''******************************************************************************
''Private Sub WriteCard1320()
''    pFile.WriteLine ("c1320 BMP SITE Pollutant Decay/Loss rates")
''    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
''    pFile.WriteLine ("c QUALDECAYi = First-order decay rate for pollutant i (hr^-1)")
''    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  QUALDECAY1  QUALDECAY2 ... QUALDECAYN")
''    pFile.Write StrCard1320DecayFactors
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub
'******************************************************************************
'Subroutine: WriteCard765
'Author:     Mira Chokshi
'Purpose:    Write BMP Pollutant Decay/Loss rates
'******************************************************************************
Private Sub WriteCard765()
On Error GoTo ShowError
    pFile.WriteLine ("c765 BMP SITE Pollutant Decay/Loss rates")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
    pFile.WriteLine ("c QUALDECAYi = First-order decay rate for pollutant i (hr^-1)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  QUALDECAY1  QUALDECAY2 ... QUALDECAYN")
    pFile.WriteLine ("c BMPSITE" & vbTab & "QUALDECAY1" & vbTab & "QUALDECAY2" & vbTab & "..." & vbTab & "QUALDECAYN")
    pFile.Write StrCard765DecayFactors
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
Private Sub WriteCard766()
On Error GoTo ShowError
    pFile.WriteLine ("c766 Pollutant K' values (applies when pollutant removal method is kadlec and knight method in card 740)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
    pFile.WriteLine ("C K 'i        = Constant rate for pollutant i (ft/yr)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from card 705)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  QUALK'1  QUALK'2 ... QUALK'N")
    pFile.WriteLine ("c BMPSITE" & vbTab & "QUALK'1" & vbTab & "QUALK'2" & vbTab & "..." & vbTab & "QUALK'N")
    pFile.Write StrCard766KFactors
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
Private Sub WriteCard767()
On Error GoTo ShowError
    pFile.WriteLine ("c767 Pollutant C* values (applies when pollutant removal method is kadlec and knight method in card 740)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
    pFile.WriteLine ("c C*i        = Background concentration for pollutant i (mg/l)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from card 705)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  QUALC*1  QUALC*2 ... QUALC*N")
    pFile.WriteLine ("c BMPSITE" & vbTab & "QUALC*1" & vbTab & "QUALC*2" & vbTab & "..." & vbTab & "QUALC*N")
    pFile.Write StrCard767CValues
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub


'******************************************************************************
'Subroutine: WriteCard775
'Author:     Sabu Paul
'Purpose:    Write sediment parameters
'******************************************************************************
Private Sub WriteCard775()
On Error GoTo ShowError
    pFile.WriteLine ("c775 Sediment General Parameters (required if pollutant type is sediment in card 705)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
    pFile.WriteLine ("c BEDWID     = Bed width (ft) - this is constant for the entire simulation period")
    pFile.WriteLine ("c BEDDEP     = Initial bed depth (ft)")
    pFile.WriteLine ("c BEDPOR     = Bed sediment porosity")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   BEDWID   BEDDEP   BEDPOR")
    pFile.WriteLine ("c BMPSITE" & vbTab & "BEDWID" & vbTab & "BEDDEP" & vbTab & "BEDPOR")
    pFile.Write StrCard775Sediment
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'******************************************************************************
'Subroutine: WriteCard780
'Author:     Sabu Paul
'Purpose:    Write sediment parameters
'******************************************************************************
Private Sub WriteCard780()
On Error GoTo ShowError
    pFile.WriteLine ("c780 Sand Transport Parameters (required if pollutant type is sediment in card 705)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   = BMP site identifier in card 715")
    pFile.WriteLine ("c D         = Effective diameter of the transported sand particles (in)")
    pFile.WriteLine ("c W         = The corresponding fall velocity in still water (in/sec)")
    pFile.WriteLine ("c RHO       = The density of the sand particles (lb/ft3)")
    pFile.WriteLine ("c KSAND     = The coefficient in the sandload power function formula")
    pFile.WriteLine ("c EXPSND    = The exponent in the sandload power function formula")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   D        W        RHO      KSAND    EXPSND")
    pFile.WriteLine ("c BMPSITE" & vbTab & "D" & vbTab & "W" & vbTab & "RHO" & vbTab & "KSAND" & vbTab & "EXPSND")
    pFile.Write StrCard780SandTransport
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'******************************************************************************
'Subroutine: WriteCard785
'Author:     Sabu Paul
'Purpose:    Write sediment parameters
'******************************************************************************
Private Sub WriteCard785()
On Error GoTo ShowError
    pFile.WriteLine ("c785 Silt Transport Parameters (required if pollutant type is sediment in card 705)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   = BMP site identifier in card 715")
    pFile.WriteLine ("c D         = Effective diameter of the transported silt particles (in)")
    pFile.WriteLine ("c W         = The corresponding fall velocity in still water (in/sec)")
    pFile.WriteLine ("c RHO       = The density of the silt particles (lb/ft3)")
    pFile.WriteLine ("c TAUCD     = The critical bed shear stress for deposition (lb/ft2)")
    pFile.WriteLine ("c TAUCS     = The critical bed shear stress for scour (lb/ft2)")
    pFile.WriteLine ("c M         = The erodibility coefficient of the silt particles (lb/ft2/day)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   D        W        RHO      TAUCD    TAUCS    M")
    pFile.WriteLine ("c BMPSITE" & vbTab & "D" & vbTab & "W" & vbTab & "RHO" & vbTab & "TAUCD" & vbTab & "TAUCS" & vbTab & "M")
    pFile.Write StrCard785SiltTransport
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

'******************************************************************************
'Subroutine: WriteCard786
'Author:     Sabu Paul
'Purpose:    Write sediment parameters
'******************************************************************************
Private Sub WriteCard786()
On Error GoTo ShowError
    pFile.WriteLine ("c786 Clay Transport Parameters (required if pollutant type is sediment in card 705)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE   = BMP site identifier in card 715")
    pFile.WriteLine ("c D         = Effective diameter of the transported clay particles (in)")
    pFile.WriteLine ("c W         = The corresponding fall velocity in still water (in/sec)")
    pFile.WriteLine ("c RHO       = The density of the silt/clay particles (lb/ft3)")
    pFile.WriteLine ("c TAUCD     = The critical bed shear stress for deposition (lb/ft2)")
    pFile.WriteLine ("c TAUCS     = The critical bed shear stress for scour (lb/ft2)")
    pFile.WriteLine ("c M         = The erodibility coefficient of the clay particles (lb/ft2/day)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE   D        W        RHO      TAUCD    TAUCS    M")
    pFile.WriteLine ("c BMPSITE" & vbTab & "D" & vbTab & "W" & vbTab & "RHO" & vbTab & "TAUCD" & vbTab & "TAUCS" & vbTab & "M")
    pFile.Write StrCard786ClayTransport
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub


''******************************************************************************
''Subroutine: WriteCard1330
''Author:     Mira Chokshi
''Purpose:    Write BMP Underdrain Pollutant Percent Removal (applies when underdrain is on in card 740)
''******************************************************************************
'Private Sub WriteCard1330()
'    pFile.WriteLine ("c1330 BMP Underdrain Pollutant Percent Removal (applies when underdrain is on in card 740)")
'    pFile.WriteLine ("c BMPSITE     = BMPSITE identifier in card 715")
'    pFile.WriteLine ("c QUALPCTREMi = Perecent Removal for pollutant i through underdrain (0-1)")
'    pFile.WriteLine ("c               Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
'    pFile.WriteLine ("c")
'    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
'    pFile.WriteLine ("c BMPSITE  QUALPCTREM1  QUALPCTREM2 ... QUALPCTREMN")
'    pFile.Write StrCard1330PercentRemoval
'    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
'End Sub

'******************************************************************************
'Subroutine: WriteCard770
'Author:     Mira Chokshi
'Purpose:    Write BMP Underdrain Pollutant Percent Removal (applies when underdrain is on in card 740)
'******************************************************************************
Private Sub WriteCard770()
On Error GoTo ShowError
    pFile.WriteLine ("c770 BMP Underdrain Pollutant Percent Removal (applies when underdrain is on in card 740)")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE     = BMPSITE identifier in card 715")
    pFile.WriteLine ("c QUALPCTREMi = Perecent Removal for pollutant i through underdrain (0-1)")
    pFile.WriteLine ("c               Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  QUALPCTREM1  QUALPCTREM2 ... QUALPCTREMN")
    pFile.WriteLine ("c BMPSITE" & vbTab & "QUALPCTREM1" & vbTab & "QUALPCTREM2" & vbTab & "..." & vbTab & "QUALPCTREMN")
    pFile.Write StrCard770PercentRemoval
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1400
'''Author:     Mira Chokshi
'''Purpose:    Write Adjustable BMP parameters (applies when parameter is selected for optimization)
'''******************************************************************************
''Private Sub WriteCard1400()
''    pFile.WriteLine ("c1400 BMP SITE Adjustable Parameters")
''    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
''    pFile.WriteLine ("c VARIABLE   = Variable name")
''    pFile.WriteLine ("c FROM       = From value in the range")
''    pFile.WriteLine ("c TO         = To value in the range")
''    pFile.WriteLine ("c STEP       = Increment step")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE  VARIABLE  FROM   TO   STEP")
''    pFile.Write StrCard1400AdjustParameter
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard810
'Author:     Mira Chokshi
'Purpose:    Write Adjustable BMP parameters (applies when parameter is selected for optimization)
'******************************************************************************
Private Sub WriteCard810()
  On Error GoTo ErrorHandler

    pFile.WriteLine ("c810 BMP SITE Adjustable Parameters")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE    = BMP site identifier in card 715")
    pFile.WriteLine ("c VARIABLE   = Variable name (must use the exact same keyword)")
    pFile.WriteLine ("c            LENGTH --- BMP length,")
    pFile.WriteLine ("c            NUMUNIT --- number of units,")
    pFile.WriteLine ("c            WEIRH --- weir height,")
    pFile.WriteLine ("c            SDEPTH --- soil media depth,")
    pFile.WriteLine ("c            DCIA --- directly connected impervious area for area BMP type,")
    pFile.WriteLine ("c            MAXDEPTH --- maximum surface storage depth for swale,")
    pFile.WriteLine ("c            CECURVE --- cost-effectiveness curve for Tier-1 solution")

    pFile.WriteLine ("c FROM       = From value in the range")
    pFile.WriteLine ("c TO         = To value in the range")
    pFile.WriteLine ("c STEP       = Increment step")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE  VARIABLE  FROM   TO   STEP")
    pFile.WriteLine ("c BMPSITE" & vbTab & "VARIABLE" & vbTab & "FROM" & vbTab & "TO" & vbTab & "STEP")
    pFile.Write StrCard810AdjustParameter
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

  Exit Sub
ErrorHandler:
    gHasInFileError = True
End Sub
'''******************************************************************************
'''Subroutine: WriteCard1410
'''Author:     Mira Chokshi
'''Purpose:    BMP Cost Functions
'''Modified:   Sabu Paul added this new card. The BMP cost is a function of Depth,
'''            Area and other constants.
'''******************************************************************************
''Private Sub WriteCard1410()
''    pFile.WriteLine ("c1410 BMP Cost Functions")
''    pFile.WriteLine ("c Cost ($) = (Aa) Area^(Ab)x(Da)Depth^(Db) + (LdCost)Area + (ConstCost)")
''    pFile.WriteLine ("c Depth = WeirHeight + SoilDepth + UnderdrainDepth")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c BMPSITE = BMP site identifier in card 715")
''    pFile.WriteLine ("c")
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''    pFile.WriteLine ("c BMPSITE       Aa      Ab      Da      Db      LdCost  ConstCost")
''    pFile.Write StrCard1410BMPCost
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'******************************************************************************
'Subroutine: WriteCard805
'Author:     Mira Chokshi
'Purpose:    BMP Cost Functions
'Modified:   Sabu Paul added this new card. The BMP cost is a function of Depth,
'            Area and other constants.
'******************************************************************************
Private Sub WriteCard805()
  On Error GoTo ErrorHandler

'    pFile.WriteLine ("c805 BMP Cost Functions")
'    pFile.WriteLine ("c Cost ($) = (Aa) Area^(Ab)x(Da)Depth^(Db) + (LdCost)Area + (ConstCost)")
'    pFile.WriteLine ("c Depth = WeirHeight + SoilDepth + UnderdrainDepth")
'    pFile.WriteLine ("c")
'    pFile.WriteLine ("c BMPSITE = BMP site identifier in card 715")
'    pFile.WriteLine ("c")
'    pFile.WriteLine ("c BMPSITE       Aa      Ab      Da      Db      LdCost  ConstCost")
'    pFile.Write StrCard805BMPCost
'    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    pFile.WriteLine ("c805 BMP Cost Functions")
    
    'pFile.WriteLine ("c Cost ($) = ((LinearCost) Length + (AreaCost)Area + (TotalVolumeCost)TotalVolume + (MediaVolumeCost)SoilMediaVolume + (UnderDrainVolumeCost)UnderDrainVolume+ (ConstantCost)) * (1+PercentCost/100)")
    'pFile.WriteLine ("c Cost ($) = ((LinearCost)Length^(LengthExp)  + (AreaCost)Area^(AreaExp)  + (TotalVolumeCost)TotalVolume^(TotalVolExp)")
    'pFile.WriteLine ("c          + (MediaVolumeCost)SoilMediaVolume^(MediaVolExp) + (UnderDrainVolumeCost)UnderDrainVolume^(UDVolExp)")
    'pFile.WriteLine ("c          + (Unitcost) +  (ConstantCost)) * (1+PercentCost/100)")
    pFile.WriteLine ("c Cost ($) = ((LinearCost)*Length^(LengthExp) + (AreaCost)*Area^(AreaExp) + (TotalVolumeCost)*TotalVolume^(TotalVolExp)")
    pFile.WriteLine ("c (MediaVolumeCost) * SoilMediaVolume ^ (MediaVolExp) + (UnderDrainVolumeCost) * UnderDrainVolume ^ (UDVolExp)")
    pFile.WriteLine ("c          + (ConstantCost)) * (1+PercentCost/100)")

    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE                 = BMP site identifier in card 715")
    pFile.WriteLine ("c LinearCost              = Cost per unit length of the BMP structure ($/ft)")
    pFile.WriteLine ("c AreaCost                = Cost per unit area of the BMP structure ($/ft^2)")
    pFile.WriteLine ("c TotalVolumeCost         = Cost per unit total volume of the BMP structure ($/ft^3)")
    pFile.WriteLine ("c MediaVolumeCost         = Cost per unit volume of the soil media ($/ft^3)")
    pFile.WriteLine ("c UnderDrainVolumeCost    = Cost per unit volume of the under drain structure ($/ft^3)")
    'pFile.WriteLine ("c UnitCost                = Cost per unit of the functional component ($/num)")
    pFile.WriteLine ("c ConstantCost            = Constant cost ($)")
    pFile.WriteLine ("c PercentCost             = Cost in percentage of all other cost (%)")
    pFile.WriteLine ("c LengthExp               = Exponent for linear unit")
    pFile.WriteLine ("c AreaExp                 = Exponent for area unit")
    pFile.WriteLine ("c TotalVolExp             = Exponent for total volume unit")
    pFile.WriteLine ("c MediaVolExp             = Exponent for soil media volume unit")
    pFile.WriteLine ("c UDVolExp                = Exponent for underdrain volume unit")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE       LinearCost      AreaCost      TotalVolumeCost MediaVolumeCost UnderDrainVolumeCost    ConstantCost  PercentCost   LengthExp   AreaExp     TotalVolExp   MediaVolExp    UDVolExp")
    pFile.WriteLine ("c BMPSITE" & vbTab & "LinearCost" & vbTab & "AreaCost" & vbTab & "TotalVolumeCost" & vbTab & "MediaVolumeCost" & vbTab & "UnderDrainVolumeCost" & vbTab & "ConstantCost" & vbTab & "PercentCost" & vbTab & "LengthExp" & vbTab & "AreaExp" & vbTab & "TotalVolExp" & vbTab & "MediaVolExp" & vbTab & "UDVolExp")
    pFile.Write StrCard805BMPCost
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

  Exit Sub
ErrorHandler:
    gHasInFileError = True
  'HandleError False, "WriteCard805 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.description, 4
End Sub

'''******************************************************************************
'''Subroutine: WriteCard1420
'''Author:     Mira Chokshi
'''Purpose:    Assessment point details
'''Modified:   Sabu Paul added new card details to enter additional information
'''            for BMPs added as assessment points.
'''******************************************************************************
''Private Sub WriteCard1420()
''    GetAssessPointOptimizationDetails
''    pFile.WriteLine ("c1420 Assessment Point and Evaluation Factor")
''    pFile.WriteLine ("c Option -- Optimization options")
''    pFile.WriteLine ("c     0 = no optimization")
''    pFile.WriteLine ("c     1 = specific control target and minimize cost")
''    pFile.WriteLine ("c     2 = cost limit and maximize control")
''    pFile.WriteLine ("c     3 = generate trade-off curve")
''    pFile.WriteLine ("c StopDelta -- Criteria for stopping the optimization iteration")
''    pFile.WriteLine ("c              in $ for option 1, meaning if the cost not improved by this criteria, stop the search")
''    pFile.WriteLine ("c              in % for option 2, meaning if the Evaluation Factor not improved by this criteria, stop the search")
''    pFile.WriteLine ("c MaxRunTime(hr) -- Maximum search time (in Hour) allowed")
''    pFile.WriteLine ("c NumBest -- Number of best solutions for output")
''    pFile.WriteLine ("c BMPSITE -- BMP site identifier in card 715 if it is an assessment point")
''    pFile.WriteLine ("c FactorGroup -- -1 for flow related, positive number for pollutant column order in card 765 and 770")
''    pFile.WriteLine ("c FactorType -- Evaluation Factor Type (negative number for flow related and positive number for pollutant related)")
''    pFile.WriteLine ("c    -1 = AAFV Annual Average Flow Volume (ft3/yr)")
''    pFile.WriteLine ("c    -2 = PDF  Peak Discharge Flow (cfs)")
''    pFile.WriteLine ("c    -3 = FEF  Flow Exceeding frequency (cfs)")
''    pFile.WriteLine ("c     1 = AAL  Annual Average Load (kg/yr)")
''    pFile.WriteLine ("c     2 = AAC  Annual Average Concentration (mg/L)")
''    pFile.WriteLine ("c     3 = MAC  Maximum #days Average Concentraion (mg/L)")
''    pFile.WriteLine ("c CalcDays --  FactorType 3 (MAC): Maxmimum #Days;")
''    pFile.WriteLine ("c       FactorType -3 (FEF): Threshold (cfs)")
''    pFile.WriteLine ("c CalcMode -- Evaluation Factor Calculation Mode")
''    pFile.WriteLine ("c   -99 =        Option 0: no optimizaiton, only calculate EF")
''    pFile.WriteLine ("c     0 =        Option 2: Maximize Control")
''    pFile.WriteLine ("c     1 = %      percent of value under existing condition (0-100)")
''    pFile.WriteLine ("c     2 = S      scale between pre-develop and existing condition (0-1)")
''    pFile.WriteLine ("c     3 = Value  absolute value in the unit as shown in Factor_ID block")
''    pFile.WriteLine ("c Target_Value -- Target value for Option 1 and Priority Factor (0-10) for Option 2")
''    pFile.WriteLine ("c TargetVal_Low and TargetVal_Up -- Target Value Lower and Upper Limit for Option 3")
''    pFile.WriteLine ("c Factor_Name = Evaluation factor name, e.g. FlowVolume or SEDIMENT")
''    pFile.WriteLine ("c---------------------------------------------------------------------------------------------")
''    pFile.Write StrCard1420Assess
''    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
''End Sub

'New card for optimization controls
'******************************************************************************
'Subroutine: WriteCard800
'Author:     Mira Chokshi
'Purpose:    Assessment point details
'Modified:   Sabu Paul added new card details to enter additional information
'            for BMPs added as assessment points.
'******************************************************************************
Private Sub WriteCard800()
On Error GoTo ShowError
    GetAssessPointOptimizationDetails
    
    pFile.WriteLine ("c800 Optimization Controls")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c Technique -- Optimization Techniques")
    pFile.WriteLine ("c     0 = no optimization")
    pFile.WriteLine ("c     1 = Scatter Search")
    pFile.WriteLine ("c     2 = NSGAII")
    pFile.WriteLine ("c Option -- Optimization options")
    pFile.WriteLine ("c     0 = no optimization")
    pFile.WriteLine ("c     1 = specific control target and minimize cost")
    'pFile.WriteLine ("c     3 = cost limit and maximize control")
    pFile.WriteLine ("c     2 = generate cost effectiveness curve")
    pFile.WriteLine ("c StopDelta -- Criteria for stopping the optimization iteration")
    pFile.WriteLine ("c              in dollars($), meaning if the cost not improved by this criteria, stop the search (for Option 1)")
    pFile.WriteLine ("c MaxRuns -- Maximum number of iterations")
    pFile.WriteLine ("c NumBest -- Number of best solutions for output (for Option 1)")
    'pFile.WriteLine ("c NumBreak  Number of break points between the lower and upper target values of trade-off curve.")
    pFile.WriteLine ("c")
    pFile.Write StrCard800OptimizationControls
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub

Private Sub WriteCard814()
On Error GoTo ShowError

    pFile.WriteLine ("c814 Predeveloped Timeseries at Assessment Point for Flow Duration Curve")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE = BMP site identifier in card 715 if it is an assessment point")
    pFile.WriteLine ("c NumBins = Number of bins for flow duration curve")
    pFile.WriteLine ("c PreDevFlag = Pre-developed timeseries option (1-internal,2-external)")
    pFile.WriteLine ("c PreDevFile = Pre-developed timeseries file path for external option")
    pFile.WriteLine ("c              The timeseries file format (AssessmentPoint_ID               Year       Month  Day        Hour      Minute Flow_cfs)")
    pFile.WriteLine ("c              The first line is skipped (comment line) and data start from the second line in the required format.")
    pFile.WriteLine ("c")
    'pFile.WriteLine ("c BMPSITE    NumBins  PreDevFlag   PreDevFile")
    pFile.WriteLine ("c BMPSITE" & vbTab & "NumBins" & vbTab & "PreDevFlag" & vbTab & "PreDevFile")

    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
End Sub
'******************************************************************************
'Subroutine: WriteCard815
'Author:     Mira Chokshi
'Purpose:    Assessment point details
'Modified:   Sabu Paul added new card details to enter additional information
'            for BMPs added as assessment points.
'******************************************************************************
Private Sub WriteCard815()
  On Error GoTo ErrorHandler

    GetAssessPointOptimizationDetails
    pFile.WriteLine ("c815 Assessment Point and Evaluation Factor")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c BMPSITE -- BMP site identifier in card 715 if it is an assessment point")
    pFile.WriteLine ("c FactorGroup -- Flow or pollutant related evaluation factor group")
    pFile.WriteLine ("c    -1 = flow related evaluation factor")
    pFile.WriteLine ("c     # = pollutant ID in card 705")
    
    pFile.WriteLine ("c FactorType -- Evaluation Factor Type (negative number for flow related and positive number for pollutant related)")
    pFile.WriteLine ("c    -1 = AAFV Annual Average Flow Volume (ft3/yr)")
    pFile.WriteLine ("c    -2 = PDF  Peak Discharge Flow (cfs)")
    pFile.WriteLine ("c    -3 = FEF  Flow Exceeding frequency  (#times/year)")
    pFile.WriteLine ("c    -4 = FDC  Flow Duration Curve (sum of sorted flow difference with pre-developed condition, cfs)")
    pFile.WriteLine ("c     1 = AAL  Annual Average Load (lb/yr)")
    pFile.WriteLine ("c     2 = AAC  Annual Average Concentration (mg/L)")
    pFile.WriteLine ("c     3 = MAC  Maximum #days Average Concentraion (mg/L)")
    
    pFile.WriteLine ("c FactorVal1 -- if FactorType = 3 (MAC): Maximum #Days;")

    pFile.WriteLine ("c            -- if FactorType = -3 (FEF): Threshold (cfs)")
    pFile.WriteLine ("c            -- if FactorType = -4 (FDC): Low flow limit (cfs)")
    pFile.WriteLine ("c            -- all other FactorType : -99")

    pFile.WriteLine ("c FactorVal2 -- if FactorType = -3 (FEF): Minimum inter-exceedance time (hr)")
    pFile.WriteLine ("c                                    if =  0 then daily running average flow exceeding frequency")
    pFile.WriteLine ("c                                    if = -1 then daily average flow exceeding frequency")
    pFile.WriteLine ("c                                    otherwise minimum inter-exceedance time for simulation interval")
    pFile.WriteLine ("c            -- if FactorType = -4 (FDC): High flow limit (cfs)")
    pFile.WriteLine ("c            -- all other FactorType : -99")

    pFile.WriteLine ("c CalcMode -- Evaluation Factor Calculation Mode")
    pFile.WriteLine ("c   -99 for Option 0 (card 800): no optimizaiton")
    pFile.WriteLine ("c     1 = % percent of value under existing condition (0-100)")
    pFile.WriteLine ("c     2 = S scale between pre-develop and existing condition (0-1)")
    pFile.WriteLine ("c     3 = V  absolute value in the unit as shown in FactorType (third block in this card)")
    
    pFile.WriteLine ("c TargetVal1 -- Target value for evaluation factor calculation mode")

    pFile.WriteLine ("c     -99 for Option 0 (card 800): no optimizaiton")

    pFile.WriteLine ("c     Target value for minimize cost Option 1 (card 800)")

    pFile.WriteLine ("c     Lower limit of target value for cost-effective curve Option 2 (card 800)")

    pFile.WriteLine ("c TargetVal2 -- Target value for evaluation factor calculation mode")

    pFile.WriteLine ("c     -99 for Option 0 (card 800): no optimizaiton")

    pFile.WriteLine ("c     -99 for Option 1 (card 800): minimize cost")

    pFile.WriteLine ("c     Upper limit of target value for cost-effective curve Option 2 (card 800)")

    pFile.WriteLine ("c Factor_Name -- Evaluation factor name (user specified without any space), e.g. FlowVolume or SEDIMENT")
    pFile.WriteLine ("c")
    pFile.Write StrCard815Assess
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")

  Exit Sub
ErrorHandler:
    gHasInFileError = True
  'HandleError False, "WriteCard815 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.description, 4
End Sub
'*******************************************************************************
'Subroutine : GetPollutantDetails
'Purpose    : Read Pollutants table and print the list of pollutants
'Note       :
'Arguments  :
'Author     : Sabu Paul
'History    :
'*******************************************************************************
Public Function GetPollutantDetails() As String
On Error GoTo ShowError
    Dim result As String
    result = ""
    
    Dim pTable As iTable
    Set pTable = GetInputDataTable("Pollutants")
    If (pTable Is Nothing) Then
        GetPollutantDetails = result
        MsgBox "Missing pollutants table: Define pollutants first"
        Exit Function
    End If
    
    Dim iNameFld As Integer
    iNameFld = pTable.FindField("Name")
    
    Dim iMultiplierFld As Integer
    iMultiplierFld = pTable.FindField("Multiplier")
    
    Dim iSedFlagFld As Integer
    iSedFlagFld = pTable.FindField("Sediment")
    
    Dim iSedAssoc As Integer
    iSedAssoc = pTable.FindField("SedAssoc")
    Dim iSandFrac As Integer
    iSandFrac = pTable.FindField("SandFrac")
    Dim iSiltFrac As Integer
    iSiltFrac = pTable.FindField("SiltFrac")
    Dim iClayFrac As Integer
    iClayFrac = pTable.FindField("ClayFrac")
       
    Dim pCursor As ICursor
    Dim pRow As iRow
    
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    pQueryFilter.WhereClause = "ID > 0 Order by ID"
    
    Set pCursor = pTable.Search(pQueryFilter, False)
    Set pRow = pCursor.NextRow
    
    Dim sedFlag As Integer
    Dim sedAssoc As Integer
    Dim sandFrac As Double
    Dim siltFrac As Double
    Dim clayFrac As Double
        
    If gInternalSimulation Then
        sedAssoc = 0
        sandFrac = 0
        siltFrac = 0
        clayFrac = 0
    End If
        
    Dim rCount As Integer
    rCount = 1
    Do Until pRow Is Nothing
        Select Case UCase(pRow.value(iSedFlagFld))
            Case "SEDIMENT":
                sedFlag = 4
            Case "SAND":
                sedFlag = 1
            Case "SILT":
                sedFlag = 2
            Case "CLAY":
                sedFlag = 3
            Case Else
                sedFlag = 0
        End Select
        If gExternalSimulation Then
            sedAssoc = pRow.value(iSedAssoc)
            If sedAssoc = 1 Then
                sandFrac = pRow.value(iSandFrac)
                siltFrac = pRow.value(iSiltFrac)
                clayFrac = pRow.value(iClayFrac)
            Else
                sandFrac = 0
                siltFrac = 0
                clayFrac = 0
            End If
        End If
        'result = result & vbTab & rCount & vbTab & pRow.value(iNameFld) & vbTab & pRow.value(iMultiplierFld) & vbTab & pRow.value(iSedFlagFld) & vbNewLine
        'result = result & rCount & vbTab & pRow.value(iNameFld) & vbTab & pRow.value(iMultiplierFld) & vbTab & sedFlag & vbTab & sedAssoc & vbTab & FormatNumber(sandFrac, 3) & vbTab & FormatNumber(siltFrac, 3) & vbTab & FormatNumber(clayFrac, 3) & vbNewLine
        result = result & rCount & vbTab & pRow.value(iNameFld) & vbTab & pRow.value(iMultiplierFld) & vbTab & sedFlag & vbTab & sedAssoc & vbTab & Format$(sandFrac, "##0.000") & vbTab & Format$(siltFrac, "##0.000") & vbTab & Format$(clayFrac, "##0.000") & vbNewLine
        
        Set pRow = pCursor.NextRow
        rCount = rCount + 1
    Loop
    
    GetPollutantDetails = result
    GoTo CleanUp
ShowError:
    MsgBox "Error in GetPollutantDetails:" & Err.description
CleanUp:
    Set pTable = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pQueryFilter = Nothing
End Function
'******************************************************************************
'Subroutine: GetLandTypeDetails
'Author:     Mira Chokshi
'Purpose:    Read the lureclass table, gets all parameters for land use types.
'*****************************************************************************
Private Function GetLandTypeDetails() As String
On Error GoTo ShowError

    Dim fso As Scripting.FileSystemObject
    Set fso = CreateObject("Scripting.FileSystemObject")
    
    Dim pTable As iTable
    Dim pTableName As String
    
    If gExternalSimulation Then
        pTableName = "TSAssigns"
    Else
       pTableName = "LUReclass"
    End If
    
    Set pTable = GetInputDataTable(pTableName)
    If (pTable Is Nothing) Then
        MsgBox pTableName & " table not found."
        Exit Function
    End If
        
    Dim iLuGroupID As Long
    iLuGroupID = pTable.FindField("LUGroupID")
    Dim iLuGroup As Long
    iLuGroup = pTable.FindField("LUGroup")
    Dim iLUType As Long
    iLUType = pTable.FindField("Impervious")
    Dim iTimeSeries As Long
    If gExternalSimulation Then iTimeSeries = pTable.FindField("TimeSeries")
    
    Dim iSand As Long
    iSand = pTable.FindField("SandFrac")
    Dim iSilt As Long
    iSilt = pTable.FindField("SiltFrac")
    Dim iClay As Long
    iClay = pTable.FindField("ClayFrac")
    
    Dim pTimeSeriesFile As String
    pMaxLandTypeGroupID = 0
    Dim pReturnString As String
    Dim pImpervious As Integer
    Dim LUgroupCount As Integer
    LUgroupCount = 0
    
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    pQueryFilter.WhereClause = "LUGroupID > 0"
    'pQueryFilter.WhereClause = "LUGroupID >= 0"
    
    Dim pTableSort As ITableSort
    Set pTableSort = New esriGeoDatabase.TableSort
    
    With pTableSort
      .Fields = "LUGroupID, LuGroup"
      .Ascending("LUGroupID") = True
      .Ascending("LuGroup ") = True
      Set .QueryFilter = pQueryFilter
      Set .Table = pTable
    End With

    pTableSort.Sort Nothing
           
    Dim pCursor As ICursor
    'Set pCursor = pTable.Search(Nothing, True)
    Dim pRow As iRow
    'Set pRow = pCursor.NextRow
    
    Set pCursor = pTableSort.Rows
    Set pRow = pCursor.NextRow
    
    Do While Not pRow Is Nothing
        If (pMaxLandTypeGroupID <> pRow.value(iLuGroupID)) Then
            pMaxLandTypeGroupID = pRow.value(iLuGroupID)
            LUgroupCount = LUgroupCount + 1
            pTimeSeriesFile = pRow.value(iTimeSeries)
            'fso.CopyFile pTimeSeriesFile, pInputFolder
            If gExternalSimulation Then
                'Remain the full path
                'pTimeSeriesFile = fso.GetFileName(pTimeSeriesFile)
            Else
                pTimeSeriesFile = "Internal Simulation"
            End If
            'Modified to add sand, silt and clay fraction - June 14, 2007
            If pRow.value(iLUType) = "1" Then
            pReturnString = pReturnString & _
                            pRow.value(iLuGroupID) & vbTab & _
                            Replace(pRow.value(iLuGroup), " ", "_") & "_Impervious" & vbTab & _
                            pRow.value(iLUType) & vbTab & _
                            pTimeSeriesFile & vbTab & _
                            pRow.value(iSand) & vbTab & _
                            pRow.value(iSilt) & vbTab & _
                            pRow.value(iClay) & vbNewLine
            
            Else
            'pReturnString = pReturnString & _
                            pRow.value(iLuGroupID) & vbTab & _
                            Replace(pRow.value(iLuGroup), " ", "_") & vbTab & _
                            pRow.value(iLUType) & vbTab & _
                            pTimeSeriesFile & vbTab & _
                            pRow.value(iSand) & vbTab & _
                            pRow.value(iSilt) & vbTab & _
                            pRow.value(iClay) & vbNewLine
            pReturnString = pReturnString & _
                            pRow.value(iLuGroupID) & vbTab & _
                            Replace(pRow.value(iLuGroup), " ", "_") & "_Pervious" & vbTab & _
                            pRow.value(iLUType) & vbTab & _
                            pTimeSeriesFile & vbTab & _
                            pRow.value(iSand) & vbTab & _
                            pRow.value(iSilt) & vbTab & _
                            pRow.value(iClay) & vbNewLine
                            
            End If
        End If
        Set pRow = pCursor.NextRow
    Loop
    


    'Return the LandType Information
    GetLandTypeDetails = pReturnString
    GoTo CleanUp
ShowError:
    MsgBox "GetLandTypeDetails: " & Err.description
CleanUp:
    Set pTable = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set fso = Nothing
End Function
Private Function GetAquifersDetails() As String
On Error GoTo ShowError
    Dim pReturnString As String
                            
    Dim pTable As iTable
    Dim pTableName As String
    
    pTableName = "Aquifers"
    
    Set pTable = GetInputDataTable(pTableName)
        
    Dim pQueryFilter As IQueryFilter
    Dim pCursor As ICursor
    Dim pRow As iRow

    Dim iFieldPropValue As Long
    iFieldPropValue = pTable.FindField("PropValue")
    Dim iFieldPropName As Long
    iFieldPropName = pTable.FindField("PropName")
                           
    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        Set pQueryFilter = New QueryFilter
        pQueryFilter.WhereClause = "PropName='Name' And ID=" & CStr(ii + 1)
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        If Not pRow Is Nothing Then
            pReturnString = pReturnString & _
                            CStr(ii + 1) & vbTab & _
                            Replace(pRow.value(iFieldPropValue), " ", "_") & vbTab
        End If
        
        pQueryFilter.WhereClause = "PropName='Initial Storage' And ID=" & CStr(ii + 1)
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        If Not pRow Is Nothing Then
            pReturnString = pReturnString & _
                            pRow.value(iFieldPropValue) & vbTab
        End If
        pQueryFilter.WhereClause = "PropName='Recession Coef' And ID=" & CStr(ii + 1)
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        If Not pRow Is Nothing Then
            pReturnString = pReturnString & _
                            pRow.value(iFieldPropValue) & vbTab
        End If
        pQueryFilter.WhereClause = "PropName='Seepage Coef' And ID=" & CStr(ii + 1)
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        If Not pRow Is Nothing Then
            pReturnString = pReturnString & _
                            pRow.value(iFieldPropValue) & vbNewLine
        End If
    Next ii
    GetAquifersDetails = pReturnString
    GoTo CleanUp
ShowError:
    MsgBox "GetAquifersDetails: " & Err.description
CleanUp:
    Set pTable = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing

End Function

Public Function GetAquiferParameters() As String
On Error GoTo ShowError
    Dim pTable As iTable
    Set pTable = GetInputDataTable("Aquifers")
    If pTable Is Nothing Then
        Exit Function
    End If
    
    Dim pCursor As esriGeoDatabase.ICursor
    Dim pRow As iRow
        
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    
    Dim bmpDict As Scripting.Dictionary
    Set bmpDict = New Scripting.Dictionary
    Dim propNameFld As Integer
    Dim propValueFld As Integer
    propNameFld = pTable.FindField("PropName")
    propValueFld = pTable.FindField("PropValue")
    Dim IDFld As Long
    IDFld = pTable.FindField("ID")
    
    Dim strStatGrid As String, strElevGrid As String
    Dim statGrids, elevGrids
    Dim i As Integer
    Dim numOfStations As Integer
    Dim AquiferCountDict As Dictionary
    Set AquiferCountDict = New Dictionary
    Dim iDistinctID As Integer
    Dim pCursorDistinctID As ICursor
    Set pCursorDistinctID = pTable.Search(Nothing, False)
    Dim pRowDistinctID As iRow
    Set pRowDistinctID = pCursorDistinctID.NextRow
    Do Until pRowDistinctID Is Nothing
        If Not AquiferCountDict.Exists(pRowDistinctID.value(IDFld)) Then
            AquiferCountDict.add pRowDistinctID.value(IDFld), pRowDistinctID.value(IDFld)
        End If
        Set pRowDistinctID = pCursorDistinctID.NextRow
    Loop
    Dim pReturnString As String
    
    pReturnString = ""
    Dim transectId
    For Each transectId In AquiferCountDict
        'pQueryFilter.WhereClause = " ID = " & CInt(transectId)
        pQueryFilter.WhereClause = " ID = " & CInt(transectId) & " AND PropName LIKE 'C%'"
        pReturnString = pReturnString & CStr(transectId)
        Set pCursor = pTable.Search(pQueryFilter, False)
        Set pRow = pCursor.NextRow
        Do Until pRow Is Nothing
            bmpDict.Item(pRow.value(propNameFld)) = pRow.value(propValueFld)
            pReturnString = pReturnString & vbTab & pRow.value(propValueFld)
            Set pRow = pCursor.NextRow
        Loop
        pReturnString = pReturnString & vbNewLine
        
    Next
    GetAquiferParameters = pReturnString
    GoTo CleanUp
ShowError:
    gHasInFileError = True
    MsgBox "Error in GetIrregularCSParameters : " & Err.description
CleanUp:
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set bmpDict = Nothing
End Function

Private Function GetPointSourceDetails() As String
On Error GoTo ShowError
    Dim pReturnString As String
    pReturnString = ""
    
    Dim pTable As iTable
    Dim pCursor As ICursor
    Dim pRow As iRow
    
    Set pTable = GetInputDataTable("ExternalTS")
    If pTable Is Nothing Then
        GetPointSourceDetails = pReturnString
        Exit Function
    Else
        Set pCursor = pTable.Search(Nothing, True)
        
        Dim iBMPIndex As Long
        iBMPIndex = pTable.FindField("BMPID")
        Dim iLUDesc As Long
        iLUDesc = pTable.FindField("LUDescrip")
        Dim iExtTimeSeries As Long
        iExtTimeSeries = pTable.FindField("TimeSeries")
        'Added three new fields - June 18, 2007
        Dim iSandFracIndex As Long
        iSandFracIndex = pTable.FindField("SandFrac")
        Dim iSiltFracIndex As Long
        iSiltFracIndex = pTable.FindField("SiltFrac")
        Dim iClayFracIndex As Long
        iClayFracIndex = pTable.FindField("ClayFrac")
        
        Dim iMultiplier As Long
        iMultiplier = pTable.FindField("Multiplier")
        
        Dim iLUDescrip As Long
        iLUDescrip = pTable.FindField("LUDescrip")
        
        Dim pRowCount As Integer
        Dim pBMPID As String
        Dim pLUDescrip As String
        Dim pMultiplier As Double
        Dim pTimeSeriesFile As String
        Dim pSandFrac As Double
        Dim pSiltFrac As Double
        Dim pClayFrac As Double
        
        pRowCount = 1
        
        Set pRow = pCursor.NextRow
        Do Until pRow Is Nothing
            pBMPID = pRow.value(iBMPIndex)
            pTimeSeriesFile = pRow.value(iExtTimeSeries)
            'fso.CopyFile pTimeSeriesFile, pInputFolder
            'pTimeSeriesFile = fso.GetFileName(pTimeSeriesFile)
            'Modified to add sand, silt and clay fraction - June 14, 2007
            pMultiplier = CDbl(CStr(pRow.value(iMultiplier)))
            pLUDescrip = pRow.value(iLUDescrip)
            pLUDescrip = Replace(pLUDescrip, " ", "_")
            
            pSandFrac = CDbl(CStr(pRow.value(iSandFracIndex)))
            pSiltFrac = CDbl(CStr(pRow.value(iSiltFracIndex)))
            pClayFrac = CDbl(CStr(pRow.value(iClayFracIndex)))
            
            
''            pReturnString = pReturnString & _
''                            LUgroupCount & vbTab & _
''                            pRow.value(iLUDesc) & vbTab & _
''                            "1" & vbTab & _
''                            pTimeSeriesFile & vbNewLine
                            
            pReturnString = pReturnString & _
                            pRowCount & vbTab & _
                            pLUDescrip & vbTab & _
                            pBMPID & vbTab & _
                            pMultiplier & vbTab & _
                            pTimeSeriesFile & vbTab & _
                            CStr(pSandFrac) & vbTab & _
                            CStr(pSiltFrac) & vbTab & _
                            CStr(pClayFrac) & vbNewLine
                            
            Set pRow = pCursor.NextRow
            pRowCount = pRowCount + 1
        Loop
    End If
    'Added by Ji to set the return value
    GetPointSourceDetails = pReturnString
    GoTo CleanUp
ShowError:
    MsgBox "Error in GetPointSourceDetails :" & Err.description
CleanUp:
    Set pRow = Nothing
    Set pCursor = Nothing
    Set pTable = Nothing
End Function

'******************************************************************************
'Subroutine: CustomSplit
'Author:     Sabu Paul
'Purpose:    Removes all spaces in a string.
'*****************************************************************************
Public Function CustomSplit(pString As String) As String()
  On Error GoTo ErrorHandler
    Dim res() As String
    If Trim(pString) = "" Then
        CustomSplit = Split(Trim(pString))
        Exit Function
    End If
    pString = Replace(pString, " ", vbTab)
    Dim tmpRes
    'tmpRes = Split(pString, " ", -1, vbTextCompare)
    tmpRes = Split(pString, vbTab, , vbTextCompare)
    Dim resWords As Long
    resWords = 0
    Dim incr As Integer
    For incr = 0 To UBound(tmpRes)
        If Trim(tmpRes(incr)) <> "" Then
            ReDim Preserve res(resWords)
            res(resWords) = tmpRes(incr)
            resWords = resWords + 1
        End If
    Next incr
    CustomSplit = res

  Exit Function
ErrorHandler:
  'HandleError True, "CustomSplit " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.description, 4
End Function


'******************************************************************************
'Subroutine: BMPCardsDetail
'Author:     Mira Chokshi
'Purpose:    Get the dimension properties of BMPs. Also get the soil properties,
'            cost values, assessment point evaluation factors, growth index.
'*****************************************************************************
Public Sub BMPCardsDetail(BMPID As Integer)
On Error GoTo ShowError
    'MsgBox " BMPCardsDetail 1 for " & bmpId
    Dim pReturnString As String
    Dim pFeatureLayer As IFeatureLayer
    Set pFeatureLayer = GetInputFeatureLayer("BMPs")
    If (pFeatureLayer Is Nothing) Then
        MsgBox "BMPs featurelayer not found."
        Exit Sub
    End If
    Dim pTotalBMPs As Integer
    pTotalBMPs = pFeatureLayer.FeatureClass.FeatureCount(Nothing)
'    Dim pTable As iTable
'    Set pTable = GetInputDataTable("BMPDetail")
'    If (pTable Is Nothing) Then
'        MsgBox "BMPDetail table not found."
'        Exit Sub
'    End If
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    Dim pCursor As ICursor
    Dim pRow As iRow
'    Dim iPropName As Long
'    iPropName = pTable.FindField("PropName")
'    Dim iPropValue As Long
'    iPropValue = pTable.FindField("PropValue")
    
    Dim iBMP As Integer
    Dim strBmpFeatType As String
    Dim pBMPType As String
    Dim pBMPClass As String
    Dim isAssessPoint As Boolean
    
    'Modified so that bmp ids are read from BMPs feature table
    Dim pFeatureclass As IFeatureClass
    Set pFeatureclass = pFeatureLayer.FeatureClass
    Dim pFeature As IFeature
    Dim pFeatureCursor As IFeatureCursor
    Dim pFeatIDIndex As Long
    pFeatIDIndex = pFeatureclass.FindField("ID")
    Dim pFeatTypeIndex As Long
    pFeatTypeIndex = pFeatureclass.FindField("Type")

    'Define watershed feature layer
    Dim pTotalDrainageArea As Double
        
    'set whereclause of the bmp feature layer
    pQueryFilter.WhereClause = "ID = " & BMPID
    Set pFeatureCursor = pFeatureLayer.Search(pQueryFilter, False)
    Set pFeature = pFeatureCursor.NextFeature
    Dim pPoint As IPoint
    Dim pXpos As Double
    Dim pYpos As Double
    
    Dim pAgg_BMP_Cat_Dict As Scripting.Dictionary
    
    Dim bmpCardStr As String
    While Not pFeature Is Nothing
        'MsgBox " BMPCardsDetail 2 for " & bmpId
        iBMP = pFeature.value(pFeatIDIndex)
        strBmpFeatType = pFeature.value(pFeatTypeIndex)
        'Set the point to find X and Y coordinates
        Set pPoint = pFeature.Shape
        pPoint.QueryCoords pXpos, pYpos
        
        'Format pXpos, pYpos
        pXpos = FormatNumber(pXpos, 1)
        pYpos = FormatNumber(pYpos, 1)
        
        If UCase(strBmpFeatType) = "AGGREGATE" Then
            'Set pAgg_BMP_Cat_Dict = Get_OnMap_BMP_Categories(iBMP)
            Set pAgg_BMP_Cat_Dict = GetAggBMPTypes(iBMP)
            
            'Get individual BMP properties
            Dim catIndex As Integer
            If Not pAgg_BMP_Cat_Dict Is Nothing Then
                For catIndex = 0 To pAgg_BMP_Cat_Dict.Count - 1
                    If pAgg_BMP_Cat_Dict.Exists(pAgg_BMP_Cat_Dict.keys(catIndex)) Then
                        bmpCardStr = iBMP & "_" & pAgg_BMP_Cat_Dict.keys(catIndex)
                        pBMPDetailDict.RemoveAll
                        If gBMPDrainAreaDict.Exists(bmpCardStr) Then
                            pTotalDrainageArea = gBMPDrainAreaDict.Item(bmpCardStr)
                        Else
                            pTotalDrainageArea = 0
                        End If
                        Set pBMPDetailDict = GetBMPDetailDict(CInt(pAgg_BMP_Cat_Dict.keys(catIndex)), "AgBMPDetail")
                        Call GetBMPDetails_Classes(iBMP, pBMPType, pXpos, pYpos, pTotalDrainageArea, bmpCardStr)
                    End If
                Next
            End If

            'bmpCardStr = iBMP & "_E"
            bmpCardStr = iBMP '& "_0"
            pBMPDetailDict.RemoveAll
            Dim pAgBmpLuDTable As iTable
            Set pAgBmpLuDTable = GetInputDataTable("AgLuDistribution")
            If Not (pAgBmpLuDTable Is Nothing) Then
                pQueryFilter.WhereClause = "LuGroup = 'AquiferID' AND BMPID = " & bmpCardStr
                Set pCursor = pAgBmpLuDTable.Search(pQueryFilter, True)
                Set pRow = pCursor.NextRow
                If Not pRow Is Nothing Then
                    Dim dicAquiferIDToName As Dictionary
                    Set dicAquiferIDToName = GetDictAquiferIDToName
                    If Not dicAquiferIDToName Is Nothing Then
                        Dim lAquiferID As Long
                        lAquiferID = pRow.value(pRow.Fields.FindField("LuGrpID"))
                        If dicAquiferIDToName.Exists(lAquiferID) Then
                            pBMPDetailDict.Item("Aquifer") = CStr(dicAquiferIDToName.Item(lAquiferID))
                        End If
                    End If

                    
                End If
                
            End If
            pBMPDetailDict.Item("BMPType") = "Junction"
            pBMPDetailDict.Item("BMPClass") = "X"
            If gBMPDrainAreaDict.Exists(bmpCardStr) Then
                pTotalDrainageArea = gBMPDrainAreaDict.Item(bmpCardStr)
            Else
                pTotalDrainageArea = 0
            End If
            GetDummyBMPParameters bmpCardStr, pXpos, pYpos, pTotalDrainageArea
        
        Else
        'Check if BMPDetail table exist
            Dim pTable As iTable
            Set pTable = GetInputDataTable("BMPDetail")
            If (pTable Is Nothing) Then
                MsgBox "BMPDetail table not found."
                Exit Sub
            End If
            Dim iPropName As Long
            iPropName = pTable.FindField("PropName")
            Dim iPropValue As Long
            iPropValue = pTable.FindField("PropValue")

            pQueryFilter.WhereClause = "ID = " & iBMP
            Set pCursor = pTable.Search(pQueryFilter, True)
            Set pRow = pCursor.NextRow
            'Remove all the parameters present in the dictionary
            pBMPDetailDict.RemoveAll
            
            Do While Not pRow Is Nothing
                pBMPDetailDict.Item(pRow.value(iPropName)) = pRow.value(iPropValue)
                Set pRow = pCursor.NextRow
            Loop
            pBMPType = Trim(pBMPDetailDict.Item("BMPType"))
            pBMPClass = Trim(pBMPDetailDict.Item("BMPClass"))
            If pBMPType = "" Then
                pBMPType = pFeature.value(pFeatureclass.FindField("TYPE"))
            End If
            
            If gBMPDrainAreaDict.Exists(iBMP) Then
                pTotalDrainageArea = gBMPDrainAreaDict.Item(iBMP)
            Else
                pTotalDrainageArea = 0
            End If
            
            Call GetBMPDetails_Classes(iBMP, pBMPType, pXpos, pYpos, pTotalDrainageArea)
        End If
        Set pFeature = pFeatureCursor.NextFeature
    Wend
    GoTo CleanUp
ShowError:
    MsgBox "BMPCardsDetail: " & Err.description
CleanUp:
    Set pFeatureLayer = Nothing
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pFeatureclass = Nothing
    Set pFeature = Nothing
    Set pFeatureCursor = Nothing
    Set pPoint = Nothing
'    Set pWatershedLayer = Nothing
'    Set pWatershedClass = Nothing
'    Set pWatershedCursor = Nothing
'    Set pWatershedFeature = Nothing
End Sub

Public Sub VFSCardsDetail_New(BMPID As Integer)
On Error GoTo ShowError
    'MsgBox " BMPCardsDetail 1 for " & bmpId
    Dim pReturnString As String
    Dim pFeatureLayer As IFeatureLayer
    Set pFeatureLayer = GetInputFeatureLayer("VFS")
    If (pFeatureLayer Is Nothing) Then
        Exit Sub
    End If
    Dim pTotalBMPs As Integer
    pTotalBMPs = pFeatureLayer.FeatureClass.FeatureCount(Nothing)
    Dim pTable As iTable
    Set pTable = GetInputDataTable("BMPDetail")
    If (pTable Is Nothing) Then
        MsgBox "BMPDetail table not found."
        Exit Sub
    End If
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    Dim pCursor As ICursor
    Dim pRow As iRow
    Dim iPropName As Long
    iPropName = pTable.FindField("PropName")
    Dim iPropValue As Long
    iPropValue = pTable.FindField("PropValue")
    
    Dim iBMP As Integer
    Dim strBmpFeatType As String
    Dim pBMPType As String
    Dim pBMPClass As String
    Dim isAssessPoint As Boolean
    
    'Modified so that bmp ids are read from BMPs feature table
    Dim pFeatureclass As IFeatureClass
    Set pFeatureclass = pFeatureLayer.FeatureClass
    Dim pFeature As IFeature
    Dim pFeatureCursor As IFeatureCursor
    Dim pFeatIDIndex As Long
    pFeatIDIndex = pFeatureclass.FindField("ID")
    Dim pFeatTypeIndex As Long
    pFeatTypeIndex = pFeatureclass.FindField("Type")

    'Define watershed feature layer
    Dim pTotalDrainageArea As Double
        
    'set whereclause of the bmp feature layer
    pQueryFilter.WhereClause = "ID = " & BMPID
    Set pFeatureCursor = pFeatureLayer.Search(pQueryFilter, False)
    Set pFeature = pFeatureCursor.NextFeature
    Dim pPoint As IPoint
    Dim pXpos As Double
    Dim pYpos As Double
    
    Dim pAgg_BMP_Cat_Dict As Scripting.Dictionary
    
    Dim bmpCardStr As String
    While Not pFeature Is Nothing
        'MsgBox " BMPCardsDetail 2 for " & bmpId
        iBMP = pFeature.value(pFeatIDIndex)
        
        pQueryFilter.WhereClause = "ID = " & iBMP
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        'Remove all the parameters present in the dictionary
        pBMPDetailDict.RemoveAll
        
        Do While Not pRow Is Nothing
            pBMPDetailDict.Item(pRow.value(iPropName)) = pRow.value(iPropValue)
            Set pRow = pCursor.NextRow
        Loop
        pBMPType = Trim(pBMPDetailDict.Item("BMPType"))
        pBMPClass = Trim(pBMPDetailDict.Item("BMPClass"))
        If pBMPType = "" Then
            pBMPType = pFeature.value(pFeatureclass.FindField("TYPE"))
        End If
        
        If gBMPDrainAreaDict.Exists(iBMP) Then
            pTotalDrainageArea = gBMPDrainAreaDict.Item(iBMP)
        Else
            pTotalDrainageArea = 0
        End If
        
        'Call GetBMPDetails_Classes(iBMP, pBMPType, pXpos, pYpos, pTotalDrainageArea)
        Call GetBMPDetails_Classes(iBMP, pBMPType, 0, 0, pTotalDrainageArea)
            
        Set pFeature = pFeatureCursor.NextFeature
    Wend
    GoTo CleanUp
ShowError:
    MsgBox "BMPCardsDetail: " & Err.description
CleanUp:
    Set pFeatureLayer = Nothing
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pFeatureclass = Nothing
    Set pFeature = Nothing
    Set pFeatureCursor = Nothing
    Set pPoint = Nothing
'    Set pWatershedLayer = Nothing
'    Set pWatershedClass = Nothing
'    Set pWatershedCursor = Nothing
'    Set pWatershedFeature = Nothing
End Sub



'******************************************************************************
'Subroutine: ConduitCardsDetails
'Author:     Mira Chokshi   02/08/2005
'Purpose:    Get the dimension and cross section information for conduits
'*****************************************************************************
Private Sub ConduitCardsDetails(BMPID As Integer)
On Error GoTo ShowError
   
    '*** Check for BMP layer to get the number of BMPs
    Dim pBMPFeatureLayer As IFeatureLayer
    Set pBMPFeatureLayer = GetInputFeatureLayer("BMPs")
    If (pBMPFeatureLayer Is Nothing) Then
        Exit Sub
    End If
    Dim pBMPFeatureClass As IFeatureClass
    Set pBMPFeatureClass = pBMPFeatureLayer.FeatureClass
    
    Dim lNumBMPs
    lNumBMPs = pBMPFeatureClass.FeatureCount(Nothing)
    
    Dim pVFSFeatureLayer As IFeatureLayer
    Set pVFSFeatureLayer = GetInputFeatureLayer("VFS")
    If Not (pVFSFeatureLayer Is Nothing) Then
        Dim pVFSFeatureClass As IFeatureClass
        Set pVFSFeatureClass = pVFSFeatureLayer.FeatureClass
        lNumBMPs = lNumBMPs + pVFSFeatureClass.FeatureCount(Nothing)
    End If
    If lNumBMPs = 1 Then
        Exit Sub
    End If

   'Get the Conduits feature layer
    Dim pFeatureLayer As IFeatureLayer
    Set pFeatureLayer = GetInputFeatureLayer("Conduits")
    If (pFeatureLayer Is Nothing) Then
        
        bInputFileWritingError = True
        Exit Sub
    End If
    'Get the BMPDetail table
    Dim pTable As iTable
    Set pTable = GetInputDataTable("BMPDetail")
    If (pTable Is Nothing) Then
        bInputFileWritingError = True
        MsgBox "BMPDetail table not found."
        Exit Sub
    End If
    'Define the query, cursor, row for conduits detail table
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    Dim pCursor As ICursor
    Dim pRow As iRow
    Dim iPropName As Long
    iPropName = pTable.FindField("PropName")
    Dim iPropValue As Long
    iPropValue = pTable.FindField("PropValue")
    'Define the feature class, feature cursor, feature for conduit feature layer
    Dim pFeatureclass As IFeatureClass
    Set pFeatureclass = pFeatureLayer.FeatureClass
    Dim pFeatureCursor As IFeatureCursor
    Dim pFeature As IFeature
    Dim iIDFld As Long
    Dim pConduitID As Integer
    iIDFld = pFeatureclass.FindField("ID")
    Dim iFROMFld As Long
    Dim iTOFld As Long
    iFROMFld = pFeatureclass.FindField("CFROM")
    iTOFld = pFeatureclass.FindField("CTO")
    pQueryFilter.WhereClause = "ID = " & BMPID
    Set pFeatureCursor = pFeatureclass.Search(pQueryFilter, True)
    Set pFeature = pFeatureCursor.NextFeature
    'Get the ID of each conduit feature, get details for each feature from conduit details table
    Do While Not pFeature Is Nothing
        'Get the Conduit ID value
        pConduitID = pFeature.value(iIDFld)
        pQueryFilter.WhereClause = "ID = " & pConduitID
        Set pCursor = pTable.Search(pQueryFilter, True)
        Set pRow = pCursor.NextRow
        'Remove all parameters from bmpdetaildict
        pBMPDetailDict.RemoveAll
        pBMPDetailDict.add "INLETNODE", pFeature.value(iFROMFld)
        pBMPDetailDict.add "OUTLETNODE", pFeature.value(iTOFld)
        'Get each property name
        Do While Not pRow Is Nothing
            pBMPDetailDict.add pRow.value(iPropName), pRow.value(iPropValue)
            Set pRow = pCursor.NextRow  'Next row
        Loop
        'Get Decay Factors for conduits
        GetBMPDecayFactors CStr(pConduitID)
        GetBMP_K_C_factors CStr(pConduitID)  '766, 767
        'Get Class C - Conduits parameters
        GetConduitParameters pConduitID
        'Iterate to next feature
        Set pFeature = pFeatureCursor.NextFeature
    Loop

    GoTo CleanUp
ShowError:
    MsgBox "ConduitCardsDetail: " & Err.description
CleanUp:
    Set pFeatureLayer = Nothing
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pFeatureclass = Nothing
    Set pFeatureCursor = Nothing
    Set pFeature = Nothing
End Sub



'******************************************************************************
'Subroutine: VFSCardsDetail
'Author:     Mira Chokshi   02/08/2005
'Purpose:    Get the dimension and cross section information for conduits
'*****************************************************************************
Private Sub VFSCardsDetail(BMPID As Integer)
On Error GoTo ShowError
    'Get the Conduits feature layer
    Dim pFeatureLayer As IFeatureLayer
    Set pFeatureLayer = GetInputFeatureLayer("VFS")
    If (pFeatureLayer Is Nothing) Then
        Exit Sub
    End If
    'Get the BMPDetail table
    Dim pTable As iTable
    'Set pTable = GetInputDataTable("VFSDetail")
    Set pTable = GetInputDataTable("BMPDetail")
    
    If (pTable Is Nothing) Then
        MsgBox "BMPDetail table not found."
        Exit Sub
    End If
    'Define the query, cursor, row for conduits detail table
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    Dim pCursor As ICursor
    Dim pRow As iRow
    Dim iPropName As Long
    iPropName = pTable.FindField("PropName")
    Dim iPropValue As Long
    iPropValue = pTable.FindField("PropValue")
    'Define the feature class, feature cursor, feature for conduit feature layer
    Dim pFeatureclass As IFeatureClass
    Set pFeatureclass = pFeatureLayer.FeatureClass
    Dim pFeatureCursor As IFeatureCursor
    Dim pFeature As IFeature
    Dim iIDFld As Long
    iIDFld = pFeatureclass.FindField("ID")
    Dim pVFSID As Integer
    pQueryFilter.WhereClause = "ID = " & BMPID
    Set pFeatureCursor = pFeatureclass.Search(pQueryFilter, True)
    Set pFeature = pFeatureCursor.NextFeature
    'Get the ID of each conduit feature, get details for each feature from conduit details table
    Do While Not pFeature Is Nothing
        'Get the Conduit ID value
        pVFSID = pFeature.value(iIDFld)
        pQueryFilter.WhereClause = "ID = " & pVFSID
        Set pCursor = pTable.Search(pQueryFilter, pVFSID)
        Set pRow = pCursor.NextRow
        'Remove all parameters from bmpdetaildict
        pBMPDetailDict.RemoveAll
        'Get each property name
        Do While Not pRow Is Nothing
            pBMPDetailDict.add pRow.value(iPropName), pRow.value(iPropValue)
            Set pRow = pCursor.NextRow  'Next row
        Loop
        'Get Class D - Buffer Strip parameters
        GetVFSParameters pVFSID
        'Iterate to next feature
        Set pFeature = pFeatureCursor.NextFeature
    Loop

    GoTo CleanUp
ShowError:
    MsgBox "VFSCardsDetail: " & Err.description
CleanUp:
    Set pFeatureLayer = Nothing
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pFeatureclass = Nothing
    Set pFeatureCursor = Nothing
    Set pFeature = Nothing
End Sub

'******************************************************************************
'Subroutine: GetBMPParametersA
'Author:     Mira Chokshi
'Purpose:    Get the dimention details of BMP Type A. BMP Type A could be a
'            rain barrel or water cistern. In case of cistern, also get the
'            24 hour, hourly water release information.
'*****************************************************************************
'Private Sub GetBMPParametersA(iBMPindex As Integer, pXpos As Double, pYpos As Double, pDrainageArea As Double)
Private Sub GetBMPParametersA(strBMPindex As String, pXpos As Double, pYpos As Double, pDrainageArea As Double)
On Error GoTo ShowError
    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim basinlength As Double
    Dim basinwidth As Double
    Dim basinsurf_area As Double
    Dim orificediam As Double
    Dim orificeheight As Double
    Dim orifice_area As Double
    Dim orificecoef As Double
    Dim weirheight As Double
    Dim weirwidth As Double
    Dim exittype As Integer
    Dim WeirType As Integer
    Dim weirangle As Double
    Dim releaseoption As String
    Dim releasetype As Integer
    Dim NumDays As Integer
    Dim NumPeople As Integer
    Dim strCisternFlow As String
    Dim pCisternFlow
    Dim NumUnits As Integer
    Dim ddArea As Double
    Dim AquiferID As Integer
    Dim AquiferStr As String
    Dim Mulitiplier As Double
    Dim Pump_Flag As Integer
    Dim Depth_On As Double
    Dim Depth_Off As Double
    Dim Pump_Curve As String
        
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    basinlength = CDbl(pBMPDetailDict.Item("BMPLength"))
    basinwidth = CDbl(pBMPDetailDict.Item("BMPWidth"))
    orificeheight = CDbl(pBMPDetailDict.Item("BMPOrificeHeight"))
    orificediam = CDbl(pBMPDetailDict.Item("BMPOrificeDiameter"))
    weirheight = CDbl(pBMPDetailDict.Item("BMPWeirHeight"))
    weirwidth = CDbl(pBMPDetailDict.Item("BMPRectWeirWidth"))
    exittype = CInt(pBMPDetailDict.Item("ExitType"))
    WeirType = CInt(pBMPDetailDict.Item("WeirType"))
    releaseoption = CStr(pBMPDetailDict.Item("ReleaseOption"))
    NumDays = CInt(pBMPDetailDict.Item("NumDays"))
    NumPeople = CInt(pBMPDetailDict.Item("NumPeople"))
    weirangle = CDbl(pBMPDetailDict.Item("BMPTriangularWeirAngle"))
    orificecoef = CDbl(pBMPDetailDict.Item("OrificeCoef"))
    NumUnits = CInt(pBMPDetailDict.Item("NumUnits"))
    ddArea = CDbl(pBMPDetailDict.Item("DrainArea"))
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    
    Mulitiplier = CDbl(pBMPDetailDict.Item("Multiplier"))
    Pump_Flag = CInt(pBMPDetailDict.Item("PumpOption"))
    Depth_On = CDbl(pBMPDetailDict.Item("PumpOnDepth"))
    Depth_Off = CDbl(pBMPDetailDict.Item("PumpOffDepth"))
    Pump_Curve = CStr(pBMPDetailDict.Item("PumpCurve"))

    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
            AquiferID = ii + 1
        End If
    Next ii
    If gInternalSimulation = True Then
        AquiferID = 0
    End If

    Select Case releaseoption
        Case "Cistern":
            releasetype = 1
            strCisternFlow = pBMPDetailDict.Item("CisternFlow")
            pCisternFlow = Split(strCisternFlow, ";")
        Case "RainBarrel":
            releasetype = 2
        Case "None":
            releasetype = 3
    End Select

    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists(BMPType) Then
            bmpTypeID = gBmpTypeClassDict.Item(BMPType)
        End If
    End If

    'Define card 715
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If

    StrCard715BMPTypes = StrCard715BMPTypes & _
                         strBMPindex & vbTab & _
                         BMPName & vbTab & _
                         bmpTypeID & vbTab & _
                         Format$(pDrainageArea, "##0.000") & vbTab & _
                         NumUnits & vbTab & _
                         ddArea & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine

    'Add Class A Parameters
    StrCard725BMPClassA = StrCard725BMPClassA & _
                          strBMPindex & vbTab & _
                          basinwidth & vbTab & _
                          basinlength & vbTab & _
                          orificeheight & vbTab & _
                          orificediam & vbTab & _
                          exittype & vbTab & _
                          releasetype & vbTab & _
                          NumPeople & vbTab & _
                          NumDays & vbTab & _
                          WeirType & vbTab & _
                          weirheight & vbTab & _
                          weirwidth & vbTab & _
                          weirangle & vbTab & _
                          Mulitiplier & vbTab & _
                          Pump_Flag & vbTab & _
                          Depth_On & vbTab & _
                          Depth_Off & vbTab & _
                          Pump_Curve & vbNewLine

    'Add Cistern Flow
    StrCard730BMPClassA = StrCard730BMPClassA & _
                          strBMPindex

    Dim f As Integer
    If (Trim(strCisternFlow) = "") Then
        ReDim pCisternFlow(0 To 23) As Double
    End If

    For f = LBound(pCisternFlow) To UBound(pCisternFlow)
         StrCard730BMPClassA = StrCard730BMPClassA & vbTab & CDbl(pCisternFlow(f))
    Next

    StrCard730BMPClassA = StrCard730BMPClassA & vbNewLine

    GoTo CleanUp
ShowError:
    MsgBox "GetBMPParametersA: " & Err.description
CleanUp:
    
End Sub


'******************************************************************************
'Subroutine: GetBMPParametersB
'Author:     Mira Chokshi
'Purpose:    Get the spatial and dimensional parameters for BMP Type B.
'*****************************************************************************
'Private Sub GetBMPParametersB(iBMPindex As Integer, pXpos As Double, pYpos As Double, pDrainageArea As Double)
Private Sub GetBMPParametersB(strBMPindex As String, pXpos As Double, pYpos As Double, pDrainageArea As Double)
On Error GoTo ShowError

    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim channellength As Double 'BMP Properties
    Dim channelwidth As Double
    Dim channeldepth As Double
    Dim slope1 As Double
    Dim slope2 As Double
    Dim slope3 As Double
    Dim manncoeff As Double
    Dim AquiferID As Integer
    Dim AquiferStr As String
    Dim Mulitiplier As Double
    
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    channellength = CDbl(pBMPDetailDict.Item("BMPLength"))  'BMP Properties
    channelwidth = CDbl(pBMPDetailDict.Item("BMPWidth"))
    channeldepth = CDbl(pBMPDetailDict.Item("BMPMaxDepth"))
    slope1 = CDbl(pBMPDetailDict.Item("BMPSlope1"))
    slope2 = CDbl(pBMPDetailDict.Item("BMPSlope2"))
    slope3 = CDbl(pBMPDetailDict.Item("BMPSlope3"))
    manncoeff = CDbl(pBMPDetailDict.Item("BMPManningsN"))
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    
    Mulitiplier = CDbl(pBMPDetailDict.Item("Multiplier"))
    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
    
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
        
            AquiferID = ii + 1
        End If
    Next ii
    If gInternalSimulation = True Then
        AquiferID = 0
    End If
    
    Dim NumUnits As Integer
    Dim ddArea As Double
    
    NumUnits = CInt(pBMPDetailDict.Item("NumUnits"))
    ddArea = CDbl(pBMPDetailDict.Item("DrainArea"))
    
    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists(BMPType) Then
            bmpTypeID = gBmpTypeClassDict.Item(BMPType)
        End If
    End If
    

    
    'X, y coods not needed
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If
    
    StrCard715BMPTypes = StrCard715BMPTypes & _
                         strBMPindex & vbTab & _
                         BMPName & vbTab & _
                         bmpTypeID & vbTab & _
                         Format$(pDrainageArea, "##0.000") & vbTab & _
                         NumUnits & vbTab & _
                         ddArea & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine
                             
    'Define card 735 string
    StrCard735BMPClassB = StrCard735BMPClassB & _
                          strBMPindex & vbTab & _
                          channelwidth & vbTab & _
                          channellength & vbTab & _
                          channeldepth & vbTab & _
                          slope1 & vbTab & _
                          slope2 & vbTab & _
                          slope3 & vbTab & _
                          manncoeff & vbTab & _
                          Mulitiplier & vbNewLine
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPParametersB: " & Err.description
CleanUp:
End Sub

Private Sub GetBMPParametersEF(strBMPindex As String, pXpos As Double, pYpos As Double, pDrainageArea As Double, bKCFactor As Boolean)
On Error GoTo ShowError
    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim NumUnits As Integer
    Dim ddArea As Double
    Dim AquiferID As Integer
    Dim AquiferStr As String
        
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    NumUnits = CInt(pBMPDetailDict.Item("NumUnits"))
    ddArea = CDbl(pBMPDetailDict.Item("DrainArea"))
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    

    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
            AquiferID = ii + 1
        End If
    Next ii
    If gInternalSimulation = True Then
        AquiferID = 0
    End If


    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists(BMPType) Then
            bmpTypeID = gBmpTypeClassDict.Item(BMPType)
        End If
    End If
    'Define card 715
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If
    
    StrCard715BMPTypes = StrCard715BMPTypes & _
                         strBMPindex & vbTab & _
                         BMPName & vbTab & _
                         bmpTypeID & vbTab & _
                         Format$(pDrainageArea, "##0.000") & vbTab & _
                         NumUnits & vbTab & _
                         ddArea & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine
                         
    If bKCFactor Then
        GetBMP_K_C_factors strBMPindex  'added to record K, C factors
    End If
    
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPParametersEF: " & Err.description
CleanUp:
    
End Sub


Private Sub GetInitialMoisture(strBMPindex As String, pXpos As Double, pYpos As Double, pDrainageArea As Double)
On Error GoTo ShowError
    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim IniMoisture As Double
    Dim IniSurfaceWaterDepth As Double
        
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    IniMoisture = CDbl(pBMPDetailDict.Item("txtInitialMoist"))
    IniSurfaceWaterDepth = CDbl(pBMPDetailDict.Item("txtInitialSerfaceWater"))
    
    
    'Add Class A Parameters
    StrCard747BMPClassABF = StrCard747BMPClassABF & _
                          strBMPindex & vbTab & _
                          Format$(IniSurfaceWaterDepth, "##0.000") & vbTab & _
                          Format$(IniMoisture, "##0.000") & vbNewLine
                          
    GoTo CleanUp
ShowError:
    MsgBox "GetInitialMoisture: " & Err.description
CleanUp:
    
End Sub

Private Sub GetDiscoImperParameters(strBMPindex As String)
On Error GoTo ShowError
    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim Area As Double
    Dim FlowLength As Double
    Dim DepresStorage As Double
    Dim Manning_N As Double
    Dim Slope As Double
    Dim Sat_Infilt As Double
    Dim PolRemm As Integer
    Dim DCIA As Double
    Dim TotalImpDrainArea As Double
    
        
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    Area = CDbl(pBMPDetailDict.Item("BMPLength"))
    FlowLength = CDbl(pBMPDetailDict.Item("BMPWidth"))
    DepresStorage = CDbl(pBMPDetailDict.Item("DepressionStorage"))
    Slope = CDbl(pBMPDetailDict.Item("OverlandSlope"))
    Manning_N = CDbl(pBMPDetailDict.Item("OverlandManning"))
    Sat_Infilt = CDbl(pBMPDetailDict.Item("SoilLayerInfiltration"))
    PolRemm = CDbl(pBMPDetailDict.Item("Pollutant Removal Method"))
    DCIA = CDbl(pBMPDetailDict.Item("txtUserInputImp"))
    TotalImpDrainArea = CDbl(pBMPDetailDict.Item("txtImpArea")) * 0.0000229568411 ' convert from square feet to acre
    
    
    
    'Add Class E Parameters
    StrCard762BMPClassE = StrCard762BMPClassE & _
                          strBMPindex & vbTab & _
                          Format$(Area, "##0.000") & vbTab & _
                          Format$(FlowLength, "##0.000") & vbTab & _
                          Format$(DepresStorage, "##0.000") & vbTab & _
                          Format$(Slope, "##0.000") & vbTab & _
                          Format$(Manning_N, "##0.000") & vbTab & _
                          Format$(Sat_Infilt, "##0.000") & vbTab & _
                          CStr(PolRemm) & vbTab & _
                          Format$(DCIA, "##0.000") & vbTab & _
                          Format$(TotalImpDrainArea, "##0.00") & vbNewLine

                          
    GoTo CleanUp
ShowError:
    MsgBox "GetDiscoImperParameters: " & Err.description
CleanUp:
    
End Sub


Private Sub GetBufferstripParameters(strBMPindex As String)
On Error GoTo ShowError
    Dim BMPName As String
    Dim BMPType As String
    Dim bmpClass As String
    Dim Width As Double
    Dim FlowLength As Double
    Dim DepresStorage As Double
    Dim Manning_N As Double
    Dim Slope As Double
    Dim PolRemm As Integer
    Dim ETMultiplier As Double
    
    
        
    BMPName = pBMPDetailDict.Item("BMPName")
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    Width = CDbl(pBMPDetailDict.Item("BMPLength"))
    FlowLength = CDbl(pBMPDetailDict.Item("BMPWidth"))
    DepresStorage = CDbl(pBMPDetailDict.Item("DepressionStorage"))
    Slope = CDbl(pBMPDetailDict.Item("OverlandSlope"))
    Manning_N = CDbl(pBMPDetailDict.Item("OverlandManning"))
    PolRemm = CInt(pBMPDetailDict.Item("Pollutant Removal Method"))
    ETMultiplier = CDbl(pBMPDetailDict.Item("Multiplier"))
    
    
    'Add Class F Parameters
    StrCard761BMPClassF = StrCard761BMPClassF & _
                          strBMPindex & vbTab & _
                          Format$(FlowLength, "##0.000") & vbTab & _
                          Format$(Width, "##0.000") & vbTab & _
                          Format$(DepresStorage, "##0.000") & vbTab & _
                          Format$(Slope, "##0.000") & vbTab & _
                          Format$(Manning_N, "##0.000") & vbTab & _
                          CStr(PolRemm) & vbTab & _
                          Format$(ETMultiplier, "##0.00") & vbNewLine

                          
    GoTo CleanUp
ShowError:
    MsgBox "GetBufferstripParameters: " & Err.description
CleanUp:
    
End Sub



'******************************************************************************
'Subroutine: GetConduitParameters
'Author:     Mira Chokshi
'Purpose:    Get the dimensional parameters for BMP Type C - Conduit.
'*****************************************************************************
Private Sub GetConduitParameters(iConduitIndex As Integer)
On Error GoTo ShowError

    '*** define all conduit parameters
    Dim bmpClass As String
    Dim conduitShape As String
    Dim conduitInletNode As Integer
    Dim conduitOutletNode As Integer
    Dim conduitBarrels As Integer
    Dim conduitGeom1 As Double  'Max. depth
    Dim conduitGeom2 As Double
    Dim conduitGeom3 As Double
    Dim conduitGeom4 As Double
    Dim conduitLength As Double
    Dim conduitManning As Double
    Dim conduitInitFlow As Double
    Dim conduitHeadLossEnt As Double
    Dim conduitHeadLossExit As Double
    Dim conduitHeadLossAvg As Double
    Dim conduitSlopeEnt As Double
    Dim conduitSlopeExit As Double
    Dim conduitSlopeAvg As Double
    Dim AquiferID As Integer
    Dim AquiferStr As String
    Dim conduitTransectName As String
    
    'Get all BMP type C - Conduit parameter names
    bmpClass = pBMPDetailDict.Item("BMPClass")
    conduitShape = pBMPDetailDict.Item("TYPE")
    conduitInletNode = pBMPDetailDict.Item("INLETNODE")
    conduitOutletNode = pBMPDetailDict.Item("OUTLETNODE")
    conduitBarrels = pBMPDetailDict.Item("BARRELS")
    conduitGeom1 = pBMPDetailDict.Item("GEOM1")
    conduitGeom2 = pBMPDetailDict.Item("GEOM2")
    conduitGeom3 = pBMPDetailDict.Item("GEOM3")
    conduitGeom4 = pBMPDetailDict.Item("GEOM4")
    conduitLength = pBMPDetailDict.Item("LENGTH")
    conduitManning = pBMPDetailDict.Item("MANN_N")
    conduitInitFlow = pBMPDetailDict.Item("INIFLOW")
    conduitHeadLossEnt = pBMPDetailDict.Item("ENTLOSS")
    conduitHeadLossExit = pBMPDetailDict.Item("EXTLOSS")
    conduitHeadLossAvg = pBMPDetailDict.Item("AVGLOSS")
    conduitSlopeEnt = pBMPDetailDict.Item("ENTINVERTLEV")
    conduitSlopeExit = pBMPDetailDict.Item("EXTINVERTLEV")
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
            AquiferID = ii + 1
        End If
    Next ii
    If gInternalSimulation = True Then
        AquiferID = 0
    End If
    
    Dim bedWidth As Double
    Dim bedDepth As Double
    Dim bedPorosity As Double
    Dim SAND_FRAC As Double
    Dim SILT_FRAC As Double
    Dim CLAY_FRAC As Double
    Dim sandDiameter As Double
    Dim sandVelocity As Double
    Dim sandDensity As Double
    Dim sandCoeff As Double
    Dim sandExponent As Double
    Dim siltDiameter As Double
    Dim siltVelocity As Double
    Dim siltDensity As Double
    Dim siltTaucd As Double
    Dim siltTaucs As Double
    Dim siltM As Double
    
    bedWidth = pBMPDetailDict.Item("Bed width")
    bedDepth = pBMPDetailDict.Item("Bed depth")
    bedPorosity = pBMPDetailDict.Item("Porosity")
    SAND_FRAC = pBMPDetailDict.Item("Sand fraction")
    SILT_FRAC = pBMPDetailDict.Item("Silt fraction")
    CLAY_FRAC = pBMPDetailDict.Item("Clay fraction")
    sandDiameter = pBMPDetailDict.Item("Sand effective diameter")
    sandVelocity = pBMPDetailDict.Item("Sand velocity")
    sandDensity = pBMPDetailDict.Item("Sand density")
    sandCoeff = pBMPDetailDict.Item("Sand coefficient")
    sandExponent = pBMPDetailDict.Item("Sand exponent")
    siltDiameter = pBMPDetailDict.Item("Silt effective diameter")
    siltVelocity = pBMPDetailDict.Item("Silt velocity")
    siltDensity = pBMPDetailDict.Item("Silt density")
    siltTaucd = pBMPDetailDict.Item("Silt Deposition stress")
    siltTaucs = pBMPDetailDict.Item("Silt Scour stress")
    siltM = pBMPDetailDict.Item("Silt Erodibility")
    
    Dim clayDiameter As Double
    Dim clayVelocity As Double
    Dim clayDensity As Double
    Dim clayTaucd As Double
    Dim clayTaucs As Double
    Dim clayM As Double
    
    clayDiameter = pBMPDetailDict.Item("Clay effective diameter")
    clayVelocity = pBMPDetailDict.Item("Clay velocity")
    clayDensity = pBMPDetailDict.Item("Clay density")
    clayTaucd = pBMPDetailDict.Item("Clay Deposition stress")
    clayTaucs = pBMPDetailDict.Item("Clay Scour stress")
    clayM = pBMPDetailDict.Item("Clay Erodibility")
    
    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists("Conduit") Then
            bmpTypeID = gBmpTypeClassDict.Item("Conduit")
        End If
    End If
    
    
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If
    StrCard715BMPTypes = StrCard715BMPTypes & _
                         iConduitIndex & vbTab & _
                         "Conduit" & vbTab & _
                         bmpTypeID & vbTab & _
                         0 & vbTab & _
                         1 & vbTab & _
                         0 & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine
                         
    'Define card 750 string
    StrCard750ConduitDimensions = StrCard750ConduitDimensions & _
                         iConduitIndex & vbTab & _
                         conduitInletNode & vbTab & _
                         conduitOutletNode & vbTab & _
                         conduitLength & vbTab & _
                         conduitManning & vbTab & _
                         conduitSlopeEnt & vbTab & _
                         conduitSlopeExit & vbTab & _
                         conduitInitFlow & vbTab & _
                         conduitHeadLossEnt & vbTab & _
                         conduitHeadLossExit & vbTab & _
                         conduitHeadLossAvg & vbNewLine
                             
    'Define card 755 string
    'StrCard755ConduitCrossSections = StrCard755ConduitCrossSections & _
                          iConduitIndex & vbTab & _
                          conduitShape & vbTab & _
                          conduitGeom1 & vbTab & _
                          conduitGeom2 & vbTab & _
                          conduitGeom3 & vbTab & _
                          conduitGeom4 & vbTab & _
                          conduitBarrels & vbNewLine
                          
    If conduitShape = "IRREGULAR" Then
        conduitTransectName = pBMPDetailDict.Item("Transect")
        If conduitTransectName = "" Then
            conduitTransectName = "0"
        End If
        StrCard755ConduitCrossSections = StrCard755ConduitCrossSections & _
                          iConduitIndex & vbTab & _
                          conduitShape & vbTab & _
                          conduitTransectName & vbTab & _
                          conduitGeom2 & vbTab & _
                          conduitGeom3 & vbTab & _
                          conduitGeom4 & vbTab & _
                          conduitBarrels & vbNewLine

    Else
        StrCard755ConduitCrossSections = StrCard755ConduitCrossSections & _
                          iConduitIndex & vbTab & _
                          conduitShape & vbTab & _
                          conduitGeom1 & vbTab & _
                          conduitGeom2 & vbTab & _
                          conduitGeom3 & vbTab & _
                          conduitGeom4 & vbTab & _
                          conduitBarrels & vbNewLine
    End If
                          
                          
    'StrCard775Sediment = StrCard775Sediment & _
                        iConduitIndex & vbTab & _
                        bedWidth & vbTab & _
                        bedDepth & vbTab & _
                        bedPorosity & vbTab & _
                        SAND_FRAC & vbTab & _
                        SILT_FRAC & vbTab & _
                        CLAY_FRAC & vbNewLine
                        
    StrCard775Sediment = StrCard775Sediment & _
                        iConduitIndex & vbTab & _
                        bedWidth & vbTab & _
                        bedDepth & vbTab & _
                        bedPorosity & vbNewLine
                       
                        
                        
    StrCard780SandTransport = StrCard780SandTransport & _
                        iConduitIndex & vbTab & _
                        sandDiameter & vbTab & _
                        sandVelocity & vbTab & _
                        sandDensity & vbTab & _
                        sandCoeff & vbTab & _
                        sandExponent & vbNewLine
    StrCard785SiltTransport = StrCard785SiltTransport & _
                        iConduitIndex & vbTab & _
                        siltDiameter & vbTab & _
                        siltVelocity & vbTab & _
                        siltDensity & vbTab & _
                        siltTaucd & vbTab & _
                        siltTaucs & vbTab & _
                        siltM & vbNewLine
    StrCard786ClayTransport = StrCard786ClayTransport & _
                        iConduitIndex & vbTab & _
                        clayDiameter & vbTab & _
                        clayVelocity & vbTab & _
                        clayDensity & vbTab & _
                        clayTaucd & vbTab & _
                        clayTaucs & vbTab & _
                        clayM & vbNewLine
   'Commented the following - June 18, 2007
''    'Define card 1190 string
''    StrCard1190ConduitLosses = StrCard1190ConduitLosses & _
''                          iConduitIndex & vbTab & _
''                          conduitHeadLossEnt & vbTab & _
''                          conduitHeadLossExit & vbTab & _
''                          conduitHeadLossAvg & vbTab & _
''                          "NO" & vbNewLine
                          
    GoTo CleanUp
ShowError:
    MsgBox "GetConduitParameters: " & Err.description
CleanUp:
End Sub

Public Sub GetIrregularCSParameters()
On Error GoTo ShowError
    Dim pTable As iTable
    Set pTable = GetInputDataTable("Transects")
    If pTable Is Nothing Then
        Exit Sub
    End If
    
    Dim pCursor As esriGeoDatabase.ICursor
    Dim pRow As iRow
        
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    
    Dim bmpDict As Scripting.Dictionary
    Set bmpDict = New Scripting.Dictionary
    Dim propNameFld As Integer
    Dim propValueFld As Integer
    propNameFld = pTable.FindField("PropName")
    propValueFld = pTable.FindField("PropValue")
    Dim IDFld As Long
    IDFld = pTable.FindField("ID")
    
    Dim strStatGrid As String, strElevGrid As String
    Dim statGrids, elevGrids
    Dim i As Integer
    Dim numOfStations As Integer
    
    Set gTransCountDict = New Dictionary
    Dim iDistinctID As Integer
    Dim pCursorDistinctID As ICursor
    Set pCursorDistinctID = pTable.Search(Nothing, False)
    Dim pRowDistinctID As iRow
    Set pRowDistinctID = pCursorDistinctID.NextRow
    Do Until pRowDistinctID Is Nothing
        If Not gTransCountDict.Exists(pRowDistinctID.value(IDFld)) Then
            gTransCountDict.add pRowDistinctID.value(IDFld), pRowDistinctID.value(IDFld)
        End If
        Set pRowDistinctID = pCursorDistinctID.NextRow
    Loop
    
    Dim transectId
    For Each transectId In gTransCountDict
        pQueryFilter.WhereClause = " ID = " & CInt(transectId)
        Set pCursor = pTable.Search(pQueryFilter, False)
        Set pRow = pCursor.NextRow
        Do Until pRow Is Nothing
            bmpDict.Item(pRow.value(propNameFld)) = pRow.value(propValueFld)
            Set pRow = pCursor.NextRow
        Loop
        
        strStatGrid = bmpDict.Item("Stations_grid")
        strElevGrid = bmpDict.Item("Elevations_grid")
        statGrids = CustomSplit_New(strStatGrid, ";")
        elevGrids = CustomSplit_New(strElevGrid, ";")
        numOfStations = UBound(statGrids) + 1
        StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & "NC" & vbTab & bmpDict.Item("Left Bank") & vbTab & bmpDict.Item("Right Bank") & vbTab & bmpDict.Item("Channel") & vbNewLine & _
                    "X1" & vbTab & bmpDict.Item("Name") & vbTab & numOfStations & vbTab & bmpDict.Item("Left") & vbTab & bmpDict.Item("Right") & "   0   0   0" & vbTab & bmpDict.Item("Stations") & vbTab & bmpDict.Item("Elevations") & vbNewLine
        
        StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & "GR"
        For i = 0 To numOfStations - 1
            'StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & vbTab & CDbl(statGrids(i)) & vbTab & CDbl(elevGrids(i))
            StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & vbTab & CDbl(elevGrids(i)) & vbTab & CDbl(statGrids(i))
            
            If (i + 1) Mod 5 = 0 Then
                StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & vbNewLine & "GR"
            End If
        Next
        StrCard760IrregularCrossSections = StrCard760IrregularCrossSections & vbNewLine
    Next
    GoTo CleanUp
ShowError:
    gHasInFileError = True
    MsgBox "Error in GetIrregularCSParameters : " & Err.description
CleanUp:
    Set pTable = Nothing
    Set pQueryFilter = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set bmpDict = Nothing
End Sub

Public Function CustomSplit_New(pString As String, Optional delimiter As String) As String()
  On Error GoTo ErrorHandler
    Dim res() As String
    If Trim(pString) = "" Then
        CustomSplit_New = Split(Trim(pString))
        Exit Function
    End If
    pString = Replace(pString, " ", vbTab)
    If delimiter = "" Then delimiter = vbTab
    
    Dim tmpRes
    'tmpRes = Split(pString, " ", -1, vbTextCompare)
    tmpRes = Split(pString, delimiter, , vbTextCompare)
    Dim resWords As Long
    resWords = 0
    Dim incr As Integer
    For incr = 0 To UBound(tmpRes)
        If Trim(tmpRes(incr)) <> "" Then
            ReDim Preserve res(resWords)
            res(resWords) = tmpRes(incr)
            resWords = resWords + 1
        End If
    Next incr
    CustomSplit_New = res

  Exit Function
ErrorHandler:
  'HandleError True, "CustomSplit " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.description, 4
End Function


'******************************************************************************
'Subroutine: GetVFSParameters
'Author:     Mira Chokshi
'Purpose:    Get the dimensional parameters for BMP Type D - Buffer Strip
'*****************************************************************************
Private Sub GetVFSParameters(iVFSIndex As Integer)
On Error GoTo ShowError
    
    Dim i As Integer
        
    '*** define all conduit parameters
    Dim bmpClass As String
    Dim vfsLength As Double
    Dim vfsWidth As Double
    Dim AquiferID As Integer
    Dim AquiferStr As String
        
    'Get all BMP type D - Buffer Strip parameter names
    bmpClass = "D"  'pBMPDetailDict.Item("BMPClass")
'    vfsLength = pBMPDetailDict.Item("Length")
'    vfsWidth = pBMPDetailDict.Item("Width")
    
    'Change in control name - Sabu Paul, June 7 2007
    vfsLength = pBMPDetailDict.Item("BufferLength")
    vfsWidth = pBMPDetailDict.Item("BufferWidth")

    Dim NPROP As Integer
    NPROP = CInt(pBMPDetailDict.Item("NPROP"))
    
    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists("VFS") Then
            bmpTypeID = gBmpTypeClassDict.Item("VFS")
        End If
    End If
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
            AquiferID = ii + 1
        End If
    Next ii
    
    If gInternalSimulation = True Then
        AquiferID = 0
    End If


    Dim vfsDA As Double
    vfsDA = 0
    If gBMPDrainAreaDict.Exists(iVFSIndex) Then vfsDA = gBMPDrainAreaDict.Item(iVFSIndex)
    
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If
    
    StrCard715BMPTypes = StrCard715BMPTypes & _
                         iVFSIndex & vbTab & _
                         pBMPDetailDict.Item("Name") & vbTab & _
                         bmpTypeID & vbTab & _
                         Format$(vfsDA, "##0.000") & vbTab & _
                         1 & vbTab & _
                         0 & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine
                         
    'Define card 901 string
    StrCard901_VFS_Dim = StrCard901_VFS_Dim & _
                         iVFSIndex & vbTab & _
                         pBMPDetailDict.Item("Name") & vbTab & _
                         vfsWidth & vbTab & _
                         vfsLength & vbTab & _
                         NPROP & vbNewLine
                      
    'Define card 902 string
    For i = 1 To NPROP
        StrCard902_VFS_SegDetails = StrCard902_VFS_SegDetails & _
                    iVFSIndex & vbTab & _
                    i & vbTab & _
                    pBMPDetailDict.Item("SX" & i) & vbTab & _
                    pBMPDetailDict.Item("RNA" & i) & vbTab & _
                    pBMPDetailDict.Item("SOA" & i) & vbNewLine
                         
    Next
    
    'Define card 903 string
    StrCard903_VFS_SoilProps = StrCard903_VFS_SoilProps & _
                    iVFSIndex & vbTab & _
                    pBMPDetailDict.Item("VKS") & vbTab & _
                    pBMPDetailDict.Item("Sav") & vbTab & _
                    pBMPDetailDict.Item("OS") & vbTab & _
                    pBMPDetailDict.Item("OI") & vbTab & _
                    pBMPDetailDict.Item("SM") & vbTab & _
                    pBMPDetailDict.Item("SCHK") & vbNewLine
    
    'Define card 904 string
    StrCard904_VFS_Buf_Sed = StrCard904_VFS_Buf_Sed & _
                    iVFSIndex & vbTab & _
                    pBMPDetailDict.Item("SS") & vbTab & _
                    pBMPDetailDict.Item("VN") & vbTab & _
                    pBMPDetailDict.Item("H") & vbTab & _
                    pBMPDetailDict.Item("Vn2") & vbTab & _
                    0 & vbNewLine 'card ICO - no feed back
    
     'Define card 905 string
    StrCard905_VFS_Sed_Filt = StrCard905_VFS_Sed_Filt & _
                    iVFSIndex & vbTab & _
                    pBMPDetailDict.Item("NPARTSand") & vbTab & _
                    pBMPDetailDict.Item("NPARTSilt") & vbTab & _
                    pBMPDetailDict.Item("NPARTClay") & vbTab & _
                    pBMPDetailDict.Item("COARSESand") & vbTab & _
                    pBMPDetailDict.Item("COARSESilt") & vbTab & _
                    pBMPDetailDict.Item("COARSEClay") & vbTab & _
                    pBMPDetailDict.Item("PORSand") & vbTab & _
                    pBMPDetailDict.Item("PORSilt") & vbTab & _
                    pBMPDetailDict.Item("PORClay") & vbTab & _
                    pBMPDetailDict.Item("DPSand") & vbTab & _
                    pBMPDetailDict.Item("DPSilt") & vbTab & _
                    pBMPDetailDict.Item("DPClay") & vbTab & _
                    pBMPDetailDict.Item("SGSand") & vbTab & _
                    pBMPDetailDict.Item("SGSilt") & vbTab & _
                    pBMPDetailDict.Item("SGClay") & vbNewLine
    
    Call CreatePollutantList
    Dim pTotalPollutants As Integer
    pTotalPollutants = UBound(gPollutants) + 1
    
   
    
    StrCard906_VFS_Sed_Fracion = StrCard906_VFS_Sed_Fracion & iVFSIndex
    StrCard907_VFS_FO_Adsorbed = StrCard907_VFS_FO_Adsorbed & iVFSIndex
    StrCard908_VFS_FO_Dissolved = StrCard908_VFS_FO_Dissolved & iVFSIndex
    StrCard909_VFS_TC_Adsorbed = StrCard909_VFS_TC_Adsorbed & iVFSIndex
    StrCard910_VFS_TC_Dissolved = StrCard910_VFS_TC_Dissolved & iVFSIndex
    
    Dim iR As Integer
    For iR = 1 To pTotalPollutants
        'If iR <> sedPollInd Then
        If pBMPDetailDict.Exists("SedFrac" & iR) Then
            StrCard906_VFS_Sed_Fracion = StrCard906_VFS_Sed_Fracion & vbTab & pBMPDetailDict.Item("SedFrac" & iR)
            StrCard907_VFS_FO_Adsorbed = StrCard907_VFS_FO_Adsorbed & vbTab & pBMPDetailDict.Item("SedDec" & iR)
            StrCard908_VFS_FO_Dissolved = StrCard908_VFS_FO_Dissolved & vbTab & pBMPDetailDict.Item("WatDec" & iR)
            StrCard909_VFS_TC_Adsorbed = StrCard909_VFS_TC_Adsorbed & vbTab & pBMPDetailDict.Item("SedCorr" & iR)
            StrCard910_VFS_TC_Dissolved = StrCard910_VFS_TC_Dissolved & vbTab & pBMPDetailDict.Item("WatCorr" & iR)
        End If
    Next
    
    StrCard906_VFS_Sed_Fracion = StrCard906_VFS_Sed_Fracion & vbNewLine
    StrCard907_VFS_FO_Adsorbed = StrCard907_VFS_FO_Adsorbed & vbNewLine
    StrCard908_VFS_FO_Dissolved = StrCard908_VFS_FO_Dissolved & vbNewLine
    StrCard909_VFS_TC_Adsorbed = StrCard909_VFS_TC_Adsorbed & vbNewLine
    StrCard910_VFS_TC_Dissolved = StrCard910_VFS_TC_Dissolved & vbNewLine

    Exit Sub
ShowError:
    MsgBox "Error in GetVFSParameters: " & Err.description
CleanUp:
End Sub


'******************************************************************************
'Subroutine: GetDummyBMPParameters
'Author:     Mira Chokshi
'Purpose:    Get the spatial and dimensional parameters for DUMMY BMP.
'*****************************************************************************
'Private Sub GetDummyBMPParameters(iBMPindex As Integer, pXpos As Double, pYpos As Double, pDrainageArea As Double)
Private Sub GetDummyBMPParameters(strBMPindex As String, pXpos As Double, pYpos As Double, pDrainageArea As Double)
On Error GoTo ShowError
    
    Dim BMPType As String
    Dim bmpClass As String
    Dim AquiferID As Integer
    Dim AquiferStr As String
    
    BMPType = pBMPDetailDict.Item("BMPType")
    bmpClass = pBMPDetailDict.Item("BMPClass")
    
    Dim bmpTypeID As String
    If Not gBmpTypeClassDict Is Nothing Then
        If gBmpTypeClassDict.Exists(BMPType) Then
            bmpTypeID = gBmpTypeClassDict.Item(BMPType)
        End If
    End If
    AquiferStr = pBMPDetailDict.Item("Aquifer")
    If AquiferStr = "" Then
        AquiferID = 0
    End If
    Dim ii As Integer
    For ii = 0 To FrmAquiferDef.lstAquifers.ListCount - 1
        If AquiferStr = FrmAquiferDef.lstAquifers.List(ii) Then
            AquiferID = ii + 1
        End If
    Next ii
     If gInternalSimulation = True Then
        AquiferID = 0
    End If
   
    
    If pPredevelopedLanduse = "" Then
        pPredevelopedLanduse = "0"
    End If
    
    StrCard715BMPTypes = StrCard715BMPTypes & _
                         strBMPindex & vbTab & _
                         BMPType & vbTab & _
                         bmpTypeID & vbTab & _
                         Format$(pDrainageArea, "##0.000") & vbTab & _
                         1 & vbTab & _
                         0 & vbTab & _
                         pPredevelopedLanduse & vbTab & _
                         AquiferID & vbTab & _
                         "0" & vbTab & _
                         "No" & vbNewLine
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPParametersB: " & Err.description
CleanUp:
End Sub

'''******************************************************************************
'''Subroutine: GetAssessPointDetails
'''Author:     Mira Chokshi
'''Purpose:    Assessment point is assumed to be a class A BMP with zero dimensions
'''            Get the spatial information for assessment point and evaluation
'''            factor.
'''            Mira Chokshi, Sabu Paul modified 08/23/04 to change output format
'''*****************************************************************************
''Private Sub GetAssessPointDetails(iBMPindex As Integer, pXpos As Double, pYpos As Double, pDrainageArea As Double)
''On Error GoTo ShowError
''
''    Dim BMPName As String
''    Dim bmpType As String
''    Dim bmpClass As String
''
''    Dim pBMPKeys
''    pBMPKeys = pBMPDetailDict.Keys
''
''    BMPName = pBMPDetailDict.Item("BMPName")
''    bmpType = pBMPDetailDict.Item("BMPType")
''    bmpClass = pBMPDetailDict.Item("BMPClass")
''
''    If bmpClass = "X" Then
''        StrCard1100BMPTypes = StrCard1100BMPTypes & _
''                     iBMPindex & vbTab & _
''                     "AssessmentPoint" & vbTab & _
''                     bmpClass & vbTab & _
''                     pDrainageArea & vbTab & _
''                     pPredevelopedLanduse & vbTab & _
''                     pXpos & vbTab & _
''                     pYpos & vbNewLine
''    End If
''
''    GoTo CleanUp
''ShowError:
''    MsgBox "GetAssessPointDetails: " & Err.description
''CleanUp:
''    Set pBMPKeys = Nothing
''End Sub

'******************************************************************************
'Subroutine: GetSoilIndex
'Author:     Mira Chokshi
'Purpose:    Get the spatial and dimensional parameters for BMP Type B.
'*****************************************************************************
Private Sub GetSoilIndex(strBMPindex As String) '(iBMPindex As Integer)
On Error GoTo ShowError
    Dim SoilDepth As Double     'Soil Growth Properties
    Dim SoilPorosity As Double
    Dim udsoildepth As Double
    Dim udsoilporosity As Double
    Dim udfinalf As Double
    Dim finalf As Double
    Dim vegparma As Double
    Dim underdrain_on As Integer
    
    Dim infiltm As Integer
    Dim polRoutM As Integer
    Dim polRemovM As Integer
    Dim soilFC As Double
    Dim soilWP As Double
    Dim Suction As Double
    Dim HydCon  As Double
    Dim IMDmax As Double
    
    Dim HortonMaxInfil As Double
    Dim HortonDecay As Double
    Dim HortonDryTime As Double
    Dim HortonMaxVolume As Double

    HortonMaxInfil = CDbl(pBMPDetailDict.Item("HortonMaxInfiltration"))
    HortonDecay = CDbl(pBMPDetailDict.Item("HortonDecayConstant"))
    HortonDryTime = CDbl(pBMPDetailDict.Item("HortonDryTime"))
    HortonMaxVolume = CDbl(pBMPDetailDict.Item("HortonMaxVolume"))

    SoilDepth = CDbl(pBMPDetailDict.Item("SoilDepth"))
    SoilPorosity = CDbl(pBMPDetailDict.Item("SoilPorosity"))
    udsoildepth = CDbl(pBMPDetailDict.Item("StorageDepth"))
    udsoilporosity = CDbl(pBMPDetailDict.Item("VoidFraction"))
    udfinalf = CDbl(pBMPDetailDict.Item("BackgroundInfiltration"))
    vegparma = CDbl(pBMPDetailDict.Item("VegetativeParam"))
    finalf = CDbl(pBMPDetailDict.Item("SoilLayerInfiltration"))
    underdrain_on = 0
    If (CBool(pBMPDetailDict.Item("UnderDrainON")) = True) Then
        underdrain_on = 1
    End If
    
    infiltm = pBMPDetailDict.Item("Infiltration Method")

    polRoutM = pBMPDetailDict.Item("Pollutant Routing Method")
    polRemovM = pBMPDetailDict.Item("Pollutant Removal Method")
    soilFC = CDbl(pBMPDetailDict.Item("SoilFieldCapacity"))
    soilWP = CDbl(pBMPDetailDict.Item("SoilWiltingPoint"))
    Suction = CDbl(pBMPDetailDict.Item("SuctionHead"))
    IMDmax = CDbl(pBMPDetailDict.Item("InitialDeficit"))

    'StrCard740BMPSoilIndex = StrCard740BMPSoilIndex & _
                             strBMPindex & vbTab & _
                             infiltm & vbTab & _
                             polRoutM & vbTab & _
                             polRemovM & vbTab & _
                             SoilDepth & vbTab & _
                             SoilPorosity & vbTab & _
                             soilFC & vbTab & _
                             soilWP & vbTab & _
                             vegparma & vbTab & _
                             finalf & vbTab & _
                             underdrain_on & vbTab & _
                             udsoildepth & vbTab & _
                             udsoilporosity & vbTab & _
                             udfinalf & vbTab & _
                             Suction & vbTab & _
                             HydCon & vbTab & _
                             IMDmax & vbNewLine
    StrCard740BMPSoilIndex = StrCard740BMPSoilIndex & _
                             strBMPindex & vbTab & _
                             infiltm & vbTab & _
                             polRoutM & vbTab & _
                             polRemovM & vbTab & _
                             SoilDepth & vbTab & _
                             SoilPorosity & vbTab & _
                             soilFC & vbTab & _
                             soilWP & vbTab & _
                             vegparma & vbTab & _
                             finalf & vbTab & _
                             underdrain_on & vbTab & _
                             udsoildepth & vbTab & _
                             udsoilporosity & vbTab & _
                             udfinalf & vbTab & _
                             Suction & vbTab & _
                             IMDmax & vbTab & _
                             HortonMaxInfil & vbTab & _
                             HortonDecay & vbTab & _
                             HortonDryTime & vbTab & _
                             HortonMaxVolume & vbNewLine
    'if pollutant removal is kadlec and knight method
    GetBMP_K_C_factors strBMPindex  'iBMPindex
''    If polRemovM = 1 Then
''        GetBMP_K_C_factors iBMPindex
''    End If
    
    GoTo CleanUp
ShowError:
    MsgBox "GetSoilIndex: " & Err.description
CleanUp:
End Sub


'******************************************************************************
'Subroutine: GetBMPDecayFactors
'Author:     Mira Chokshi
'Purpose:    Get the decay factors for each bmp/nutrient and write to a string.
'*****************************************************************************
Public Sub GetBMPDecayFactors(pBMPIdStr As String) '(pBMPId As Integer)
On Error GoTo ShowError
    'pBmpDetailDict

    Dim pStrCard765ForBMP As String
    pStrCard765ForBMP = ""
    Dim pDecayFactorValue As Double
    Dim i As Integer
    For i = 1 To pTotalPollutantCount
        pDecayFactorValue = 0
        If pBMPDetailDict.Exists("Decay" & i) Then pDecayFactorValue = pBMPDetailDict.Item("Decay" & i)
        'pStrCard765ForBMP = pStrCard765ForBMP & Format$(pDecayFactorValue / 24, "##0.0000") & vbTab
        pStrCard765ForBMP = pStrCard765ForBMP & Format$(pDecayFactorValue, "##0.0000") & vbTab
    Next
    If (pStrCard765ForBMP <> "") Then
         StrCard765DecayFactors = StrCard765DecayFactors & pBMPIdStr & vbTab & pStrCard765ForBMP & vbNewLine
    End If
    
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPDecayFactors: " & Err.description
CleanUp:
'    Set pTable = Nothing
'    Set pCursor = Nothing
'    Set pRow = Nothing
End Sub


'******************************************************************************
'Subroutine: GetBMPPercentRemovalFactors
'Author:     Mira Chokshi
'Purpose:    Get the percent removal factors for each bmp/nutrient
'            and write to a string.
'*****************************************************************************
Private Sub GetBMPPercentRemovalFactors(pBMPIdStr As String) '(pBMPId As Integer)
On Error GoTo ShowError
    '***
    Dim pStrCard770ForBMP As String
    pStrCard770ForBMP = ""
    Dim i As Integer
    Dim pPercentRemovalValue As Double
    For i = 1 To pTotalPollutantCount
        pPercentRemovalValue = 0
        If pBMPDetailDict.Exists("PctRem" & i) Then pPercentRemovalValue = pBMPDetailDict.Item("PctRem" & i)
        pStrCard770ForBMP = pStrCard770ForBMP & pPercentRemovalValue & vbTab
    Next
    If (pStrCard770ForBMP <> "") Then
        StrCard770PercentRemoval = StrCard770PercentRemoval & pBMPIdStr & vbTab & pStrCard770ForBMP & vbNewLine
    End If
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPPercentRemovalFactors: " & Err.description
CleanUp:
'    Set pRow = Nothing
'    Set pCursor = Nothing
'    Set pTable = Nothing
'    Set pQueryFilter = Nothing
End Sub

Private Sub GetBMP_K_C_factors(pBMPID As String) 'pBMPId As Integer)
On Error GoTo ShowError
    '***

    Dim i As Integer
    Dim kValue As Double
    Dim cValue As Double
    
    StrCard766KFactors = StrCard766KFactors & pBMPID
    StrCard767CValues = StrCard767CValues & pBMPID
    
    For i = 1 To pTotalPollutantCount
        kValue = 0
        If pBMPDetailDict.Exists("K" & i) Then kValue = pBMPDetailDict.Item("K" & i)
        StrCard766KFactors = StrCard766KFactors & vbTab & kValue
        
        cValue = 0
        If pBMPDetailDict.Exists("C" & i) Then cValue = pBMPDetailDict.Item("C" & i)
        StrCard767CValues = StrCard767CValues & vbTab & cValue
    Next
    StrCard766KFactors = StrCard766KFactors & vbNewLine
    StrCard767CValues = StrCard767CValues & vbNewLine
    GoTo CleanUp
ShowError:
    MsgBox "GetBMP_K_C_factors: " & Err.description
CleanUp:
'    Set pRow = Nothing
'    Set pCursor = Nothing
'    Set pTable = Nothing
'    Set pQueryFilter = Nothing
End Sub

'******************************************************************************
'Subroutine: GetBMPCosts
'Author:     Mira Chokshi
'Purpose:    Get the cost function parameters for each bmp and write to a string.
'*****************************************************************************
Private Sub GetBMPCosts(strBMPindex As String) 'iBMPindex As Integer
On Error GoTo ShowError
''    Dim bmpAa As Double
''    Dim bmpAb As Double
''    Dim bmpDa As Double
''    Dim bmpDb As Double
''    Dim bmpLdCost As Double
''    Dim bmpConstCost As Double
''
''    bmpAa = CDbl(pBmpDetailDict.Item("Aa"))
''    bmpAb = CDbl(pBmpDetailDict.Item("Ab"))
''    bmpDa = CDbl(pBmpDetailDict.Item("Da"))
''    bmpDb = CDbl(pBmpDetailDict.Item("Db"))
''    bmpLdCost = CDbl(pBmpDetailDict.Item("LdCost"))
''    bmpConstCost = CDbl(pBmpDetailDict.Item("ConstCost"))
''
''    StrCard805BMPCost = StrCard805BMPCost & _
''                            iBMPindex & vbTab & _
''                            bmpAa & vbTab & _
''                            bmpAb & vbTab & _
''                            bmpDa & vbTab & _
''                            bmpDb & vbTab & _
''                            bmpLdCost & vbTab & _
''                            bmpConstCost & vbNewLine

    Dim costUnits
    Dim costAdjUnitCosts
    Dim costVolTypes
    Dim costNumUnits
    
    Dim volCostTotal As Double: volCostTotal = 0#
    Dim areaCost As Double: areaCost = 0#
    Dim linearCost As Double: linearCost = 0#
    Dim numCost As Double: numCost = 0#
    Dim consCost As Double: consCost = 0#
    Dim percCost As Double: percCost = 0#
    Dim volCostMedia As Double: volCostMedia = 0#
    Dim volCostUnderDrain As Double: volCostUnderDrain = 0#
           
    Dim adjCost As Double
    Dim i As Integer
           
    Dim costExps
    Dim costExpDict As Scripting.Dictionary
    Set costExpDict = New Scripting.Dictionary
    
    If pBMPDetailDict.Exists("CostComponents") Then
        costUnits = Split(pBMPDetailDict.Item("CostUnits"), ";", , vbTextCompare)
        costAdjUnitCosts = Split(pBMPDetailDict.Item("CostAdjUnitCosts"), ";")
        
        costVolTypes = Split(pBMPDetailDict.Item("CostVolTypes"), ";", , vbTextCompare)
        costNumUnits = Split(pBMPDetailDict.Item("CostNumUnits"), ";", , vbTextCompare)
        costExps = Split(pBMPDetailDict.Item("costExponents"), ";", , vbTextCompare)
        
        For i = 0 To UBound(costUnits)
            adjCost = CDbl(costAdjUnitCosts(i))
            costExpDict.Item(UCase(costUnits(i)) & "_" & CInt(costVolTypes(i))) = costExps(i)
            
            Select Case UCase(costUnits(i))
                Case "FEET"
                    linearCost = linearCost + adjCost
                Case "SQUARE FEET"
                    areaCost = areaCost + adjCost
                Case "CUBIC FEET"
                    If CInt(costVolTypes(i)) = COST_VOLUME_TYPE_TOTAL Then
                        volCostTotal = volCostTotal + adjCost
                    ElseIf CInt(costVolTypes(i)) = COST_VOLUME_TYPE_MEDIA Then
                        volCostMedia = volCostMedia + adjCost
                    Else
                        volCostUnderDrain = volCostUnderDrain + adjCost
                    End If
                Case "PERCENTAGE"
                    percCost = percCost + adjCost
                Case "CONSTANT"
                    consCost = consCost + adjCost
                Case "PER UNIT"
                    'numCost = numCost + (adjCost * CInt(costNumUnits(i)))
                    consCost = consCost + (adjCost * CInt(costNumUnits(i)))
            End Select
        Next
    End If
        
    
'    StrCard805BMPCost = StrCard805BMPCost & _
'                            strBMPindex & vbTab & _
'                            linearCost & vbTab & _
'                            areaCost & vbTab & _
'                            volCostTotal & vbTab & _
'                            volCostMedia & vbTab & _
'                            volCostUnderDrain & vbTab & _
'                            numCost & vbTab & _
'                            consCost & vbTab & _
'                            percCost & vbNewLine
'    StrCard805BMPCost = StrCard805BMPCost & _
'                            strBMPindex & vbTab & _
'                            linearCost & vbTab & _
'                            areaCost & vbTab & _
'                            volCostTotal & vbTab & _
'                            volCostMedia & vbTab & _
'                            volCostUnderDrain & vbTab & _
'                            consCost & vbTab & _
'                            percCost & vbNewLine
    Dim ftExp As String
    ftExp = 1
    If costExpDict.Exists("FEET_1") Then ftExp = costExpDict.Item("FEET_1")
    Dim sqftExp As String
    sqftExp = 1
    If costExpDict.Exists("SQUARE FEET_1") Then sqftExp = costExpDict.Item("SQUARE FEET_1")
    Dim cubftExp1 As String, cubftExp2 As String, cubftExp3 As String
    cubftExp1 = 1: cubftExp2 = 1: cubftExp3 = 1
    If costExpDict.Exists("CUBIC FEET_" & COST_VOLUME_TYPE_TOTAL) Then cubftExp1 = costExpDict.Item("CUBIC FEET_" & COST_VOLUME_TYPE_TOTAL)
    If costExpDict.Exists("CUBIC FEET_" & COST_VOLUME_TYPE_MEDIA) Then cubftExp2 = costExpDict.Item("CUBIC FEET_" & COST_VOLUME_TYPE_MEDIA)
    If costExpDict.Exists("CUBIC FEET_" & COST_VOLUME_TYPE_UNDERDRAIN) Then cubftExp3 = costExpDict.Item("CUBIC FEET_" & COST_VOLUME_TYPE_UNDERDRAIN)
    
    StrCard805BMPCost = StrCard805BMPCost & _
                            strBMPindex & vbTab & _
                            linearCost & vbTab & _
                            areaCost & vbTab & _
                            volCostTotal & vbTab & _
                            volCostMedia & vbTab & _
                            volCostUnderDrain & vbTab & _
                            consCost & vbTab & _
                            percCost & vbTab & _
                            ftExp & vbTab & _
                            sqftExp & vbTab & _
                            cubftExp1 & vbTab & _
                            cubftExp2 & vbTab & _
                            cubftExp3 & vbNewLine
      
    GoTo CleanUp
ShowError:
    MsgBox "GetBMPCosts: " & Err.description
CleanUp:
End Sub


'******************************************************************************
'Subroutine: GetOptimizationParameters
'Author:     Mira Chokshi
'Purpose:    Get the parameters that will be used for optimization.
'            The optimized parameter will have lower value, higher value and
'            increment value.
'*****************************************************************************
Private Sub GetOptimizationParameters(strBMPindex As String) 'iBMPindex As Integer)
On Error GoTo ShowError

    Dim lengthMin As Double     'Adjustable parameters
    Dim lengthMax As Double
    Dim lengthStep As Double
    Dim widthMin As Double
    Dim widthMax As Double
    Dim widthStep As Double
    Dim weirHMin As Double
    Dim weirHMax As Double
    Dim weirHStep As Double
    
    Dim soilDMin As Double
    Dim soilDMax As Double
    Dim soilDStep As Double
    
        
    If pBMPDetailDict.Exists("BLengthOptimized") Then
        If (CBool(pBMPDetailDict.Item("BLengthOptimized"))) Then
            lengthMin = CDbl(pBMPDetailDict.Item("MinBasinLength"))
            lengthMax = CDbl(pBMPDetailDict.Item("MaxBasinLength"))
            lengthStep = CDbl(pBMPDetailDict.Item("BasinLengthIncr"))
           
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "Length" & vbTab & _
                                            lengthMin & vbTab & _
                                            lengthMax & vbTab & _
                                            lengthStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("BWidthOptimized") Then
        If (CBool(pBMPDetailDict.Item("BWidthOptimized"))) Then
            widthMin = CDbl(pBMPDetailDict.Item("MinBasinWidth"))
            widthMax = CDbl(pBMPDetailDict.Item("MaxBasinWidth"))
            widthStep = CDbl(pBMPDetailDict.Item("BasinWidthIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "Width" & vbTab & _
                                            widthMin & vbTab & _
                                            widthMax & vbTab & _
                                            widthStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("WHeightOptimized") Then
        If (CBool(pBMPDetailDict.Item("WHeightOptimized"))) Then
            weirHMin = CDbl(pBMPDetailDict.Item("MinWeirHeight"))
            weirHMax = CDbl(pBMPDetailDict.Item("MaxWeirHeight"))
            weirHStep = CDbl(pBMPDetailDict.Item("WeirHeightIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "WeirH" & vbTab & _
                                            weirHMin & vbTab & _
                                            weirHMax & vbTab & _
                                            weirHStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("SoilDOptimized") Then
        If (CBool(pBMPDetailDict.Item("SoilDOptimized"))) Then
            soilDMin = CDbl(pBMPDetailDict.Item("MinSoilDepth"))
            soilDMax = CDbl(pBMPDetailDict.Item("MaxSoilDepth"))
            soilDStep = CDbl(pBMPDetailDict.Item("SoilDepthIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "SDepth" & vbTab & _
                                            soilDMin & vbTab & _
                                            soilDMax & vbTab & _
                                            soilDStep & vbNewLine
        End If
     End If
     
    'Additional parameters -- Sabu Paul
    Dim depthMin As Double
    Dim depthMax As Double
    Dim depthStep As Double
    If pBMPDetailDict.Exists("BLengthBOptimized") Then
        If (CBool(pBMPDetailDict.Item("BLengthBOptimized"))) Then
            lengthMin = CDbl(pBMPDetailDict.Item("MinBasinBLength"))
            lengthMax = CDbl(pBMPDetailDict.Item("MaxBasinBLength"))
            lengthStep = CDbl(pBMPDetailDict.Item("BasinBLengthIncr"))
           
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "Length" & vbTab & _
                                            lengthMin & vbTab & _
                                            lengthMax & vbTab & _
                                            lengthStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("BWidthBOptimized") Then
        If (CBool(pBMPDetailDict.Item("BWidthBOptimized"))) Then
            widthMin = CDbl(pBMPDetailDict.Item("MinBasinBWidth"))
            widthMax = CDbl(pBMPDetailDict.Item("MaxBasinBWidth"))
            widthStep = CDbl(pBMPDetailDict.Item("BasinBWidthIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "Width" & vbTab & _
                                            widthMin & vbTab & _
                                            widthMax & vbTab & _
                                            widthStep & vbNewLine
        End If
    End If
    If pBMPDetailDict.Exists("BDepthBOptimized") Then
        If (CBool(pBMPDetailDict.Item("BDepthBOptimized"))) Then
            depthMin = CDbl(pBMPDetailDict.Item("MinBasinBDepth"))
            depthMax = CDbl(pBMPDetailDict.Item("MaxBasinBDepth"))
            depthStep = CDbl(pBMPDetailDict.Item("BasinBDepthIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "Width" & vbTab & _
                                            depthMin & vbTab & _
                                            depthMax & vbTab & _
                                            depthStep & vbNewLine
        End If
    End If
    
    Dim numMin As Double
    Dim numMax As Double
    Dim numStep As Double
    
    If pBMPDetailDict.Exists("NumUnitsOptimized") Then
        If (CBool(pBMPDetailDict.Item("NumUnitsOptimized"))) Then
            numMin = CDbl(pBMPDetailDict.Item("MinNumUnits"))
            numMax = CDbl(pBMPDetailDict.Item("MaxNumUnits"))
            numStep = CDbl(pBMPDetailDict.Item("NumUnitsIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "NUMUNIT" & vbTab & _
                                            numMin & vbTab & _
                                            numMax & vbTab & _
                                            numStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("NumUnitsOptimizedB") Then
        If (CBool(pBMPDetailDict.Item("NumUnitsOptimizedB"))) Then
            numMin = CDbl(pBMPDetailDict.Item("MinNumUnitsB"))
            numMax = CDbl(pBMPDetailDict.Item("MaxNumUnitsB"))
            numStep = CDbl(pBMPDetailDict.Item("NumUnitsIncrB"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "NUMUNIT" & vbTab & _
                                            numMin & vbTab & _
                                            numMax & vbTab & _
                                            numStep & vbNewLine
        End If
    End If
    
    If pBMPDetailDict.Exists("ImpDisOptimized") Then           'for Area BMP
        If (CBool(pBMPDetailDict.Item("ImpDisOptimized"))) Then
            numMin = CDbl(pBMPDetailDict.Item("MinImpDis"))
            numMax = CDbl(pBMPDetailDict.Item("MaxImpDis"))
            numStep = CDbl(pBMPDetailDict.Item("ImpDisIncr"))
            
            StrCard810AdjustParameter = StrCard810AdjustParameter & _
                                            strBMPindex & vbTab & _
                                            "DCIA" & vbTab & _
                                            numMin & vbTab & _
                                            numMax & vbTab & _
                                            numStep & vbNewLine
        End If
    End If

               
     GoTo CleanUp
ShowError:
    MsgBox "GetOptimizationParameters: " & Err.description
CleanUp:
End Sub


'******************************************************************************
'Subroutine: GetGrowthIndex
'Author:     Mira Chokshi
'Purpose:    Get the growth index parameters for each bmp. Growth index varies
'            each month, hence has monthly values.
'*****************************************************************************
Private Sub GetGrowthIndex(strBMPindex As String) 'iBMPindex As Integer)
On Error GoTo ShowError
    Dim strGrowthIndex As String    'Growth Index
    Dim iGrowthIndex() As String
    Dim i As Integer
    
    strGrowthIndex = CStr(pBMPDetailDict.Item("GrowthIndex"))
    iGrowthIndex = Split(strGrowthIndex, ";")
    
    If (UBound(iGrowthIndex) > LBound(iGrowthIndex)) Then
        StrCard745BMPGrowthIndex = StrCard745BMPGrowthIndex & strBMPindex & vbTab
        For i = LBound(iGrowthIndex) To UBound(iGrowthIndex)
            StrCard745BMPGrowthIndex = StrCard745BMPGrowthIndex & iGrowthIndex(i) & vbTab
        Next
        StrCard745BMPGrowthIndex = StrCard745BMPGrowthIndex & vbNewLine
    End If
    
    GoTo CleanUp
    
ShowError:
    MsgBox "GetGrowthIndex: " & Err.description
CleanUp:
    
End Sub



'******************************************************************************
'Subroutine: GetLandTypeRouting
'Author:     Mira Chokshi
'Purpose:    Call the subroutine to summarize the land use area over the subbasin.
'            It returns a dictionary with keys as subbasin id, and values as
'            dictionary of land use code and area mapping. Determine the bmps each
'            land use flows into and output to a string variable.
'*****************************************************************************
Private Sub GetLandTypeRouting()
On Error GoTo ShowError
    StrCard790LandTypeRouting = ""
    Set gBMPDrainAreaDict = New Scripting.Dictionary
    
    If (gSubWaterLandUseDict Is Nothing) Then
        'Call FindAndConvertWatershedFeatureLayerToRaster
        Call ComputeLanduseAreaForEachSubBasin 'Generate dictionary --  gSubWaterLandUseDict
    End If

    'Call the function to do land use reclassification for subwatershed
    'Call ComputeLanduseAreaForEachSubBasin  'Generate dictionary --  gSubWaterLandUseDict
    
    'Call the function to get the basin to bmp routing and save into a dictionary
    Call DefineBasinToBMPDictionary  'Generate -- pBasinToBMPDict
    
    Dim pTable As iTable
    Dim pTableName As String
    If gExternalSimulation Then
        pTableName = "TSAssigns"
    Else
        pTableName = "LUReclass"
    End If
    
    Set pTable = GetInputDataTable(pTableName) '"LUReclass")
    If (pTable Is Nothing) Then
        MsgBox pTableName & " table not found."
        Exit Sub
    End If
    Dim pCursor As ICursor
    Dim pRow As iRow
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    Dim pLucode As Integer
    Dim iLuCode As Long
    iLuCode = pTable.FindField("LUCode")
    
    Dim pLanduseDict As Scripting.Dictionary
    Set pLanduseDict = CreateObject("Scripting.Dictionary")
    'Get subwater and landtype dictionary
    Dim pSubWaterKeys
    pSubWaterKeys = gSubWaterLandUseDict.keys
    Dim pLandTypeKeys
    Dim iGroupArea As Double
    Dim iXpos As Long
    Dim iYpos As Long
    Dim idsBMP As Integer
    Dim iCount As Integer
    iCount = 1
    Dim iGroup, iSubWater, iLandType As Integer
    Dim iDrainageBasin As Integer
    Dim pLandUseAreaDict As Scripting.Dictionary
    
    'Get the pervious/impervious percentage - Sabu Paul, Sep 17, 2004
    Dim iPercentageCode As Long
    iPercentageCode = pTable.FindField("Percentage")
    Dim pPercentage As Double
    Dim pLUAreaPerGroup As Double
    
    Dim pAggBmpLuDict As Scripting.Dictionary
    Dim pAggBMPArea As Double
    Dim pAggLuArea As Double
'    Dim agg_bmp_cat_Ids, catIndex As Integer
'    agg_bmp_cat_Ids = Array("A", "B", "C", "D", "E")
    Dim bmpCardStr As String
    Dim catIndex As Integer

''    'Loop over each landtype group
''    For iGroup = 1 To pMaxLandTypeGroupID
''        'Get each subwater, find the area for each land type
''        For iSubWater = 0 To gSubWaterLandUseDict.Count - 1
''            iDrainageBasin = pSubWaterKeys(iSubWater)
''
''            Set pLandUseAreaDict = Nothing
''            Set pLandUseAreaDict = gSubWaterLandUseDict.Item(iDrainageBasin)
''
''            'Get the draining BMP from the basin to bmp dictionary
''            idsBMP = pBasinToBMPDict.Item(iDrainageBasin)
''
''            'Query the subwatershed for same group land use type
''            pQueryFilter.WhereClause = "LUGroupID = " & iGroup
''            Set pCursor = pTable.Search(pQueryFilter, True)
''            Set pRow = pCursor.NextRow
''            'Find all landtype subgroup and sum their area
''            iGroupArea = 0
''            Do While Not pRow Is Nothing
''                pLUCode = pRow.value(iLUCode)
''                pPercentage = pRow.value(iPercentageCode)
''                pLandTypeKeys = pLandUseAreaDict.keys
''                For iLandType = 0 To pLandUseAreaDict.Count - 1
''                    If (pLandTypeKeys(iLandType) = pLUCode) Then
''                        pLUAreaPerGroup = 0
''                        If (pLandUseAreaDict.Exists(pLandTypeKeys(iLandType))) Then
''                             pLUAreaPerGroup = pLandUseAreaDict.Item(pLandTypeKeys(iLandType))
''                        End If
''                        iGroupArea = iGroupArea + ((pLUAreaPerGroup * pPercentage) / 4046.856) ' area in acres -- Sabu Paul, Aug 24, 2004
''                    End If
''                Next
''                'Go to next landtype
''                Set pRow = pCursor.NextRow
''            Loop
''            Set pRow = Nothing
''            Set pCursor = Nothing
''            If (iGroupArea > 0) Then
''                StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
''                                            Format(iGroupArea, "#.##") & vbTab & idsBMP & vbNewLine
''                iCount = iCount + 1
''            End If
''
''            'Check to see if it is aggregage BMP
''
''            'Check for Aggregate BMP distribution
''            Set pAggBmpLuDict = Get_Agg_BMP_Lu_Distrib(idsBMP)
''
''            gBMPDrainAreaDict.Item(idsBMP) = gBMPDrainAreaDict.Item(idsBMP) + CDbl(pHruDictTmp.Item(iHru))
''
''        Next
''    Next
    
    Dim pSQAcreFactor As Double
    If gMetersPerUnit = 0# Then GetMetersPerLinearUnit
    pSQAcreFactor = gMetersPerUnit * gMetersPerUnit * 0.0002471044       'sq meter to acre conversion

    Dim strAggBMPIds As String
    Dim listAggBmpIds
    Dim strLuDist As String, listLuDist
    
    Dim lineNumber As String
    
    'Get each subwater, find the area for each land type
    For iSubWater = 0 To gSubWaterLandUseDict.Count - 1
        lineNumber = "SubWater = " & iSubWater & " 1 "
        
        iDrainageBasin = pSubWaterKeys(iSubWater)
            
        Set pLandUseAreaDict = Nothing
        Set pLandUseAreaDict = gSubWaterLandUseDict.Item(iDrainageBasin)
        
        'Get the draining BMP from the basin to bmp dictionary
        idsBMP = pBasinToBMPDict.Item(iDrainageBasin)
        
        Set pAggBmpLuDict = Nothing
        'Check to see if it is aggregage BMP
        If gAggBMPFlagDict.Exists(idsBMP) Then
            If gAggBMPFlagDict.Item(idsBMP) Then
                'Check for Aggregate BMP distribution
                Set pAggBmpLuDict = Get_Agg_BMP_Lu_Distrib(idsBMP)
                If pAggBmpLuDict.Exists(-98) Then
                    strAggBMPIds = pAggBmpLuDict.Item(-98)(1)
                    listAggBmpIds = Split(strAggBMPIds, ",")
                End If
            End If
        End If
        
        lineNumber = "SubWater = " & iSubWater & " 2 "
        'Loop over each landtype group
        For iGroup = 1 To pMaxLandTypeGroupID
                                
            'Query the subwatershed for same group land use type
            pQueryFilter.WhereClause = "LUGroupID = " & iGroup
            Set pCursor = pTable.Search(pQueryFilter, True)
            Set pRow = pCursor.NextRow
            'Find all landtype subgroup and sum their area
            iGroupArea = 0
            Do While Not pRow Is Nothing
                pLucode = pRow.value(iLuCode)
                pPercentage = pRow.value(iPercentageCode)
                pLandTypeKeys = pLandUseAreaDict.keys
                For iLandType = 0 To pLandUseAreaDict.Count - 1
                    If (pLandTypeKeys(iLandType) = pLucode) Then
                        pLUAreaPerGroup = 0
                        If (pLandUseAreaDict.Exists(pLandTypeKeys(iLandType))) Then
                             pLUAreaPerGroup = pLandUseAreaDict.Item(pLandTypeKeys(iLandType))
                        End If
                        iGroupArea = iGroupArea + ((pLUAreaPerGroup * pPercentage) * pSQAcreFactor) ' area in acres -- Sabu Paul, Aug 24, 2004
                    End If
                Next
                'Go to next landtype
                Set pRow = pCursor.NextRow
            Loop
            Set pRow = Nothing
            Set pCursor = Nothing
            lineNumber = "SubWater = " & iSubWater & " 3 "
            If (iGroupArea > 0) Then
                If pAggBmpLuDict Is Nothing Then
                    lineNumber = "SubWater = " & iSubWater & " 4 Group = " & iGroup
                    'StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
                    '                            Format(iGroupArea, "#.##") & vbTab & idsBMP & vbNewLine
                    'StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
                                                FormatNumber(iGroupArea, 2) & vbTab & idsBMP & vbNewLine
                    StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
                                                Format$(iGroupArea, "##0.000") & vbTab & idsBMP & vbNewLine
                    
                    gBMPDrainAreaDict.Item(idsBMP) = gBMPDrainAreaDict.Item(idsBMP) + iGroupArea
                    lineNumber = "SubWater = " & iSubWater & " 4_1 Group = " & iGroup
                    iCount = iCount + 1
                Else
                    lineNumber = "SubWater = " & iSubWater & " 5 Group = " & iGroup
                    If pAggBmpLuDict.Exists(CInt(iGroup)) Then
                        lineNumber = "SubWater = " & iSubWater & " 5_1 Group = " & iGroup
                        'pAggBMPArea = pAggBmpLuDict.Item(CInt(iGroup))(0)
'                        For catIndex = 0 To UBound(agg_bmp_cat_Ids)
'
'                            lineNumber = "SubWater = " & iSubWater & " 5_2 Group = " & iGroup
'                            bmpCardStr = idsBMP & "_" & agg_bmp_cat_Ids(catIndex)
'                            pAggLuArea = CDbl(pAggBmpLuDict.Item(CInt(iGroup))(catIndex + 1)) / 100 * iGroupArea
'                            lineNumber = "SubWater = " & iSubWater & " 5_3 Group = " & iGroup
'                            If pAggLuArea > 0 Then
'                                lineNumber = "SubWater = " & iSubWater & " 5_4 Group = " & iGroup
'                                StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
'                                                    Format(pAggLuArea, "#.##") & vbTab & bmpCardStr & vbNewLine
'                                gBMPDrainAreaDict.Item(bmpCardStr) = gBMPDrainAreaDict.Item(bmpCardStr) + pAggLuArea
'                                lineNumber = "SubWater = " & iSubWater & " 5_5 Group = " & iGroup
'                                iCount = iCount + 1
'                            End If
'
'                        Next
                      
                        If pAggBmpLuDict.Exists(CInt(iGroup)) Then
                            strLuDist = pAggBmpLuDict.Item(CInt(iGroup))(1)
                            listLuDist = Split(strLuDist, ",")
                            For catIndex = 0 To UBound(listLuDist)
                                If CInt(listAggBmpIds(catIndex)) = 0 Then
                                    bmpCardStr = idsBMP
                                Else
                                    bmpCardStr = idsBMP & "_" & listAggBmpIds(catIndex)
                                End If
                                pAggLuArea = CDbl(listLuDist(catIndex)) / 100 * iGroupArea
                                If pAggLuArea > 0 Then
                                    lineNumber = "SubWater = " & iSubWater & " 5_4 Group = " & iGroup
'                                    StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
'                                                        Format(pAggLuArea, "#.##") & vbTab & bmpCardStr & vbNewLine
                                    
'                                    StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
                                                        FormatNumber(pAggLuArea, 2) & vbTab & bmpCardStr & vbNewLine
                                    
                                    StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & iGroup & vbTab & _
                                                        Format$(pAggLuArea, "##0.000") & vbTab & bmpCardStr & vbNewLine
                                                        
                                    gBMPDrainAreaDict.Item(bmpCardStr) = gBMPDrainAreaDict.Item(bmpCardStr) + pAggLuArea
                                    lineNumber = "SubWater = " & iSubWater & " 5_5 Group = " & iGroup
                                    iCount = iCount + 1
                                End If
                            Next
                        End If
                    End If
                End If
                
            End If
            
            

        Next
    Next
    
    lineNumber = "SubWater = " & iSubWater & " 6 = "
    

    '*** Add additional details of external timeseries file
    '*** Remove this part (table "ExternalTS") from card790
    'Dim LUgroupCount As Integer
    'LUgroupCount = pMaxLandTypeGroupID
    'Set pTable = GetInputDataTable("ExternalTS")
    'If Not (pTable Is Nothing) Then
    '    Set pCursor = pTable.Search(Nothing, True)
    '    Set pRow = pCursor.NextRow
    '    Dim iBMPID As Long
    '    iBMPID = pCursor.FindField("BMPID")
    '    Dim iMultiplier As Long
    '    iMultiplier = pCursor.FindField("Multiplier")
    '    Do While Not pRow Is Nothing
    '            LUgroupCount = LUgroupCount + 1
    '            StrCard790LandTypeRouting = StrCard790LandTypeRouting & iCount & vbTab & LUgroupCount & vbTab & _
                                            "0.00" & vbTab & pRow.value(iBMPID) & vbNewLine
    '            iCount = iCount + 1
    '        Set pRow = pCursor.NextRow
    '    Loop
    'End If
    
    'GetLandTypeRouting = StrCard790LandTypeRouting
    
    GoTo CleanUp
    
ShowError:
    'MsgBox lineNumber
    gHasInFileError = True
    MsgBox "GetLandTypeRouting: " & Err.description
CleanUp:
    Set pTable = Nothing
    Set pCursor = Nothing
    Set pRow = Nothing
    Set pQueryFilter = Nothing
    Set pLanduseDict = Nothing
    Set pSubWaterKeys = Nothing
    Set pLandTypeKeys = Nothing
    Set pLandUseAreaDict = Nothing
End Sub


'******************************************************************************
'Subroutine: DefineBasinToBMPDictionary
'Author:     Mira Chokshi
'Purpose:    Get the BMP Identifier into which a watershed flows into.
'*****************************************************************************
Private Sub DefineBasinToBMPDictionary()
On Error GoTo ShowError
    'Define variables for watershed feature layer
    Dim pWatershedFLayer As IFeatureLayer
    Set pWatershedFLayer = GetInputFeatureLayer("Watershed")
    If (pWatershedFLayer Is Nothing) Then
        MsgBox "Watershed feature layer not found."
        Exit Sub
    End If
    Dim pWatershedFClass As IFeatureClass
    Set pWatershedFClass = pWatershedFLayer.FeatureClass
    Dim pWatershedFCursor As IFeatureCursor
    Dim pWatershedFeature As IFeature
    'Define variables to access watershed feature layer fields
    Dim iBasinIDFld As Long
    iBasinIDFld = pWatershedFClass.FindField("ID")
    Dim iBmpIdFld As Long
    iBmpIdFld = pWatershedFClass.FindField("BMPID")
    'Iterate over different watershed feature layer features
    Set pWatershedFCursor = pWatershedFClass.Search(Nothing, True)
    Set pWatershedFeature = pWatershedFCursor.NextFeature
    
    'Initialize the dictionary for subbasin to bmp routing
    Set pBasinToBMPDict = CreateObject("Scripting.Dictionary")
    Do While Not pWatershedFeature Is Nothing
        'KEY = Basin ID, VALUE = Draining BMP
        pBasinToBMPDict.add pWatershedFeature.value(iBasinIDFld), pWatershedFeature.value(iBmpIdFld)
        Set pWatershedFeature = pWatershedFCursor.NextFeature
    Loop
    GoTo CleanUp

ShowError:
    MsgBox "DefineBasinToBMPDictionary: " & Err.description
CleanUp:
    Set pWatershedFLayer = Nothing
    Set pWatershedFClass = Nothing
    Set pWatershedFCursor = Nothing
    Set pWatershedFeature = Nothing
End Sub

Private Function SetAggBMPNetwork(curBmpID As Integer, usBmpId As Integer, _
                    dsBmpId As Integer, pBMPOutletType As Integer, Optional bUsOnly As Boolean) As String
On Error GoTo ShowError
    Dim pAgg_BMP_Cat_Dict As Scripting.Dictionary
'    Dim agg_bmp_cats
'    agg_bmp_cats = Array("On-Site Interception", "On-Site Treatment", "Routing Attenuation", "Regional Storage/Treatment")
'    Dim agg_bmp_cat_Ids
'    agg_bmp_cat_Ids = Array("A", "B", "C", "D", "E")
    
    
    Dim catIndex As Integer, catIndex2 As Integer
    Dim bmpCardStr As String, bmpCardStr2 As String
    Dim bDsFound As Boolean, bUsBmpSet As Boolean
    Set pAgg_BMP_Cat_Dict = Nothing
    bUsBmpSet = False
    
    Set pAgg_BMP_Cat_Dict = GetAggBMPTypes(curBmpID) 'Get_OnMap_BMP_Categories(curBmpID)
    If pAgg_BMP_Cat_Dict Is Nothing Then Exit Function
    
    Dim strAggBMPIds As String
    Dim strAggBMPDsIds As String
    Dim listAggBmpIds, listAggDsIds
    Dim strLuDist As String, listLuDist
    Dim pAggBmpLuDict As Scripting.Dictionary
    Set pAggBmpLuDict = Get_Agg_BMP_Lu_Distrib(curBmpID)
    If pAggBmpLuDict.Exists(-98) And pAggBmpLuDict.Exists(-99) Then
        strAggBMPIds = pAggBmpLuDict.Item(-98)(1)
        listAggBmpIds = Split(strAggBMPIds, ",")
        strAggBMPDsIds = pAggBmpLuDict.Item(-99)(1)
        listAggDsIds = Split(strAggBMPDsIds, ",")
    End If
        
'    Dim upstreamBMP As String
'    If pAggBmpLuDict.Exists(-97) Then
'        upstreamBMP = pAggBmpLuDict.Item(-97)(1)
'    End If
    Dim resultString As String
    resultString = ""
    'Set the routing network for aggregate BMP A-B-C-D
''    For catIndex = 0 To UBound(agg_bmp_cats)
''        bDsFound = False
''        If pAgg_BMP_Cat_Dict.Exists(CStr(agg_bmp_cats(catIndex))) Then
''            bmpCardStr = curBmpID & "_" & agg_bmp_cat_Ids(catIndex)
''
''            If Not bUsBmpSet Then
''                bUsBmpSet = True
''                If usBmpId <> 0 Then
''                    resultString = resultString & _
''                                  usBmpId & vbTab & _
''                                  pBMPOutletType & vbTab & _
''                                  bmpCardStr & vbNewLine
''                End If
''            End If
''
''            If Not bUsOnly Then
''                For catIndex2 = catIndex + 1 To UBound(agg_bmp_cats)
''                    If pAgg_BMP_Cat_Dict.Exists(CStr(agg_bmp_cats(catIndex2))) Then
''                        bmpCardStr2 = curBmpID & "_" & agg_bmp_cat_Ids(catIndex2)
''                        resultString = resultString & _
''                                          bmpCardStr & vbTab & _
''                                          pBMPOutletType & vbTab & _
''                                          bmpCardStr2 & vbNewLine
''                        bDsFound = True
''                        Exit For
''                    End If
''                Next
''
''                If Not bDsFound Then
''                    resultString = resultString & _
''                                      bmpCardStr & vbTab & _
''                                      pBMPOutletType & vbTab & _
''                                      curBmpID & "_E" & vbNewLine
''                End If
''            End If
''        End If
''    Next
    
    'if this bmp is the most upstream
'    If Not bUsBmpSet Then
'        bUsBmpSet = True
        If usBmpId <> 0 Then
            resultString = resultString & _
                          usBmpId & vbTab & _
                          pBMPOutletType & vbTab & _
                          curBmpID & vbNewLine  'curBmpID & "_0" & vbNewLine
        End If
'    End If
    
    For catIndex = 0 To UBound(listAggBmpIds) - 1
        'bmpCardStr = curBmpID & "_" & listAggBmpIds(catIndex)
        
        If Not bUsOnly Then
            'resultString = resultString & _
                        curBmpID & "_" & listAggBmpIds(catIndex) & vbTab & _
                        pBMPOutletType & vbTab & _
                        curBmpID & "_" & listAggDsIds(catIndex) & vbNewLine
            If CInt(listAggBmpIds(catIndex)) = 0 Then
                resultString = resultString & curBmpID
            Else
                resultString = resultString & curBmpID & "_" & listAggBmpIds(catIndex)
            End If
            
            If CInt(listAggDsIds(catIndex)) = 0 Then
                resultString = resultString & vbTab & _
                        pBMPOutletType & vbTab & _
                        curBmpID & vbNewLine
            Else
                resultString = resultString & vbTab & _
                        pBMPOutletType & vbTab & _
                        curBmpID & "_" & listAggDsIds(catIndex) & vbNewLine
            End If
            
        End If
    Next

    'bmpCardStr = curBmpID & "_E"
    bmpCardStr = curBmpID '& "_0"
'    If Not bUsBmpSet Then
'        If usBmpId <> 0 Then
'            resultString = resultString & _
'                              usBmpId & vbTab & _
'                              pBMPOutletType & vbTab & _
'                              bmpCardStr & vbNewLine
'        End If
'    End If
    If Not bUsOnly Then
        'Set the routing network for aggregate BMP E- outside
        resultString = resultString & _
                        bmpCardStr & vbTab & _
                        pBMPOutletType & vbTab & _
                        dsBmpId & vbNewLine
    End If
    
    SetAggBMPNetwork = resultString
    Exit Function
ShowError:
    MsgBox "Error in SetAggBMPNetwork:" & Err.description
End Function
'******************************************************************************
'Subroutine: GetBMPNetworkRouting
'Author:     Mira Chokshi
'Purpose:    Get the bmp network. Each bmp has three outlets. If all three flow
'            flow to same d/s bmp, the outlet_type is 1, else outlet_type is
'            2, 3, 4 with individual d/s bmp specified. This information is
'            already present in BMPNetwork table.
'*****************************************************************************
Private Function GetBMPNetworkRouting()
On Error GoTo ShowError

    Dim pTable As iTable
    Set pTable = GetInputDataTable("BMPNetwork")
    If (pTable Is Nothing) Then
        MsgBox "BMPNetwork table not found."
        Exit Function
    End If
    Dim pFeatureLayer As IFeatureLayer
    Set pFeatureLayer = GetInputFeatureLayer("Conduits")
    If (pFeatureLayer Is Nothing) Then
        'MsgBox "Conduits feature layer not found."
        Exit Function
    End If
    
    Dim pCursor As ICursor
    Set pCursor = pTable.Search(Nothing, True)
    Dim pRow As iRow
    Set pRow = pCursor.NextRow
    'Get field index
    Dim iID As Long
    iID = pCursor.FindField("ID")
    Dim iOUTLETTYPE As Long
    iOUTLETTYPE = pCursor.FindField("OutletType")
    Dim iDSID As Long
    iDSID = pCursor.FindField("DSID")
    'Get field values
    Dim StrCard795BMPNetworkRouting As String
    Dim pBMPID As Integer
    Dim pBMPdsId As Integer
    Dim pBMPOutletType As Integer
    'Iterate over bmpnetwork table to get values for bmp's flowing to downstream
    
    Dim bIsAggBMP As Boolean
    Do While Not pRow Is Nothing
        pBMPID = pRow.value(iID)
        pBMPOutletType = pRow.value(iOUTLETTYPE)
        pBMPdsId = pRow.value(iDSID)
        bIsAggBMP = False
        If gAggBMPFlagDict.Exists(pBMPID) Then
            If gAggBMPFlagDict.Item(pBMPID) Then bIsAggBMP = True
        End If
            
        If (pBMPdsId = 0) Then
            If Not bIsAggBMP Then
                StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & _
                                              pBMPID & vbTab & _
                                              pBMPOutletType & vbTab & _
                                              pBMPdsId & vbNewLine
            Else
                StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & SetAggBMPNetwork(pBMPID, 0, pBMPdsId, pBMPOutletType)
            End If
        End If
        'Move to next table record
        Set pRow = pCursor.NextRow
    Loop
    
    'Iterate over conduits feature table to get bmp-->conduit-->bmp network
    Dim pQueryFilter As IQueryFilter
    Dim pFeatureclass As IFeatureClass
    Set pFeatureclass = pFeatureLayer.FeatureClass
    Dim pFeatureCursor As IFeatureCursor
    Set pFeatureCursor = pFeatureclass.Search(Nothing, True)
    Dim pFeature As IFeature
    Dim iFROM As Long
    iFROM = pFeatureclass.FindField("CFROM")
    Dim iTO As Long
    iTO = pFeatureclass.FindField("CTO")
    Dim iCID As Long
    iCID = pFeatureclass.FindField("ID")
    Dim iOUTTYPE As Long
    iOUTTYPE = pFeatureclass.FindField("OUTLETTYPE")
    Dim pFROM As Integer
    Dim pTO As Integer
    Dim pCID As Integer
    Dim pOUTLETType As Integer
    
    'Loop over all features
    Set pFeature = pFeatureCursor.NextFeature
    Do While Not pFeature Is Nothing
        pFROM = pFeature.value(iFROM)
        pTO = pFeature.value(iTO)
        pCID = pFeature.value(iCID)
        pOUTLETType = pFeature.value(iOUTTYPE)
        
        bIsAggBMP = False
        If gAggBMPFlagDict.Exists(pFROM) Then
            If gAggBMPFlagDict.Item(pFROM) Then bIsAggBMP = True
        End If
        
        'First line for u/s bmp to conduit, next line for conduit to d/s bmp
        If Not bIsAggBMP Then
            StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & _
                                          pFROM & vbTab & _
                                          pOUTLETType & vbTab & _
                                          pCID & vbNewLine
        Else
            StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & SetAggBMPNetwork(pFROM, 0, pCID, pOUTLETType)
        End If
        
        bIsAggBMP = False
        If gAggBMPFlagDict.Exists(pTO) Then
            If gAggBMPFlagDict.Item(pTO) Then bIsAggBMP = True
        End If
        
        pOUTLETType = 3  'Channel, since conduit is a channel
        If Not bIsAggBMP Then
            'Line for conduit to d/s bmp
            StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & _
                                          pCID & vbTab & _
                                          pOUTLETType & vbTab & _
                                          pTO & vbNewLine
        Else
            StrCard795BMPNetworkRouting = StrCard795BMPNetworkRouting & SetAggBMPNetwork(pTO, pCID, 0, pBMPOutletType, True)
        End If
        'Move to next conduit
        Set pFeature = pFeatureCursor.NextFeature
    Loop
    
    
    'Return the bmp network routing
    GetBMPNetworkRouting = StrCard795BMPNetworkRouting
GoTo CleanUp
    
ShowError:
    MsgBox "GetBMPNetworkRouting: " & Err.description
CleanUp:
    Set pRow = Nothing
    Set pCursor = Nothing
    Set pTable = Nothing
End Function


'******************************************************************************
'Subroutine: GetAssessPointOptimizationDetails
'Author:     Mira Chokshi
'Purpose:    Get details for assessment point - optimization parameters
'*****************************************************************************
Private Function GetAssessPointOptimizationDetails()
On Error GoTo ShowError

    Dim pTable As iTable
    Set pTable = GetInputDataTable("OptimizationDetail")
    If (pTable Is Nothing) Then
        MsgBox "OptimizationDetail table not found."
        Exit Function
    End If
    
    Dim pQueryFilter As IQueryFilter
    Set pQueryFilter = New QueryFilter
    pQueryFilter.WhereClause = "ID = 0"
    Dim pCursor As ICursor
    Set pCursor = pTable.Search(pQueryFilter, True)

    'Get field index
    Dim iID As Long
    iID = pCursor.FindField("ID")
    Dim iPropName As Long
    iPropName = pCursor.FindField("PropName")
    Dim iPropValue As Long
    iPropValue = pCursor.FindField("PropValue")
    Dim pRow As iRow
    Set pRow = pCursor.NextRow
    
    '*** Option header
    'StrCard800OptimizationControls = "c Technique Option    StopDelta   MaxRuns  NumBest" & vbNewLine
    StrCard800OptimizationControls = "c Technique" & vbTab & "Option" & vbTab & "StopDelta" & vbTab & "MaxRuns" & vbTab & "NumBest" & vbNewLine
    Dim pTechniqueVal As Integer 'New parameter - June 18, 2007
    Dim pOptionVal As Integer
'    Dim pCostLimit As Double
    Dim pStopDelta As Double
    Dim pMaxRunTime As Double
    Dim pNumBest As Integer
'    Dim pNumBreak As Integer
'    pNumBreak = 0
    
    Do While Not pRow Is Nothing
        Select Case (pRow.value(iPropName))
            Case "Technique":
                    pTechniqueVal = CDbl(pRow.value(iPropValue))
            Case "Option":
                    pOptionVal = CInt(pRow.value(iPropValue))
'            Case "CostLimit":
'                    pCostLimit = CDbl(pRow.value(iPropValue))
            Case "StopDelta":
                    pStopDelta = CDbl(pRow.value(iPropValue))
            Case "MaxRunTime":
                    pMaxRunTime = CDbl(pRow.value(iPropValue))
            Case "NumBest":
                    pNumBest = CDbl(pRow.value(iPropValue))
        End Select
        Set pRow = pCursor.NextRow
    Loop
    
    Set pCursor = pTable.Search(pQueryFilter, True)
    Set pRow = pCursor.NextRow
    Do While Not pRow Is Nothing
        Select Case (pRow.value(iPropName))
            Case "MaxRunTime":
                'If pTechniqueVal = 1 Then
                '    Dim NumDV As Integer
                '    NumDV = GetNumofOptimizations
                '    pMaxRunTime = CStr(80 * NumDV)
                'Else
                    pMaxRunTime = CDbl(pRow.value(iPropValue))
                'End If
        End Select
        Set pRow = pCursor.NextRow
    Loop
    StrCard800OptimizationControls = StrCard800OptimizationControls & pTechniqueVal & vbTab & _
                                        pOptionVal & vbTab & _
                                        pStopDelta & vbTab & _
                                        pMaxRunTime & vbTab & _
                                        pNumBest & vbNewLine
    '*** BMP Assessment optimization header
    'StrCard815Assess = "c   BMPSITE     FactorGroup     FactorType     FactorVal1    FactorVal2   CalcMode    TargetVal1    TargetVal2    FactorName" & vbNewLine
    StrCard815Assess = "c BMPSITE" & vbTab & "FactorGroup" & vbTab & "FactorType" & vbTab & "FactorVal1" & vbTab & "FactorVal2" & vbTab & "CalcMode" & vbTab & "TargetVal1" & vbTab & "TargetVal2" & vbTab & "FactorName" & vbNewLine
    
    pQueryFilter.WhereClause = "ID <> 0"
    Set pCursor = pTable.Search(pQueryFilter, True)
    Set pRow = pCursor.NextRow
    Dim pIDString As String
    Dim pParamString As String
    Do While Not (pRow Is Nothing)
        pIDString = pRow.value(iID) & vbTab
        pParamString = pRow.value(iPropValue)
        pParamString = Replace(pParamString, ",", vbTab)
        StrCard815Assess = StrCard815Assess & pIDString & pParamString & vbNewLine
        Set pRow = pCursor.NextRow
    Loop
GoTo CleanUp
    
ShowError:
    MsgBox "GetAssessPointOptimizationDetails: " & Err.description
CleanUp:
    Set pRow = Nothing
    Set pCursor = Nothing
    Set pTable = Nothing
End Function

Private Sub WriteCard900()
On Error GoTo ShowError
    pFile.WriteLine ("c900 Vegetative Filter Strip Simulation control card")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c N             = Number of nodes in the domain(must be an odd number for a quadratic finite element solution")
    pFile.WriteLine ("c THETAW        = Time-weight factor for the Crank-Nicholson solution (0.5 recommended)")
    pFile.WriteLine ("c CR            = Courant number for the calculation of time step from 0.5 - 0.8 (recommended).")
    pFile.WriteLine ("c MAXITER       = Maximum number of iterations alowed in the Picard loop (integer).")
    pFile.WriteLine ("c NPOL          = Number of nodal points over each element")
    pFile.WriteLine ("c IELOUT        = Flag to output elemental information - (1-Feeeback, 0-No Feedback - SUSTAIN Always -0 )")
    pFile.WriteLine ("c KPG           = Flag to choose the Petrov-Galerkin solution (1) or regular finite element (0)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c N THETAW CR MAXITER NPOL IELOUT KPG")
    pFile.WriteLine ("c N" & vbTab & "THETAW" & vbTab & "CR" & vbTab & "MAXITER" & vbTab & "NPOL" & vbTab & "IELOUT" & vbTab & "KPG")
    pFile.Write GetVFSSimulationOptions
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 900:" & Err.description
End Sub

Private Sub WriteCard901()
On Error GoTo ShowError
    pFile.WriteLine ("c901 Vegetative Filter Strip (VFS) Overland Flow Solution")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier")
    pFile.WriteLine ("c LABEL         = Filter strip name for identification")
    pFile.WriteLine ("c FWIDTH        = Width of the filter strip (ft)")
    pFile.WriteLine ("c VL            = Lenght of the filter strip (ft)")
    pFile.WriteLine ("c NPROP         = Number of segments with different surface properties (slope or roughness)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID LABEL FWIDTH VL NPROP")
    pFile.WriteLine ("c VFSID" & vbTab & "LABEL" & vbTab & "FWIDTH" & vbTab & "VL" & vbTab & "NPROP")
    pFile.Write StrCard901_VFS_Dim
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 901:" & Err.description
End Sub

Private Sub WriteCard902()
On Error GoTo ShowError
    pFile.WriteLine ("c902 VFS Segment Details")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID           = Unique Filter strip identifier")
    pFile.WriteLine ("c SegID           = Segment identifier")
    pFile.WriteLine ("c SX(SegID)       = Horizontal distance from the beginning on the filter (ft)")
    pFile.WriteLine ("c RNA(SegID)      = Mannings roughness for each segment (s.ft^(-1/3))")
    pFile.WriteLine ("c SOA(SegID)      = Slope at each segment")
    pFile.WriteLine ("c                  SX, RNA, SOA will be for SegID = 1 to NPROP")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID SEGID SX(SegID) RNA(SegID) SOA(SegID)")
    pFile.WriteLine ("c VFSID" & vbTab & "SEGID" & vbTab & "SX(SegID)" & vbTab & "RNA(SegID)" & vbTab & "SOA(SegID)")
    pFile.Write StrCard902_VFS_SegDetails
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 902:" & Err.description
End Sub

Private Sub WriteCard903()
On Error GoTo ShowError
    pFile.WriteLine ("c903 VFS Soil Properties")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID           = Unique Filter strip identifier ")
    pFile.WriteLine ("c VKS             = Saturated hydraulic conductivity (in/hr)")
    pFile.WriteLine ("c SAV             = Green-Ampts average suction at wet front(ft)")
    pFile.WriteLine ("c OS              = Saturated soil-water content(ft^3/ft^3)")
    pFile.WriteLine ("c OI              = Initial soil-water content (ft^3/ft^3)")
    pFile.WriteLine ("c SM              = Maximum surface storage (ft)")
    pFile.WriteLine ("c SCHK            = Relative distance from de upper filter edge where the check for ponding conditions is made (i.e. 1= end filter, 0.5= mid point, 0= beginning)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID VKS SAV OS OI SM SCHK ")
    pFile.WriteLine ("c VFSID" & vbTab & "VKS" & vbTab & "SAV" & vbTab & "OS" & vbTab & "OI" & vbTab & "SM" & vbTab & "SCHK")
    pFile.Write StrCard903_VFS_SoilProps
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 903:" & Err.description
End Sub

Private Sub WriteCard904()
On Error GoTo ShowError
    pFile.WriteLine ("c904 VFS Buffer Properties for Sediment Filtration Model")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier ")
    pFile.WriteLine ("c SS            = Spacing of the filter media elements (in)")
    pFile.WriteLine ("c VN            = Filter media Manning's n (0.012 for cylindrical media) (s.in^(-1/3))")
    pFile.WriteLine ("c H             = Filter media height (ft)")
    pFile.WriteLine ("c VN2           = Bare surface Manning's n for sediment inundated area and overland flow (s.ft^(-1/3))")
    pFile.WriteLine ("c ICO           = Flag to feedback the change in slope and surface roughness at the sediment wedge for each time step (0= no feedback; 1= feedback - SUSTAIN always 0)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    pFile.Write StrCard904_VFS_Buf_Sed
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 904:" & Err.description
End Sub

Private Sub WriteCard905()
On Error GoTo ShowError
    pFile.WriteLine ("c905 VFS Sediment Properties for Sediment Filtration Model")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier ")
    pFile.WriteLine ("c NPARTi        = Incoming sediment particle class according to the USDA (1975) particle classes for 3 soil size classes(where i=1,2,3)")
    pFile.WriteLine ("c COARSEi       = % of particles from incoming sediment with diameter > 0.0037 cm (coarse fraction that will be routed through wedge) (unit fraction, i.e. 100% = 1.0) for 3 soil size classes(where i=1,2,3)")
    pFile.WriteLine ("c PORi          = Porosity of deposited sediment (unit fraction, i.e. 43.4% = 0.434) for 3 soil size classes(where i=1,2,3)")
    pFile.WriteLine ("c DPi           = Sediment particle size, diameter(in), required only if NPART=7 for 3 soil size classes(where i=1,2,3)")
    pFile.WriteLine ("c SGi           = Sediment particle density (lb/ft^3), required only if NPART=7 for 3 soil size classes(where i=1,2,3)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    pFile.Write StrCard905_VFS_Sed_Filt
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 905:" & Err.description
End Sub

Private Sub WriteCard906()
On Error GoTo ShowError

    pFile.WriteLine ("c906 VFS Pollutant Sediment Fraction for the pollutants")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier")
    pFile.WriteLine ("c QUALSEDFRACi  = Sediment fraction for pollutant i (hr^-1)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID  QUALSEDFRAC1  QUALSEDFRAC2 ... QUALSEDFRACN")
    pFile.WriteLine ("c VFSID" & vbTab & "QUALSEDFRAC1" & vbTab & "QUALSEDFRAC2" & vbTab & "..." & vbTab & "QUALSEDFRACN")
    pFile.Write StrCard906_VFS_Sed_Fracion
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 906:" & Err.description
End Sub

Private Sub WriteCard907()
On Error GoTo ShowError
    pFile.WriteLine ("c907 VFS Pollutant First Order Decay Factors in the adsorbed fraction")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier ")
    pFile.WriteLine ("c QUALDECAYi    = First-order decay rate for pollutant i (hr^-1)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID  QUALDECAY1  QUALDECAY2 ... QUALDECAYN")
    pFile.WriteLine ("c VFSID" & vbTab & "QUALDECAY1" & vbTab & "QUALDECAY2" & vbTab & "..." & vbTab & "QUALDECAYN")
    pFile.Write StrCard907_VFS_FO_Adsorbed
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 907:" & Err.description
End Sub
Private Sub WriteCard908()
On Error GoTo ShowError
    pFile.WriteLine ("c908 VFS Pollutant First Order Decay Factors in the dissolved fraction")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID         = Unique Filter strip identifier ")
    pFile.WriteLine ("c QUALDECAYi    = First-order decay rate for pollutant i (hr^-1)")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID  QUALDECAY1  QUALDECAY2 ... QUALDECAYN")
    pFile.WriteLine ("c VFSID" & vbTab & "QUALDECAY1" & vbTab & "QUALDECAY2" & vbTab & "..." & vbTab & "QUALDECAYN")
    pFile.Write StrCard908_VFS_FO_Dissolved
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 908:" & Err.description
End Sub
Private Sub WriteCard909()
On Error GoTo ShowError
    pFile.WriteLine ("c909 VFS Pollutant Temperature Correction Factors in the adsorbed fraction")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID          = Unique Filter strip identifier ")
    pFile.WriteLine ("c TEMPCORRi      = Temperature correction for pollutant i ")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID  TEMPCORR1  TEMPCORR2 ... TEMPCORRN")
    pFile.WriteLine ("c VFSID" & vbTab & "TEMPCORR1" & vbTab & "TEMPCORR2" & vbTab & "..." & vbTab & "TEMPCORRN")
    pFile.Write StrCard909_VFS_TC_Adsorbed
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 909:" & Err.description
End Sub
Private Sub WriteCard910()
On Error GoTo ShowError
    pFile.WriteLine ("c910 VFS Pollutant Temperature Correction Factors in the dissolved fraction")
    pFile.WriteLine ("c")
    pFile.WriteLine ("c VFSID           = Unique Filter strip identifier ")
    pFile.WriteLine ("c TEMPCORRi       = Temperature correction for pollutant i ")
    pFile.WriteLine ("c              Where i = 1 to N (N = Number of QUAL from TIMESERIES FILES)")
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    'pFile.WriteLine ("c VFSID  TEMPCORR1  TEMPCORR2 ... TEMPCORRN")
    pFile.WriteLine ("c VFSID" & vbTab & "TEMPCORR1" & vbTab & "TEMPCORR2" & vbTab & "..." & vbTab & "TEMPCORRN")
  
    pFile.Write StrCard910_VFS_TC_Dissolved
    pFile.WriteLine ("c--------------------------------------------------------------------------------------------")
    Exit Sub
ShowError:
    gHasInFileError = True
    MsgBox "Card 910:" & Err.description
End Sub

Public Sub GetBMPDetails_Classes(ByVal iBMP As Integer, pBMPType As String, _
        pXpos As Double, pYpos As Double, pTotalDrainageArea As Double, Optional strBmpCardId As String)
    
On Error GoTo ShowError
    'ByVal pBmpDetailDict As Scripting.Dictionary, _

    If strBmpCardId = "" Then strBmpCardId = CStr(iBMP)
    
    Dim pBMPClass As String
    pBMPClass = Trim(pBMPDetailDict.Item("BMPClass"))
    Dim isAssessPoint As Boolean
    isAssessPoint = CBool(pBMPDetailDict.Item("isAssessmentPoint"))
    'Get BMP A/B Dimension Parameters
    If (pBMPClass = "A") Then
        GetBMPParametersA strBmpCardId, pXpos, pYpos, pTotalDrainageArea     '715,725,730
        GetInitialMoisture strBmpCardId, pXpos, pYpos, pTotalDrainageArea    '747 (new card)
        'Get BMP Decay Factors for bmp's
        GetBMPDecayFactors strBmpCardId                                      '765
        'Get BMP Percent Removal Factor
        GetBMPPercentRemovalFactors strBmpCardId                             '770
        
    ElseIf (pBMPClass = "B") Then
        GetBMPParametersB strBmpCardId, pXpos, pYpos, pTotalDrainageArea     '715, 735
        GetInitialMoisture strBmpCardId, pXpos, pYpos, pTotalDrainageArea    '747 (new card)
        'Get BMP Decay Factors for bmp's
        GetBMPDecayFactors strBmpCardId                                      '765
        'Get BMP Percent Removal Factor
        GetBMPPercentRemovalFactors strBmpCardId                             '770
    'Get Dummy BMP Dimension Parameters
    ElseIf (pBMPClass = "X") Then
        GetDummyBMPParameters strBmpCardId, pXpos, pYpos, pTotalDrainageArea '715
    End If
    
    If (pBMPClass = "E") Then
        GetBMPParametersEF strBmpCardId, 0, 0, pTotalDrainageArea, True      '715 record K, C factors
        GetBMPDecayFactors strBmpCardId                                      '765
        GetDiscoImperParameters strBmpCardId                                 '762 (new card)
    End If
    
    If (pBMPClass = "F") Then
        GetBMPParametersEF strBmpCardId, 0, 0, pTotalDrainageArea, False     '715
        GetBMPDecayFactors strBmpCardId                                      '765
        GetInitialMoisture strBmpCardId, 0, 0, pTotalDrainageArea            '747 (new card)
        GetBufferstripParameters strBmpCardId                                '763 (new card)
    End If


    If (pBMPClass <> "X" And pBMPType <> "VirtualOutlet") Then
        If (pBMPClass <> "E") Then
            GetSoilIndex strBmpCardId     'Get soil properties               '740 'record K, C factiors
            GetGrowthIndex strBmpCardId  'Get growth index                   '745
        End If
        GetBMPCosts strBmpCardId      'Get the Cost parameters               '805
        GetOptimizationParameters strBmpCardId 'iBMP                         '810       'Get Adjustable parameter range and increment
        GetBMPSedimentParameters strBmpCardId                                '775, 780, 785, 786
    End If
    Exit Sub
    
ShowError:
    gHasInFileError = True
    MsgBox "Error in GetBMPDetails_Classes: " & Err.description
End Sub


Public Sub GetBMPSedimentParameters(iBMPIndex As String)
On Error GoTo ShowError
    Dim bedWidth As Double
    Dim bedDepth As Double
    Dim bedPorosity As Double
    Dim SAND_FRAC As Double
    Dim SILT_FRAC As Double
    Dim CLAY_FRAC As Double
    Dim sandDiameter As Double
    Dim sandVelocity As Double
    Dim sandDensity As Double
    Dim sandCoeff As Double
    Dim sandExponent As Double
    Dim siltDiameter As Double
    Dim siltVelocity As Double
    Dim siltDensity As Double
    Dim siltTaucd As Double
    Dim siltTaucs As Double
    Dim siltM As Double
   
    bedWidth = pBMPDetailDict.Item("Bed width")
    bedDepth = pBMPDetailDict.Item("Bed depth")
    bedPorosity = pBMPDetailDict.Item("Porosity")
    SAND_FRAC = pBMPDetailDict.Item("Sand fraction")
    SILT_FRAC = pBMPDetailDict.Item("Silt fraction")
    CLAY_FRAC = pBMPDetailDict.Item("Clay fraction")
    sandDiameter = pBMPDetailDict.Item("Sand effective diameter")
    sandVelocity = pBMPDetailDict.Item("Sand velocity")
    sandDensity = pBMPDetailDict.Item("Sand density")
    sandCoeff = pBMPDetailDict.Item("Sand coefficient")
    sandExponent = pBMPDetailDict.Item("Sand exponent")
    siltDiameter = pBMPDetailDict.Item("Silt effective diameter")
    siltVelocity = pBMPDetailDict.Item("Silt velocity")
    siltDensity = pBMPDetailDict.Item("Silt density")
    siltTaucd = pBMPDetailDict.Item("Silt Deposition stress")
    siltTaucs = pBMPDetailDict.Item("Silt Scour stress")
    siltM = pBMPDetailDict.Item("Silt Erodibility")
    
    Dim clayDiameter As Double
    Dim clayVelocity As Double
    Dim clayDensity As Double
    Dim clayTaucd As Double
    Dim clayTaucs As Double
    Dim clayM As Double
    
    clayDiameter = pBMPDetailDict.Item("Clay effective diameter")
    clayVelocity = pBMPDetailDict.Item("Clay velocity")
    clayDensity = pBMPDetailDict.Item("Clay density")
    clayTaucd = pBMPDetailDict.Item("Clay Deposition stress")
    clayTaucs = pBMPDetailDict.Item("Clay Scour stress")
    clayM = pBMPDetailDict.Item("Clay Erodibility")
    
    'StrCard775Sediment = StrCard775Sediment & _
                        iBMPIndex & vbTab & _
                        bedWidth & vbTab & _
                        bedDepth & vbTab & _
                        bedPorosity & vbTab & _
                        SAND_FRAC & vbTab & _
                        SILT_FRAC & vbTab & _
                        CLAY_FRAC & vbNewLine
                        
     StrCard775Sediment = StrCard775Sediment & _
                        iBMPIndex & vbTab & _
                        bedWidth & vbTab & _
                        bedDepth & vbTab & _
                        bedPorosity & vbNewLine
                       
    StrCard780SandTransport = StrCard780SandTransport & _
                        iBMPIndex & vbTab & _
                        sandDiameter & vbTab & _
                        sandVelocity & vbTab & _
                        sandDensity & vbTab & _
                        sandCoeff & vbTab & _
                        sandExponent & vbNewLine
    StrCard785SiltTransport = StrCard785SiltTransport & _
                        iBMPIndex & vbTab & _
                        siltDiameter & vbTab & _
                        siltVelocity & vbTab & _
                        siltDensity & vbTab & _
                        siltTaucd & vbTab & _
                        siltTaucs & vbTab & _
                        siltM & vbNewLine
    StrCard786ClayTransport = StrCard786ClayTransport & _
                        iBMPIndex & vbTab & _
                        clayDiameter & vbTab & _
                        clayVelocity & vbTab & _
                        clayDensity & vbTab & _
                        clayTaucd & vbTab & _
                        clayTaucs & vbTab & _
                        clayM & vbNewLine
    Exit Sub
ShowError:
    MsgBox "Error in GetBMPSedimentParameters: " & Err.description
End Sub
