// BMPRunner.cpp: implementation of the CBMPRunner class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Global.h"
#include "LandUse.h"
#include "BMPSite.h"
#include "SiteLandUse.h"
#include "SitePointSource.h"
#include "Sediment.h"
#include "BMPData.h"
#include "BMPRunner.h"
#include <math.h>
#include <afxtempl.h>
#include "ProgressWnd.h"	 
#include "BMPOptimizer.h"	 
#include "Aquifer.h"	 
#include "Pump.h"
#include "Ftable.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define SMALLNUM	1.0E-15

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBMPRunner::CBMPRunner()
{
	optcounter = 0;
	outcounter = 0;
	nMaxRun = 0;	
	pBMPData = NULL;
	pWndProgress = NULL;	
	time_i = COleDateTime(1890,1,1,0,0,0);
	fp = NULL;
	fp_bin = NULL;
	nRUN_BATCH = 0;
}

CBMPRunner::CBMPRunner(CBMPData* bmpData)
{
	optcounter = 0;
	outcounter = 0;
	nMaxRun = 0;	
	pBMPData = bmpData;
	pWndProgress = NULL;	
	time_i = COleDateTime(1890,1,1,0,0,0);
	fp = NULL;
	fp_bin = NULL;
	nRUN_BATCH = 0;
}

CBMPRunner::~CBMPRunner()
{

}

void CBMPRunner::bmp_a(int nInfiltMethod,int nInfil_Index,bool underdrain_on,int LANDtimestep,int BMPtimestep,int npeople,int ddays,
					   int releasetype,int weirtype,int& counter,double oinflow,double BMParea,double orifice_dia,double orificeheight,
					   double orificecoef,double weirwidth,double weirheight,double weirangle,double cisternoutflow,double soildepth,
					   double soilporosity,double finalf,double vegparma,double holtpar,double udfinalf,double udsoildepth,
					   double udsoilporosity,double FC,double WP,double ETrate,double& AET,double& perc,double& ovolume,double& ostage,
					   double& infilt,double& orifice,double& weir,double& osa,double& ostorage,double& udout,double& seepage,double& pump,
					   BMP_A* pBMP,int nFtableFlag,CFtable* pFtable)
{
	double bot_area = BMParea;		// bottom area (ft2)
	double BMPdepth = weirheight;	// ft

	if (bot_area <= 0 || BMPdepth <= 0)
		return;

	// number of intervals per inflow timestep
	int nivl = int(LANDtimestep / (double)BMPtimestep + 0.5);
	
	float tStep = BMPtimestep * 60.0;	//sec/BMPivl
	double delts = LANDtimestep * 60.;			// sec/LANDivl
	double deltd = LANDtimestep / (60. * 24.);	// day/LANDivl
	double g = 32.2;			// ft/sec2
	double r__1 = 0., r__2 = 0., r__3 = 0.,h = 0., f = 0.;
	double nweir = 0.,norifice = 0.,nudout = 0.,ninfilt = 0.,nperc = 0.0;
	double nseepage = 0.,nAET = 0.,nAETsurf = 0.,nAETudrain = 0.;
	double tempweir = 0.,temporifice = 0.,tempudout = 0.,tempinfilt = 0.,tempperc = 0.;
	double tempseepage = 0.,tempAET = 0.;
	double orifice_area = (pi/4.)*(orifice_dia*orifice_dia);  //ft2 

	double npump = 0.,temppump = 0.;
	int pump_flag = pBMP->m_pPumpControl.m_nPumpFlag;	//pump option (0-off, 1-on)
	bool pump_ON = pBMP->m_pPumpControl.m_bIsPumping;	//pumping status
	double depth_on = pBMP->m_pPumpControl.m_lfDepth_ON;	//ft
	double depth_off = pBMP->m_pPumpControl.m_lfDepth_OFF;	//ft

	//calculate the maximum surface area (ft2)
	double s_area_max = 0.,sur_area = 0.;	 

	//use Ftable if provided
	if (nFtableFlag > 0 && pFtable != NULL)
		s_area_max = pFtable->Depth_lookup(BMPdepth,pFtable->m_lfArea);
	else
		s_area_max = bot_area;	//box structure 

	// at the begining of the timestep
	double AWC      = soildepth*s_area_max*(FC - WP);//available water capacity for plants only (ft3)
	double nvolume  = ovolume;				//available water in the water column (ft3)
	double nsa      = osa/12.0*s_area_max;	//available pores of the soil column (ft3)
	double nstorage = ostorage/12.0*bot_area;//available pores of the underdrain column (ft3)
	double nstage   = ostage;				//ft

	//use Ftable if provided
	if (nFtableFlag > 0 && pFtable != NULL)
		sur_area = pFtable->Depth_lookup(nstage,pFtable->m_lfArea);
	else
		sur_area = bot_area;	

	if (sur_area > s_area_max)
		sur_area = s_area_max;

	//calculate the max available space in soil column (in)
	double nsamax = soildepth * s_area_max * (soilporosity - WP); 

	if (nsamax < SMALLNUM)
		nsamax = 0.0;

	//calculate the max available space in underdrain column (in)
	double nstoragemax = udsoildepth*bot_area*udsoilporosity;

	if (nstoragemax < SMALLNUM)
		nstoragemax = 0.0;

	//update the volume
	double nvolume2 = nvolume + oinflow;	// ft3/LANDivl

	//update the stage
	if (nFtableFlag > 0 && pFtable != NULL)
	{
		//check if volume exceeds the maximum volume given in Ftable
		int nLast = pFtable->m_nRecords-1;
		if(nvolume2 > pFtable->m_lfVolume[nLast])
			nstage = pFtable->m_lfDepth[nLast]+
			(nvolume2 - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
		else
			//get the water depth from the given Ftable
			nstage = pFtable->Volume_lookup(nvolume2,pFtable->m_lfDepth);
	}
	else
	{
		nstage = nvolume2 / bot_area;  //ft
	}

//=========================================================
// Step 1: Calculate pump withdrawal from the water column 
//=========================================================
	if (nstage > SMALLNUM)			
	{
		//check if the pump option is ON
		if (pump_flag == 1)
		{
			//check if the pump is ON
			if (pump_ON == true)
			{
				//check the water depth (ft)
				if (nstage > depth_off)
				{
					//get the pump outflow
					double lfFlow = 0.0;
					if (pBMP->m_pPumpCurve != NULL)
						lfFlow = pBMP->m_pPumpCurve->table_lookup(nstage);	//cfs
					npump = lfFlow * (double)LANDtimestep * 60.0; //ft3
					// --- do not allow reverse flow through pump
					if (npump < 0) 
						npump = 0.0;
					else
						npump = min(npump, nvolume2);
				}
				else
				{
					//turn off the pump
					pump_ON = false;
					
					// no pump outflow
					npump = 0.0;
				}
			}
			else
			{
				//check the water depth
				if (nstage >= depth_on)
				{
					//turn on the pump
					pump_ON = true;

					//get the pump outflow (cfs)
					double lfFlow = 0.0;
					if (pBMP->m_pPumpCurve != NULL)
						lfFlow = pBMP->m_pPumpCurve->table_lookup(nstage);
					npump = lfFlow * (double)LANDtimestep * 60.0; //ft3
					// --- do not allow reverse flow through pump
					if (npump < 0) 
						npump = 0.0;
					else
						npump = min(npump, nvolume2);
				}
				else
				{
					// no pump outflow
					npump = 0.0;
				}
			}
		}
		else
		{
			// no pump outflow
			npump = 0.0;
		}
	}
	else
	{
		// no pump outflow
		npump = 0.0;
	}

	double pumpoutflow = npump / nivl;	//ft3/LANDivl to ft3/BMPivl
	double inflow = oinflow / nivl;		//ft3/LANDivl to ft3/BMPivl

	if (inflow > SMALLNUM)
		counter = 0;
	else
		counter++;

	//for infiltration use the maximum surface area
	float finflow = inflow/(s_area_max*tStep);//ft/sec
	float fpump = pumpoutflow/(s_area_max*tStep);//ft/sec

	for (int i=0; i<nivl; i++) 
	{
		ninfilt  = 0.0;		// surface infiltration (ft3/ivl)
		norifice = 0.0;		// orifice outflow (ft3/ivl)
		nweir    = 0.0;		// weir outflow (ft3/ivl)
		nudout   = 0.0;		// underdrain outflow (ft3/ivl)
		nperc    = 0.0;		// percolation to underdrain storage (ft3/ivl)
		nAETsurf = 0.0;		// actual evapotranspiration from the surface (ft3/ivl) 
		nAET     = 0.0;		// actual evapotranspiration from the subsurface (ft3/ivl)
		nAETudrain = 0.0;	// actual evapotranspiration from the underdrain (ft3/ivl)
		nseepage = 0.0;		// deep percolation to groundwater storage (ft3/ivl)
		npump = pumpoutflow;// pump outflow (ft3/ivl)

		//find depth before any inflow or outflow
		float fdepth = nvolume/s_area_max;//ft
		
		//update the volume
		nvolume += inflow;	// ft3

		//remove the pump outflow
		if (npump > 0)
		{
			// RECALCULATE VOLUME
			if (npump > nvolume)
				npump = nvolume;

			//update the volume
			nvolume -= npump;	//ft3
		}
		
		//update the stage
		if (nFtableFlag > 0 && pFtable != NULL)
		{
			//check if volume exceeds the maximum volume given in Ftable
			int nLast = pFtable->m_nRecords-1;
			if(nvolume > pFtable->m_lfVolume[nLast])
				nstage = pFtable->m_lfDepth[nLast]+
				(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
			else
				//calculate the water depth from the given Ftable
				nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

			sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
			if (sur_area > s_area_max)
				sur_area = s_area_max;
		}
		else
		{
			nstage = nvolume / bot_area;  //ft
		}
				
//==============================================================
// Step 2: Calculate the evaporation loss from the water column
//==============================================================
		if (nstage > SMALLNUM)			
		{
			// calculate the evaporation from the surface (ft3/ivl)
			nAETsurf = min(nvolume, ETrate/12.0/nivl*sur_area);
				
			//update the volume
			nvolume -= nAETsurf;// ft3

			//update the stage
			if (nFtableFlag > 0 && pFtable != NULL)
			{
				//check if volume exceeds the maximum volume given in Ftable
				int nLast = pFtable->m_nRecords-1;
				if(nvolume > pFtable->m_lfVolume[nLast])
					nstage = pFtable->m_lfDepth[nLast]+
					(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
				else
					//calculate the water depth from the given Ftable
					nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

				sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
				if (sur_area > s_area_max)
					sur_area = s_area_max;
			}
			else
			{
				nstage = nvolume / bot_area;  //ft
			}

			if (nstage <= SMALLNUM)
			{
				nAETsurf += nvolume;//add to surface evaporation(ft3/ivl)
				nstage = 0.0;		//dry surface
				nvolume = 0.0;		//no surface water
				norifice = 0.0;		//no orifice outflow
				nweir    = 0.0;		//no weir outflow
			}

//=============================================================================
// Step 3: Calculate the infiltration from the water column to the soil column
//=============================================================================
			if (nstage > SMALLNUM)	
			{
				if (nInfiltMethod == 0)
				{
					float fEvap = (float)nAETsurf / ((float)s_area_max * tStep);//ft/sec
					float irate = finflow - fEvap - fpump;//ft/sec
					if (irate < 0)
					{
						irate = 0;
						fdepth = float(nvolume/s_area_max);//ft
					}
					
					// calculate infiltration rate per hour using Green-Ampt Method (ft/sec)
					f = infil_getInfil(nInfil_Index,GREEN_AMPT,tStep,irate,fdepth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else if (nInfiltMethod == 1)
				{
					float fEvap = (float)nAETsurf / ((float)s_area_max * tStep);//ft/sec
					float irate = finflow - fEvap - fpump;
					if (irate < 0)
					{
						irate = 0;
						fdepth = float(nvolume/s_area_max);//ft
					}

					// calculate infiltration rate per hour using Horton Method (ft/sec)
					f = infil_getInfil(nInfil_Index,HORTON,tStep,irate,fdepth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else
				{
					// calculate infiltration potential per hour using Holtan Method (in/hr)
					f = holtpar * vegparma * pow(nsa/s_area_max*12., 1.4) + finalf;
					
					// f is converted from in/hr to in/sec
					f /= 3600.;	

					// f is converted from in/sec to in/ivl
					f *= (double)tStep;
				}
				
				//infiltration is the min of potential infiltration or available water in basin
				ninfilt = min(f/12.0*s_area_max, nvolume);	//ft3/ivl

				//update the volume
				nvolume -= ninfilt;	// ft3

				//update the stage
				if (nFtableFlag > 0 && pFtable != NULL)
				{
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pFtable->m_nRecords-1;
					if(nvolume > pFtable->m_lfVolume[nLast])
						nstage = pFtable->m_lfDepth[nLast]+
						(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

					sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
					if (sur_area > s_area_max)
						sur_area = s_area_max;
				}
				else
				{
					nstage = nvolume / bot_area;  //ft
				}

				if (nstage <= SMALLNUM)
				{
					ninfilt += nvolume;	//add to infiltrated water(ft3/ivl)
					nstage = 0.0;		//dry surface
					nvolume = 0.0;		//no surface water
					norifice = 0.0;		//no orifice outflow
					nweir    = 0.0;		//no weir outflow
				}
			}
			else
			{
				nAETsurf += nvolume;//add small water to evaporated water (ft3/ivl)
				nstage = 0.0;		//dry surface
				nvolume = 0.0;		//no surface water storage
				ninfilt = 0.0;		//no water available for infiltration
				norifice = 0.0;		//no orifice outflow
				nweir    = 0.0;		//no weir outflow
			}
		} 
		else  
		{
			nAETsurf = nvolume;	//all water evaporated (ft3/ivl)
			nstage = 0.0;		//dry surface
			nvolume = 0.0;		//no surface storage
			ninfilt = 0.0;		//no infiltration
			norifice = 0.0;		//no orifice outflow
			nweir = 0.0;		//no weir outflow
		}

//====================================================================
// Step 4: Calculate the evapotranspiration loss from the soil column
//====================================================================
		if (nsamax > 0)
		{
			// extra water after saturation (ft3)
			double excesswater = ninfilt - nsa;
			
			//sub-surface evapotranspiration (ft3/ivl)
			nAET = ETrate/12.0/nivl*s_area_max - nAETsurf;
			nAET = max(0.0, nAET);

			if (nAET > (nsamax + excesswater))
				nAET = nsamax + excesswater;
				
//=================================================================================
// Step 5: Calculate the percolation from the soil column to the underdrain column
//=================================================================================
			if (excesswater >= 0) // saturation 
			{
				nperc = min(finalf/12.0/3600.* (double)tStep * s_area_max, excesswater + nsamax - nAET - AWC);//finalf is in/hr
				nperc = max(0.0, nperc);//ft3

				double excessinfilt = max(0.0, excesswater - nAET - nperc);
				ninfilt -= excessinfilt;//ft3	
				
				//update the volume
				nvolume += excessinfilt;// ft3

				//update the stage
				if (nFtableFlag > 0 && pFtable != NULL)
				{
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pFtable->m_nRecords-1;
					if(nvolume > pFtable->m_lfVolume[nLast])
						nstage = pFtable->m_lfDepth[nLast]+
						(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

					sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
					if (sur_area > s_area_max)
						sur_area = s_area_max;
				}
				else
				{
					nstage = nvolume / bot_area;  //ft
				}

				//update the soil available storage
				nsa = nsa - ninfilt + nperc + nAET;
			}
			else   // no saturation
			{
				if (-excesswater < nsamax)	// there is water to percolate
				{
					r__1 = (nsamax + excesswater) / nsamax;
					r__1 = (0.5 + 0.5 * r__1);
					r__1 = max(0.0, r__1);

					nperc = min(r__1 * finalf/12.0/3600.* (double)tStep *s_area_max, excesswater + nsamax - nAET - AWC);//finalf is in/hr	
					nperc = max(0.0, nperc);
					nsa   = nsa - ninfilt + nperc + nAET;
				}
				else
				{
					// no percolation to underdrain storage
					nperc = 0.0; 
					nsa   = nsamax;
				}
			}
		}
		else
		{
			// there is no storage capacity
			nAET  = 0.0;
			nsa   = 0.0;
			nperc = ninfilt; //ft3
		}

		nsa = max(nsa, 0.0);	
		nsa = min(nsa, nsamax);	

//==========================================================================
// Step 6: Calculate the evapotranspiration loss from the underdrain column
//==========================================================================
		if(nstoragemax > 0)
		{
			// extra water after saturation (ft3)
			double excesswater = nperc - nstorage;

			//under-drain evapotranspiration (ft3/ivl)
			nAETudrain = ETrate/12.0/nivl*bot_area - nAETsurf - nAET;
			nAETudrain = max(0.0, nAETudrain);

			if (nAETudrain > (nstoragemax + excesswater))
				nAETudrain = nstoragemax + excesswater;

			//Limit the ET rate for underdrain to be 50% of the normal ET rate
			double PETudrain = ETrate/12.0/nivl/2.*bot_area; //ft3
			if (nAETudrain > PETudrain)
				nAETudrain = PETudrain;

//===============================================================
// Step 7: Calculate the seepage loss from the underdrain column
//===============================================================
			if (excesswater >= 0) // saturation 
			{
				nseepage = min(udfinalf/12.0/3600.* (double)tStep *bot_area, excesswater + nstoragemax - nAETudrain);//udfinalf is in/hr
				nudout   = 0.0;

				if(underdrain_on)
				{
					nudout = max(0.0, excesswater - nseepage - nAETudrain);
				}
				else
				{
					double excessperc = max(0.0, excesswater - nseepage - nAETudrain);
					nperc -= excessperc;

					//check if soil column has space to hold this excess water
					if(nsa > 0)
					{
						//check the available space
						if(nsa < excessperc)
						{
							ninfilt -= (excessperc - nsa);
							nsa = 0.0;	// saturated (no more space)

							//update the volume
							nvolume += (excessperc - nsa);// ft3

							//update the stage
							if (nFtableFlag > 0 && pFtable != NULL)
							{
								//check if volume exceeds the maximum volume given in Ftable
								int nLast = pFtable->m_nRecords-1;
								if(nvolume > pFtable->m_lfVolume[nLast])
									nstage = pFtable->m_lfDepth[nLast]+
									(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
								else
									//calculate the water depth from the given Ftable
									nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

								sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
								if (sur_area > s_area_max)
									sur_area = s_area_max;
							}
							else
							{
								nstage = nvolume / bot_area;  //ft
							}
						}
						else
						{
							nsa -= excessperc;
						}
					}
					else
					{
						//return this excess water to water column
						ninfilt -= excessperc;

						//update the volume
						nvolume += excessperc;// ft3

						//update the stage
						if (nFtableFlag > 0 && pFtable != NULL)
						{
							//check if volume exceeds the maximum volume given in Ftable
							int nLast = pFtable->m_nRecords-1;
							if(nvolume > pFtable->m_lfVolume[nLast])
								nstage = pFtable->m_lfDepth[nLast]+
								(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
							else
								//calculate the water depth from the given Ftable
								nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

							sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
							if (sur_area > s_area_max)
								sur_area = s_area_max;
						}
						else
						{
							nstage = nvolume / bot_area;  //ft
						}
					}
				}

				nstorage = nstorage - nperc + nseepage + nudout + nAETudrain;
			}
			else	// no saturation (no underdrain outflow)
			{
				nudout = 0.0;

				if (-excesswater < nstoragemax)	// there is water to percolate
				{
					nseepage = min(udfinalf/12.0/3600.* (double)tStep *bot_area, excesswater + nstoragemax - nAETudrain);//udfinalf is in/hr
					nstorage = nstorage - nperc + nseepage + nAETudrain;
				}
				else
				{
					nseepage = 0.0; 
					nstorage = nstoragemax;
				}
			}
		}
		else
		{
			// no underdrain outflow
			nudout   = 0.0;
			nstorage = 0.0;
			nAETudrain = 0.0;
			nseepage = nperc;// percolate will go to groundwater storage
		}

		nstorage = max(nstorage, 0.0);	
		nstorage = min(nstorage, nstoragemax);
		
//======================================================================
// Step 8: Calculate the weir and channel outflow from the water column
//======================================================================
		if (nstage > SMALLNUM)			
		{
			//use Ftable if provided
			if (nFtableFlag == 1 && pFtable != NULL)
			{
				//calculate the orifice outflow rate from the given Ftable
				norifice = pFtable->Depth_lookup(nstage,pFtable->m_lfOrificeOutflow);
				norifice *= (double)tStep;	//ft3/ivl

				if (norifice > nvolume)
					norifice = nvolume;

				//update the volume (ft3/ivl)
				nvolume -= norifice;

				//calculate the weir outflow rate from the given Ftable
				nweir = pFtable->Depth_lookup(nstage,pFtable->m_lfWeirOutflow);
				nweir *= (double)tStep;	//ft3/ivl

				if (nweir > nvolume)
					nweir = nvolume;

				//update the volume (ft3/ivl)
				nvolume -= nweir;

				//check if volume exceeds the maximum volume given in Ftable
				int nLast = pFtable->m_nRecords-1;
				if(nvolume > pFtable->m_lfVolume[nLast])
					nstage = pFtable->m_lfDepth[nLast]+
					(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
				else
					//calculate the water depth from the given Ftable
					nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);
			}
			else
			{
				// CALCULATE ORIFICE FLOW
				if (nstage > orificeheight)
				{
					h = nstage - orificeheight;			// height of water above the orifice
					double norifice_max = h * bot_area;	// ft3

					//use Ftable if provided
					if (nFtableFlag == 2 && pFtable != NULL)
					{
						double volume1 = pFtable->Depth_lookup(nstage,pFtable->m_lfVolume);
						double volume2 = pFtable->Depth_lookup(orificeheight,pFtable->m_lfVolume);
						norifice_max = volume1 - volume2;
					}

					if (norifice_max > nvolume)
						norifice_max = nvolume;

					if (releasetype == 1) 
					{
						// cistern outflow
						norifice = cisternoutflow / nivl * npeople;	//ft3/ivl
						if (norifice > norifice_max)
							norifice = norifice_max;
					}
					else if (releasetype == 2)
					{
						if (counter > int(ddays / deltd + 0.5))
						{
							double frac = 1.0;
							if ( h < orifice_dia ) 
								frac = h / orifice_dia;
							
							if ( frac < 1.0 )
							{
								// --- case where orifice is only partly full
								 norifice = sqrt(2 * g * orifice_dia) * orifice_area * orificecoef * pow(frac, 1.5);	//cfs
							}
							else
							{
								// --- case where orifice is submerged
								norifice = sqrt(2 * g * h) * orifice_area  * orificecoef;  //cfs 	
							}

							norifice = norifice * (double)tStep;	//ft3/ivl
							if (norifice > norifice_max)
								norifice = norifice_max;
						}
						else
						{
							norifice = 0.0;
						}
					}
					else
					{
						double frac = 1.0;
						if ( h < orifice_dia ) 
							frac = h / orifice_dia;
						
						if ( frac < 1.0 )
						{
							// --- case where orifice is only partly full
							 norifice = sqrt(2 * g * orifice_dia) * orifice_area * orificecoef * pow(frac, 1.5);	//cfs
						}
						else
						{
							// --- case where orifice is submerged
							norifice = sqrt(2 * g * h) * orifice_area  * orificecoef;  //cfs 	
						}
						norifice = norifice * (double)tStep;	//ft3/ivl
						if (norifice > norifice_max)
							norifice = norifice_max;
					}
				}
				else
				{
					// no orifice outflow
					norifice = 0.0;
				}

				//update the volume (ft3/ivl)
				nvolume -= norifice;

				// CALCULATE WEIR OVERFLOW  
				if (nstage > weirheight)
				{
					h = nstage - weirheight;	// ft 

					double nweir_max = h * bot_area;	// ft3												

					//use Ftable if provided
					if (nFtableFlag == 2 && pFtable != NULL)
					{
						double volume1 = pFtable->Depth_lookup(nstage,pFtable->m_lfVolume);
						double volume2 = pFtable->Depth_lookup(weirheight,pFtable->m_lfVolume);
						nweir_max = volume1 - volume2;
					}

					if (nweir_max > nvolume)
						nweir_max = nvolume;

					if (weirtype == 1)
					{
						//Chow Equation, Replaces Linsey lookup table for weir coefficient
						double c_weir = 0.0;

						if (h / weirheight < 10)
							c_weir = 3.27 + 0.4 * h / weirheight;
						else
							c_weir = 5.68 * pow(1 + weirheight / h, 1.5);

						nweir = c_weir * weirwidth * sqrt(h * h * h);  //cfs 
						nweir = nweir * (double)tStep;	//ft3/ivl
						if (nweir > nweir_max)
							nweir = nweir_max;
					}
					else // weirtype == 2
					{
						weirangle = min(179.9, max(1.0, weirangle));
						r__1      = 3.141592653 * weirangle / 180.0;
						r__2      = tan(r__1 / 2);
						r__3      = pow(h, 2.5);
						nweir     = 2.4824 * r__2 * r__3;	//cfs 
						nweir     = nweir * (double)tStep;	//ft3/ivl
						if (nweir > nweir_max)
							nweir = nweir_max;
					}
				}
				else
				{
					// no weir outflow
					nweir = 0.0;
				}

				//update the volume (ft3/ivl)
				nvolume -= nweir;

				//update the stage
				if (nFtableFlag == 2 && pFtable != NULL)
				{
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pFtable->m_nRecords-1;
					if(nvolume > pFtable->m_lfVolume[nLast])
						nstage = pFtable->m_lfDepth[nLast]+
						(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);
				}
				else
				{
					nstage  = nvolume / bot_area;  //ft
				}
			}
		} 
		else   
		{
			nAETsurf += nvolume;//all water is evaporated (in/ivl)
			norifice = 0.0;	//no orifice outflow
			nweir    = 0.0;	//no weir outflow
			nstage   = 0.0;	//dry surface
			nvolume  = 0.0;	//no surface water
		}

		// Green_Ampt method
		if (nInfiltMethod == 0)
		{
			//update the soil moisture
			double loss = (nperc + nAET)/s_area_max;	//ft/ivl

            //GAInfil[nInfil_Index].F -= loss;
            GAInfil[nInfil_Index].FU -= loss;
            if ( GAInfil[nInfil_Index].FU <= 0.0 )
            {
                GAInfil[nInfil_Index].FU = 0.0;
                GAInfil[nInfil_Index].F = 0.0;
                GAInfil[nInfil_Index].IMD = GAInfil[nInfil_Index].IMDmax;
			}
		}
		else if (nInfiltMethod == 1 && nstage <= SMALLNUM)
		{
			//update the regeneration
			float recoveryfactor = 1;
			float df   = HortInfil[nInfil_Index].f0 - HortInfil[nInfil_Index].fmin;
			float kd   = HortInfil[nInfil_Index].decay;
			float tp   = HortInfil[nInfil_Index].tp;
			float kr   = HortInfil[nInfil_Index].regen;
			kr *= recoveryfactor;

			if ( df > 0.0 && kd > 0.0 && kr > 0.0 ) 
			{
				tp = -log(1.0 - exp(-kr * tStep)*(1.0 - exp(-kd * tp))) / kd;
			}
			HortInfil[nInfil_Index].tp = tp;
		}
		// accumulate the values
		temporifice += norifice;					// ft3/hr
		tempweir    += nweir;						// ft3/hr
		temppump    += npump;						// ft3/hr
		tempudout   += nudout;						// ft3/hr
		tempinfilt  += ninfilt;						// ft3/hr
		tempperc    += nperc;						// ft3/hr
		tempseepage += nseepage;					// ft3/hr
		tempAET     += (nAETsurf+nAET+nAETudrain);	// ft3/hr
    }

	// OUTPUT PARAMETERS  
	ostage    = nstage;					//ft
    osa       = nsa/s_area_max*12.0;	//in
	ostorage  = nstorage/bot_area*12.0;	//in
    ovolume   = nvolume;				//ft3	
	orifice   = temporifice / delts;	//cfs	
	weir      = tempweir    / delts;	//cfs	
	pump      = temppump    / delts;	//cfs	
	udout     = tempudout   / delts;	//cfs	
	infilt    = tempinfilt  / delts;	//cfs	
	perc      = tempperc    / delts;	//cfs	
	AET       = tempAET     / delts;	//cfs	
	seepage   = tempseepage / delts;	//cfs	
	pBMP->m_pPumpControl.m_bIsPumping = pump_ON;	//pumping status

	return;
}

void CBMPRunner::bmp_b(int nInfiltMethod,int nInfil_Index, bool underdrain_on,int LANDtimestep,int BMPtimestep,double oinflow,double BMParea,
					   double BMPdepth,double BMPwidth,double BMPlength,double slope1,double slope2,double slope3,double man_n,
					   double soildepth,double soilporosity,double finalf,double vegparma,double holtpar,double udfinalf,
					   double udsoildepth,double udsoilporosity,double FC,double WP,double ETrate,double& AET,double& perc,
					   double& ovolume,double& ostage,double& infilt,double& channel,double& weir,double& osa,double& ostorage,
					   double& udout,double& seepage,int nFtableFlag,CFtable* pFtable)
{
	double bot_area = BMParea;	// bottom area (ft2)

	if (bot_area <= 0 || BMPdepth <= 0)
		return;

	if (nFtableFlag != 1 && (slope1 <= 0 || slope2 <= 0 || man_n <= 0))
		return;

	// number of intervals per inflow timestep 
	int nivl = int(LANDtimestep / (double)BMPtimestep + 0.5);

	double delts = LANDtimestep * 60.;			// sec/LANDivl
	double g = 32.2;			// ft/sec2
	double r__1 = 0., r__2 = 0., h = 0., f = 0.;
	double nweir = 0.,nchannel = 0.,nudout = 0.,ninfilt = 0.,nperc = 0.;
	double nseepage = 0.,nAET = 0.,nAETsurf = 0.,overflow_max = 0.,nAETudrain = 0.;
	double tempweir = 0.,tempchannel = 0.,tempudout = 0.,tempinfilt = 0.,tempperc = 0.;
	double tempseepage = 0.,tempAET = 0.;
	double x_area = 0., a = 0., b = 0., c = 0., d = 0.;
	double s_area_max = 0.,sur_area = 0.,vol_max = 0.;	 

	//calculate the maximum surface area (ft2)

	//use Ftable if provided
	if (nFtableFlag > 0 && pFtable != NULL)
	{
		s_area_max = pFtable->Depth_lookup(BMPdepth,pFtable->m_lfArea);
	}
	else
	{
		//calculate the top width (ft)
		double top_width = (BMPdepth/slope1 + BMPdepth/slope2 + BMPwidth); 
		//calculate the maximum surface area (ft2)
		s_area_max = top_width * BMPlength;	 
	}

	// at the begining of the timestep
	double AWC      = soildepth*s_area_max*(FC - WP);//available water capacity for plants only (ft3)
	double nvolume  = ovolume;				//available water in the water column (ft3)
	double nsa      = osa/12.0*s_area_max;	//available pores of the soil column (ft3)
	double nstorage = ostorage/12.0*bot_area;//available pores of the underdrain column (ft3)
	double inflow   = oinflow / nivl;		//ft3/LANDivl to ft3/BMPivl
	double nstage   = ostage;				//ft

	// calculate water surface area (ft2)

	//use Ftable if provided
	if (nFtableFlag > 0 && pFtable != NULL)
		sur_area = pFtable->Depth_lookup(nstage,pFtable->m_lfArea);
	else
		sur_area = (nstage/slope1 + nstage/slope2 + BMPwidth) * BMPlength;	

	if (sur_area > s_area_max)
		sur_area = s_area_max;

	//calculate the max capacity of the water column (ft3)

	//use Ftable if provided
	if (nFtableFlag > 0 && pFtable != NULL)
	{
		vol_max = pFtable->Depth_lookup(BMPdepth,pFtable->m_lfVolume);
	}
	else
	{
		//calculate the top width (ft)
		double top_width = (BMPdepth/slope1 + BMPdepth/slope2 + BMPwidth); 
		//calculate the max capacity of the water column (ft3)
		vol_max = BMPdepth * (top_width + BMPwidth) / 2.0 * BMPlength;
	}

	//calculate the max available space in soil column (ft3)
	double nsamax = soildepth * s_area_max * (soilporosity - WP); 

	if (nsamax < SMALLNUM)
		nsamax = 0.0;

	//calculate the max available space in under-drain column (ft3)
	double nstoragemax = udsoildepth*bot_area*udsoilporosity;

	if (nstoragemax < SMALLNUM)
		nstoragemax = 0.0;

	float tStep = BMPtimestep * 60.0;//sec/ivl

	//for infiltration use the maximum surface area
	float finflow = inflow/(s_area_max*tStep);//ft/sec

	for (int i=0; i<nivl; i++) 
	{
		ninfilt  = 0.0;		// surface infiltration (ft3/ivl)
		nchannel = 0.0;		// channel outflow (ft3/ivl)
		nweir    = 0.0;		// weir outflow (ft3/ivl)
		nudout   = 0.0;		// underdrain outflow (ft3/ivl)
		nperc    = 0.0;		// percolation to underdrain storage (ft3/ivl)
		nAETsurf = 0.0;		// actual evapotranspiration from the surface (ft3/ivl) 
		nAET     = 0.0;		// actual evapotranspiration (ft3/ivl)
		nAETudrain = 0.0;	// actual evapotranspiration from the underdrain (ft3/ivl) 
		nseepage = 0.0;		// deep percolation to groundwater storage (ft3/ivl)

		float fdepth = nvolume/s_area_max;//ft

		//update the volume
		nvolume += inflow;	// ft3

		//use Ftable if provided
		if (nFtableFlag > 0 && pFtable != NULL)
		{
			x_area = nvolume / pFtable->m_lfLength;
			//check if volume exceeds the maximum volume given in Ftable
			int nLast = pFtable->m_nRecords-1;
			if(nvolume > pFtable->m_lfVolume[nLast])
				nstage = pFtable->m_lfDepth[nLast]+
				(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
			else
				//calculate the water depth from the given Ftable
				nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

			sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
			if (sur_area > s_area_max)
				sur_area = s_area_max;
		}
		else
		{
			//update cross-section area (ft2), stage (ft), and surface area (ft2)
			UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
				slope1, slope2, x_area, nstage, sur_area);
		}

//==============================================================
// Step 1: Calculate the evaporation loss from the water column
//==============================================================
		if (nstage > SMALLNUM)			
		{
			// calculate the evaporation from the surface (ft3/ivl)
			nAETsurf = min(nvolume, ETrate/12.0/nivl*sur_area);
				
			//update the volume
			nvolume -= nAETsurf;// ft3

			//use Ftable if provided
			if (nFtableFlag > 0 && pFtable != NULL)
			{
				x_area = nvolume / pFtable->m_lfLength;
				//check if volume exceeds the maximum volume given in Ftable
				int nLast = pFtable->m_nRecords-1;
				if(nvolume > pFtable->m_lfVolume[nLast])
					nstage = pFtable->m_lfDepth[nLast]+
					(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
				else
					//calculate the water depth from the given Ftable
					nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

				sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
				if (sur_area > s_area_max)
					sur_area = s_area_max;
			}
			else
			{
				//update cross-section area, stage, and surface area
				UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
					slope1, slope2, x_area, nstage, sur_area);
			}

			if (nstage <= SMALLNUM)	
			{
				nAETsurf += nvolume;//add small portion to evaporation
				nstage = 0.0;	//dry surface
				nvolume = 0.0;	//no surface water
				nweir = 0.0;	//no weir outflow
				nchannel = 0.0;	//no orifice outflow
			}

//=============================================================================
// Step 2: Calculate the infiltration from the water column to the soil column
//=============================================================================
			if (nstage > SMALLNUM)	
			{
				if (nInfiltMethod == 0)
				{
					float fEvap = (float)nAETsurf / ((float)s_area_max * tStep);//ft/sec
					float irate = finflow - fEvap;
					if (irate < 0)
					{
						irate = 0;
						fdepth = float(nvolume/s_area_max);//ft
					}
					
					// calculate infiltration rate per hour using Green-Ampt Method (ft/sec)
					f = infil_getInfil(nInfil_Index,GREEN_AMPT,tStep,irate,fdepth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else if (nInfiltMethod == 1)
				{
					//use Horton method
					float fEvap = (float)nAETsurf / ((float)s_area_max * tStep);//ft/sec
					float irate = finflow - fEvap;
					if (irate < 0)
					{
						irate = 0;
						fdepth = float(nvolume/s_area_max);//ft
					}

					// calculate infiltration rate per hour using Horton Method (ft/sec)
					f = infil_getInfil(nInfil_Index,HORTON,tStep,irate,fdepth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else
				{
					// calculate infiltration potential per hour using Holtan Method (in/hr)
					f = holtpar * vegparma * pow(nsa/s_area_max*12., 1.4) + finalf;

					// f is converted from in/hr to in/sec
					f /= 3600.;

					// f is converted from in/sec to in/ivl
					f *= (double)tStep;
				}

				//infiltration is the min of potential infiltration or available water in basin
				ninfilt = min(f/12.0*s_area_max, nvolume);	//ft3/ivl

				//update the volume
				nvolume -= ninfilt;	// ft3

				//use Ftable if provided
				if (nFtableFlag > 0 && pFtable != NULL)
				{
					x_area = nvolume / pFtable->m_lfLength;
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pFtable->m_nRecords-1;
					if(nvolume > pFtable->m_lfVolume[nLast])
						nstage = pFtable->m_lfDepth[nLast]+
						(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

					sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
					if (sur_area > s_area_max)
						sur_area = s_area_max;
				}
				else
				{
					//update cross-section area, stage, and surface area
					UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
						slope1, slope2, x_area, nstage, sur_area);
				}

				if (nstage <= SMALLNUM)	
				{
					nAETsurf += nvolume;//add small portion to evaporation
					nstage = 0.0;	//dry surface
					nvolume = 0.0;	//no surface water
					nweir = 0.0;	//no weir outflow
					nchannel = 0.0;	//no orifice outflow
				}
			}
			else
			{
				nAETsurf += nvolume;	//ft3
				nstage = 0.0;			//dry surface
				nvolume = 0.0;			//no surface water
				ninfilt = 0.0;			//no water available
				nweir = 0.0;			//no weir outflow
				nchannel = 0.0;			//no orifice outflow
			}
		} 
		else  // if there is no water currently in the basin... 
		{
			nAETsurf = nvolume;
			nstage = 0.0;	//dry surface
			nvolume = 0.0;	//no surface water
			ninfilt = 0.0;	//no water available
			nweir = 0.0;	//no weir outflow
			nchannel = 0.0;	//no orifice outflow
		}

//====================================================================
// Step 3: Calculate the evapotranspiration loss from the soil column
//====================================================================
		if (nsamax > 0)
		{
			// extra water after saturation (ft3)
			double excesswater = ninfilt - nsa;
			
			//sub-surface evapotranspiration (ft3/ivl)
			nAET = ETrate/12.0/nivl*s_area_max - nAETsurf;
			nAET = max(0.0, nAET);

			if (nAET > (nsamax + excesswater))
				nAET = nsamax + excesswater;
				
//=================================================================================
// Step 4: Calculate the percolation from the soil column to the underdrain column
//=================================================================================
			if (excesswater >= 0) // saturation 
			{
				nperc = min(finalf/12.0/3600.* (double)tStep * s_area_max, excesswater + nsamax - nAET - AWC);//finalf is in/hr
				nperc = max(0.0, nperc);//ft3

				double excessinfilt = max(0.0, excesswater - nAET - nperc);
				ninfilt -= excessinfilt;//ft3

				//update the volume
				nvolume += excessinfilt;// ft3

				//use Ftable if provided
				if (nFtableFlag > 0 && pFtable != NULL)
				{
					x_area = nvolume / pFtable->m_lfLength;
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pFtable->m_nRecords-1;
					if(nvolume > pFtable->m_lfVolume[nLast])
						nstage = pFtable->m_lfDepth[nLast]+
						(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

					sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
					if (sur_area > s_area_max)
						sur_area = s_area_max;
				}
				else
				{
					//update cross-section area, stage, and surface area
					UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
						slope1, slope2, x_area, nstage, sur_area);
				}

				//update the soil available storage
				nsa = nsa - ninfilt + nperc + nAET;
			}
			else   // no saturation
			{
				if (-excesswater < nsamax)	// there is water to percolate
				{
					r__1 = (nsamax + excesswater) / nsamax;
					r__1 = (0.5 + 0.5 * r__1);
					r__1 = max(0.0, r__1);

					nperc = min(r__1 * finalf/12.0/3600.* (double)tStep *s_area_max, excesswater + nsamax - nAET - AWC);//finalf is in/hr	
					nperc = max(0.0, nperc);
					nsa   = nsa - ninfilt + nperc + nAET;
				}
				else
				{
					// no percolation to underdrain storage
					nperc = 0.0; 
					nsa   = nsamax;
				}
			}
		}
		else
		{
			// there is no storage capacity
			nAET  = 0.0;
			nsa   = 0.0;
			nperc = ninfilt; //ft3
		}

		nsa = max(nsa, 0.0);	
		nsa = min(nsa, nsamax);	

//==========================================================================
// Step 5: Calculate the evapotranspiration loss from the underdrain column
//==========================================================================
		if(nstoragemax > 0)
		{
			// extra water after saturation (ft3)
			double excesswater = nperc - nstorage;

			//under-drain evapotranspiration (ft3/ivl)
			nAETudrain = ETrate/12.0/nivl*bot_area - nAETsurf - nAET;
			nAETudrain = max(0.0, nAETudrain);

			if (nAETudrain > (nstoragemax + excesswater))
				nAETudrain = nstoragemax + excesswater;

			//Limit the ET rate for underdrain to be 50% of the normal ET rate
			double PETudrain = ETrate/12.0/nivl/2.*bot_area;
			if (nAETudrain > PETudrain)
				nAETudrain = PETudrain;

//===============================================================
// Step 6: Calculate the seepage loss from the underdrain column
//===============================================================
			if (excesswater >= 0) // saturation 
			{
				nseepage = min(udfinalf/12.0/3600.* (double)tStep *bot_area, excesswater + nstoragemax - nAETudrain);//udfinalf is in/hr
				nudout   = 0.0;

				if(underdrain_on)
				{
					nudout = max(0.0, excesswater - nseepage - nAETudrain);
				}
				else
				{
					double excessperc = max(0.0, excesswater - nseepage - nAETudrain);
					nperc -= excessperc;

					//check if soil column has space to hold this excess water
					if(nsa > 0)
					{
						//check the available space
						if(nsa < excessperc)
						{
							ninfilt -= (excessperc - nsa);
							nsa = 0.0;	// saturated (no more space)
							
							//update the volume
							nvolume += (excessperc - nsa);// ft3

							//use Ftable if provided
							if (nFtableFlag > 0 && pFtable != NULL)
							{
								x_area = nvolume / pFtable->m_lfLength;
								//check if volume exceeds the maximum volume given in Ftable
								int nLast = pFtable->m_nRecords-1;
								if(nvolume > pFtable->m_lfVolume[nLast])
									nstage = pFtable->m_lfDepth[nLast]+
									(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
								else
									//calculate the water depth from the given Ftable
									nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

								sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
								if (sur_area > s_area_max)
									sur_area = s_area_max;
							}
							else
							{
								//update cross-section area, stage, and surface area
								UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
									slope1, slope2, x_area, nstage, sur_area);
							}
						}
						else
						{
							nsa -= excessperc;
						}
					}
					else
					{
						//return this excess water to water column
						ninfilt -= excessperc;

						//update the volume
						nvolume += excessperc;// ft3

						//use Ftable if provided
						if (nFtableFlag > 0 && pFtable != NULL)
						{
							x_area = nvolume / pFtable->m_lfLength;
							//check if volume exceeds the maximum volume given in Ftable
							int nLast = pFtable->m_nRecords-1;
							if(nvolume > pFtable->m_lfVolume[nLast])
								nstage = pFtable->m_lfDepth[nLast]+
								(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
							else
								//calculate the water depth from the given Ftable
								nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

							sur_area = pFtable->Volume_lookup(nvolume,pFtable->m_lfArea);
							if (sur_area > s_area_max)
								sur_area = s_area_max;
						}
						else
						{
							//update cross-section area, stage, and surface area
							UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
								slope1, slope2, x_area, nstage, sur_area);
						}
					}
				}

				nstorage = nstorage - nperc + nseepage + nudout + nAETudrain;
			}
			else	// no saturation (no underdrain outflow)
			{
				nudout = 0.0;

				if (-excesswater < nstoragemax)	// there is water to percolate
				{
					nseepage = min(udfinalf/12.0/3600.* (double)tStep *bot_area, excesswater + nstoragemax - nAETudrain);//udfinalf is in/hr
					nstorage = nstorage - nperc + nseepage + nAETudrain;
				}
				else
				{
					nseepage = 0.0; 
					nstorage = nstoragemax;
				}
			}
		}
		else
		{
			// no underdrain outflow
			nudout   = 0.0;
			nstorage = 0.0;
			nAETudrain = 0.0;
			nseepage = nperc;// percolate will go to groundwater storage
		}

		nstorage = max(nstorage, 0.0);	
		nstorage = min(nstorage, nstoragemax);
		
//======================================================================
// Step 7: Calculate the weir and channel outflow from the water column
//======================================================================
		if (nstage > SMALLNUM)			
		{
			//use Ftable if provided
			if (nFtableFlag == 1 && pFtable != NULL)
			{
				//calculate the weir outflow rate from the given Ftable
				nweir = pFtable->Depth_lookup(nstage,pFtable->m_lfWeirOutflow);
				nweir *= (double)tStep;	//ft3/ivl

				if (nweir > nvolume)
					nweir = nvolume;

				//update the volume (ft3/ivl)
				nvolume -= nweir;

				//calculate the channel outflow rate from the given Ftable
				nchannel = pFtable->Depth_lookup(nstage,pFtable->m_lfOrificeOutflow);
				nchannel *= (double)tStep;	//ft3/ivl

				if (nchannel > nvolume)
					nchannel = nvolume;

				//update the volume (ft3/ivl)
				nvolume -= nchannel;

				//check if volume exceeds the maximum volume given in Ftable
				int nLast = pFtable->m_nRecords-1;
				if(nvolume > pFtable->m_lfVolume[nLast])
					nstage = pFtable->m_lfDepth[nLast]+
					(nvolume - pFtable->m_lfVolume[nLast])/pFtable->m_lfArea[nLast];
				else
					//calculate the water depth from the given Ftable
					nstage = pFtable->Volume_lookup(nvolume,pFtable->m_lfDepth);

				if (nstage < SMALLNUM)
				{
					//all the water leaves
					nchannel += nvolume; //ft3
					nstage = 0.0;
					nvolume = 0.0;
				}
			}
			else
			{
				if (nstage > BMPdepth)
				{
					// CALCULATE SWALE CAPACITY OVERFLOW USING WEIR OVERFLOW FROM SIDE OF CHANNEL  
					h = nstage - BMPdepth;	// ft 

					double nweir_max = h * s_area_max;	// ft3												// h is the lookup index1 for weir coefficient
					if (nweir_max > nvolume)
						nweir_max = nvolume;

					//Chow Equation, Replaces Linsey lookup table for weir coefficient
					double c_weir = 0.0;

					if (h / BMPdepth < 10)
						c_weir = 3.27 + 0.4 * h / BMPdepth;
					else
						c_weir = 5.68 * pow(1 + BMPdepth / h, 1.5);
					
					nweir = c_weir * BMPlength * sqrt(h * h * h);  //cfs 
					nweir = nweir * (double)tStep;	//ft3/ivl
					if (nweir > nweir_max)
						nweir = nweir_max;

					//update the volume
					nvolume -= nweir;// ft3
					
					//update cross-section area, stage, and surface area
					UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
						slope1, slope2, x_area, nstage, sur_area);
				}
				else
				{
					// no weir outflow
					nweir = 0.0;
				}

				// COMPUTE CHANNEL OUTFLOW  
				// solve for velocity using manning's equation
				
				h = nstage;
				r__1 = h / slope1;
				r__2 = h / slope2;
				
				double wet_p = BMPwidth + sqrt(h*h + r__1*r__1) + sqrt(h*h + r__2*r__2); //ft
				
				double HR = x_area / wet_p; //ft
				
				double velocity = 1.49 / man_n * pow(HR, 2/3) * pow(slope3, 0.5); //ft/sec

				// solve for channel flow using continuity and manning's velocity
				nchannel = velocity * x_area;	//cfs 
				nchannel = nchannel * tStep;	//ft3/ivl
				if (nchannel > nvolume)
					nchannel = nvolume;
						
				//update the volume
				nvolume -= nchannel;// ft3

				//update cross-section area, stage, and surface area
				UpdateXareaStageSarea(nvolume, vol_max, s_area_max, BMPdepth, BMPwidth, BMPlength,
					slope1, slope2, x_area, nstage, sur_area);
			}
		} 
		else  // if there is no water currently in the basin... 
		{
			nAETsurf += nvolume;//ft3
			nstage = 0.0;		//dry surface
			nvolume = 0.0;		//no surface water
			nweir    = 0.0;		//no weir outflow
			nchannel = 0.0;		//no orifice outflow
		}

		// Green_Ampt method
		if (nInfiltMethod == 0)
		{
			//update the soil moisture
			double loss = (nperc + nAET)/s_area_max;	//ft/ivl

            GAInfil[nInfil_Index].FU -= loss;
            if ( GAInfil[nInfil_Index].FU <= 0.0 )
            {
                GAInfil[nInfil_Index].FU = 0.0;
                GAInfil[nInfil_Index].F = 0.0;
                GAInfil[nInfil_Index].IMD = GAInfil[nInfil_Index].IMDmax;
			}
		}
		else if (nInfiltMethod == 1 && nstage <= SMALLNUM)
		{
			//update the regeneration
			float recoveryfactor = 1;
			float df   = HortInfil[nInfil_Index].f0 - HortInfil[nInfil_Index].fmin;
			float kd   = HortInfil[nInfil_Index].decay;
			float tp   = HortInfil[nInfil_Index].tp;
			float kr   = HortInfil[nInfil_Index].regen;
			kr *= recoveryfactor;

			if ( df > 0.0 && kd > 0.0 && kr > 0.0 ) 
			{
				tp = -log(1.0 - exp(-kr * tStep)*(1.0 - exp(-kd * tp))) / kd;
			}
			HortInfil[nInfil_Index].tp = tp;
		}

		// accumulate the values
		tempchannel += nchannel;					// ft3/hr
		tempweir    += nweir;						// ft3/hr
		tempudout   += nudout;						// ft3/hr
		tempinfilt  += ninfilt;						// ft3/hr
		tempperc    += nperc;						// ft3/hr
		tempseepage += nseepage;					// ft3/hr
		tempAET     += (nAET+nAETsurf+nAETudrain);	// ft3/hr
    }

	// OUTPUT PARAMETERS  
	ostage    = nstage;					//ft
    osa       = nsa/s_area_max*12.0;	//in
	ostorage  = nstorage/bot_area*12.0;	//in
    ovolume   = nvolume;				//ft3	
	channel   = tempchannel / delts;	//cfs	
	weir      = tempweir    / delts;	//cfs	
	udout     = tempudout   / delts;	//cfs	
	infilt    = tempinfilt  / delts;	//cfs	
	perc      = tempperc    / delts;	//cfs	
	AET       = tempAET     / delts;	//cfs	
	seepage   = tempseepage / delts;	//cfs	

	return;
}

void CBMPRunner::bmp_d(int nInfiltMethod,int nInfil_Index,int LANDtimestep,int BMPtimestep,double oinflow,
					   double BMParea,double soildepth,double soilporosity,double finalf,double vegparma,
					   double holtpar,double FC,double WP,double ETrate,double& AET,double& ovolume,
					   double& ostage,double& infilt,double& weir,double& osa,double& seepage,CBMPSite* pBMPSite)
{
	if (BMParea <= 0)
		return;

	BMP_D* pBMP = (BMP_D*) pBMPSite->m_pSiteProp;

	int nivl = LANDtimestep / BMPtimestep;		// number of intervals per inflow timestep 
	double delts = LANDtimestep * 60.;			// sec/LANDivl
	double deltd = LANDtimestep / (60. * 24.);	// day/LANDivl
	double g = 32.2;			// ft/sec2
	double r__1 = 0., r__2 = 0., r__3 = 0.,h = 0., f = 0.;
	double nweir = 0.,ninfilt = 0.;
	double nseepage = 0.,nAET = 0., nAETsurf = 0.;
	double tempweir = 0.,tempinfilt = 0.;
	double tempseepage = 0.,tempAET = 0.;

	// at the begining of the timestep
	double AWC      = soildepth*12.0*(FC - WP);	//available water capacity for plants only (in)
	double nvolume  = ovolume;				//available water in the water column (ft3)
	double nsa      = osa;					//available pores of the soil column (in)
	double nstage   = ostage;				//surface ponded water depth (ft)

	//calculate the max available space in soil column (in)
	double nsamax = soildepth*12.0*(soilporosity - WP); 

	if (nsamax < SMALLNUM)
		nsamax = 0.0;

	//update the volume
	double nvolume2 = nvolume + oinflow;	// ft3/LANDivl

	//update the stage
	nstage = nvolume2 / BMParea;  //ft

	double inflow = oinflow / nivl;		//ft3/LANDivl to ft3/BMPivl
	float tStep = BMPtimestep * 60.0;			//sec/BMPivl
	float finflow = inflow / (BMParea * tStep);	//ft/sec

	for (int i=0; i<nivl; i++) 
	{
		ninfilt  = 0.0;		// surface infiltration (in/ivl)
		nweir    = 0.0;		// weir outflow (ft3/ivl)
		nAETsurf = 0.0;		// actual evapotranspiration from the surface (in/ivl) 
		nAET     = 0.0;		// actual evapotranspiration from the subsurface (in/ivl)
		nseepage = 0.0;		// deep percolation to groundwater storage (in/ivl)

		float depth = nvolume / BMParea;  	//ft

		//update the volume
		nvolume += inflow;	// ft3

		//update the stage
		nstage = nvolume / BMParea;  //ft

		//WATER COLUMN
		if (nstage > SMALLNUM)			
		{
			// calculate the evaporation from the surface (in/ivl)
			nAETsurf = min(nstage*12.0,ETrate / nivl);
				
			//update the stage (ft)
			nstage -= nAETsurf/12.0;

			if (nstage > SMALLNUM)	
			{
				if (nInfiltMethod == 0)
				{
					float fEvap = (float)nAETsurf / (12.0 * tStep);	//ft/sec
					float irate = finflow - fEvap;
					if (irate < 0)
					{
						irate = 0;
						depth = (float)nstage;  //ft
					}
					
					// calculate infiltration rate per hour using Green-Ampt Method (ft/sec)
					f = infil_getInfil(nInfil_Index,GREEN_AMPT,tStep,irate,depth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else if (nInfiltMethod == 1)
				{
					float fEvap = (float)nAETsurf / (12.0 * tStep);	//ft/sec
					float irate = finflow - fEvap;
					if (irate < 0)
					{
						irate = 0;
						depth = (float)nstage;  //ft
					}

					// calculate infiltration rate per hour using Horton Method (ft/sec)
					f = infil_getInfil(nInfil_Index,HORTON,tStep,irate,depth);

					//convert infiltration rate from ft/sec to in/ivl
					f = f * 12.0 * (double)tStep;
				}
				else
				{
					// calculate infiltration potential per hour using Holtan Method (in/hr)
					f = holtpar * vegparma * pow(nsa, 1.4) + finalf;
					
					// f is converted from in/hr to in/sec
					f /= 3600.;	

					// f is converted from in/sec to in/ivl
					f *= (double)tStep;
				}
				
				//infiltration is the min of potential infiltration or available water in basin
				ninfilt = min(f, nstage*12.0);	// in/ivl

				//update the stage (ft)
				nstage -= ninfilt/12.0;

				if (nstage > SMALLNUM)	
				{
					//update the surface storage
					nvolume = nstage * BMParea; //ft3

					//calculate the orifice and weir outflow
				}
				else
				{
					ninfilt += (nstage * 12);	//add to infiltrated water(in/ivl)
					nstage = 0.0;	//dry surface
					nvolume = 0.0;	//no surface water
					nweir    = 0.0;	//no weir outflow
				}
			}
			else
			{
				nAETsurf += nstage * 12;//add small water to evaporated water (in/ivl)
				nstage = 0.0;	//dry surface
				nvolume = 0.0;	//no surface water storage
				ninfilt = 0.0;	//no water available for infiltration
				nweir    = 0.0;	//no weir outflow
			}
		} 
		else  // if there is no water currently in the basin... 
		{
			nAETsurf = nstage * 12;	//all water evaporated (in/ivl)
			nstage = 0.0;	//dry surface
			nvolume = 0.0;	//no surface storage
			ninfilt = 0.0;	//no infiltration
			nweir = 0.0;	//no weir outflow
		}

		// SOIL LAYER
		if (nsamax > 0)
		{
			// extra water after saturation (in)
			double excesswater = ninfilt - nsa;
			
			//sub-surface evapotranspiration (in/ivl)
			nAET = ETrate / nivl - nAETsurf;

			if (nAET > (nsamax + excesswater))
				nAET = nsamax + excesswater;
				
			if (excesswater >= 0) // saturation 
			{
				nseepage = min(finalf/3600.* (double)tStep, excesswater + nsamax - nAET - AWC);	//finalf is in/hr
				nseepage = max(0.0, nseepage);

				double excessinfilt = max(0.0, excesswater - nAET - nseepage);
				ninfilt -= excessinfilt;//reduce the amount of infiltartion	
				
				// send back to surface storage
				nstage += excessinfilt/12; //ft
				nvolume = nstage * BMParea; //ft3

				//update the soil available storage
				nsa = nsa - ninfilt + nseepage + nAET;
			}
			else   // no saturation
			{
				if (-excesswater < nsamax)	// there is water to seep
				{
					r__1 = (nsamax + excesswater) / nsamax;
					r__1 = (0.5 + 0.5 * r__1);
					r__1 = max(0.0, r__1);

					nseepage = min(r__1 * finalf/3600.* (double)tStep, excesswater + nsamax - nAET - AWC);	//finalf is in/hr
					nseepage = max(0.0, nseepage);
					nsa   = nsa - ninfilt + nseepage + nAET;
				}
				else
				{
					// no seepage to groundwate recharge
					nseepage = 0.0; 
					nsa   = nsamax;
				}
			}
		}
		else
		{
			// there is no storage capacity
			nAET  = 0.0;
			nsa   = 0.0;
			nseepage = ninfilt; 
		}

		nsa = max(nsa, 0.0);	
		nsa = min(nsa, nsamax);	
		
		//WATER COLUMN
		if (nstage > SMALLNUM)			
		{
			float precip = 0.0;					
			float evap = ETrate / (12.0 * delts);	//ft/sec
			float surfInfil = (float)ninfilt / (12.0 * tStep);	//ft/sec
			float runoff = pBMP->m_lfrunoff;	//ft/sec

			pBMPSite->getSubareaRunoff2(finflow, precip, evap, tStep, &surfInfil, &depth, &runoff);

			pBMP->m_lfrunoff = runoff;	//ft/sec
			nweir = (double)runoff * BMParea * tStep;	//ft3/ivl

			if (nweir > 0)
			{
				// RECALCULATE VOLUME
				r__1    = nvolume - nweir;	//ft3
				nvolume = max(r__1, 0.0);

				//update the stage
				nstage  = nvolume / BMParea;  //ft
				if (nstage < SMALLNUM)
				{
					nAETsurf += nstage * 12; //ft3
					nstage = 0.0;
					nvolume = 0.0;
				}
			}
		} 
		else  // if there is no water currently in the basin... 
		{
			nAETsurf += nstage * 12;//all water evaporated (in/ivl)
			nweir    = 0.0;	//no weir outflow
			nstage   = 0.0;	//dry surface
			nvolume  = 0.0;	//no surface water
		}

		// Green_Ampt method
		if (nInfiltMethod == 0)
		{
			//update the soil moisture
			double loss = (nseepage + nAET)/12.0;	//ft/ivl

            GAInfil[nInfil_Index].FU -= loss;
            if ( GAInfil[nInfil_Index].FU <= 0.0 )
            {
                GAInfil[nInfil_Index].FU = 0.0;
                GAInfil[nInfil_Index].F = 0.0;
                GAInfil[nInfil_Index].IMD = GAInfil[nInfil_Index].IMDmax;
			}
		}
		else if (nInfiltMethod == 1 && nstage <= SMALLNUM)
		{
			//update the regeneration
			float recoveryfactor = 1;
			float df   = HortInfil[nInfil_Index].f0 - HortInfil[nInfil_Index].fmin;
			float kd   = HortInfil[nInfil_Index].decay;
			float tp   = HortInfil[nInfil_Index].tp;
			float kr   = HortInfil[nInfil_Index].regen;
			kr *= recoveryfactor;

			if ( df > 0.0 && kd > 0.0 && kr > 0.0 ) 
			{
				tp = -log(1.0 - exp(-kr * tStep)*(1.0 - exp(-kd * tp))) / kd;
			}
			HortInfil[nInfil_Index].tp = tp;
		}
		// accumulate the values
		tempweir    += nweir;								// ft3/ivl
		tempinfilt  += (ninfilt  / 12 * BMParea);			// ft3/ivl
		tempseepage += (nseepage / 12 * BMParea);			// ft3/ivl
		tempAET     += ((nAETsurf+nAET) / 12 * BMParea);	// ft3/ivl
    }

	// OUTPUT PARAMETERS  
	ostage    = nstage;					//ft
    osa       = nsa;					//in
    ovolume   = nvolume;				//ft3	
	weir      = tempweir    / delts;	//cfs	
	infilt    = tempinfilt  / delts;	//cfs	
	AET       = tempAET     / delts;	//cfs	
	seepage   = tempseepage / delts;	//cfs	

	return;
}

void CBMPRunner::UpdateXareaStageSarea(double nvolume,double vol_max,double s_area_max,
	 double BMPdepth,double BMPwidth,double BMPlength,double slope1,double slope2,
	 double& x_area,double& nstage,double& sur_area)
{
		//update cross-section area and stage
		if (nvolume > vol_max)
		{
			x_area = nvolume / BMPlength;// ft2
			
			double overflow_max = nvolume - vol_max;// ft3
			nstage = overflow_max / s_area_max + BMPdepth;// ft
		}
		else
		{
			x_area = nvolume / BMPlength; // ft2
			
			// quadratic formula coefficients for calculating stage
			double a = slope1 + slope2;
			double b = BMPwidth * 2 * slope1 * slope2;
			double c = x_area * -2 * slope1 * slope2;
			double d = b * b - 4 * a * c;// (b^2 - 4ac)
			nstage = (-b + pow(d, 0.5)) / (2 * a);// (-b + (b^2 - 4ac)^0.5 ) / 2a
		}
		
		// update surface area (ft2)
		sur_area = (nstage/slope1 + nstage/slope2 + BMPwidth) * BMPlength;	

		if (sur_area > s_area_max)
			sur_area = s_area_max;

	return;
}

void CBMPRunner::advect(double imat,double svol,double sro,double evol,double ero,
	 double delts,double crrat,double& conc,double& romat)
{
	double js = 0.0;
    double sconc = conc;// save starting concentration 

	if (fabs(sro) > fThreshold)
	{
		double rat = svol / (sro * delts);
		if (rat < crrat)
			js = rat / crrat;
		else
			js = 1.0;
	}
    
	double cojs = 1.0 - js;
	double srovol = js * sro * delts;
	double erovol = cojs * ero * delts;

    if (fabs(evol) > fThreshold)
	{
		// reach/res contains water; perform advection normally  
		// calculate new concentration of material in reach/res based  
		// on quantity of material entering during interval (imat),  
		// weighted volume of outflow based on conditions at start of  
		// ivl (srovol), and weighted volume of outflow based on  
		// conditions at end of ivl (erovol)  
		conc = (imat + sconc * (svol - srovol)) / (evol + erovol);				
		if (conc < 0)	
			conc = 0;
		// calculate total amount of material leaving reach/res during  
		// interval  

//		if (ero > 0.001)//cfs
//		{
			romat = srovol * sconc + erovol * conc;									
			if (romat < 0)	
				romat = 0;
//		}
//		else
//		{
//			romat = 0;	
//		}
    }
	else
	{
		// reach/res has gone dry during the interval; set conc equal to  
		// an undefined value  
		// conc = -1e10;
		conc = 0;
		// calculate total amount of material leaving during interval;  
		// this is equal to material inflow + material initially present  

//		if (ero > 0.001)	//cfs
//		{
			romat = imat + sconc * svol;	
			if (romat < 0)	
				romat = 0;
//		}
//		else
//		{
//			romat = 0;
//		}
    }
	return;
}

void CBMPRunner::RunModel(CString strRunMode, int nRunMode)
{
	//define local variables
	int  i=0, j=0, k=0;	//loop counters
	int  NPOL  = 0;		//number of pollutants (before spliting sediment);
	int  NWQ   = 0;		//number of pollutants (after spliting sediment);
	int  NLAND = 0;		//number of land use types
	int  NBMP  = 0;		//number of BMPs
	int  NBMPtype = 0;  //number of unique BMP types
	long t=0, N=0;		//simulation counters (N = total)
	int nLeapYears = 0;	//number of leap years during the simulation period
	bool bLeapFlag = false;
	double lfNumOfYears = 0.0;		//number of years
	double lfWetDaysPerYear = 0.0;	//number of wet days per year
	double lfAll = 0.0;				//total number of seconds for the simulation period

	SYSTEMTIME tm;					//system time
	COleDateTime tStart, tEnd;		//date time for the start and end of simulation
	COleDateTimeSpan tSpan, tSpan0;	//time spans
	POSITION pos, pos1;				//position pointers 
	CSiteLandUse *pSiteLU;			//land use pointer
	CSitePointSource *pSitePS;		//point source pointer
	CBMPSite *pBMPSite, *pBMPSiteUp;//BMP site pointers
	US_BMPSITE *pUS;				//upstream BMP pointer

	//check the run counter to update the progress bar
	if (nRunMode == RUN_OPTIMIZE)
		optcounter++;
	else if (nRunMode == RUN_OUTPUT)
		outcounter++;

	//get the system time at the beginning of the simulation
	if (nRunMode == RUN_PREDEV)
	{
		GetLocalTime(&tm);
		time_i = COleDateTime(tm);
	}

	NWQ = pBMPData->nNWQ;
	NPOL = pBMPData->nPollutant;
	NLAND = pBMPData->siteluList.GetCount();
	NBMP = pBMPData->routeList.GetCount();
	NBMPtype = pBMPData->nBMPtype;
	//initialize the BMP cost
	for (i=0; i<NBMPtype; i++)
		pBMPData->m_pBMPcost[i].m_lfCost = 0.0;
	
	tStart = pBMPData->startDate;
	tEnd = pBMPData->endDate;
	tSpan0 = tEnd - tStart;

	long deltm = pBMPData->nLANDTimeStep;	//min/LANDivl
	N = long(tSpan0.GetTotalMinutes()/deltm + 0.5);
	lfAll = tSpan0.GetTotalSeconds();

	int nNumBins = pBMPData->nBins;	//(optional)
	int	Landtimestep = pBMPData->nLANDTimeStep;	// minutes  
	int	BMPtimestep = pBMPData->nBMPTimeStep;	// minutes
	int dayflag = 0;

	// number of intervals per inflow timestep
	int nivl = int(Landtimestep / (double)BMPtimestep + 0.5);
	float tStep = BMPtimestep * 60.;			// sec/BMPivl
	double delts = Landtimestep * 60.;			// sec/LANDivl
	double delth = Landtimestep / 60.;			// hour/LANDivl
	double deltd = Landtimestep / (60. * 24.);	// day/LANDivl

	//initialize the progress bar
	pWndProgress->SetRange(0, 100);			 
	pWndProgress->SetText("");

	//define local dynamic arrays
	int    *counter_p      = new int[NBMP];				
    double *BmpFlowInput_per = new double[NBMP];
    double *BmpFlowInput_imp = new double[NBMP];
    double *BmpFlowInput   = new double[NBMP];
    double *LandGWrecharge = new double[NBMP];
	double *bmpoflow       = new double[NBMP];
	double *vol_p          = new double[NBMP];
	double *bmpvol_p       = new double[NBMP];
	double *osa_p          = new double[NBMP];
	double *ostorage_p     = new double[NBMP];
	double *weir_p         = new double[NBMP];
	double *orifice_p      = new double[NBMP];
	double *infilt_p       = new double[NBMP];
	double *undrain_p      = new double[NBMP];
	double *seepage_p      = new double[NBMP];
	//output variables
	double *bmpvol_s       = new double[NBMP];
	double *bmpstage_s     = new double[NBMP];
    double *BmpFlowInput_s = new double[NBMP];
	double *weir_s         = new double[NBMP];
	double *orifice_s      = new double[NBMP];
	double *bmpudout_s     = new double[NBMP];
	double *bmpbypass_s    = new double[NBMP];
	double *bmpoutflow_s   = new double[NBMP];
	double *infilt_s       = new double[NBMP];
	double *perc_s         = new double[NBMP];
	double *AET_s          = new double[NBMP];
	double *seepage_s      = new double[NBMP];
//	double *usstorage_s    = new double[NBMP];
//	double *udstorage_s    = new double[NBMP];
	double *bmpoflow_w     = new double[NBMP];			
	double *bmpoflow_o     = new double[NBMP];			
	double *bmpoflow_ud    = new double[NBMP];			
	double *bmpoflow_ut    = new double[NBMP];	//untreated for the exceeding design drainage area
	double *rbsedtot	   = new double[NBMP];

	double *BmpWqInput_per = new double[NBMP*NWQ];
	double *BmpWqInput_imp = new double[NBMP*NWQ];
	double *BmpWqInput     = new double[NBMP*NWQ];
	double *bmpc           = new double[NBMP*NWQ];
	double *bmpc2          = new double[NBMP*NWQ];
	double *bmpqconc_sand  = new double[NBMP*NWQ];	//lb of qual per lb of sand
	double *bmpqconc_silt  = new double[NBMP*NWQ];	//lb of qual per lb of silt
	double *bmpqconc_clay  = new double[NBMP*NWQ];	//lb of qual per lb of clay
	double *bmpmassout     = new double[NBMP*NWQ];
	double *bmpmassout_w   = new double[NBMP*NWQ];
	double *bmpmassout_o   = new double[NBMP*NWQ];
	double *bmpmassout_ud  = new double[NBMP*NWQ];
	double *bmpmassout_ut  = new double[NBMP*NWQ];//untreated for the exceeding design drainage area
	double *bmudconc       = new double[NBMP*NWQ];
	double *romat_o2       = new double[NBMP*NWQ];
	double *romat_ut2      = new double[NBMP*NWQ];
	double *rbsed		   = new double[NBMP*NWQ];

	//output variables
//	double *BmpWqInput_s   = new double[NBMP*NPOL];
//	double *bmpmass_w_s    = new double[NBMP*NPOL];
//	double *bmpmass_o_s    = new double[NBMP*NPOL];
//	double *bmpmass_ud_s   = new double[NBMP*NPOL];
//	double *bmpmass_ut_s   = new double[NBMP*NPOL];
//	double *bmpmassout_s   = new double[NBMP*NPOL];
//	double *bmpconcout_s   = new double[NBMP*NPOL];
	double *BmpWqInput_s   = new double[NBMP*NWQ];
	double *bmpmass_w_s    = new double[NBMP*NWQ];
	double *bmpmass_o_s    = new double[NBMP*NWQ];
	double *bmpmass_ud_s   = new double[NBMP*NWQ];
	double *bmpmass_ut_s   = new double[NBMP*NWQ];
	double *bmpmassout_s   = new double[NBMP*NWQ];
	double *bmpconcout_s   = new double[NBMP*NWQ];

	// evaluation factor calculation
	int	   *nExceedFlag    = new int[NBMP];			// number of times flow exceeds threshold flow 
	double *bmpTotalFlow   = new double[NBMP];		// cumulative hourly flows (ft3/simulation)
	double *bmpAAFlowVol   = new double[NBMP];		// annual average flow (ft3/yr)
	double *bmpPkDisFlow   = new double[NBMP];		// peak flow per simulation (cfs) 
	double *bmpFlowExcFreq = new double[NBMP];		// exceeding frequency of flow compared to threshold flow (/yr) 
	double *bmpTotalLoad   = new double[NBMP*NPOL];	// cumulative hourly load (lb/simulation)
	double *bmpAALoad      = new double[NBMP*NPOL];	// annual average load (lb/yr)
	double *bmpAAConc      = new double[NBMP*NPOL];	// annual average concentration (mg/l)
	double *bmpMAConc      = new double[NBMP*NPOL];	// maximum running average concentration (mg/l)
	double *bmpmassout2    = new double[NBMP*NPOL]; // lbs
	//optional
	int	   *nExceedFlow    = new int[NBMP];			// number of times flow exceeds threshold flow 
	int	   *nInterEvent    = new int[NBMP];			// an inter-event duration of an overflow event (min)
	int	   *nExceedConc    = new int[NBMP*NPOL];	// number of times conc exceeds threshold conc during a wet day 
	double *lfDailyFlow	   = new double[NBMP];		// ft3/day (each day of simulation)
	double *lfSumFlow	   = new double[NBMP];		// ft3/day (previous day of each timestep)
	double *lfSumMass	   = new double[NBMP*NPOL];	// lb/day
	double *bmpConcExcDays = new double[NBMP*NPOL];	// number of days conc exceeds threshold conc per year 
	//optional
	int	   *nExceedFlow_bin    = NULL;		// number of times flow exceeds threshold flow 
	double *bmpTotalFlow_bin   = NULL;		// cumulative hourly flows (ft3/simulation)
	double *bmpAAFlowVol_bin   = NULL;		// annual average flow (ft3/yr)
	double *bmpPkDisFlow_bin   = NULL;		// peak flow per simulation (cfs) 
	double *bmpFlowExcFreq_bin = NULL;		// exceeding frequency of flow compared to threshold flow (/yr) 
	double *bmpTotalLoad_bin   = NULL;		// cumulative hourly load (lb/simulation)
	double *bmpAALoad_bin      = NULL;		// annual average load (lb/yr)
	double *bmpAAConc_bin      = NULL;		// annual average concentration (mg/l)
	double *bmpMAConc_bin      = NULL;		// maximum running average concentration (mg/l)

	if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
	{
	    nExceedFlow_bin    = new int[NBMP*nNumBins];			// number of times flow exceeds threshold flow 
		bmpTotalFlow_bin   = new double[NBMP*nNumBins];	// cumulative hourly flows (ft3/simulation)
		bmpAAFlowVol_bin   = new double[NBMP*nNumBins];	// annual average flow (ft3/yr)
		bmpPkDisFlow_bin   = new double[NBMP*nNumBins];	// peak flow per simulation (cfs) 
		bmpFlowExcFreq_bin = new double[NBMP*nNumBins];	// exceeding frequency of flow compared to threshold flow (/yr) 
		bmpTotalLoad_bin   = new double[NBMP*NPOL*nNumBins];// cumulative hourly load (lb/simulation)
		bmpAALoad_bin      = new double[NBMP*NPOL*nNumBins];// annual average load (lb/yr)
		bmpAAConc_bin      = new double[NBMP*NPOL*nNumBins];// annual average concentration (mg/l)
		bmpMAConc_bin      = new double[NBMP*NPOL*nNumBins];// maximum running average concentration (mg/l)
	}

	//---------------------------------------------------------------------

	for(i=0; i<NBMP; i++)
	{
		// get model parameters
		pos = pBMPData->routeList.FindIndex(i);
		pBMPSite = (CBMPSite*) pBMPData->routeList.GetAt(pos);

		double WP			  = pBMPSite->m_lfWPoint;	//ft/ft	
		double soildepth      = pBMPSite->m_lfSoilDepth;//ft
		double soilporosity   = pBMPSite->m_lfPorosity;	//ft/ft
		double udsoildepth    = pBMPSite->m_lfUndDepth;	//ft
		double udsoilporosity = pBMPSite->m_lfUndVoid;	//ft/ft	

		bmpvol_p[i] = 0.0;	//ft3
		osa_p[i] = soildepth * (soilporosity - WP) * 12;	//max water holding capacity (in)
		//assume underdrain storage media can release all water under gravity
		ostorage_p[i] = udsoildepth * udsoilporosity * 12;	//max water holding capacity (in)

		if (pBMPSite->m_nBMPClass == CLASS_A)
		{
			BMP_A* pBMP = (BMP_A*) pBMPSite->m_pSiteProp;
			int releasetype = pBMP->m_nORelease;		    
			double WatDep_i = pBMP->m_lfWatDep_i;			//ft
			double Theta_i = pBMP->m_lfTheta_i;				//ft/ft
			double basinlength = pBMP->m_lfBasinLength;		//ft
			double basinwidth = pBMP->m_lfBasinWidth;		//ft
			double BMParea = basinlength * basinwidth;		//ft2	 
			
			// check if this BMP is cistern or rainbarrel, if so then 
			// basinlength = diameter (ft) 
			// basinwidth = number of devices

			if (releasetype == 1 || releasetype == 2)
				BMParea = 3.142857/4.0*pow(basinlength,2);	// ft2 

			if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
				bmpvol_p[i] = pBMPSite->m_pFtable->Depth_lookup(WatDep_i,pBMPSite->m_pFtable->m_lfVolume);
			else
				bmpvol_p[i] = BMParea * WatDep_i;	//ft3
			
			osa_p[i] = osa_p[i] - (Theta_i - WP) * soildepth * 12.;	//in
			osa_p[i] = max(0.0, osa_p[i]);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_B)
		{
			BMP_B* pBMP = (BMP_B*) pBMPSite->m_pSiteProp;
			double WatDep_i = pBMP->m_lfWatDep_i;		//ft
			double Theta_i = pBMP->m_lfTheta_i;			//ft/ft
			double BMPlength = pBMP->m_lfBasinLength;	//ft
			double BMPwidth = pBMP->m_lfBasinWidth;		//ft
			double BMPdepth = pBMP->m_lfMaximumDepth;	//ft
			double slope1 = pBMP->m_lfSideSlope1;		//ft/ft
			double slope2 = pBMP->m_lfSideSlope2;		//ft/ft

			//calculate the top width (ft)
			double top_width = (BMPdepth/slope1 + BMPdepth/slope2 + BMPwidth);// ft
			double BMParea = BMPlength * (top_width + BMPwidth)/2.;	// ft2		 

			if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
				bmpvol_p[i] = pBMPSite->m_pFtable->Depth_lookup(WatDep_i,pBMPSite->m_pFtable->m_lfVolume);
			else
				bmpvol_p[i] = BMParea * WatDep_i;		//ft3
			
			osa_p[i] = osa_p[i] - (Theta_i - WP) * soildepth * 12.;	//in
			osa_p[i] = max(0.0, osa_p[i]);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_D)
		{
			BMP_D* pBMP = (BMP_D*) pBMPSite->m_pSiteProp;
			double WatDep_i = pBMP->m_lfWatDep_i;		//ft
			double Theta_i = pBMP->m_lfTheta_i;			//ft/ft
			double BMPlength = pBMP->m_lfFlowLength;	//ft
			double BMPwidth = pBMP->m_lfBufferWidth;	//ft
			double BMParea = BMPlength * BMPwidth;	// ft2		 

			bmpvol_p[i] = BMParea * WatDep_i;		//ft3
			osa_p[i] = osa_p[i] - (Theta_i - WP) * soildepth * 12.;	//in
			osa_p[i] = max(0.0, osa_p[i]);
		}

		bmpvol_s[i]       = 0.0;
		vol_p[i]          = 0.0;
		infilt_s[i]       = 0.0;
		perc_s[i]         = 0.0;
		AET_s[i]          = 0.0;
		seepage_s[i]      = 0.0;
//		usstorage_s[i]    = 0.0;
//		udstorage_s[i]    = 0.0;
		weir_p[i]         = 0.0;
		orifice_p[i]      = 0.0;
		infilt_p[i]       = 0.0;
		undrain_p[i]      = 0.0;
		seepage_p[i]      = 0.0;
		weir_s[i]         = 0.0;
		orifice_s[i]      = 0.0;
		bmpoutflow_s[i]   = 0.0;
		bmpoflow[i]       = 0.0;
		bmpoflow_w[i]     = 0.0;		
		bmpoflow_o[i]     = 0.0;		
		bmpoflow_ud[i]    = 0.0;		
		bmpoflow_ut[i]    = 0.0;
		BmpFlowInput_s[i] = 0.0;
		bmpstage_s[i]     = 0.0;
		bmpudout_s[i]     = 0.0;
		bmpbypass_s[i]    = 0.0;
		counter_p[i]      = 0;					
		bmpTotalFlow[i]	  =	0.0;
		bmpAAFlowVol[i]	  =	0.0;	
		bmpPkDisFlow[i]	  = 0.0;	
		bmpFlowExcFreq[i] =	0.0;	
		nExceedFlow[i]    =	0;	
		nInterEvent[i]    = 0;
		nExceedFlag[i]    =	0;	
		rbsedtot[i]       = 0.0;		
		lfSumFlow[i]	  = 0.0;
		lfDailyFlow[i]	  = 0.0;
	}

	for(i=0; i<NBMP*NWQ; i++)
	{
		bmpc[i]         = 0.0;
		bmpc2[i]        = 0.0;
		bmpqconc_sand[i]= 0.0;
		bmpqconc_silt[i]= 0.0;
		bmpqconc_clay[i]= 0.0;
		bmpmassout[i]   = 0.0;
		bmpmassout_w[i] = 0.0;
		bmpmassout_o[i] = 0.0;
		bmpmassout_ud[i]= 0.0;
		bmpmassout_ut[i]= 0.0;
		bmudconc[i]     = 0.0;
		romat_o2[i]     = 0.0;		
		romat_ut2[i]    = 0.0;		
		rbsed[i]        = 0.0;	
		BmpWqInput_s[i] = 0.0;
		bmpmass_w_s[i] = 0.0;
		bmpmass_o_s[i] = 0.0;
		bmpmass_ud_s[i] = 0.0;
		bmpmass_ut_s[i] = 0.0;
		bmpmassout_s[i] = 0.0;
		bmpconcout_s[i] = 0.0;
	}

	for(i=0; i<NBMP*NPOL; i++)
	{
//		BmpWqInput_s[i] = 0.0;
//		bmpmass_w_s[i] = 0.0;
//		bmpmass_o_s[i] = 0.0;
//		bmpmass_ud_s[i] = 0.0;
//		bmpmass_ut_s[i] = 0.0;
//		bmpmassout_s[i] = 0.0;
//		bmpconcout_s[i] = 0.0;
		bmpTotalLoad[i] = 0.0;
		bmpAALoad[i]    = 0.0;		
		bmpAAConc[i]    = 0.0;		
		bmpMAConc[i]    = 0.0;	
		bmpmassout2[i]  = 0.0;
		nExceedConc[i]  = 0;
		lfSumMass[i]	= 0.0;
		bmpConcExcDays[i] = 0.0;
	}

	if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
	{
		for(i=0; i<NBMP*nNumBins; i++)
		{
			nExceedFlow_bin[i]    = 0;
			bmpTotalFlow_bin[i]   = 0.0;
			bmpAAFlowVol_bin[i]	  =	0.0;	
			bmpPkDisFlow_bin[i]	  = 0.0;	
			bmpFlowExcFreq_bin[i] =	0.0;	
		}
		for(i=0; i<NBMP*NPOL*nNumBins; i++)
		{
			bmpTotalLoad_bin[i]   = 0.0;
			bmpAALoad_bin[i]	  =	0.0;	
			bmpAAConc_bin[i]	  = 0.0;
			bmpMAConc_bin[i]	  = 0.0; 
		}
	}

	// calculate BMP cost for each BMP site
	// Cost ($) = (LinearCost * Length + AreaCost * Area + TotalVolumeCost * TotalVolume 
	// + MediaVolumeCost * SoilMediaVolume + UnderDrainVolumeCost * UnderDrainVolume
	// + Unitcost * Count + ConstantCost) * (1 + PercentCost / 100)

	int ii = 0;
	double totalCost = 0.0;
	pos = pBMPData->routeList.GetHeadPosition();
	while (pos != NULL)
	{
		pBMPSite = (CBMPSite*) pBMPData->routeList.GetNext(pos);

		pBMPSite->m_lfCost = 0.0;
		pBMPSite->m_lfSurfaceArea = 0.0;
		pBMPSite->m_lfExcavatnVol = 0.0;
		pBMPSite->m_lfSurfStorVol = 0.0;
		pBMPSite->m_lfSoilStorVol = 0.0;
		pBMPSite->m_lfUdrnStorVol = 0.0;

		double lfLinearCost = pBMPSite->m_costParam.m_lfLinearCost;			
		double lfAreaCost = pBMPSite->m_costParam.m_lfAreaCost;            				
		double lfTotalVolumeCost = pBMPSite->m_costParam.m_lfTotalVolumeCost;     				
		double lfMediaVolumeCost = pBMPSite->m_costParam.m_lfMediaVolumeCost;     				
		double lfUnderDrainVolumeCost = pBMPSite->m_costParam.m_lfUnderDrainVolumeCost;				
		double lfConstantCost = pBMPSite->m_costParam.m_lfConstantCost;        				
		double lfPercentCost = pBMPSite->m_costParam.m_lfPercentCost;  
		
		// initialize BMP Cost parameters			 
		double lfLengthExp = pBMPSite->m_costParam.m_lfLengthExp;
		double lfAreaExp = pBMPSite->m_costParam.m_lfAreaExp;
		double lfTotalVolExp = pBMPSite->m_costParam.m_lfTotalVolExp;
		double lfMediaVolExp = pBMPSite->m_costParam.m_lfMediaVolExp;
		double lfUDVolExp = pBMPSite->m_costParam.m_lfUDVolExp;

		double soilporosity   = pBMPSite->m_lfPorosity;
		double udsoilporosity = pBMPSite->m_lfUndVoid;	
		double sqft2acre = 2.295675e-005;
		double cuft2acft = 2.295675e-005;

		//assign array size
		if (pBMPSite->m_pConc != NULL)
			delete []pBMPSite->m_pConc;

		if (pBMPSite->m_nPolRotMethod*NWQ > 0)
			pBMPSite->m_pConc = new double[pBMPSite->m_nPolRotMethod*NWQ];
		
		//initialize the values
		for (i=0; i<pBMPSite->m_nPolRotMethod*NWQ; i++)
			pBMPSite->m_pConc[i] = 0.0;

		// initialize running average parameters
		int qsize = int(1/deltd+0.5);	// intervals per day 
		double value1 = 0.0;
		while (!pBMPSite->qFlow.empty())
			pBMPSite->qFlow.pop();
		while (pBMPSite->qFlow.size() != qsize)
			pBMPSite->qFlow.push(value1);

		pBMPSite->m_nInterEvent = 0;
		pBMPSite->m_lfThreshFlow = 0;
		if (pBMPSite->m_RAConc != NULL)
			delete []pBMPSite->m_RAConc;
		//pBMPSite->m_RAConc = new POLLUT_RAConc[NWQ];
		pBMPSite->m_RAConc = new POLLUT_RAConc[NPOL];	//don't split TSS

		//for (i=0; i<NWQ; ++i)
		for (i=0; i<NPOL; ++i)
		{
			pBMPSite->m_RAConc[i].m_nRDays = 0;
			pBMPSite->m_RAConc[i].m_lfRFlow = NULL;
			pBMPSite->m_RAConc[i].m_lfRLoad = NULL;
			pBMPSite->m_RAConc[i].m_lfThreshConc = 0.0;	

			//initialze
			while (!pBMPSite->m_RAConc[i].qMass.empty())
				pBMPSite->m_RAConc[i].qMass.pop();
			while (pBMPSite->m_RAConc[i].qMass.size() != qsize)
				pBMPSite->m_RAConc[i].qMass.push(value1);

			pos1 = pBMPSite->m_factorList.GetHeadPosition();
			while (pos1 != NULL)
			{
				EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
				if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == FEF)
				{
					pBMPSite->m_lfThreshFlow = ef->m_lfThreshold;
					pBMPSite->m_nInterEvent = ef->m_nInterEvent; // hr
				}
				if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == FDC)
				{
					pBMPSite->m_lfLowFlowLimit = ef->m_lfLowFlowLimit;	//cfs
					pBMPSite->m_lfHighFlowLimit = ef->m_lfHighFlowLimit;	//cfs
				}
				if ((ef->m_nFactorGroup-1) == i && ef->m_nFactorType == CEF)	
					pBMPSite->m_RAConc[i].m_lfThreshConc = ef->m_lfConcThreshold;
				if ((ef->m_nFactorGroup-1) == i && ef->m_nFactorType == MAC)
					pBMPSite->m_RAConc[i].m_nRDays = ef->m_nCalcDays;		 
			}
			if (pBMPSite->m_RAConc[i].m_nRDays > 0)
			{
				int arraysize = pBMPSite->m_RAConc[i].m_nRDays/deltd+0.5;
				pBMPSite->m_RAConc[i].m_lfRFlow = new double[arraysize];
				pBMPSite->m_RAConc[i].m_lfRLoad = new double[arraysize];
				for (j=0; j<arraysize; j++)
				{
					pBMPSite->m_RAConc[i].m_lfRFlow[j] = 0.0;
					pBMPSite->m_RAConc[i].m_lfRLoad[j] = 0.0;
				}
			}
		}

		int nInfiltMethod = pBMPSite->m_nInfiltMethod;
		int nInfil_Index = pBMPSite->m_nInfil_Index;
		double soildepth = pBMPSite->m_lfSoilDepth;	
		double udsoildepth = pBMPSite->m_lfUndDepth;	
		double volsediment = 0.0;

		if (pBMPSite->m_nBMPClass == CLASS_A)
		{
			BMP_A* pBMP = (BMP_A*) pBMPSite->m_pSiteProp;

			// copy data
			if (nInfiltMethod == 0)
				CopyGAInfil(&pBMP->m_pGAInfil, &GAInfil[nInfil_Index]);
			else if (nInfiltMethod == 1)
				CopyHortonInfil(&pBMP->m_pHortonInfil, &HortInfil[nInfil_Index]);

			int    releasetype = pBMP->m_nORelease;		    
			double basinlength = pBMP->m_lfBasinLength;		//ft
			double basinwidth = pBMP->m_lfBasinWidth;		//ft
			double weirheight = pBMP->m_lfWeirHeight;		//ft
			double BMParea = basinlength * basinwidth;		//ft2	 
			
			// check if this BMP is cistern or rainbarrel, if so then 
			// basinlength = diameter (ft) and basinwidth = number of devices
			if (releasetype == 1 || releasetype == 2)
				BMParea = 3.142857/4.0*pow(basinlength,2);	// ft2 
			
			double bot_area = BMParea; //ft2
			double BMPdepth2 = weirheight + soildepth + udsoildepth;// ft	 
			double BMPSurfaceVolume = BMParea * weirheight;
			double BMPvolume = BMParea * BMPdepth2;

			//check for the Ftable
			if(pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
			{
				basinlength = pBMPSite->m_pFtable->m_lfLength;//ft
				bot_area = pBMPSite->m_pFtable->m_lfArea[0];//ft2
				BMParea = pBMPSite->m_pFtable->Depth_lookup(weirheight,pBMPSite->m_pFtable->m_lfArea);
				BMPSurfaceVolume = pBMPSite->m_pFtable->Depth_lookup(weirheight,pBMPSite->m_pFtable->m_lfVolume);
				BMPvolume = BMParea * (weirheight + soildepth) + bot_area * udsoildepth;
			}

			//initilaize the cost
			pBMPSite->m_lfCost = 0.0;
			
			if (BMPvolume > 0)
			{
				//Cost ($) = ((LinearCost)Length^(LengthExp) 
				//+ (AreaCost)Area^(AreaExp)  
				//+ (TotalVolumeCost)TotalVolume^(TotalVolExp) 
				//+ (MediaVolumeCost)SoilMediaVolume^(MediaVolExp) 
				//+ (UnderDrainVolumeCost)UnderDrainVolume^(UDVolExp) 
				//+ (ConstantCost)) * (1+PercentCost/100)

				pBMPSite->m_lfCost = (lfLinearCost * pow(basinlength, lfLengthExp) 
				+ lfAreaCost * pow(BMParea, lfAreaExp) 
				+ lfTotalVolumeCost	* pow(BMPvolume, lfTotalVolExp) 
				+ lfMediaVolumeCost	* pow((BMParea * soildepth), lfMediaVolExp) 
				+ lfUnderDrainVolumeCost * pow((bot_area * udsoildepth), lfUDVolExp) 
				+ lfConstantCost) * (1 + lfPercentCost / 100);
			}
				
			//get the total cost for all units
			pBMPSite->m_lfCost *= pBMPSite->m_lfBMPUnit;

			//assign the cost to its unique BMPtype
			for (i=0; i<NBMPtype; i++)
			{
				if (pBMPData->m_pBMPcost[i].m_strBMPType.CompareNoCase(pBMPSite->m_strType) == 0)
				{
					pBMPData->m_pBMPcost[i].m_lfCost += pBMPSite->m_lfCost;
					break;
				}
			}

			pBMPSite->m_lfSurfaceArea = BMParea * pBMPSite->m_lfBMPUnit * sqft2acre;//acre
			pBMPSite->m_lfExcavatnVol = BMPvolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSurfStorVol = BMPSurfaceVolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSoilStorVol = BMParea * soildepth * soilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfUdrnStorVol = bot_area * udsoildepth * udsoilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			
			// calculate initial volume of sediment in the bed (ft3)
			volsediment = basinlength * pBMPSite->m_sediment.m_lfBEDWID
						* pBMPSite->m_sediment.m_lfBEDDEP
						* (1.0 - pBMPSite->m_sediment.m_lfBEDPOR);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_B)
		{
			BMP_B* pBMP = (BMP_B*) pBMPSite->m_pSiteProp;

			// copy data
			if (nInfiltMethod == 0)
				CopyGAInfil(&pBMP->m_pGAInfil, &GAInfil[nInfil_Index]);
			else if (nInfiltMethod == 1)
				CopyHortonInfil(&pBMP->m_pHortonInfil, &HortInfil[nInfil_Index]);

			double BMPlength   = pBMP->m_lfBasinLength;
			double BMPwidth    = pBMP->m_lfBasinWidth;
			double BMPdepth    = pBMP->m_lfMaximumDepth;
			double slope1      = pBMP->m_lfSideSlope1;
			double slope2      = pBMP->m_lfSideSlope2;

			//calculate the top width (ft)
			double top_width   = (BMPdepth/slope1 + BMPdepth/slope2 + BMPwidth);// ft
			double BMParea     = BMPlength * top_width;							// ft2		 
			double bot_area    = BMPlength * BMPwidth; //ft2
			double BMPSurfaceVolume = BMParea * BMPdepth;
			double BMPvolume = BMParea * (BMPdepth + soildepth) + bot_area * udsoildepth;

			//check for the Ftable
			if(pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
			{
				BMPlength = pBMPSite->m_pFtable->m_lfLength;//ft
				bot_area = pBMPSite->m_pFtable->m_lfArea[0];//ft2
				BMParea = pBMPSite->m_pFtable->Depth_lookup(BMPdepth,pBMPSite->m_pFtable->m_lfArea);
				BMPSurfaceVolume = pBMPSite->m_pFtable->Depth_lookup(BMPdepth,pBMPSite->m_pFtable->m_lfVolume);
				BMPvolume = BMParea * (BMPdepth + soildepth) + bot_area * udsoildepth;
			}

			//initilaize the cost
			pBMPSite->m_lfCost = 0.0;

			if (BMPvolume > 0)
			{
				//Cost ($) = ((LinearCost)Length^(LengthExp) 
				//+ (AreaCost)Area^(AreaExp)  
				//+ (TotalVolumeCost)TotalVolume^(TotalVolExp) 
				//+ (MediaVolumeCost)SoilMediaVolume^(MediaVolExp) 
				//+ (UnderDrainVolumeCost)UnderDrainVolume^(UDVolExp) 
				//+ (Unitcost) +  (ConstantCost)) * (1+PercentCost/100)

				pBMPSite->m_lfCost = (lfLinearCost * pow(BMPlength, lfLengthExp) 
				+ lfAreaCost * pow(BMParea, lfAreaExp) 
				+ lfTotalVolumeCost	* pow(BMPvolume, lfTotalVolExp) 
				+ lfMediaVolumeCost	* pow((BMParea * soildepth), lfMediaVolExp) 
				+ lfUnderDrainVolumeCost * pow((bot_area * udsoildepth), lfUDVolExp) 
				+ lfConstantCost) * (1 + lfPercentCost / 100);
			}

			//get the total cost for all units
			pBMPSite->m_lfCost *= pBMPSite->m_lfBMPUnit;

			//assign the cost to its unique BMPtype
			for (i=0; i<NBMPtype; i++)
			{
				if (pBMPData->m_pBMPcost[i].m_strBMPType.CompareNoCase(pBMPSite->m_strType) == 0)
				{
					pBMPData->m_pBMPcost[i].m_lfCost += pBMPSite->m_lfCost;
					break;
				}
			}

			pBMPSite->m_lfSurfaceArea = BMParea * pBMPSite->m_lfBMPUnit * sqft2acre;//acre
			pBMPSite->m_lfExcavatnVol = BMPvolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSurfStorVol = BMPSurfaceVolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSoilStorVol = BMParea * soildepth * soilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfUdrnStorVol = bot_area * udsoildepth * udsoilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			
			// calculate initial volume of sediment in the bed (ft3)
			volsediment = BMPlength * pBMPSite->m_sediment.m_lfBEDWID
						* pBMPSite->m_sediment.m_lfBEDDEP
						* (1.0 - pBMPSite->m_sediment.m_lfBEDPOR);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_C)
		{
			//  Purpose: initializes a link's state variables at start of simulation.
			BMP_C* pBMP = (BMP_C*) pBMPSite->m_pSiteProp;

			int nIndex = pBMP->m_nIndex;
			float Length = Conduit[Link[nIndex].subIndex].length;

			// copy data
			CopyLink(&pBMP->m_pTLink, &Link[nIndex], NWQ);
			CopyConduit(&pBMP->m_pTConduit, &Conduit[nIndex]);
			CopyTransect(&pBMP->m_pTTransect, &Transect[nIndex]);

			// check if conduit type is DUMMY
			if(Link[nIndex].xsect.type != DUMMY ||
				(Link[nIndex].xsect.type == DUMMY && pBMPSite->m_nFtableFlag > 0))
			{
				// --- assign initial flow to both ends of conduit
				k = Link[nIndex].subIndex;
				if ((float)Conduit[k].barrels > 0)
					Conduit[k].q1 = Link[nIndex].newFlow / (float)Conduit[k].barrels;
				else
					Conduit[k].q1 = 0.0;;

				Conduit[k].q2 = Conduit[k].q1;

				Conduit[k].q1Old = Conduit[k].q1;
				Conduit[k].q2Old = Conduit[k].q2;

				//check for the Ftable
				if(pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
				{
					Length = pBMPSite->m_pFtable->m_lfLength;//ft
					Conduit[k].a1 = Link[nIndex].q0*delts/(float)Conduit[k].barrels/Length;
					Conduit[k].a2 = Conduit[k].a1;
				}
				else
				{
					// --- find areas based on initial flow depth
					Conduit[k].a1 = xsect_getAofY(&Link[nIndex].xsect, Link[nIndex].newDepth);
					Conduit[k].a2 = Conduit[k].a1;
				}

				// --- compute initial volume from area
//				Link[nIndex].newVolume = max(0.0, Conduit[k].a1 * Conduit[k].length *
//									(float)Conduit[k].barrels);
				Link[nIndex].newVolume = max(0.0, Conduit[k].a1 * Length *
									(float)Conduit[k].barrels);
				Link[nIndex].oldVolume = Link[nIndex].newVolume;

				// --- initialize flow
				Link[nIndex].oldFlow = Link[nIndex].q0;
				Link[nIndex].newFlow = Link[nIndex].q0;

				//check for the Ftable
				if(pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
				{
					// --- initialize water depth
					//check if volume exceeds the maximum volume given in Ftable
					int nLast = pBMPSite->m_pFtable->m_nRecords-1;
					float newvol = Link[nIndex].newVolume/(float)Conduit[k].barrels;
					if(newvol > pBMPSite->m_pFtable->m_lfVolume[nLast])
						Link[nIndex].newDepth = pBMPSite->m_pFtable->m_lfDepth[nLast]+
						(newvol - pBMPSite->m_pFtable->m_lfVolume[nLast])/
						pBMPSite->m_pFtable->m_lfArea[nLast];
					else
						//calculate the water depth from the given Ftable
						Link[nIndex].newDepth = pBMPSite->m_pFtable->Volume_lookup(newvol,pBMPSite->m_pFtable->m_lfDepth);//ft;
				}
				else
				{
					// --- initialize water depth
					if ((float)Conduit[k].barrels > 0)
						Link[nIndex].newDepth = link_getYnorm(nIndex, Link[nIndex].q0 / (float)Conduit[k].barrels);
					else
						Link[nIndex].newDepth	= 0;
				}

				Link[nIndex].oldDepth = Link[nIndex].newDepth;
			}

			// calculate initial volume of sediment in the bed (ft3)
			volsediment = Length * (float)Conduit[k].barrels 
								 * pBMPSite->m_sediment.m_lfBEDWID
								 * pBMPSite->m_sediment.m_lfBEDDEP
								 * (1.0 - pBMPSite->m_sediment.m_lfBEDPOR);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_D)
		{
			BMP_D* pBMP = (BMP_D*) pBMPSite->m_pSiteProp;

			// copy data
			if (nInfiltMethod == 0)
				CopyGAInfil(&pBMP->m_pGAInfil, &GAInfil[nInfil_Index]);
			else if (nInfiltMethod == 1)
				CopyHortonInfil(&pBMP->m_pHortonInfil, &HortInfil[nInfil_Index]);

			double length = pBMP->m_lfFlowLength;	//ft
			double width = pBMP->m_lfBufferWidth;	//ft
			double depth = pBMP->m_lfdStore;		//ft
			double BMParea = length * width;		//ft2	 
			double BMPdepth2 = depth + soildepth + udsoildepth;	// ft	 
			double BMPSurfaceVolume = BMParea * depth;
			double BMPvolume = BMParea * BMPdepth2;

			//initilaize the cost
			pBMPSite->m_lfCost = 0.0;
			
			if (BMParea > 0 && BMPdepth2 > 0)		 
				
				//Cost ($) = ((LinearCost)Length^(LengthExp) 
				//+ (AreaCost)Area^(AreaExp)  
				//+ (TotalVolumeCost)TotalVolume^(TotalVolExp) 
				//+ (MediaVolumeCost)SoilMediaVolume^(MediaVolExp) 
				//+ (UnderDrainVolumeCost)UnderDrainVolume^(UDVolExp) 
				//+ (ConstantCost)) * (1+PercentCost/100)

				pBMPSite->m_lfCost = (lfLinearCost * pow(length, lfLengthExp) 
				+ lfAreaCost * pow(BMParea, lfAreaExp) 
				+ lfTotalVolumeCost	* pow(BMPvolume, lfTotalVolExp) 
				+ lfMediaVolumeCost	* pow((BMParea * soildepth), lfMediaVolExp) 
				+ lfUnderDrainVolumeCost * pow((BMParea * udsoildepth), lfUDVolExp) 
				+ lfConstantCost) * (1 + lfPercentCost / 100);
			
			//get the total cost for all units
			pBMPSite->m_lfCost *= pBMPSite->m_lfBMPUnit;

			//assign the cost to its unique BMPtype
			for (i=0; i<NBMPtype; i++)
			{
				if (pBMPData->m_pBMPcost[i].m_strBMPType.CompareNoCase(pBMPSite->m_strType) == 0)
				{
					pBMPData->m_pBMPcost[i].m_lfCost += pBMPSite->m_lfCost;
					break;
				}
			}

			pBMPSite->m_lfSurfaceArea = BMParea * pBMPSite->m_lfBMPUnit * sqft2acre;//acre
			pBMPSite->m_lfExcavatnVol = BMPvolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSurfStorVol = BMPSurfaceVolume * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfSoilStorVol = BMParea * soildepth * soilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			pBMPSite->m_lfUdrnStorVol = BMParea * udsoildepth * udsoilporosity * pBMPSite->m_lfBMPUnit * cuft2acft;//ac-ft
			
			// calculate initial volume of sediment in the bed (ft3)
			volsediment = length * pBMPSite->m_sediment.m_lfBEDWID
						* pBMPSite->m_sediment.m_lfBEDDEP
						* (1.0 - pBMPSite->m_sediment.m_lfBEDPOR);
		}
		else if (pBMPSite->m_nBMPClass == CLASS_E)
		{
			BMP_E* pBMP = (BMP_E*) pBMPSite->m_pSiteProp;

			double BMParea = pBMP->m_lfsArea;	//ft2
			double BMPdepth = pBMP->m_lfdStore;	//ft
			double BMPvolume = BMParea * BMPdepth;

			//initilaize the cost
			pBMPSite->m_lfCost = 0.0;
			
			if (BMParea > 0 && pBMP->m_lfDCIA < 100 && pBMPSite->m_lfDDarea > 0)		 
				pBMPSite->m_lfCost = (lfConstantCost/pBMPSite->m_lfDDarea) * (100.0 - pBMP->m_lfDCIA) / 100.0 * pBMP->m_lfTotImpArea;
			
			//assign the cost to its unique BMPtype
			for (i=0; i<NBMPtype; i++)
			{
				if (pBMPData->m_pBMPcost[i].m_strBMPType.CompareNoCase(pBMPSite->m_strType) == 0)
				{
					pBMPData->m_pBMPcost[i].m_lfCost += pBMPSite->m_lfCost;
					break;
				}
			}

			pBMPSite->m_lfSurfaceArea = BMParea * sqft2acre;//acre
			pBMPSite->m_lfExcavatnVol = 0.0;//ac-ft
			pBMPSite->m_lfSurfStorVol = BMPvolume * cuft2acft;//ac-ft
			pBMPSite->m_lfSoilStorVol = 0.0;//ac-ft
			pBMPSite->m_lfUdrnStorVol = 0.0;//ac-ft
			
			// calculate initial volume of sediment in the bed (ft3)
			volsediment = pBMP->m_lfFlowLength * pBMPSite->m_sediment.m_lfBEDWID
						* pBMPSite->m_sediment.m_lfBEDDEP
						* (1.0 - pBMPSite->m_sediment.m_lfBEDPOR);
		}
		
		for (j=0; j<NWQ; j++)
		{
			if(pBMPData->nSedflag[j] == 2)		// silt 
				rbsed[ii*NWQ+j] = volsediment * pBMPSite->m_sediment.m_lfSILT_FRAC * pBMPSite->m_silt.m_lfRHO;
			else if(pBMPData->nSedflag[j] == 3)	// clay 
				rbsed[ii*NWQ+j] = volsediment * pBMPSite->m_sediment.m_lfCLAY_FRAC * pBMPSite->m_clay.m_lfRHO;
		}

		//check if the bmpsite has tradeoff curve
		if (pBMPSite->m_nBreakPoints > 0)
		{
			//break point id: 0 for initial, -1 for PreDev, and -2 for PostDev condition
			//get the break point index
			int nBrPtIndex = int(pBMPSite->m_lfBreakPtID + 2.0);
			
			if (nRunMode == RUN_POSTDEV)
				nBrPtIndex = 0;				//index = -2+2=0
			else if (nRunMode == RUN_PREDEV)
				nBrPtIndex = 1;				//index (-1+2=1)
			else if(nRunMode == RUN_INIT)
				nBrPtIndex = 2;				//index = 0+2=2

			pBMPSite->m_lfCost = pBMPSite->m_TradeOff[nBrPtIndex].m_lfCost*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;

			if (pBMPSite->m_nBreakPoints > 1)
			{
				// load time series data for tradeoff curve
				if (!pBMPSite->LoadTradeOffCurveData(pBMPData->nLANDTimeStep, nBrPtIndex, pBMPData->startDate, pBMPData->endDate))
				{
					CString strErr;
					strErr.Format("Check Cost-Effectiveness Curve Data for BMPSite ID: %s", pBMPSite->m_strID);
					AfxMessageBox(strErr);
					goto L001;
				}  
			}
		}

		ii++;
		totalCost += pBMPSite->m_lfCost;

		//initialize the groundwater storage (3-11-2013)
		if (pBMPSite->m_pAquifer != NULL)
			pBMPSite->m_pAquifer->m_lfGWStorage_u = pBMPSite->m_pAquifer->m_lfGWStorage_i;	//acre-ft

		//flow duration curve
		if (nRunMode == RUN_PREDEV && pBMPSite->m_nPreDevFlag == 1)
		{
			pos1 = pBMPSite->m_factorList.GetHeadPosition();
			while (pos1 != NULL)
			{
				EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
				if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == FDC)
				{
					// allocate memory
					if(pBMPSite->m_pPreDevFlow!= NULL)	
						delete[]pBMPSite->m_pPreDevFlow;
					pBMPSite->m_pPreDevFlow = new double[N];

					//initialize values
					pBMPSite->m_lfMinFlow = 1e12;
					pBMPSite->m_lfMaxFlow = 0.;
					for (long jj=0; jj<N; jj++)
						pBMPSite->m_pPreDevFlow[jj] = 0.;
				}
			}
		}
		else if (nRunMode != RUN_PREDEV && pBMPSite->m_nPreDevFlag > 0)
		{
			pos1 = pBMPSite->m_factorList.GetHeadPosition();
			while (pos1 != NULL)
			{
				EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
				if (ef->m_nFactorGroup == -1 && ef->m_nFactorType == FDC)
				{
					// allocate memory
					if(pBMPSite->m_pFlow!= NULL)	
						delete[]pBMPSite->m_pFlow;
					pBMPSite->m_pFlow = new double[N];

					//initialize
					pBMPSite->m_lfMinFlow = 1e12;
					pBMPSite->m_lfMaxFlow = 0.;
					for (long jj=0; jj<N; jj++)
						pBMPSite->m_pFlow[jj] = 0.;
				}
			}
		}
	}
	
// start of simulation
	dayflag = 0;
	for(t=0; t<N; t++)
	{
		COleDateTimeSpan tSpan1(0, 0, t*deltm, 0);
		COleDateTime tCurrent = pBMPData->startDate + tSpan1;
		double lfPart = tSpan1.GetTotalSeconds();
		int nYear  = tCurrent.GetYear();
		int mon    = tCurrent.GetMonth() - 1;
		int day    = tCurrent.GetDay() - 1;
		int hour   = tCurrent.GetHour();
		int minute = tCurrent.GetMinute();
		int nxtmon = (mon + 1) % 12;

		int ndays  = 31; // for Jan, Mar, May, Jul, Aug, Oct, Dec
		if (mon == 3 || mon == 5 || mon == 8 || mon == 10) // for Apr, Jun, Sep, Nov
			ndays = 30;
		if(mon == 1) // for Feb
		{
			if(nYear%400 == 0)
			{
				ndays = 29;
				if (!bLeapFlag)
				{
					nLeapYears++;
					bLeapFlag = true;
				}
			}
			else if (nYear%4 == 0 && nYear%100 != 0)
			{
				ndays = 29;
				if (!bLeapFlag)
				{
					nLeapYears++;
					bLeapFlag = true;
				}
			}
			else
			{
				ndays = 28;
				bLeapFlag = false;
			}
		}

		// initialization
		for(i=0; i<NBMP; i++)
		{
			BmpFlowInput_per[i] = 0.0;
			BmpFlowInput_imp[i] = 0.0;
			BmpFlowInput[i] = 0.0;
			LandGWrecharge[i] = 0.0;
		}
		for(i=0; i<NBMP*NWQ; i++)
		{
			BmpWqInput_per[i] = 0.0;
			BmpWqInput_imp[i] = 0.0;
			BmpWqInput[i] = 0.0;
		}
		
		// sum landuse flow and wq
		if (nRunMode == RUN_PREDEV)
		{
			i = 0;	// BMP array
			pos = pBMPData->routeList.GetHeadPosition();
			while (pos != NULL)
			{
				pBMPSite = (CBMPSite*) pBMPData->routeList.GetNext(pos);

				// for the pre-developed LU flowing into this bmp site, summarize the flow and wq
				if (pBMPData->nLandSimulation == 0)
				{
					//external land simulation option
					if (pBMPSite->m_preLU->m_nType == 0)	//pervious land
						BmpFlowInput_per[i] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum]*pBMPSite->m_lfSiteDArea;	// in-acres/hr
					else if (pBMPSite->m_preLU->m_nType == 1)	//impervious land
						BmpFlowInput_imp[i] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum]*pBMPSite->m_lfSiteDArea;	// in-acres/hr
					
					BmpFlowInput[i] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum]*pBMPSite->m_lfSiteDArea;	// in-acres/hr
					LandGWrecharge[i] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+1]*pBMPSite->m_lfSiteDArea;	// in-acres/hr
					
					int nIndex = 0;
					for (j=0; j<NPOL; j++)
					{
						if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
						{
							// need to split the TSS into sand, silt, and clay
							double SED_FR[3];
							SED_FR[0] = pBMPSite->m_preLU->m_lfsand_fr;
							SED_FR[1] = pBMPSite->m_preLU->m_lfsilt_fr;
							SED_FR[2] = pBMPSite->m_preLU->m_lfclay_fr;
							for (k=0; k<3; k++)
							{
								if (pBMPSite->m_preLU->m_nType == 0)	//pervious land
									BmpWqInput_per[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea*SED_FR[k];	// lbs/hr
								else if (pBMPSite->m_preLU->m_nType == 1)	//impervious land
									BmpWqInput_imp[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea*SED_FR[k];	// lbs/hr
								
								BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea*SED_FR[k];	// lbs/hr
								nIndex++;
							}
						}
						else
						{
							if (pBMPSite->m_preLU->m_nType == 0)	//pervious land
								BmpWqInput_per[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea;	// lbs/hr
							else if (pBMPSite->m_preLU->m_nType == 1)	//impervious land
								BmpWqInput_imp[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea;	// lbs/hr
							
							BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_preLU->m_pData[t*pBMPSite->m_preLU->m_nQualNum+2+j]*pBMPSite->m_lfSiteDArea;	// lbs/hr
							nIndex++;
						}
					}
				}
				else if (pBMPSite->m_pDataPreLU != NULL)
				{
					//internal land simulation option
					BmpFlowInput[i] += pBMPSite->m_pDataPreLU[t*pBMPSite->m_nQualNum];	// in-acres/ivl
					
					//pollutant index
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += pBMPSite->m_pDataPreLU[t*pBMPSite->m_nQualNum+1+j];	// lbs/ivl
				}

				//add point source data
				pos1 = pBMPSite->m_sitepsList.GetHeadPosition();
				while (pos1 != NULL)
				{
					pSitePS = (CSitePointSource*) pBMPSite->m_sitepsList.GetNext(pos1);
						
					BmpFlowInput[i] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum]*pSitePS->m_lfMult;	// in-acres/hr
					
					int nIndex = 0;
					for (j=0; j<NPOL; j++)
					{
						if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
						{
							// need to split the TSS into sand, silt, and clay
							double SED_FR[3];
							SED_FR[0] = pSitePS->m_lfSand;
							SED_FR[1] = pSitePS->m_lfSilt;
							SED_FR[2] = pSitePS->m_lfClay;
							for (k=0; k<3; k++)
							{
								BmpWqInput[i*NWQ+nIndex] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum+1+j]*pSitePS->m_lfMult*SED_FR[k];	// lbs/hr
								nIndex++;
							}
						}
						else
						{
							BmpWqInput[i*NWQ+nIndex] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum+1+j]*pSitePS->m_lfMult;	// lbs/hr
							nIndex++;
						}
					}
				}

				//add trade off curve data if needed for predeveloped condition
				int nBrPtIndex = 1;	//index (-1+2=1)

				if (pBMPSite->m_nBreakPoints > 0)
				{
					if (pBMPSite->m_TradeOff != NULL)
					{
						BmpFlowInput[i] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// in-acres/ivl
						//TSS was already splitted into sand, silt, and clay in teir 1 run.
						for (j=0; j<NWQ; j++)
							BmpWqInput[i*NWQ+j] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// lbs/ivl

//						int nIndex = 0;
//						for (j=0; j<NPOL; j++)
//						{
//							if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
//							{
								// need to split the TSS into sand, silt, and clay
//								double SED_FR[3];
//								SED_FR[0] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfSand;
//								SED_FR[1] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfSilt;
//								SED_FR[2] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfClay;
//								for (k=0; k<3; k++)
//								{
//									BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult*SED_FR[k];	// lbs/hr
//									nIndex++;
//								}
//							}
//							else
//							{
//								BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// lbs/hr
//								nIndex++;
//							}
//						}
					}
				}

				i++;
			}
		}
		else
		{
			// sum landuse flow and wq
			i = 0;
			pos = pBMPData->routeList.GetHeadPosition();
			while (pos != NULL)
			{
				pBMPSite = (CBMPSite*) pBMPData->routeList.GetNext(pos);

				// for all the LUs flowing into this bmp site, summarize the flow and wq
				if (pBMPData->nLandSimulation == 0)
				{
					//external land simulation option
					pos1 = pBMPSite->m_siteluList.GetHeadPosition();
					while (pos1 != NULL)
					{
						pSiteLU = (CSiteLandUse*) pBMPSite->m_siteluList.GetNext(pos1);
						if (pSiteLU->m_pLU->m_nType == 0)
							BmpFlowInput_per[i] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum]*pSiteLU->m_lfArea;	// in-acres/hr
						else if (pSiteLU->m_pLU->m_nType == 1)
							BmpFlowInput_imp[i] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum]*pSiteLU->m_lfArea;	// in-acres/hr

						BmpFlowInput[i] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum]*pSiteLU->m_lfArea;	// in-acres/hr
						LandGWrecharge[i] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+1]*pSiteLU->m_lfArea;	// in-acres/hr

						int nIndex = 0;
						for (j=0; j<NPOL; j++)
						{
							if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
							{
								// need to split the TSS into sand, silt, and clay
								double SED_FR[3];
								SED_FR[0] = pSiteLU->m_pLU->m_lfsand_fr;
								SED_FR[1] = pSiteLU->m_pLU->m_lfsilt_fr;
								SED_FR[2] = pSiteLU->m_pLU->m_lfclay_fr;
								for (k=0; k<3; k++)
								{
									if (pSiteLU->m_pLU->m_nType == 0)	//pervious land
										BmpWqInput_per[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea*SED_FR[k];	// lbs/hr
									else if (pSiteLU->m_pLU->m_nType == 1)	//impervious land
										BmpWqInput_imp[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea*SED_FR[k];	// lbs/hr

									BmpWqInput[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea*SED_FR[k];			// lbs/hr
									nIndex++;
								}
							}
							else
							{
								if (pSiteLU->m_pLU->m_nType == 0)	//pervious land
									BmpWqInput_per[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea;	// lbs/hr
								else if (pSiteLU->m_pLU->m_nType == 1)	//impervious land
									BmpWqInput_imp[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea;	// lbs/hr

								BmpWqInput[i*NWQ+nIndex] += pSiteLU->m_pLU->m_pData[t*pSiteLU->m_pLU->m_nQualNum+2+j]*pSiteLU->m_lfArea;			// lbs/hr
								nIndex++;
							}
						}
					}
				}
				else if (pBMPSite->m_pDataMixLU != NULL)
				{
					//internal land simulation option
					BmpFlowInput[i] += pBMPSite->m_pDataMixLU[t*pBMPSite->m_nQualNum];	// in-acres/ivl

					//pollutant index
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += pBMPSite->m_pDataMixLU[t*pBMPSite->m_nQualNum+1+j];	// lbs/ivl
				}

				//add point source data
				pos1 = pBMPSite->m_sitepsList.GetHeadPosition();
				while (pos1 != NULL)
				{
					pSitePS = (CSitePointSource*) pBMPSite->m_sitepsList.GetNext(pos1);
					BmpFlowInput[i] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum]*pSitePS->m_lfMult;	// in-acres/hr
					int nIndex = 0;
					for (j=0; j<NPOL; j++)
					{
						if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
						{
							// need to split the TSS into sand, silt, and clay
							double SED_FR[3];
							SED_FR[0] = pSitePS->m_lfSand;
							SED_FR[1] = pSitePS->m_lfSilt;
							SED_FR[2] = pSitePS->m_lfClay;
							for (k=0; k<3; k++)
							{
								BmpWqInput[i*NWQ+nIndex] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum+1+j]*pSitePS->m_lfMult*SED_FR[k];	// lbs/hr
								nIndex++;
							}
						}
						else
						{
							BmpWqInput[i*NWQ+nIndex] += pSitePS->m_pDataPS[t*pSitePS->m_nQualNum+1+j]*pSitePS->m_lfMult;	// lbs/hr
							nIndex++;
						}
					}
				}

				//add trade off curve data if needed

				//get the break point index
				int nBrPtIndex = int(pBMPSite->m_lfBreakPtID + 2.0);

				if (nRunMode == RUN_POSTDEV)
					nBrPtIndex = 0;				//index = -2+2=0
				else if(nRunMode == RUN_INIT)
					nBrPtIndex = 2;				//index = 0+2=2

				if (pBMPSite->m_nBreakPoints > 0 && nBrPtIndex >= 0)
				{
					if (pBMPSite->m_TradeOff != NULL)
					{
						BmpFlowInput[i] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// in-acres/ivl
						
						//TSS was already splitted into sand, silt, and clay in teir 1 run.
						for (j=0; j<NWQ; j++)
							BmpWqInput[i*NWQ+j] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// lbs/ivl

//						int nIndex = 0;
//						for (j=0; j<NPOL; j++)
//						{
//							if (pBMPData->m_pPollutant[j].m_nSedfg == TSS)
//							{
								// need to split the TSS into sand, silt, and clay
//								double SED_FR[3];
//								SED_FR[0] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfSand;
//								SED_FR[1] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfSilt;
//								SED_FR[2] = pBMPSite->m_TradeOff[nBrPtIndex].m_lfClay;
//								for (k=0; k<3; k++)
//								{
//									BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult*SED_FR[k];	// lbs/hr
//									nIndex++;
//								}
//							}
//							else
//							{
//								BmpWqInput[i*NWQ+nIndex] += pBMPSite->m_TradeOff[nBrPtIndex].m_pDataBrPt[t*pBMPSite->m_TradeOff[nBrPtIndex].m_nQualNum+1+j]*pBMPSite->m_TradeOff[nBrPtIndex].m_lfMult;	// lbs/hr
//								nIndex++;
//							}
//						}
					}
				}

				i++;
			}
		}
		
		// start loop through each BMP site
		for (i=0; i<NBMP; i++)
		{			          
			// get model parameters
			pos = pBMPData->routeList.FindIndex(i);
			pBMPSite = (CBMPSite*) pBMPData->routeList.GetAt(pos);

			pos1 = pBMPSite->m_usbmpsiteList.GetHeadPosition();
			while (pos1 != NULL)
			{
				pUS = (US_BMPSITE*) pBMPSite->m_usbmpsiteList.GetNext(pos1);
				pBMPSiteUp = pUS->m_pUSBMPSite;

				int i0 = ::FindObIndexFromList(pBMPData->routeList, pBMPSiteUp);
				
				// getting upstream flow and wq		
				if (pUS->m_nOutletType == TOTAL)
				{
					BmpFlowInput[i] += bmpoflow[i0]*delts/INACRE2CF;//in-acre/ivl
					
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += bmpmassout[i0*NWQ+j];//lb/ivl
				}
				else if (pUS->m_nOutletType == WEIR_)
				{
					BmpFlowInput[i] += bmpoflow_w[i0]*delts/INACRE2CF;//in-acre/ivl
					
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += bmpmassout_w[i0*NWQ+j];//lb/ivl
				}
				else if (pUS->m_nOutletType == ORIFICE_CHANNEL)
				{
					BmpFlowInput[i] += bmpoflow_o[i0]*delts/INACRE2CF;//in-acre/ivl
					
					//untreated bypass flow
					BmpFlowInput[i] += bmpoflow_ut[i0]*delts/INACRE2CF;//in-acre/ivl

					for (j=0; j<NWQ; j++)
					{
						BmpWqInput[i*NWQ+j] += bmpmassout_o[i0*NWQ+j];//lb/ivl
						
						//untreated bypass pollutant
						BmpWqInput[i*NWQ+j] += bmpmassout_ut[i0*NWQ+j];// lb/ivl
					}
				}
				else if (pUS->m_nOutletType == UNDERDRAIN)
				{
					BmpFlowInput[i] += bmpoflow_ud[i0]*delts/INACRE2CF;//in-acre/ivl
					
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += bmpmassout_ud[i0*NWQ+j];//lb/ivl
				}
			}

			//calculate the base flow and deep percolation loss (ft3/ivl)
			double baseflow = 0.0;		// ft3/ivl
			double deeploss = 0.0;		// ft3/ivl
			double groundwater = 0.0;	// ft3

			if (pBMPSite->m_pAquifer != NULL)
			{
				//add the land groundwater recharge to the same pool
				pBMPSite->m_pAquifer->m_lfGWStorage_u += LandGWrecharge[i]/12.0;//acre-ft
				
				groundwater = pBMPSite->m_pAquifer->m_lfGWStorage_u * ACRE2SF;	//ac-ft to ft3
				
				if(pBMPSite->m_nBMPClass == CLASS_C)
				{
					baseflow = groundwater * pBMPSite->m_pAquifer->m_lfGWrc * delth;//ft3/ivl	
					deeploss = groundwater * pBMPSite->m_pAquifer->m_lfGWsc * delth;//ft3/ivl

					//add baseflow to the total inflow (land + upstream)
					BmpFlowInput[i] += (baseflow/INACRE2CF); //acre-in

					//add background mass to the total mass coming in
					for (j=0; j<NWQ; j++)
						BmpWqInput[i*NWQ+j] += (baseflow * pBMPSite->m_pAquifer->m_pConc[j]);//lb/ivl

					//update groundwater storage (ft3)
					groundwater = groundwater - baseflow - deeploss;	//ft3
					if (groundwater < 0)	
						groundwater = 0.0;
				}
			}

			// initialize soil parameters
			int    nInfiltMethod  = pBMPSite->m_nInfiltMethod;
			int    nInfil_Index   = pBMPSite->m_nInfil_Index;
			double soildepth      = pBMPSite->m_lfSoilDepth;
			double soilporosity   = pBMPSite->m_lfPorosity;
			double FC			  = pBMPSite->m_lfFCapacity;
			double WP			  = pBMPSite->m_lfWPoint;
			double finalf         = pBMPSite->m_lfInfilt;		//in/hr  (okay)
			double udsoildepth    = pBMPSite->m_lfUndDepth;
			double udsoilporosity = pBMPSite->m_lfUndVoid;		
			double udfinalf       = pBMPSite->m_lfUndInfilt;	//in/hr	 (okay)	
			bool   underdrain_on  = pBMPSite->m_bUndSwitch;		
			double vegparma       = pBMPSite->m_holtanParam.m_lfVegA;
			double *holtm         = pBMPSite->m_holtanParam.m_lfGrowth;

			// if underdrain option is not checked, then zero all associated parameters
//			if(!underdrain_on)
//			{
//				udsoildepth    = 0.0;
//				udsoilporosity = 0.0;
//				udfinalf       = 0.0;
//			}

			//calculate the max available storage in soil column (in)
			double nsamax = soildepth*12.0*(soilporosity - WP); 

			if (nsamax < SMALLNUM)
				nsamax = 0.0;

			//calculate the max available space in under-drain column (in)
			double nstoragemax = udsoildepth*12.0*udsoilporosity;

			if (nstoragemax < SMALLNUM)
				nstoragemax = 0.0;

			double ovolume     = bmpvol_p[i];	//ft3
			double osa         = osa_p[i];		//in
			double ostorage    = ostorage_p[i];	//in
			double oinflow     = BmpFlowInput[i] * INACRE2CF;	// acre-in/ivl to ft3/ivl
			double oinflow2	   = oinflow;	//ft3/ivl
			double BMParea     = 0.0;		// ft2	
			double ostage      = 0.0;
			double weir        = 0.0;
			double pump        = 0.0;
			double orifice     = 0.0;
			double channel     = 0.0;
			double qout1       = 0.0;
			double qout2       = 0.0;
			double udout       = 0.0;
			double utout       = 0.0;
			double bmpout      = 0.0;
			double infilt      = 0.0;
			double perc        = 0.0;	// place holder for output
			double AET         = 0.0;	// place holder for output
			double seepage     = 0.0;	// place holder for output
			double crrat       = pBMPData->lfCRRAT;	
			double vol1		   = 0.0;		
			double vol2		   = 0.0;		

			double holtpar = 0.0;
			if (ndays > 0)
				holtpar = (holtm[nxtmon] - holtm[mon]) * day / ndays + holtm[mon];

			// get ET constant rate (in/day)
			int nETflag = pBMPData->nETflag;
			double ETrate = pBMPData->lfmonET[mon];	// (in/day)
			COleDateTimeSpan tsSpan = COleDateTimeSpan(0,0,t*deltm,0);
			long nTSIndex = (long)tsSpan.GetTotalDays();
			
			if (nETflag == 1)
			{
				//get ET rate from the climate file (in/day)
				double evap = pBMPData->m_pDataClimate[nTSIndex*pBMPData->m_nNum+2];
				ETrate *= evap;	//in/day
			}
			else if (nETflag == 2)
			{
				//calculate PET rate from the temperature data (in/day)
				double cts  = ETrate;			
				double lat  = pBMPData->lfLatitude;
				double Tmax = pBMPData->m_pDataClimate[nTSIndex*pBMPData->m_nNum];
				double Tmin = pBMPData->m_pDataClimate[nTSIndex*pBMPData->m_nNum+1];
				double Tavf = (Tmax + Tmin) / 2.0;
				double tavc = (Tavf - 32.0) * 5.0 / 9.0;
				double lfday = tCurrent.GetDayOfYear();
				
				ETrate = pet_Hamon(lat, cts, tavc, lfday);	//in/day
			}

			//convert ETrate from in/day to in/LANDivl
			ETrate *= deltd;	//in/ivl

			int counter = counter_p[i];
			double ndevice = pBMPSite->m_lfBMPUnit;
			double lfDDarea = pBMPSite->m_lfDDarea;		//acre
			double lfAccDArea = pBMPSite->m_lfAccDArea;	//acre

			if (pBMPSite->m_nBMPClass == CLASS_A && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
			{
				// initialize class A BMP related parameters
				BMP_A* pBMP = (BMP_A*) pBMPSite->m_pSiteProp;
				
				double basinlength    = pBMP->m_lfBasinLength;			// ft
				double basinwidth     = pBMP->m_lfBasinWidth;			// ft
				int	   npeople        = pBMP->m_nPeople;			
				int    ddays          = pBMP->m_nDays;				
				double orificeheight  = pBMP->m_lfOrificeHeight;
				double orifice_dia    = pBMP->m_lfOrificeDiameter/12.;	// ft
				double weirheight     = pBMP->m_lfWeirHeight;			// ft
				double weirwidth      = pBMP->m_lfWeirWidth;
				int    releasetype    = pBMP->m_nORelease;		
				int    weirtype       = pBMP->m_nWeirType;  
				double weirangle      = pBMP->m_lfWeirAngle;
				double PETmultiplier  = pBMP->m_lfPETmultiplier;
				double cisternoutflow = pBMP->m_lfRelease[hour]*delth;	//ft3/ivl			
				int    exittype       = pBMP->m_nExitType;
				double orificecoef    = pBMP->m_lfOrificeCoeff;

				BMParea = basinlength * basinwidth;	// ft2	
	
				// check if this BMP is cistern or rainbarrel, if so then 
				// basinlength = diameter (ft) and basinwidth = number of devices
				if (releasetype == 1 || releasetype == 2)
					BMParea = pi/4.0*pow(basinlength,2); // ft2 

				//use Ftable if provided for bottom surface area
				if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
					BMParea = pBMPSite->m_pFtable->m_lfArea[0];//ft2

				switch (exittype)
				{
					case 1:
						orificecoef = 1.0;
						break;
					case 2:
						orificecoef = 0.61;
						break;
					case 3:
						orificecoef = 0.61;
						break;
					case 4:
						orificecoef = 0.5;
						break;
					default:
//						orificecoef = 1.0;
						break;
				}

				if (BMParea > 0 && ndevice > 0)		
				{
					//check the design drainage area
					if (lfDDarea > 0 && lfAccDArea > (lfDDarea*ndevice))
					{
						utout = oinflow*(lfAccDArea-(lfDDarea*ndevice))/lfAccDArea;//ft3/ivl
						oinflow2 = oinflow - utout;//ft3/ivl
						utout /= delts;//cfs 
					}
					else
					{
						utout = 0.0;//cfs
						oinflow2 = oinflow;//ft3/ivl
					}

					ovolume /= ndevice;//ft3
					oinflow2 /= ndevice;//ft3/ivl

					//check if volume exceeds the maximum volume given in Ftable
					if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
					{
						int nLast = pBMPSite->m_pFtable->m_nRecords-1;
						if((ovolume+oinflow2) > pBMPSite->m_pFtable->m_lfVolume[nLast])
						{
							double extravolume = (ovolume+oinflow2) - pBMPSite->m_pFtable->m_lfVolume[nLast];//ft3/ivl
							double extrainflow = max(0.,extravolume - ovolume);//ft3/ivl
							//update inflow and untreated flow
							oinflow2 -= extrainflow;//ft3/ivl
							utout += (extrainflow*ndevice/delts);//cfs
						}
					}

					bmp_a(nInfiltMethod,nInfil_Index,underdrain_on,Landtimestep,
						  BMPtimestep,npeople,ddays,releasetype,weirtype,counter,
						  oinflow2,BMParea,orifice_dia,orificeheight,orificecoef,
						  weirwidth,weirheight,weirangle,cisternoutflow,soildepth,
						  soilporosity,finalf,vegparma,holtpar,udfinalf,udsoildepth,
						  udsoilporosity,FC,WP,ETrate*PETmultiplier,AET,perc,ovolume,
						  ostage,infilt,orifice,weir,osa,ostorage,udout,seepage,pump,
						  pBMP,pBMPSite->m_nFtableFlag,pBMPSite->m_pFtable);

					ovolume  *= ndevice;	//ft3
					oinflow2 *= ndevice;	//ft3
					weir     *= ndevice;	//cfs
					pump     *= ndevice;	//cfs
					orifice  *= ndevice;	//cfs
					udout    *= ndevice;	//cfs
					AET      *= ndevice;	//cfs
					infilt   *= ndevice;	//cfs
					perc     *= ndevice;	//cfs
					seepage  *= ndevice;	//cfs

					// add pump outflow to underdrain outflow
					udout += pump;
				}
				else // if area is 0 then outflow = inflow 
				{
					utout = oinflow/delts;//total inflow is untreated (cfs)
					ovolume = 0.0;	// no stored volume
					weir = 0;
					pump = 0;
					orifice = 0.0;
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
					ostage = 0.0;
				}
				groundwater += seepage*delts; // ft3
			}
			else if (pBMPSite->m_nBMPClass == CLASS_B && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
			{
				// initialize class B BMP related parameters
				BMP_B* pBMP = (BMP_B*) pBMPSite->m_pSiteProp;
				double BMPlength = pBMP->m_lfBasinLength;	// ft
				double BMPwidth  = pBMP->m_lfBasinWidth;	// ft
				double BMPdepth  = pBMP->m_lfMaximumDepth;	// ft
				double slope1    = pBMP->m_lfSideSlope1;
				double slope2    = pBMP->m_lfSideSlope2;
				double slope3    = pBMP->m_lfSideSlope3;
				double man_n     = pBMP->m_lfManning;
				double PETmultiplier  = pBMP->m_lfPETmultiplier;

				//use Ftable if provided for bottom surface area
				if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
					BMParea = pBMPSite->m_pFtable->m_lfArea[0];//ft2
				else
					BMParea = BMPlength * BMPwidth;	// ft2	
				
				//calculate the top width (ft)
				double top_width = (BMPdepth/slope1 + BMPdepth/slope2 + BMPwidth); 

				if (BMParea > 0 && BMPdepth > 0 && ndevice > 0)		
				{
					//check the design drainage area
					if (lfDDarea > 0 && lfAccDArea > (lfDDarea*ndevice))
					{
						utout = oinflow*(lfAccDArea-(lfDDarea*ndevice))/lfAccDArea;//ft3/ivl
						oinflow2 = oinflow - utout;//ft3/ivl
						utout /= delts;//cfs 
					}
					else
					{
						utout = 0.0;//cfs
						oinflow2 = oinflow;//ft3/ivl
					}

					ovolume /= ndevice;//ft3
					oinflow2 /= ndevice;//ft3/ivl

					//check if volume exceeds the maximum volume given in Ftable
					if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
					{
						int nLast = pBMPSite->m_pFtable->m_nRecords-1;
						if((ovolume+oinflow2) > pBMPSite->m_pFtable->m_lfVolume[nLast])
						{
							double extravolume = (ovolume+oinflow2) - pBMPSite->m_pFtable->m_lfVolume[nLast];//ft3/ivl
							double extrainflow = max(0.,extravolume - ovolume);//ft3/ivl
							//update inflow and untreated flow
							oinflow2 -= extrainflow;//ft3/ivl
							utout += (extrainflow*ndevice/delts);//cfs
						}
					}

					bmp_b(nInfiltMethod,nInfil_Index,underdrain_on,Landtimestep,
						  BMPtimestep,oinflow2,BMParea,BMPdepth,BMPwidth,BMPlength,
						  slope1,slope2,slope3,man_n,soildepth,soilporosity,
						  finalf,vegparma,holtpar,udfinalf,udsoildepth,
						  udsoilporosity,FC,WP,ETrate*PETmultiplier,AET,perc,
						  ovolume,ostage,infilt,channel,weir,osa,ostorage,udout,
						  seepage,pBMPSite->m_nFtableFlag,pBMPSite->m_pFtable);

					ovolume  *= ndevice;//ft3
					oinflow2 *= ndevice;//ft3
					weir     *= ndevice;//cfs
					channel  *= ndevice;//cfs
					udout    *= ndevice;//cfs
					AET      *= ndevice;//cfs
					infilt   *= ndevice;//cfs
					perc     *= ndevice;//cfs
					seepage  *= ndevice;//cfs
					orifice   = channel;
				}
				else // if area is 0 then outflow = inflow    
				{
					utout = oinflow/delts;//total inflow is untreated (cfs)
					ovolume = 0.0;	// no stored volume
					weir = 0;
					channel = 0.0;
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
					orifice = channel;
					ostage = 0.0;
				}
				groundwater += seepage*delts; // ft3
			}
			else if (pBMPSite->m_nBMPClass == CLASS_C)// && nRunMode != RUN_PREDEV) 
			{
				// initialize class C Conduit related parameters
				BMP_C* pBMP = (BMP_C*) pBMPSite->m_pSiteProp;

				int nIndex = pBMP->m_nIndex;
			    k = Link[nIndex].subIndex;

				if (Link[nIndex].xsect.type != DUMMY || 
					(Link[nIndex].xsect.type == DUMMY && pBMPSite->m_nFtableFlag > 0))
				{
					float qinflow = 0.0;	//cfs (link)
					float qoutflow = 0.0;	//cfs (link)
					float overflow = 0.0;	//cfs (link)
					float qout = 0.0;		//cfs (conduit)
					float qover = 0.0;		//cfs (conduit)
					float finflow = 0.;		//cfs
					utout = 0.;				//cfs
					oinflow2 = oinflow;		//ft3/ivl

					//check if volume exceeds the maximum volume given in Ftable
					if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
					{
						int nLast = pBMPSite->m_pFtable->m_nRecords-1;
						float LinkVolume = Link[nIndex].oldVolume;	//ft3
						float ConduitVolume = LinkVolume / (float)Conduit[k].barrels;//per barrel
						float ConduitInflow = (float)oinflow2 / (float)Conduit[k].barrels;//ft3/ivl
						
						if((ConduitVolume+ConduitInflow) > pBMPSite->m_pFtable->m_lfVolume[nLast])
						{
							double extravolume = (ConduitVolume+ConduitInflow) - pBMPSite->m_pFtable->m_lfVolume[nLast];//ft3/ivl
							double extrainflow = max(0.,extravolume - ConduitVolume);//ft3/ivl
							extrainflow *= (float)Conduit[k].barrels;//ft3/ivl
							//update inflow and untreated flow
							oinflow2 -= extrainflow;
							utout = extrainflow/delts;//cfs
						}
					}

					finflow = oinflow2 / delts;	// from ft3/ivl to cfs

					for (int ii=0; ii<nivl; ii++)	
					{
						double rsed1tot = rbsedtot[i];	// lb
						rsed1tot *= POUND2GRAM;			// gram
						rbsedtot[i] = 0.0;				// lb
						qinflow = finflow;				// cfs	
						qoutflow = 0.0;					// cfs
						double avvele = 0.0;
						double hrade = 0.0; 
						double avdepe = 0.0;
						double slope = 0.001;

						// --- replace old hydraulic state values with current ones
						link_setOldHydState(nIndex);

						//use Ftable if provided
						if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
						{
							float qin = 0.,ain = 0.,aout = 0.,xarea = 0.,wetP = 0.;
							float weir_out = 0., orif_out = 0.;
							float LinkVolume = Link[nIndex].oldVolume;	//ft3
							LinkVolume += (qinflow*tStep);
							float ConduitVolume = LinkVolume / (float)Conduit[k].barrels;//per barrel

							qin = qinflow / (float)Conduit[k].barrels;//cfs
							//calculate orifice/channel outflow (cfs)
							orif_out = pBMPSite->m_pFtable->Volume_lookup(ConduitVolume,pBMPSite->m_pFtable->m_lfOrificeOutflow);//cfs
							// calculate weir overflow (cfs)
							weir_out = pBMPSite->m_pFtable->Volume_lookup(ConduitVolume,pBMPSite->m_pFtable->m_lfWeirOutflow);//cfs
							
							//update the volume (ft3/ivl)
							if (orif_out*tStep > ConduitVolume)
								orif_out = ConduitVolume/tStep;

							ConduitVolume -= (orif_out*tStep);

							if (weir_out*tStep > ConduitVolume)
								weir_out = ConduitVolume/tStep;

							ConduitVolume -= (weir_out*tStep);

							// calculate inflow x-sectional area
							//ain = (qin * tStep) / pBMPSite->m_pFtable->m_lfLength;//ft2
							ain = Conduit[k].a1;//previous timestep
							// calculate outflow x-sectional area
							//aout = (orif_out * tStep) / pBMPSite->m_pFtable->m_lfLength;//ft2
							aout = ConduitVolume / pBMPSite->m_pFtable->m_lfLength;//ft2
							// --- save new flows and areas
							Conduit[k].q1 = qin;
							Conduit[k].a1 = ain;
							Conduit[k].q2 = orif_out;
							Conduit[k].a2 = aout;
							qinflow = Conduit[k].q1 * (float)Conduit[k].barrels;
							qoutflow = Conduit[k].q2 * (float)Conduit[k].barrels;
							overflow = weir_out * (float)Conduit[k].barrels;

							qout  += qoutflow;	// cfs
							qover += overflow;	// cfs

							Link[nIndex].newFlow = qoutflow; //cfs
							Link[nIndex].newVolume = ConduitVolume * (float)Conduit[k].barrels;//ft3
							
							//check if volume exceeds the maximum volume given in Ftable
							int nLast = pBMPSite->m_pFtable->m_nRecords-1;
							if(ConduitVolume > pBMPSite->m_pFtable->m_lfVolume[nLast])
								Link[nIndex].newDepth = pBMPSite->m_pFtable->m_lfDepth[nLast]+
								(ConduitVolume - pBMPSite->m_pFtable->m_lfVolume[nLast])/
								pBMPSite->m_pFtable->m_lfArea[nLast];
							else
								//calculate the water depth from the given Ftable
								Link[nIndex].newDepth = pBMPSite->m_pFtable->Volume_lookup(ConduitVolume,pBMPSite->m_pFtable->m_lfDepth);//ft;
							
							xarea = ConduitVolume / pBMPSite->m_pFtable->m_lfLength;//ft2
							wetP = pBMPSite->m_pFtable->Volume_lookup(ConduitVolume,pBMPSite->m_pFtable->m_lfWetPerimeter);//ft;
							//calculate hydraulic radius of a conduit
							if (wetP > 0)
								hrade = xarea / wetP;	//ft
							else
								hrade = 0.;
							//calculate average velocity in a conduit
							if (xarea > 0)
								avvele = Conduit[k].q2 / xarea;//ft/s
							else
								avvele = 0.;

							avdepe = Link[nIndex].newDepth;	// ft
							slope = pBMPSite->m_pFtable->m_lfSlope;//ft/ft
						}
						else
						{
							// routing using KW model
							kinwave_execute(nIndex, &qinflow, &qoutflow,tStep);
							
							if (qinflow > finflow)
								overflow = 0;
							else
								// calculate overflow (cfs)
								overflow = max(0.0, finflow - qinflow);

							Link[nIndex].newFlow = qoutflow; //cfs

							qout  += qoutflow;	// cfs
							qover += overflow;	// cfs

							//updates state of link after current time step 
							// --- find avg. depth from entry/exit conditions
							float a = 0.5 * (Conduit[k].a1 + Conduit[k].a2);   // avg. area
							Link[nIndex].newVolume = max(0.0, a * Conduit[k].length * (float)Conduit[k].barrels);
							
							float y1 = xsect_getYofA(&Link[nIndex].xsect, Conduit[k].a1);
							float y2 = xsect_getYofA(&Link[nIndex].xsect, Conduit[k].a2);
							Link[nIndex].newDepth = 0.5 * (y1 + y2);
							// --- get velocity (ft/sec)
							avvele = link_getVelocity(nIndex, Link[nIndex].newFlow, Link[nIndex].newDepth);//ft/s
							avdepe = Link[nIndex].newDepth;	// ft
							hrade = getHydRad(&Link[nIndex].xsect, avdepe);	//ft
							slope = Conduit[k].slope;
						}

						double avdepm = avdepe * FOOT2METER;	// m
						double hradm = hrade * FOOT2METER;	// m
						double tw = 20.0;//degreeC

						double ros  = Conduit[k].q2Old * (float)Conduit[k].barrels;	//cfs
						double ro   = Conduit[k].q2 * (float)Conduit[k].barrels;		//cfs
						double rosm = ros * CFS2CMS;//m3/s
						double rom  = ro * CFS2CMS;	//m3/s
						double vols = Link[nIndex].oldVolume;	//ft3
						double vol  = Link[nIndex].newVolume;	//ft3
						double volsm = vols * CF2CM;//m3
						double volm  = vol * CF2CM;	//m3
						
						double js   = 0.0;

						if (fabs(ros) > fThreshold)
						{
							double rat = vols / (ros * tStep);
							if (rat < crrat)
								js = rat / crrat;
							else
								js = 1.0;
						}

						double cojs    = 1.0 - js;
						double srovol  = js * ros * tStep;	//ft3
						double erovol  = cojs * ro * tStep;	//ft3
						double srovolm = srovol * CF2CM;	//m3
						double erovolm = erovol * CF2CM;	//m3

						// calculate the water quality
						for (j=0; j<NWQ; j++)
						{
							//replaces old water quality state values with current ones
							Link[nIndex].oldQual[j] = Link[nIndex].newQual[j];// lb/ft3
							double conc   = Link[nIndex].oldQual[j];	// lb/ft3
							double concs  = Link[nIndex].oldQual[j];	// lb/ft3
							double massin = BmpWqInput[i*NWQ+j]/nivl;	// lb/ivl
							double massin2 = massin;			// lb/ivl
							double rsed1  = rbsed[i*NWQ+j];		// lb
							double romat  = 0.0;				// lb/ivl

							//check if any bypass
							if (overflow > 0 && finflow > 0)
							{
								massin2 = massin * (finflow - overflow)/ finflow;	// lb/ivl
								romat_ut2[i*NWQ+j] += max(0.0, massin - massin2);	// lb/ivl
							}

							if (pBMPData->nSedflag[j] == SAND)
							{
								//simulate sand
								int    sandfg = 3;	// user-specified power function method
								double ksand  = pBMPSite->m_sand.m_lfKSAND;
								double expsnd = pBMPSite->m_sand.m_lfEXPSND;
								double db50e  = pBMPSite->m_sand.m_lfD/12.0;	//ft
								double db50m  = db50e * 304.8;					// mm
								double w = pBMPSite->m_sand.m_lfW*0.0254*tStep;	// fall velocity (m/ivl)
								double wsande = w * 3.28 / tStep;				// m/ivl to ft/sec
								double depscr = 0., rsed = 0.;
								
								//dummy variables (not required for sandfg = 3)
								double twide = 0.,fsl = 0.;

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2 *= POUND2GRAM;	//gram/ivl
									rsed1   *= POUND2GRAM;	//gram
									conc    *= LBpCFT2MGpL;	//mg/l

									sandld(massin2,volsm,srovolm,volm,erovolm,ksand,avvele,expsnd,rom,sandfg,db50e,hrade,
										   slope,tw,wsande,twide,db50m,fsl,avdepe,&conc,&rsed,&rsed1,&depscr,&romat);

									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	(g)
											rsed1  += rsed;	// mg/l*m3	(g)
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	//lb/ft3
									romat /= POUND2GRAM;	//lb/ivl
									rsed1 /= POUND2GRAM;	//lb
								}
								else
								{
									// conduit has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving conduit 
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								Link[nIndex].newQual[j] = conc;	// lb/ft3
								romat_o2[i*NWQ+j] += romat;		// lb
								rbsed[i*NWQ+j] = rsed1;			// lb
							}
							else if (pBMPData->nSedflag[j] == SILT)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_silt.m_lfW*0.0254*tStep;	// fall velocity (m/ivl)
								double taucd = pBMPSite->m_silt.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m2)
								double taucs = pBMPSite->m_silt.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m2)
								//convert erodibility coeff from lb/ft2/day to kg/m2/ivl
								double m = pBMPSite->m_silt.m_lfM*deltd/nivl*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m2/ivl)
								double gamma = 1000;	// kg/m3	 
								double tau = 0;			// (kg/m2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram/ivl
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,(double)tStep,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
									
									if (avdepm > 0.0)
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;	// (kg/m2)

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb/ivl
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// conduit has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving conduit 
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								Link[nIndex].newQual[j] = conc;	// lb/ft3
								romat_o2[i*NWQ+j] += romat;		// lb
								rbsed[i*NWQ+j] = rsed1;			// lb
								rbsedtot[i] += rsed1;			// lb
							}
							else if (pBMPData->nSedflag[j] == CLAY)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_clay.m_lfW*0.0254*tStep;	// fall velocity (m/ivl)
								double taucd = pBMPSite->m_clay.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m2)
								double taucs = pBMPSite->m_clay.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m2)
								
								//convert erodibility coeff from lb/ft2/day to kg/m2/ivl
								double m = pBMPSite->m_clay.m_lfM*deltd/nivl*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m2/ivl)
								double gamma = 1000;	// kg/m3	 
								double tau = 0;			// kg/m2		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram/ivl
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,(double)tStep,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
									
									if (avdepm > 0.0)
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;	// (kg/m2)

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb/ivl
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// conduit has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving conduit 
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								Link[nIndex].newQual[j] = conc;	// lb/ft3
								romat_o2[i*NWQ+j] += romat;		// lb
								rbsed[i*NWQ+j] = rsed1;			// lb
								rbsedtot[i] += rsed1;			// lb
							}
							else	// not sediment
							{
								double decay  = pBMPSite->m_pDecay[j]/3600.;	// per sec

								if (fabs(volm) > fThreshold)
								{
									//findLinkQual(nIndex, delts);
									findLinkQual2(nIndex,tStep,massin2,decay,conc);

									// calculate total amount of material leaving conduit 
									// during the interval  
									if (ro > 0)
									{
										romat = srovol * concs + erovol * conc; //	lb/ivl
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}
								else
								{
									// conduit has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving conduit 
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update conc
								Link[nIndex].newQual[j] = conc;	// lb/ft3
								romat_o2[i*NWQ+j] += romat;		// lb
							}
						}
					}

					if (nivl > 0)
					{
						qoutflow = qout / nivl;
						overflow = qover / nivl;
					}

					utout += overflow;					// cfs
					ovolume = Link[nIndex].newVolume;	// ft3
					ostage = Link[nIndex].newDepth;		// ft
					channel = qoutflow;					// cfs
					weir = 0.0;							// cfs
					orifice = channel;					// cfs
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
				}
				else // if area is 0 then outflow = inflow    
				{
					utout = oinflow / delts;	//total inflow is untreated (cfs)
					ovolume = 0.0;	// no stored volume
					weir = 0;
					orifice = 0.0;
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
					ostage = 0.0;
				}
			}
			else if (pBMPSite->m_nBMPClass == CLASS_D && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
			{
				// initialize class D related parameters
				BMP_D* pBMP = (BMP_D*) pBMPSite->m_pSiteProp;

				double basinlength    = pBMP->m_lfFlowLength;		// ft
				double basinwidth     = pBMP->m_lfBufferWidth;		// ft
				double PETmultiplier  = pBMP->m_lfPETmultiplier;

				BMParea = basinlength * basinwidth;	// ft2	
	
				if (BMParea > 0 && ndevice > 0)		
				{
					//check the design drainage area
					if (lfDDarea > 0 && lfAccDArea > (lfDDarea*ndevice))
					{
						utout = oinflow*(lfAccDArea-(lfDDarea*ndevice))/lfAccDArea;//ft3/ivl
						oinflow2 = oinflow - utout;//ft3/ivl
						utout /= delts;//cfs 
					}
					else
					{
						utout = 0.0;//cfs
						oinflow2 = oinflow;//ft3/ivl
					}

					ovolume /= ndevice;//ft3
					oinflow2 /= ndevice;//ft3/ivl

					bmp_d(nInfiltMethod,nInfil_Index,Landtimestep,BMPtimestep,oinflow2,BMParea,soildepth,
						  soilporosity,finalf,vegparma,holtpar,FC,WP,ETrate*PETmultiplier,AET,ovolume,ostage,
						  infilt,weir,osa,seepage,pBMPSite);

					ovolume  *= ndevice;	//ft3
					oinflow2 *= ndevice;	//ft3
					weir     *= ndevice;	//cfs
					orifice = 0.0;
					udout = 0;
					AET      *= ndevice;	//cfs
					infilt   *= ndevice;	//cfs
					perc = 0;
					seepage  *= ndevice;	//cfs
				}
				else // if area is 0 then outflow = inflow 
				{
					utout = oinflow/delts;//total inflow is untreated (cfs)
					ovolume = 0.0;	// no stored volume
					weir = 0;
					orifice = 0.0;
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
					ostage = 0.0;
				}
				groundwater += seepage*delts; // ft3
			}
			else if (pBMPSite->m_nBMPClass == CLASS_E && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
			{
				BMP_E* pBMP = (BMP_E*) pBMPSite->m_pSiteProp;
				float fBMParea = pBMP->m_lfsArea;	//ft2
				float fArea = (100.0-(float)pBMP->m_lfDCIA)/100.0;	// fraction of disconnected impervious area (0-1)
				float oinflow_per = float(BmpFlowInput_per[i]*INACRE2CF/delts);	// acre-in/ivl to cfs

				//route the upstream BMP (e.g. Rain Barrel) outflow through the area BMP
				float oinflow_imp = (oinflow / delts) - oinflow_per; //cfs

				if (fBMParea > 0 && fArea > 0)
				{
					float precip = 0.0;
					float evap = 0.0;
					float infil = 0.0;
					float depth = 0.0;
					float runoff = 0.0;
					float fdelts = BMPtimestep * 60.0;	// sec/BMPivl 
					float qin = 0.0;				// cfs
					float qout  = 0.0;				// cfs
					float qinf  = 0.0;				// cfs
					float dep_av = 0.0;				// ft
					double qoutflow = 0.0;			// cfs
					double overflow = 0.0;			// cfs
					double qinfilt = 0.0;			// cfs
					oinflow2 = 0.0;					//ft3/hr

					if (oinflow_per < 0.001)		// route impervious runoff when pervious runoff less than 0.001 cfs
					{
						if (pBMP->m_lfdepth > 0 || oinflow_imp > 0.001)
						{
							float inflow = oinflow_imp * fArea / fBMParea;	// ft/sec
							
							for (int ii=0; ii<nivl; ii++)	
							{
								qin += (inflow * fBMParea); //cfs
								oinflow2 += oinflow_imp * fArea * fdelts;		//ft3/ivl

								depth = pBMP->m_lfdepth; //ft
								runoff = pBMP->m_lfrunoff; //ft/sec

								pBMPSite->getSubareaRunoff2(inflow, precip, evap, fdelts, &infil, &depth, &runoff);

								pBMP->m_lfdepth = depth; //ft
								pBMP->m_lfrunoff = runoff; //ft/sec

								qout += (runoff * fBMParea); //cfs
								qinf += (infil * fBMParea); //cfs
								dep_av += depth; //ft
							}
							
							if (nivl > 0)
							{
								qinfilt = double(qinf / nivl); //cfs
								qoutflow = double(qout / nivl); //cfs
								overflow = (oinflow / delts) - double(qin / nivl); //cfs
								overflow = max(0.0, overflow);
								dep_av /= nivl; //ft
							}
						}
						else
						{
							//there is no surface runoff from the impervious area and no ponded water for infiltration
							overflow = (oinflow / delts); //cfs
						}
					}
					else
					{
						//there is surface runoff and no potential for further infiltration
						overflow = (oinflow / delts); //cfs
					}
					
					if (dep_av == 0.0)
						dep_av = pBMP->m_lfdepth; //ft

					utout = overflow; // cfs
					channel = qoutflow; // cfs
					weir = 0.0; //cfs
					orifice = channel; //cfs
					ovolume = double(dep_av * fBMParea); //ft3
					udout = 0;
					AET = 0;
					infilt = qinfilt; //cfs
					perc = qinfilt; //cfs
					seepage = qinfilt; //cfs
					ostage = double(dep_av); //ft
				}
				else // if area is 0 then outflow = inflow    
				{
					utout = oinflow/delts;//total inflow is untreated (cfs)
					ovolume = 0.0;	// no stored volume
					weir = 0;
					orifice = 0.0;
					udout = 0;
					AET = 0;
					infilt = 0;
					perc = 0;
					seepage = 0;
					ostage = 0.0;
				}
				groundwater += seepage*delts; // ft3
			}
			else   //Clase X		// Dummy BMP
			{
				utout = oinflow/delts;//total inflow is untreated (cfs)
				ovolume = 0.0;	// no stored volume
				weir = 0;
				orifice = 0.0;
				udout = 0;
				AET = 0;
				infilt = 0;
				perc = 0;
				seepage = 0;
				ostage = 0.0;
			}

			//update the groundwater storage
			if (pBMPSite->m_pAquifer != NULL)
				pBMPSite->m_pAquifer->m_lfGWStorage_u = groundwater/ACRE2SF; //ft3 to ac-ft

			qout1= weir_p[i] + orifice_p[i] + undrain_p[i] + seepage_p[i]; //cfs
			qout2= weir + orifice + udout + seepage;//cfs
			bmpout = weir + orifice + udout + utout;//cfs
			
			vol1 = vol_p[i];//ft3		
			vol2 = max(0.0, vol1 + oinflow2 - (qout2 + AET) * delts);	//ft3		

			double value1 = bmpout*delts; //ft3/s to ft3/ivl
			double value0 = pBMPSite->qFlow.front();
			lfSumFlow[i] -= value0;
			lfSumFlow[i] += value1;

			lfDailyFlow[i] += bmpout*delts; //ft3/s to ft3/ivl

			double lfFlowVolume = floor(lfSumFlow[i]+0.5);
			pBMPSite->qFlow.pop();
			pBMPSite->qFlow.push(value1);
			
			// evaluation factor calculation
			if (pBMPSite->m_factorList.GetCount() > 0)
			{
				bmpTotalFlow[i] += (bmpout*delts);//cumulative hourly flow for BMP (ft3/ivl)
				
				if (bmpPkDisFlow[i] < bmpout)
					bmpPkDisFlow[i] = bmpout;// keep track of peak discharge for the entire simulation period (cfs)
				
				//flow duration curve
				if (nRunMode == RUN_PREDEV && pBMPSite->m_nPreDevFlag == 1)
				{
					//internal predeveloped timeseries
					if(pBMPSite->m_pPreDevFlow!= NULL)	
						pBMPSite->m_pPreDevFlow[t] = bmpout;//cfs
					if(bmpout < pBMPSite->m_lfMinFlow)
						pBMPSite->m_lfMinFlow = bmpout;
					if(bmpout > pBMPSite->m_lfMaxFlow)
						pBMPSite->m_lfMaxFlow = bmpout;
				}
				else if (nRunMode != RUN_PREDEV && pBMPSite->m_nPreDevFlag > 0)
				{
					if(pBMPSite->m_pFlow!= NULL)	
						pBMPSite->m_pFlow[t] = bmpout;//cfs
					if(bmpout < pBMPSite->m_lfMinFlow)
						pBMPSite->m_lfMinFlow = bmpout;
					if(bmpout > pBMPSite->m_lfMaxFlow)
						pBMPSite->m_lfMaxFlow = bmpout;
				}

				if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
				{
					int nBinID = pBMPData->pBINDATA[t].nBinID;
					bmpTotalFlow_bin[i*nNumBins+nBinID] += (bmpout*delts);//cumulative hourly flow for BMP (ft3/ivl)

					if (bmpPkDisFlow_bin[i*nNumBins+nBinID] < bmpout)
						bmpPkDisFlow_bin[i*nNumBins+nBinID] = bmpout;// keep track of peak discharge for the entire simulation period (cfs)
				}
				if (pBMPSite->m_nInterEvent == -1)
				{
					if (hour == 23) //end of the day
					{
						//arithmetic daily average
						double DailyFlowRate = lfDailyFlow[i]/(24.*3600.); //cfs
						if (DailyFlowRate > pBMPSite->m_lfThreshFlow)
						{
							if (nExceedFlag[i] == 0)	// new overflow event
							{
								nExceedFlow[i]++;		// count the exceeding flow over threshold value
								if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
								{
									int nBinID = pBMPData->pBINDATA[t].nBinID;
									nExceedFlow_bin[i*nNumBins+nBinID]++;	// count the exceeding flow over threshold value
								}							
							}
							nExceedFlag[i] = 1;
						}
						else
						{
							nExceedFlag[i] = 0;
						}
						//initialize 
						lfDailyFlow[i] = 0.0;
					}
				}
				else if (pBMPSite->m_nInterEvent == 0)
				{
					//at running daily average
					double DailyFlowRate = lfSumFlow[i]/(24.*3600.); //cfs
					if (DailyFlowRate > pBMPSite->m_lfThreshFlow)
					{
						if (nExceedFlag[i] == 0)	// new overflow event
						{
							nExceedFlow[i]++;		// count the exceeding flow over threshold value
							if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
							{
								int nBinID = pBMPData->pBINDATA[t].nBinID;
								nExceedFlow_bin[i*nNumBins+nBinID]++;	// count the exceeding flow over threshold value
							}							
						}
						nExceedFlag[i] = 1;
					}
					else
					{
						nExceedFlag[i] = 0;
					}
				}
				else
				{
					//at timestep interval 
					if (bmpout > pBMPSite->m_lfThreshFlow)
					{
						if (nExceedFlag[i] == 0)	// new overflow event
						{
							nExceedFlow[i]++;		// count the exceeding flow over threshold value
							if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
							{
								int nBinID = pBMPData->pBINDATA[t].nBinID;
								nExceedFlow_bin[i*nNumBins+nBinID]++;	// count the exceeding flow over threshold value
							}							
						}
						nExceedFlag[i] = 1;
						nInterEvent[i] = 0;			// reset the inter-event counter
					}
					else
					{
						if (nInterEvent[i] >= pBMPSite->m_nInterEvent * 60)	//min
							nExceedFlag[i] = 0;
						nInterEvent[i] += Landtimestep;	// add a time step to the inter-event counter
					}
				}
			}
				
			// Pollutants 
			int	nRoutingMethod = pBMPSite->m_nPolRotMethod;	
			int	nRemovalMethod = pBMPSite->m_nPolRemMethod;	
			double rsed1tot = rbsedtot[i];	// lb
			rsed1tot *= POUND2GRAM;			// gram
			rbsedtot[i] = 0.0;				// lb

			for (int jj=0; jj<pBMPData->m_nSedQualFlag+1; jj++)
			{
				//if sediment associated qual then first simulate sediment
				//otherwise simulate the water quality
				int nPollIndex = 0;

				for (j=0; j<NWQ; j++)
				{
					double massin = BmpWqInput[i*NWQ+j];//lb/hr
					double massin2 = massin;//lb/hr
					double conc      = 0.0;
					double romat     = 0.0;
					double romat_w   = 0.0;//weir
					double romat_o   = 0.0;//orifice
					double romat_i   = 0.0;//infiltration
					double romat_ud  = 0.0;//underdrain
					double romat_ut  = 0.0;//bypass
					double decay     = 0.0;
					double lfK       = 0.0;
					double lfCstar   = 0.0;
					double BMPAREA   = 0.0;

					//check if any sediment-associated qual
					if (pBMPData->m_nSedQualFlag == 1 && jj == 0)
					{
						//need to simulate sediment before other quals
						if (pBMPData->nSedflag[j] == 0)	//not sediment
						{
							nPollIndex++;
							continue;
						}
					}
					else if (pBMPData->m_nSedQualFlag == 1 && jj == 1)
					{
						//sediment is already simulated
						if (pBMPData->nSedflag[j] != 0) //sediment
						{
							if (NWQ > NPOL)	// there is split of total sediment
								if (pBMPData->nSedflag[j] == SILT || pBMPData->nSedflag[j] == CLAY)
									nPollIndex--;
							nPollIndex++;
							continue;
						}

						//sediment-associated pollutant
						if (pBMPData->m_pPollutant[nPollIndex].m_nSedQual == 1)	
						{
							//find the index for sediment classes
							int nSandIndex = -1;
							int nSiltIndex = -1;
							int nClayIndex = -1;
							for (int kk=0; kk<NWQ; kk++)
							{
								if (pBMPData->nSedflag[kk] == SAND)
									nSandIndex = kk;
								else if (pBMPData->nSedflag[kk] == SILT)
									nSiltIndex = kk;
								else if (pBMPData->nSedflag[kk] == CLAY)
									nClayIndex = kk;
							}

							//find the BMP performance for sediment
							double lfEfficiency_Sand = 0;
							double lfEfficiency_Silt = 0;
							double lfEfficiency_Clay = 0;

							double lfBmpWqInput_Sand = 0;
							double lfBmpWqInput_Silt = 0;
							double lfBmpWqInput_Clay = 0;

							double lfbmpmassout_Sand = 0;
							double lfbmpmassout_Silt = 0;
							double lfbmpmassout_Clay = 0;

							double lfbmpmassout_ut_Sand = 0;
							double lfbmpmassout_ut_Silt = 0;
							double lfbmpmassout_ut_Clay = 0;

							double lfbmpmassout_w_Sand = 0;
							double lfbmpmassout_w_Silt = 0;
							double lfbmpmassout_w_Clay = 0;

							double lfbmpmassout_o_Sand = 0;
							double lfbmpmassout_o_Silt = 0;
							double lfbmpmassout_o_Clay = 0;

							double lfbmpmassout_ud_Sand = 0;
							double lfbmpmassout_ud_Silt = 0;
							double lfbmpmassout_ud_Clay = 0;

							double lfsand_qfr = pBMPData->m_pPollutant[nPollIndex].m_lfsand_qfr;
							double lfsilt_qfr = pBMPData->m_pPollutant[nPollIndex].m_lfsilt_qfr;
							double lfclay_qfr = pBMPData->m_pPollutant[nPollIndex].m_lfclay_qfr;
							
							if (nSandIndex != -1)
							{
								lfBmpWqInput_Sand = BmpWqInput[i*NWQ+nSandIndex];
								lfbmpmassout_Sand = bmpmassout[i*NWQ+nSandIndex];
								lfbmpmassout_ut_Sand = bmpmassout_ut[i*NWQ+nSandIndex];
								lfbmpmassout_w_Sand = bmpmassout_w[i*NWQ+nSandIndex];
								lfbmpmassout_o_Sand = bmpmassout_o[i*NWQ+nSandIndex];
								lfbmpmassout_ud_Sand = bmpmassout_ud[i*NWQ+nSandIndex];
							}
							if (nSiltIndex != -1)
							{
								lfBmpWqInput_Silt = BmpWqInput[i*NWQ+nSiltIndex];
								lfbmpmassout_Silt = bmpmassout[i*NWQ+nSiltIndex];
								lfbmpmassout_ut_Silt = bmpmassout_ut[i*NWQ+nSiltIndex];
								lfbmpmassout_w_Silt = bmpmassout_w[i*NWQ+nSiltIndex];
								lfbmpmassout_o_Silt = bmpmassout_o[i*NWQ+nSiltIndex];
								lfbmpmassout_ud_Silt = bmpmassout_ud[i*NWQ+nSiltIndex];
							}
							if (nClayIndex != -1)
							{
								lfBmpWqInput_Clay = BmpWqInput[i*NWQ+nClayIndex];
								lfbmpmassout_Clay = bmpmassout[i*NWQ+nClayIndex];
								lfbmpmassout_ut_Clay = bmpmassout_ut[i*NWQ+nClayIndex];
								lfbmpmassout_w_Clay = bmpmassout_w[i*NWQ+nClayIndex];
								lfbmpmassout_o_Clay = bmpmassout_o[i*NWQ+nClayIndex];
								lfbmpmassout_ud_Clay = bmpmassout_ud[i*NWQ+nClayIndex];
							}

							if (lfBmpWqInput_Sand > 0)
							{
								lfEfficiency_Sand = 1 - (lfBmpWqInput_Sand-lfbmpmassout_Sand)/lfBmpWqInput_Sand;
							}
							if (lfBmpWqInput_Silt > 0)
							{
								lfEfficiency_Silt = 1 - (lfBmpWqInput_Silt-lfbmpmassout_Silt)/lfBmpWqInput_Silt;
							}
							if (lfBmpWqInput_Clay > 0)
							{
								lfEfficiency_Clay = 1 - (lfBmpWqInput_Clay-lfbmpmassout_Clay)/lfBmpWqInput_Clay;
							}

							//calculate the sed-associated qual load
							double qconc_sand = bmpqconc_sand[i*NWQ+j];// lb/lb
							double qconc_silt = bmpqconc_silt[i*NWQ+j];// lb/lb
							double qconc_clay = bmpqconc_clay[i*NWQ+j];// lb/lb

							//check if there is no inflow but there is outflow
							if (massin == 0 && lfbmpmassout_Sand + lfbmpmassout_Silt + lfbmpmassout_Clay > 0)
							{
								romat = qconc_sand * lfbmpmassout_Sand + qconc_silt * lfbmpmassout_Silt + qconc_clay * lfbmpmassout_Clay;
							}
							else
							{
								romat = massin * (lfsand_qfr*lfEfficiency_Sand 
									+ lfsilt_qfr*lfEfficiency_Silt 
									+ lfclay_qfr*lfEfficiency_Clay);
							}

							double lfsandout = 0.0;
							double lfsiltout = 0.0;
							double lfclayout = 0.0;

							if (lfbmpmassout_Sand > 0)
								lfsandout = lfbmpmassout_ut_Sand/lfbmpmassout_Sand;
							if (lfbmpmassout_Silt > 0)
								lfsiltout = lfbmpmassout_ut_Silt/lfbmpmassout_Silt;
							if (lfbmpmassout_Clay > 0)
								lfclayout = lfbmpmassout_ut_Clay/lfbmpmassout_Clay;

							romat_ut = romat * (lfsand_qfr*lfsandout 
								+ lfsilt_qfr*lfsiltout + lfclay_qfr*lfclayout);

							lfsandout = 0.0;
							lfsiltout = 0.0;
							lfclayout = 0.0;

							if (lfbmpmassout_Sand > 0)
								lfsandout = lfbmpmassout_w_Sand/lfbmpmassout_Sand;
							if (lfbmpmassout_Silt > 0)
								lfsiltout = lfbmpmassout_w_Silt/lfbmpmassout_Silt;
							if (lfbmpmassout_Clay > 0)
								lfclayout = lfbmpmassout_w_Clay/lfbmpmassout_Clay;

							romat_w = romat * (lfsand_qfr*lfsandout 
								+ lfsilt_qfr*lfsiltout + lfclay_qfr*lfclayout);

							lfsandout = 0.0;
							lfsiltout = 0.0;
							lfclayout = 0.0;

							if (lfbmpmassout_Sand > 0)
								lfsandout = lfbmpmassout_o_Sand/lfbmpmassout_Sand;
							if (lfbmpmassout_Silt > 0)
								lfsiltout = lfbmpmassout_o_Silt/lfbmpmassout_Silt;
							if (lfbmpmassout_Clay > 0)
								lfclayout = lfbmpmassout_o_Clay/lfbmpmassout_Clay;

							romat_o = romat * (lfsand_qfr*lfsandout 
								+ lfsilt_qfr*lfsiltout + lfclay_qfr*lfclayout);

							lfsandout = 0.0;
							lfsiltout = 0.0;
							lfclayout = 0.0;

							if (lfbmpmassout_Sand > 0)
								lfsandout = lfbmpmassout_ud_Sand/lfbmpmassout_Sand;
							if (lfbmpmassout_Silt > 0)
								lfsiltout = lfbmpmassout_ud_Silt/lfbmpmassout_Silt;
							if (lfbmpmassout_Clay > 0)
								lfclayout = lfbmpmassout_ud_Clay/lfbmpmassout_Clay;

							romat_ud = romat * (lfsand_qfr*lfsandout 
								+ lfsilt_qfr*lfsiltout + lfclay_qfr*lfclayout);

							//calculate the sed-associated qual conc (lb of qual / lb of sed)
//							conc = bmpc[i*NWQ+j];// lb/lb
//							if (lfbmpmassout_Sand + lfbmpmassout_Silt + lfbmpmassout_Clay > 0)
//								conc = romat / (lfbmpmassout_Sand + lfbmpmassout_Silt + lfbmpmassout_Clay);

							//save values
							if (lfbmpmassout_Sand > 0)
								bmpqconc_sand[i*NWQ+j] = romat * lfsand_qfr / lfbmpmassout_Sand;//lb/lb
							if (lfbmpmassout_Silt > 0)
								bmpqconc_silt[i*NWQ+j] = romat * lfsilt_qfr / lfbmpmassout_Silt;//lb/lb
							if (lfbmpmassout_Clay > 0)
								bmpqconc_clay[i*NWQ+j] = romat * lfclay_qfr / lfbmpmassout_Clay;//lb/lb

//							goto MM;
						}
					}

					if (NWQ > NPOL)	// there is split of total sediment
						if (pBMPData->nSedflag[j] == SILT || pBMPData->nSedflag[j] == CLAY)
							nPollIndex--;

					//output variable
//					BmpWqInput_s[i*NPOL+nPollIndex] += massin;//lb/hr
					BmpWqInput_s[i*NWQ+j] += massin;//lb/hr

					//check for untreated over flow
					if (utout > 0 && oinflow > 0)
					{
						massin2 = massin * (oinflow - utout*delts)/oinflow;// lb/hr
						romat_ut = max(0.0, massin-massin2);//lb/hr
					}

					if (pBMPSite->m_nBMPClass == CLASS_A && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
					{
						BMP_A* pBMP = (BMP_A*) pBMPSite->m_pSiteProp;
						double basinlength    = pBMP->m_lfBasinLength;			// ft
						double basinwidth     = pBMP->m_lfBasinWidth;			// ft
						int    releasetype    = pBMP->m_nORelease;		

						BMPAREA = basinlength * basinwidth;	// ft^2	

						// check if this BMP is cistern or rainbarrel, if so then 
						// basinlength = diameter (ft) and basinwidth = number of devices
						if (releasetype == 1 || releasetype == 2)
							BMPAREA   = pi/4.0*pow(basinlength,2); // ft2 

						//use Ftable if provided for bottom surface area
						if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
							BMPAREA = pBMPSite->m_pFtable->m_lfArea[0];//ft2

						if (BMPAREA > SMALLNUM && ndevice > 0) 
						{
							double avdepe = ovolume/BMPAREA;// ft
							double ros  = weir_p[i] + orifice_p[i] + infilt_p[i];//cfs
							double ro   = weir + orifice + infilt;//cfs
							double rosm = ros * CFS2CMS;//m3/s
							double rom  = ro * CFS2CMS;	//m3/s
							double vols = bmpvol_p[i];//ft3
							double vol  = ovolume;//ft3
							double xarea = vol / pBMP->m_lfBasinLength;//ft2
							double avvele = ro / xarea;//ft/s
							double slope = 0.001;
							double hrade = xarea/(2*avdepe+pBMP->m_lfBasinWidth);//ft

							if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
							{
								//check if volume exceeds the maximum volume given in Ftable
								int nLast = pBMPSite->m_pFtable->m_nRecords-1;
								if(ovolume > pBMPSite->m_pFtable->m_lfVolume[nLast])
									avdepe = pBMPSite->m_pFtable->m_lfDepth[nLast]+
									(ovolume - pBMPSite->m_pFtable->m_lfVolume[nLast])/
									pBMPSite->m_pFtable->m_lfArea[nLast];
								else
									//calculate the water depth from the given Ftable
									avdepe = pBMPSite->m_pFtable->Volume_lookup(ovolume,pBMPSite->m_pFtable->m_lfDepth);//ft;

								slope = pBMPSite->m_pFtable->m_lfSlope;
								xarea = vol / pBMPSite->m_pFtable->m_lfLength;//ft2
								double wetP = pBMPSite->m_pFtable->Volume_lookup(vol,pBMPSite->m_pFtable->m_lfWetPerimeter);//ft;
								//calculate hydraulic radius of a conduit
								if (wetP > 0)
									hrade = xarea / wetP;	//ft
								else
									hrade = 0.;
								//calculate average velocity in a conduit
								if (xarea > 0)
									avvele = ro / xarea;//ft/s
								else
									avvele = 0.;
							}
							
							double volsm = vols * CF2CM;		//m3
							double volm  = vol * CF2CM;			//m3
							double avdepm = avdepe * FOOT2METER;//m
							double avvelm = avvele * FpS2MpS;	//m/s
							double hradm = hrade * FOOT2METER;	//m
							
							double tw = 20.0;	//degreeC
							double js   = 0.0;

							if (fabs(ros) > fThreshold)
							{
								double rat = vols / (ros * delts);
								if (rat < crrat)
									js = rat / crrat;
								else
									js = 1.0;
							}

							double cojs    = 1.0 - js;
							double srovol  = js * ros * delts;	//ft3
							double erovol  = cojs * ro * delts;	//ft3
							double srovolm = srovol * CF2CM;	//m3
							double erovolm = erovol * CF2CM;	//m3

							conc = bmpc[i*NWQ+j];				// lb/ft3
							double concs  = bmpc[i*NWQ+j];		// lb/ft3
							double rsed1  = rbsed[i*NWQ+j];		// lb
							//double rsed1tot = rbsedtot[i];		// lb

							//simulate sediment
							if (pBMPData->nSedflag[j] == SAND)
							{
								//simulate sand
								int    sandfg = 3;	// power function method
								double ksand  = pBMPSite->m_sand.m_lfKSAND;
								double expsnd = pBMPSite->m_sand.m_lfEXPSND;
								double db50e  = pBMPSite->m_sand.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double w = pBMPSite->m_sand.m_lfW*0.0254*delts;	// fall velocity (m/ivl)
								double wsande = w * 3.28 / delts; // from m/ivl to ft/sec
								double depscr = 0., rsed = 0.;
									
								//dummy variables (not required for sandfg = 3)
								double twide = 0.,fsl = 0.;

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2 *= POUND2GRAM;	//gram
									rsed1   *= POUND2GRAM;	//gram
									conc    *= LBpCFT2MGpL;	//mg/l

									sandld(massin2,volsm,srovolm,volm,erovolm,ksand,avvele, 
										   expsnd,rom,sandfg,db50e,hrade,slope,tw,wsande,
										   twide,db50m,fsl,avdepe,&conc,&rsed,&rsed1,&depscr,
										   &romat);

									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	(g)
											rsed1  += rsed;	// mg/l*m3	(g)
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	//lb/ft3
									romat /= POUND2GRAM;	//lb
									rsed1 /= POUND2GRAM;	//lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3
								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == SILT)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_silt.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_silt.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_silt.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_silt.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_silt.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double akappa = 0.4;	// Karman constant 
								double grav = 9.81;		// m/sec^2
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a lake- from "hydraulics
										//of sediment transport", by w.h. graf- eq.8.49
										double ustar = avvelm/(17.66+(log10(avdepm/(96.5*db50m)))*2.3/akappa);
										
										tau = gamma*pow(ustar,2)/grav;
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == CLAY)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_clay.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_clay.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_clay.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_clay.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_clay.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double akappa = 0.4;	// Karman constant 
								double grav = 9.81;		// m/sec^2
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a lake- from "hydraulics
										//of sediment transport", by w.h. graf- eq.8.49
										double ustar = avvelm/(17.66+(log10(avdepm/(96.5*db50m)))*2.3/akappa);
										
										tau = gamma*pow(ustar,2)/grav;
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else	// not sediment
							{
								if (nRoutingMethod <= 1)
								{
									conc = bmpc[i*NWQ+j];			//lb/ft3		
									
									advect(massin2,vol1,qout1,vol2,qout2,delts,crrat,conc,romat);

									if (nRemovalMethod == 0)
									{
										//1st order decay
										decay = pBMPSite->m_pDecay[j]*delth;//per ivl
										conc = conc * exp(-decay);
									}
									else if (nRemovalMethod == 1)
									{
										//kadlec and knight method
										//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
										lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
										lfCstar = pBMPSite->m_pCstar[j];//lb/ft3
										
										//maintain minimum conc of lfCstar
										conc = max(conc, lfCstar);

										double lfq = bmpout*delts/BMPAREA;
										if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
											lfq = pBMPSite->m_pFtable->Volume_lookup(bmpout*delts,pBMPSite->m_pFtable->m_lfDepth);//ft

										if (lfq > 0 && conc > lfCstar)
											conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
									}
								}
								else if (nRoutingMethod > 1)
								{
									//CSTRs in series routing
									int nCSTRs = nRoutingMethod;
									float v = vol1/nCSTRs;	//ft3
									float lfmassin = massin2/delts;	//lb/sec
									float lfinflow = oinflow2/delts;//cfs
									float lfoutflow = qout2;		//cfs

									for(k=0; k<nCSTRs; k++)
									{
										//calculate inputs for each CSTR
										float c = pBMPSite->m_pConc[j*nCSTRs+k];	//lb/ft3
										float wIn = lfmassin;						//lb/sec
										float qNet = lfinflow+(lfoutflow-lfinflow)*k/nCSTRs;		//cfs
										float qNetout = lfinflow+(lfoutflow-lfinflow)*(k+1)/nCSTRs;	//cfs

										//get the new conc (lb/ft3)
										conc = getCstrQual(c,v,wIn,qNet,(float)delts);

										//calculate massout (lb/ivl)
										romat = conc * qNetout * delts;

										if (nRemovalMethod == 0)
										{
											//1st order decay
											decay = pBMPSite->m_pDecay[j]*delth;//per ivl
											conc = conc * exp(-decay);	
										}
										else if (nRemovalMethod == 1)
										{
											//kadlec and knight method
											//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
											lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
											lfCstar = pBMPSite->m_pCstar[j];//lb/ft3

											//maintain minimum conc of lfCstar
											conc = max(conc, lfCstar);

											double lfq = lfoutflow*delts/BMPAREA;
											if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
												lfq = pBMPSite->m_pFtable->Volume_lookup(lfoutflow*delts,pBMPSite->m_pFtable->m_lfDepth);//ft
											if (lfq > 0 && conc > lfCstar)
												conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
										}

										//update conc
										pBMPSite->m_pConc[j*nCSTRs+k] = conc;

										//calculate massin to the next reactor(lb/sec)
										lfmassin = romat / delts;
									}
								}

								if (qout2 > 0)
								{
									romat_w = weir/qout2*romat;
									romat_o = orifice/qout2*romat;
									romat_ud = udout/qout2*romat;
									if (pBMPSite->m_pUndRemoval != NULL)
										romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
								}
							}
						}
						else
						{
							//no volume stored
							conc = 0.0;			
							romat_w = 0.0;
							romat_o = 0.0;
							romat_ud = 0.0;
							romat_ut = massin;	//lb/ivl
							romat = 0.0;
						}
					}
					else if (pBMPSite->m_nBMPClass == CLASS_B && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
					{
						BMP_B* pBMP = (BMP_B*) pBMPSite->m_pSiteProp;
						BMPAREA = pBMP->m_lfBasinLength * pBMP->m_lfBasinWidth;

						//use Ftable if provided for bottom surface area
						if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
							BMPAREA = pBMPSite->m_pFtable->m_lfArea[0];//ft2

						if (BMPAREA > SMALLNUM && ndevice > 0) 
						{
							double slope1 = pBMP->m_lfSideSlope1;
							double slope2 = pBMP->m_lfSideSlope2;
							double slope = pBMP->m_lfSideSlope3;
							//calculate the average surface area (ft2)
							double bot_width = pBMP->m_lfBasinWidth;
							double top_width = pBMP->m_lfMaximumDepth/slope1 + pBMP->m_lfMaximumDepth/slope2 + bot_width;
							double ave_sarea = (top_width + bot_width)/2.*pBMP->m_lfBasinLength;
							double avdepe = ovolume/ave_sarea;// ft
							double ros  = weir_p[i] + orifice_p[i] + infilt_p[i];//cfs
							double ro   = weir + orifice + infilt;//cfs
							double vols = bmpvol_p[i];//ft3
							double vol  = ovolume;//ft3
							double volsm = vols * CF2CM;//m3
							double volm  = vol * CF2CM;	//m3
							double xarea = vol / pBMP->m_lfBasinLength;//ft2
							double r__1 = avdepe / slope1;
							double r__2 = avdepe / slope2;
							double wet_p = pBMP->m_lfBasinWidth + 
										   sqrt(avdepe*avdepe + r__1*r__1) + 
										   sqrt(avdepe*avdepe + r__2*r__2); //ft
							double hrade = xarea / wet_p; //ft
							double avvele = 1.49/pBMP->m_lfManning*pow(hrade,2/3)*
											pow(slope,0.5); //ft/sec
							
							if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
							{
								//check if volume exceeds the maximum volume given in Ftable
								int nLast = pBMPSite->m_pFtable->m_nRecords-1;
								if(vol > pBMPSite->m_pFtable->m_lfVolume[nLast])
									avdepe = pBMPSite->m_pFtable->m_lfDepth[nLast]+
									(vol - pBMPSite->m_pFtable->m_lfVolume[nLast])/
									pBMPSite->m_pFtable->m_lfArea[nLast];
								else
									//calculate the water depth from the given Ftable
									avdepe = pBMPSite->m_pFtable->Volume_lookup(vol,pBMPSite->m_pFtable->m_lfDepth);//ft;

								slope = pBMPSite->m_pFtable->m_lfSlope;
								xarea = vol / pBMPSite->m_pFtable->m_lfLength;//ft2
								wet_p = pBMPSite->m_pFtable->Volume_lookup(vol,pBMPSite->m_pFtable->m_lfWetPerimeter);//ft;
								//calculate hydraulic radius of a conduit
								if (wet_p > 0)
									hrade = xarea / wet_p;	//ft
								else
									hrade = 0.;
								//calculate average velocity in a conduit
								if (pBMP->m_lfManning > 0)
									avvele = 1.49/pBMP->m_lfManning*pow(hrade,2/3)*
											 pow(slope,0.5); //ft/sec
								else
									avvele = 0.;
							}

							double avdepm = avdepe * FOOT2METER;	// m
							double rosm = ros * CFS2CMS;//m3/s
							double rom  = ro * CFS2CMS;	//m3/s
							double hradm = hrade * FOOT2METER;	// m
							double avvelm = avvele * FpS2MpS;//m/s

							double tw = 20.0;//degreeC
							double js   = 0.0;

							if (fabs(ros) > fThreshold)
							{
								double rat = vols / (ros * delts);
								if (rat < crrat)
									js = rat / crrat;
								else
									js = 1.0;
							}

							double cojs    = 1.0 - js;
							double srovol  = js * ros * delts;	//ft3
							double erovol  = cojs * ro * delts;	//ft3
							double srovolm = srovol * CF2CM;	//m3
							double erovolm = erovol * CF2CM;	//m3

							conc   = bmpc[i*NWQ+j];		// lb/ft3
							double concs  = bmpc[i*NWQ+j];
							double rsed1  = rbsed[i*NWQ+j];		// lb
							//double rsed1tot = rbsedtot[i];		// lb

							//simulate sediment
							if (pBMPData->nSedflag[j] == SAND)
							{
								//simulate sand
								int    sandfg = 3;	// user-specified power function method
								double ksand  = pBMPSite->m_sand.m_lfKSAND;
								double expsnd = pBMPSite->m_sand.m_lfEXPSND;
								double db50e  = pBMPSite->m_sand.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double w = pBMPSite->m_sand.m_lfW*0.0254*delts;	// fall velocity (m/ivl)
								double wsande = w * 3.28 / delts; // from m/ivl to ft/sec
								double depscr = 0., rsed = 0.;
									
								//dummy variables (not required for sandfg = 3)
								double twide = 0.,fsl = 0.;

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2 *= POUND2GRAM;	//gram
									rsed1   *= POUND2GRAM;	//gram
									conc    *= LBpCFT2MGpL;	//mg/l

									sandld(massin2,volsm,srovolm,volm,erovolm,ksand,avvele, 
										   expsnd,rom,sandfg,db50e,hrade,slope,tw,wsande,
										   twide,db50m,fsl,avdepe,&conc,&rsed,&rsed1,&depscr,
										   &romat);

									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	(g)
											rsed1  += rsed;	// mg/l*m3	(g)
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	//lb/ft3
									romat /= POUND2GRAM;	//lb
									rsed1 /= POUND2GRAM;	//lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3
								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == SILT)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_silt.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_silt.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_silt.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_silt.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_silt.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == CLAY)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_clay.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_clay.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_clay.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_clay.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_clay.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else	// not sediment
							{
								if (nRoutingMethod <= 1)
								{
									conc = bmpc[i*NWQ+j];			//lb/ft3		
									advect(massin2, vol1, qout1, vol2, qout2, delts, crrat, 
										conc, romat);

									if (nRemovalMethod == 0)
									{
										//1st order decay
										decay = pBMPSite->m_pDecay[j]*delth; //per ivl
										conc = conc * exp(-decay);	
									}
									else if (nRemovalMethod == 1)
									{
										//kadlec and knight method
										//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
										lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
										lfCstar = pBMPSite->m_pCstar[j];

										//maintain minimum conc of lfCstar
										conc = max(conc, lfCstar);

										double lfq = bmpout*delts/BMPAREA;
										if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
											lfq = pBMPSite->m_pFtable->Volume_lookup(bmpout*delts,pBMPSite->m_pFtable->m_lfDepth);//ft

										if (lfq > 0 && conc > lfCstar)
											conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
									}
								}
								else if (nRoutingMethod > 1)
								{
									//CSTRs in series routing
									int nCSTRs = nRoutingMethod;
									float v = vol1/nCSTRs;			//ft3
									float lfmassin = massin2/delts;	//lb/sec
									float lfinflow = oinflow2/delts;//cfs
									float lfoutflow = qout2;		//cfs

									for(k=0; k<nCSTRs; k++)
									{
										//calculate inputs for each CSTR
										float c = pBMPSite->m_pConc[j*nCSTRs+k];	//lb/ft3
										float wIn = lfmassin;						//lb/sec
										float qNet = lfinflow+(lfoutflow-lfinflow)*k/nCSTRs;		//ft3/sec
										float qNetout = lfinflow+(lfoutflow-lfinflow)*(k+1)/nCSTRs;	//ft3/sec

										//get the new conc (lb/ft3)
										conc = getCstrQual(c,v,wIn,qNet,(float)delts);

										//calculate massout (lb/ivl)
										romat = conc * qNetout * delts;

										if (nRemovalMethod == 0)
										{
											//1st order decay
											decay = pBMPSite->m_pDecay[j]*delth;	//per ivl
											conc = conc * exp(-decay);	
										}
										else if (nRemovalMethod == 1)
										{
											//kadlec and knight method
											//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
											lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
											lfCstar = pBMPSite->m_pCstar[j];
										
											//maintain minimum conc of lfCstar
											conc = max(conc, lfCstar);

											double lfq = lfoutflow*delts/BMPAREA;
											if (pBMPSite->m_nFtableFlag > 0 && pBMPSite->m_pFtable != NULL)
												lfq = pBMPSite->m_pFtable->Volume_lookup(lfoutflow*delts,pBMPSite->m_pFtable->m_lfDepth);//ft

											if (lfq > 0 && conc > lfCstar)
												conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
										}

										//update conc
										pBMPSite->m_pConc[j*nCSTRs+k] = conc;

										//calculate massin to the next reactor(lb/sec)
										lfmassin = romat / delts;
									}
								}

								if (qout2 > 0)
								{
									romat_w = weir/qout2*romat;
									romat_o = orifice/qout2*romat;
									romat_ud = udout/qout2*romat;
									if (pBMPSite->m_pUndRemoval != NULL)
										romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
								}
							}
						}
						else
						{
							//no volume stored
							//conc = massin / oinflow;//lb/ft3	
							conc = 0.0;//unknown
							romat_w = 0.0;
							romat_o = 0.0;
							romat_ud = 0.0;
							romat_ut = massin;//lb
							romat = 0.0;
						}
					}
					else if (pBMPSite->m_nBMPClass == CLASS_C)// && nRunMode != RUN_PREDEV)			
					{
						BMP_C* pBMP = (BMP_C*) pBMPSite->m_pSiteProp;
						int nIndex = pBMP->m_nIndex;
					    k = Link[nIndex].subIndex;
					
						if (Link[nIndex].xsect.type != DUMMY || 
							(Link[nIndex].xsect.type == DUMMY && pBMPSite->m_nFtableFlag > 0))
						{
							conc = Link[nIndex].newQual[j];//lb/ft3
							romat_w = 0.0;
							romat_o = romat_o2[i*NWQ+j];//lb
							romat_ud = 0.0;
							romat_ut = romat_ut2[i*NWQ+j];//lb
							romat = romat_o;
							romat_o2[i*NWQ+j] = 0.0;
							romat_ut2[i*NWQ+j] = 0.0;
						}
						else
						{
							//no volume stored
							//conc = massin / oinflow;//lb/ft3	
							conc = 0.0;//unknown
							romat_w = 0.0;
							romat_o = 0.0;
							romat_ud = 0.0;
							romat_ut = massin;//lb
							romat = 0.0;
						}
					}
					else if (pBMPSite->m_nBMPClass == CLASS_D && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
					{
						BMP_D* pBMP = (BMP_D*) pBMPSite->m_pSiteProp;
						BMPAREA = pBMP->m_lfFlowLength * pBMP->m_lfBufferWidth;

						if (BMPAREA > SMALLNUM) 
						{
							double avdepe = ovolume/BMPAREA;// ft
							double avdepm = avdepe * FOOT2METER;	// m
							double ros  = weir_p[i] + orifice_p[i] + infilt_p[i];//cfs
							double ro   = weir + orifice + infilt;//cfs
							double rosm = ros * CFS2CMS;//m3/s
							double rom  = ro * CFS2CMS;	//m3/s
							double vols = bmpvol_p[i];//ft3
							double vol  = ovolume;//ft3
							double volsm = vols * CF2CM;//m3
							double volm  = vol * CF2CM;	//m3
							double slope = pBMP->m_lfOverlandSlope;
							double hrade = max(0.0, avdepe - (double)pBMP->m_lfdStore);
							double hradm = hrade * FOOT2METER;	// m
							double tw = 20.0;//degreeC
							double avvele = 1.49/pBMP->m_lfN*pow(hrade,2/3)*
											pow(slope,0.5); //ft/sec
							double avvelm = avvele * FpS2MpS;//m/s
							
							double js   = 0.0;

							if (fabs(ros) > fThreshold)
							{
								double rat = vols / (ros * delts);
								if (rat < crrat)
									js = rat / crrat;
								else
									js = 1.0;
							}

							double cojs    = 1.0 - js;
							double srovol  = js * ros * delts;	//ft3
							double erovol  = cojs * ro * delts;	//ft3
							double srovolm = srovol * CF2CM;	//m3
							double erovolm = erovol * CF2CM;	//m3

							conc   = bmpc[i*NWQ+j];		// lb/ft3
							double concs  = bmpc[i*NWQ+j];
							double rsed1  = rbsed[i*NWQ+j];		// lb
							//double rsed1tot = rbsedtot[i];		// lb

							//simulate sediment
							if (pBMPData->nSedflag[j] == SAND)
							{
								//simulate sand
								int    sandfg = 3;	// user-specified power function method
								double ksand  = pBMPSite->m_sand.m_lfKSAND;
								double expsnd = pBMPSite->m_sand.m_lfEXPSND;
								double db50e  = pBMPSite->m_sand.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double w = pBMPSite->m_sand.m_lfW*0.0254*delts;	// fall velocity (m/ivl)
								double wsande = w * 3.28 / delts; // from m/ivl to ft/sec
								double depscr = 0., rsed = 0.;
									
								//dummy variables (not required for sandfg = 3)
								double twide = 0.,fsl = 0.;

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2 *= POUND2GRAM;	//gram
									rsed1   *= POUND2GRAM;	//gram
									conc    *= LBpCFT2MGpL;	//mg/l

									sandld(massin2,volsm,srovolm,volm,erovolm,ksand,avvele, 
										   expsnd,rom,sandfg,db50e,hrade,slope,tw,wsande,
										   twide,db50m,fsl,avdepe,&conc,&rsed,&rsed1,&depscr,
										   &romat);

									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	(g)
											rsed1  += rsed;	// mg/l*m3	(g)
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	//lb/ft3
									romat /= POUND2GRAM;	//lb
									rsed1 /= POUND2GRAM;	//lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3
								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == SILT)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_silt.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_silt.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_silt.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_silt.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_silt.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == CLAY)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_clay.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_clay.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_clay.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_clay.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_clay.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else	// not sediment
							{
								conc = bmpc[i*NWQ+j];	//lb/ft3
								
								advect(massin2, vol1, qout1, vol2, qout2, delts, crrat, conc, romat);

								if (nRemovalMethod == 0)
								{
									//1st order decay
									decay = pBMPSite->m_pDecay[j]*delth; //per ivl
									conc = conc * exp(-decay);	
								}
								else if (nRemovalMethod == 1)
								{
									//kadlec and knight method
									//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
									lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
									lfCstar = pBMPSite->m_pCstar[j];

									//maintain minimum conc of lfCstar
									conc = max(conc, lfCstar);

									double lfq = bmpout*delts/BMPAREA;
									if (lfq > 0 && conc > lfCstar)
										conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
								}

								if (qout2 > 0)
								{
									romat_w = weir/qout2*romat;
									romat_o = orifice/qout2*romat;
									romat_ud = 0.0;
								}
							}
						}
						else
						{
							//no volume stored
							//conc = massin / oinflow;//lb/ft3	
							conc = 0.0;//unknown
							romat_w = 0.0;
							romat_o = 0.0;
							romat_ud = 0.0;
							romat_ut = massin;//lb
							romat = 0.0;
						}
					}
					else if (pBMPSite->m_nBMPClass == CLASS_E && nRunMode != RUN_PREDEV && nRunMode != RUN_POSTDEV)
					{
						BMP_E* pBMP = (BMP_E*) pBMPSite->m_pSiteProp;
						float fBMPAREA = pBMP->m_lfsArea;	//ft2
						float fArea = (100.0-(float)pBMP->m_lfDCIA)/100.0;// fraction of disconnected impervious area (0-1)
						nRemovalMethod = pBMP->m_nPolRemMethod;
						double massin_per = BmpWqInput_per[i*NWQ+j];//lb/hr
						double massin_imp = BmpWqInput_imp[i*NWQ+j];//lb/hr

						massin2 = 0;
						if (qout2 > 0)
							massin2 = massin_imp * fArea;		// lb/hr

						romat_ut = max(0.0, massin - massin2);	//lb/hr
						
						if (fBMPAREA > SMALLNUM) 
						{
							double avdepe = ovolume/BMPAREA;// ft
							double avdepm = avdepe * FOOT2METER;	// m
							double ros  = weir_p[i] + orifice_p[i] + infilt_p[i];//cfs
							double ro   = weir + orifice + infilt;//cfs
							double rosm = ros * CFS2CMS;//m3/s
							double rom  = ro * CFS2CMS;	//m3/s
							double vols = bmpvol_p[i];//ft3
							double vol  = ovolume;//ft3
							double volsm = vols * CF2CM;//m3
							double volm  = vol * CF2CM;	//m3
							double slope = pBMP->m_lfOverlandSlope;
							double hrade = max(0.0, avdepe - (double)pBMP->m_lfdStore);
							double hradm = hrade * FOOT2METER;	// m
							double tw = 20.0;//degreeC
							double avvele = 1.49/pBMP->m_lfN*pow(hrade,2/3)*
											pow(slope,0.5); //ft/sec
							double avvelm = avvele * FpS2MpS;//m/s
							
							double js   = 0.0;

							if (fabs(ros) > fThreshold)
							{
								double rat = vols / (ros * delts);
								if (rat < crrat)
									js = rat / crrat;
								else
									js = 1.0;
							}

							double cojs    = 1.0 - js;
							double srovol  = js * ros * delts;	//ft3
							double erovol  = cojs * ro * delts;	//ft3
							double srovolm = srovol * CF2CM;	//m3
							double erovolm = erovol * CF2CM;	//m3

							conc   = bmpc[i*NWQ+j];		// lb/ft3
							double concs  = bmpc[i*NWQ+j];
							double rsed1  = rbsed[i*NWQ+j];		// lb
							//double rsed1tot = rbsedtot[i];		// lb

							//simulate sediment
							if (pBMPData->nSedflag[j] == SAND)
							{
								//simulate sand
								int    sandfg = 3;	// user-specified power function method
								double ksand  = pBMPSite->m_sand.m_lfKSAND;
								double expsnd = pBMPSite->m_sand.m_lfEXPSND;
								double db50e  = pBMPSite->m_sand.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double w = pBMPSite->m_sand.m_lfW*0.0254*delts;	// fall velocity (m/ivl)
								double wsande = w * 3.28 / delts; // from m/ivl to ft/sec
								double depscr = 0., rsed = 0.;
									
								//dummy variables (not required for sandfg = 3)
								double twide = 0.,fsl = 0.;

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2 *= POUND2GRAM;	//gram
									rsed1   *= POUND2GRAM;	//gram
									conc    *= LBpCFT2MGpL;	//mg/l

									sandld(massin2,volsm,srovolm,volm,erovolm,ksand,avvele, 
										   expsnd,rom,sandfg,db50e,hrade,slope,tw,wsande,
										   twide,db50m,fsl,avdepe,&conc,&rsed,&rsed1,&depscr,
										   &romat);

									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	(g)
											rsed1  += rsed;	// mg/l*m3	(g)
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	//lb/ft3
									romat /= POUND2GRAM;	//lb
									rsed1 /= POUND2GRAM;	//lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin2 + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3
								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb

								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == SILT)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_silt.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_silt.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_silt.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_silt.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_silt.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else if (pBMPData->nSedflag[j] == CLAY)
							{
								//convert settling velocity from in/sec to m/ivl
								double w = pBMPSite->m_clay.m_lfW*0.0254*delts;	// fall velocity (m/timestep)
								double taucd = pBMPSite->m_clay.m_lfTAUCD*0.4535924/0.09290304;	//critical bed shear stress for deposition (kg/m^2)
								double taucs = pBMPSite->m_clay.m_lfTAUCS*0.4535924/0.09290304;	//critical bed shear stress for scour (kg/m^2)
								//convert erodibility coeff from lb/ft^2/day to kg/m^2/ivl
								double m = pBMPSite->m_clay.m_lfM*deltd*0.4535924/0.09290304;	// erodibility coefficient of the sediment (kg/m^2/timestep)

								double db50e  = pBMPSite->m_clay.m_lfD/12.0;//ft
								double db50m  = db50e * 304.8; // mm
								double gamma = 1000;	// kg/m^3	 
								double tau = 0;			// (kg/m^2)		  

								if (fabs(volm) > fThreshold)
								{
									//convert to metric units
									massin2  *= POUND2GRAM;	//gram
									rsed1    *= POUND2GRAM;	//gram
									//rsed1tot *= POUND2GRAM;	//gram
									conc     *= LBpCFT2MGpL;//mg/l
									romat     = 0.0;

									advect(massin2,volsm,rosm,volm,rom,delts,crrat,conc,romat);
									
									double depscr = 0;		// mg/l*m3 = g
									double frcsed1 = 0.0;

									if (rsed1tot > 0.0)
									{
										frcsed1 = rsed1 / rsed1tot;
									}
									else
									{
										//no bed at start of interval, assume equal fractions 
										frcsed1 = 0.5;
									}

									//calculate exchange between bed and suspended sediment
									double rsed  = conc*volm;	// storage in suspension (g)
								  
									if (avdepm > 0.0)
									{
										//use formula appropriate to a river or stream
										tau = gamma * hradm * slope;// (kg/m^2)
									}

									if(avdepe > 0.17)
										BDEXCH(avdepm,w,tau,taucd,taucs,m,volm,frcsed1,&rsed,&rsed1,&depscr);	

									//update concentration 
									if (volm > 0)
										conc = rsed/volm;				// mg/l
									
									//set small concentrations to zero
									if (fabs(conc) < SMALLNUM) 
									{
										//small conc., set to zero
										if (depscr > 0.0) 
										{
											//deposition has occurred, add small storage to deposition
											depscr += rsed;	// mg/l*m3	
											rsed1  += rsed;	// mg/l*m3
										}
										else
										{
											//add small storage to outflow
											romat += rsed;
											depscr = 0.0;
										}
										rsed = 0.0;
										conc = 0.0;
									}

									//convert back to english units
									conc  /= LBpCFT2MGpL;	// lb/ft3
									romat /= POUND2GRAM;	// lb
									rsed1 /= POUND2GRAM;	// lb
								}
								else
								{
									// bmp has gone dry during the interval; 
									// set conc equal to zero
									conc = 0;

									// calculate total amount of material leaving  
									// during the interval;  
									// this is equal to material inflow + material initially 
									// present  
									if (ro > 0)
									{
										romat = massin + concs * vols;	
										if (romat < 0)	romat = 0;
									}
									else
									{
										romat = 0;	
									}
								}

								// update values
								bmpc[i*NWQ+j] = conc;// lb/ft3

								romat_w = 0.0;
								romat_o = 0.0;
								romat_i = 0.0;

								if (ro > 0)
								{
									romat_w = weir/ro*romat;
									romat_o = orifice/ro*romat;
									romat_i = infilt/ro*romat;
								}

								rbsed[i*NWQ+j] = rsed1;// lb
								rbsedtot[i] += rsed1;
								
								//soil and under-drain column concentration
								double conc2 = bmpc2[i*NWQ+j];//lb/ft3
								double romat2 = 0.0;//lb		
								double qout21 = undrain_p[i] + seepage_p[i];//cfs		
								double qout22 = udout + seepage;//cfs		
								double vol21 = max(0.0, vol_p[i]-bmpvol_p[i]);//ft3		
								double vol22 = max(0.0, vol2 - ovolume);//ft3	
								
								advect(romat_i,vol21,qout21,vol22,qout22,delts,crrat,conc2,romat2);
								
								bmpc2[i*NWQ+j] = conc2;// lb/ft3

								romat_ud = 0.0;
								if (qout22 > 0)
									romat_ud = udout/qout22*romat2;;
								//apply reduction
								if (pBMPSite->m_pUndRemoval != NULL)
									romat_ud *= (1.0 - pBMPSite->m_pUndRemoval[j]);
							}
							else	// not sediment
							{
								conc = bmpc[i*NWQ+j];	//lb/ft3
								
								advect(massin2, vol1, qout1, vol2, qout2, delts, crrat, conc, romat);

								if (nRemovalMethod == 0)
								{
									//1st order decay
									decay = pBMPSite->m_pDecay[j]*delth; //per ivl
									conc = conc * exp(-decay);	
								}
								else if (nRemovalMethod == 1)
								{
									//kadlec and knight method
									//Cout = Cstar + (Cin - Cstar) * exp (-k/q)
									lfK = pBMPSite->m_pK[j]*delth;	//ft/ivl
									lfCstar = pBMPSite->m_pCstar[j];

									//maintain minimum conc of lfCstar
									conc = max(conc, lfCstar);

									double lfq = bmpout*delts/fBMPAREA;
									if (lfq > 0 && conc > lfCstar)
										conc = lfCstar + (conc - lfCstar) * exp(-lfK/lfq);
								}

								if (qout2 > 0)
								{
									romat_w = weir/qout2*romat;
									romat_o = orifice/qout2*romat;
									romat_ud = 0.0;
								}
							}
						}
						else
						{
							//no volume stored
							//conc = massin / oinflow;//lb/ft3	
							conc = 0.0;//unknown
							romat_w = 0.0;
							romat_o = 0.0;
							romat_ud = 0.0;
							romat_ut = massin;//lb
							romat = 0.0;
						}
					}
					else//dummy BMP or assessment point
					{
						//no volume stored
						//conc = massin / oinflow;//lb/ft3	
						conc = 0.0;//unknown
						romat_w = 0.0;//lb
						romat_o = 0.0;//lb
						romat_ud = 0.0;//lb
						romat_ut = massin;//lb
						romat = 0.0;
					}
//MM:
					//save values
					bmpc[i*NWQ+j] = conc;//lb/ft3
					bmpmassout_w[i*NWQ+j]  = __max(0.0, romat_w);//lb
					bmpmassout_o[i*NWQ+j]  = __max(0.0, romat_o);//lb		
					bmpmassout_ud[i*NWQ+j] = __max(0.0, romat_ud);//lb
					bmpmassout_ut[i*NWQ+j] = __max(0.0, romat_ut);//lb
					bmpmassout[i*NWQ+j]    = __max(0.0, romat_w + romat_o + romat_ud + romat_ut);
					bmpmassout2[i*NPOL+nPollIndex] += bmpmassout[i*NWQ+j];
//					bmpmass_w_s[i*NPOL+nPollIndex] += romat_w;		
//					bmpmass_o_s[i*NPOL+nPollIndex] += romat_o;		
//					bmpmass_ud_s[i*NPOL+nPollIndex] += romat_ud;		
//					bmpmass_ut_s[i*NPOL+nPollIndex] += romat_ut;		
//					bmpmassout_s[i*NPOL+nPollIndex] += bmpmassout[i*NWQ+j];		
					bmpmass_w_s[i*NWQ+j] += bmpmassout_w[i*NWQ+j];		
					bmpmass_o_s[i*NWQ+j] += bmpmassout_o[i*NWQ+j];		
					bmpmass_ud_s[i*NWQ+j] += bmpmassout_ud[i*NWQ+j];		
					bmpmass_ut_s[i*NWQ+j] += bmpmassout_ut[i*NWQ+j];		
					bmpmassout_s[i*NWQ+j] += bmpmassout[i*NWQ+j];		

					nPollIndex++;
				}
			}

			// evaluation factor calculation
			for (j=0; j<NPOL; j++)
			{
				double value0 = pBMPSite->m_RAConc[j].qMass.front();
				double value1 = bmpmassout2[i*NPOL+j]; //lb
				
				lfSumMass[i*NPOL+j] -= value0;
				lfSumMass[i*NPOL+j] += value1;

				pBMPSite->m_RAConc[j].qMass.pop();
				pBMPSite->m_RAConc[j].qMass.push(value1);

				if (pBMPSite->m_factorList.GetCount() > 0)
				{
					//compute the cumulative load
					bmpTotalLoad[i*NPOL+j] += bmpmassout2[i*NPOL+j];// cumulative load (lb)

					if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
					{
						int nBinID = pBMPData->pBINDATA[t].nBinID;
						bmpTotalLoad_bin[i*NPOL*nNumBins+j*nNumBins+nBinID] += bmpmassout2[i*NPOL+j];// cumulative load (lb)
					}

					double bmpoutconc = 0.0;
					if (lfFlowVolume > 20)	//ft3/day
						bmpoutconc = lfSumMass[i*NPOL+j]/(lfSumFlow[i]*28.31685); //lb/liter
					
					if (pBMPData->nWeatherFile == 1)
					{
						if (pBMPData->pWEATHERDATA[pBMPData->lStartIndex+t].bWetInt)
						{
							if (bmpoutconc > pBMPSite->m_RAConc[j].m_lfThreshConc)
							{
								nExceedConc[i*NPOL+j]++;
							}
						}
					}

					double RTLoad = 0;
					double RTFlow = 0;
					double RAConc = 0;
					if (pBMPSite->m_RAConc[j].m_nRDays > 0)
					{
						int nRHours = pBMPSite->m_RAConc[j].m_nRDays/deltd+0.5;
						pBMPSite->m_RAConc[j].m_lfRFlow[nRHours-1] = bmpout*delts;	// ft3
						pBMPSite->m_RAConc[j].m_lfRLoad[nRHours-1] = bmpmassout2[i*NPOL+j];	// lbs

						for (k=0; k<nRHours; ++k)
						{
							RTFlow += pBMPSite->m_RAConc[j].m_lfRFlow[k];	// ft3
							RTLoad += pBMPSite->m_RAConc[j].m_lfRLoad[k];	// lbs
						}

						if (RTFlow > 0)
						RAConc = RTLoad/RTFlow*LBpCFT2MGpL;//mg/l
						
						if (bmpMAConc[i*NPOL+j] < RAConc)
							bmpMAConc[i*NPOL+j] = RAConc;

						if (nNumBins > 0 && pBMPData->pBINDATA != NULL)
						{
							int nBinID = pBMPData->pBINDATA[t].nBinID;
							if (bmpMAConc_bin[i*NPOL*nNumBins+j*nNumBins+nBinID] < RAConc)
								bmpMAConc_bin[i*NPOL*nNumBins+j*nNumBins+nBinID] = RAConc;
						}

						for (k=0; k<(nRHours-1); ++k)
						{
							pBMPSite->m_RAConc[j].m_lfRFlow[k] = pBMPSite->m_RAConc[j].m_lfRFlow[k+1];
							pBMPSite->m_RAConc[j].m_lfRLoad[k] = pBMPSite->m_RAConc[j].m_lfRLoad[k+1];
						}

						pBMPSite->m_RAConc[j].m_lfRFlow[nRHours-1] = 0.0;			
						pBMPSite->m_RAConc[j].m_lfRLoad[nRHours-1] = 0.0;
					}
				}

				//initialize to zero
				bmpmassout2[i*NPOL+j] = 0.0;
			}

			//save for the next timestep
			bmpoflow_w[i]		= weir;		//cfs							
			bmpoflow_o[i]		= orifice;	//cfs						
			bmpoflow_ud[i]		= udout;	//cfs							
			bmpoflow_ut[i]		= utout;	//cfs
			bmpoflow[i]			= bmpout;	//cfs

			vol_p[i]			= vol2;		//ft3
			bmpvol_p[i]			= ovolume;	//ft3
			osa_p[i]			= osa;		//in
			ostorage_p[i]		= ostorage;	//in
			weir_p[i]			= weir;		//cfs
			orifice_p[i]		= orifice;	//cfs
			infilt_p[i]			= infilt;	//cfs
			undrain_p[i]		= udout;	//cfs
			seepage_p[i]		= seepage;	//cfs
			counter_p[i]		= counter;

			//output variables
			BmpFlowInput_s[i]	+= oinflow/delts;// ft3/hr to cfs 
			bmpvol_s[i]			+= ovolume;	//ft3		
			bmpstage_s[i]		+= ostage;	//ft		
			infilt_s[i]			+= infilt;	//cfs		
			perc_s[i]			+= perc;	//cfs		
			AET_s[i]			+= AET;		//cfs		
			seepage_s[i]		+= seepage;	//cfs		
			weir_s[i]			+= weir;	//cfs			
			orifice_s[i]		+= orifice;	//cfs		
			bmpudout_s[i]		+= udout;	//cfs			
			bmpbypass_s[i]		+= utout;	//cfs			
			bmpoutflow_s[i]		+= bmpoflow[i];//cfs	

			// output time series results 
 			if(nRunMode != RUN_OPTIMIZE)
			{
				// save output to files
				if (pBMPSite->m_factorList.GetCount() > 0)
				{
					int printoption = 1;					//timestep
					if (pBMPData->nOutputTimeStep == 1)	
						printoption = int(1/delth+0.5);		//hourly

					if(printoption == 1 || (printoption > 1 && t%printoption == 0 && t != 0)) 
					{
						double cv = printoption;
						int nIndex = ::FindObIndexFromList(pBMPData->bmpsiteList, pBMPSite) + 1;
						CString strContent;
						strContent.Format("%s  %d  %d  %d  %d  %d\t", pBMPSite->m_strID, nYear, mon+1, day+1, hour, minute);

						CString strAdd;
						strAdd.Format("%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t",
							bmpvol_s[i]/cv,						// ft3
							bmpstage_s[i]/cv,					// ft
							BmpFlowInput_s[i]/cv,				// cfs
							weir_s[i]/cv,						// cfs
							orifice_s[i]/cv,					// cfs
							bmpudout_s[i]/cv,					// cfs
							bmpbypass_s[i]/cv,					// cfs
							bmpoutflow_s[i]/cv,					// cfs
							infilt_s[i]/cv, 					// cfs
							perc_s[i]/cv,						// cfs
							AET_s[i]/cv,						// cfs
							seepage_s[i]/cv);					// cfs
						strContent += strAdd;

//						int nNumPollutant = NPOL;
						int nNumPollutant = NWQ;
						for(j=0; j<nNumPollutant; j++)
						{
							strAdd.Format("%e\t%e\t%e\t%e\t%e\t%e\t", BmpWqInput_s[i*nNumPollutant+j], bmpmass_w_s[i*nNumPollutant+j], bmpmass_o_s[i*nNumPollutant+j], bmpmass_ud_s[i*nNumPollutant+j], bmpmass_ut_s[i*nNumPollutant+j], bmpmassout_s[i*nNumPollutant+j]);	// lb
							strContent += strAdd;

							bmpconcout_s[i*nNumPollutant+j] = 0.0;
							if (bmpoutflow_s[i] > fThreshold)
								bmpconcout_s[i*nNumPollutant+j] = bmpmassout_s[i*nNumPollutant+j]/(bmpoutflow_s[i]*delts);	//lb/ft3

							strAdd.Format("%e", bmpconcout_s[i*nNumPollutant+j]*LBpCFT2MGpL);  //mg/l

							if(j != nNumPollutant-1)
								strAdd += "\t";
							strContent += strAdd;

							//re-initialize
							BmpWqInput_s[i*nNumPollutant+j] = 0.0;	// Mass entering the BMP (lbs)
							bmpmass_w_s[i*nNumPollutant+j] = 0.0;	// Mass leaving the BMP (lbs)
							bmpmass_o_s[i*nNumPollutant+j] = 0.0;	// Mass leaving the BMP (lbs)
							bmpmass_ud_s[i*nNumPollutant+j] = 0.0;	// Mass leaving the BMP (lbs)
							bmpmass_ut_s[i*nNumPollutant+j] = 0.0;	// Mass bypassing the BMP (lbs)
							bmpmassout_s[i*nNumPollutant+j] = 0.0;	// Mass leaving the BMP (lbs)
							bmpconcout_s[i*nNumPollutant+j] = 0.0;	// Outflow concentration (mg/l)
						}

						strContent += "\n";

						//skip for the batch run
						if (nRUN_BATCH != 2 && nRUN_BATCH != 4)
						{
							fputs((LPCSTR)strContent, pBMPSite->m_fileOut);
							fflush(pBMPSite->m_fileOut);
						}

						// re-initialize
						bmpvol_s[i]       = 0.0;	// BMP volume (ft3)
						infilt_s[i]       = 0.0;	// Infiltration (cfs)
						perc_s[i]         = 0.0;	// percolation (cfs)
						AET_s[i]          = 0.0;	// actual evapotranspiration (cfs)
						seepage_s[i]      = 0.0;	// seepage (cfs)
//						usstorage_s[i]    = 0.0;	// BMP volume (ft3)
//						udstorage_s[i]    = 0.0;	// BMP volume (ft3)
						weir_s[i]         = 0.0;	// Weir outflow (cfs)
						orifice_s[i]      = 0.0;	// Orifice or channel outflow (cfs)
						BmpFlowInput_s[i] = 0.0;	// Total inflow (cfs)
						bmpstage_s[i]     = 0.0;	// Water depth (ft)
						bmpudout_s[i]     = 0.0;	// Underdrain outflow (cfs)
						bmpbypass_s[i]    = 0.0;	// untreated outflow (cfs)
						bmpoutflow_s[i]	  = 0.0;	// Total outflow (cfs)
					}
				}
			}
		}

		if(dayflag == t)
		{
			dayflag += int(1/deltd+0.5); 

			GetLocalTime(&tm);
			tSpan = COleDateTime(tm) - time_i;
			int dd_elap = int(tSpan.GetDays());
			int hh_elap = int(tSpan.GetHours());
			int mm_elap = int(tSpan.GetMinutes());
			int ss_elap = int(tSpan.GetSeconds());
			CString strMsg, strForDdg, strE;

			if (nRunMode == RUN_INIT)
			{
				strMsg.Format("BMP Simulation:\t Initial BMPs Scenario\n");
				strForDdg = strMsg;
			}
			else if (nRunMode == RUN_PREDEV)
			{
				strMsg.Format("BMP Simulation:\t Pre-Development Scenario\n");
				strForDdg = strMsg;
			}
			else if (nRunMode == RUN_POSTDEV)
			{
				strMsg.Format("BMP Simulation:\t Post-Development Scenario\n");
				strForDdg = strMsg;
			}
			else if (nRunMode == RUN_OPTIMIZE)
			{

				if (pBMPData->nStrategy == STRATEGY_GENETIC_ALGORITHM)
				{
					strMsg.Format("BMP Simulation:\t Optimization Scenario %d of %d\n", optcounter, nMaxRun);
					strForDdg = strMsg;
				}
				else
				{
					strMsg.Format("BMP Simulation:\t Optimization Scenario %d\n", optcounter);
					strForDdg = strMsg;
				}
			}
			else if (nRunMode == RUN_OUTPUT)
			{
				strMsg.Format("BMP Simulation:\t Best Solution Scenario %d of %d\n", outcounter, pBMPData->nSolution);
				strForDdg = strMsg;
			}

			strMsg.Format("Calculating:\t %02d-%02d-%04d\n", mon+1, day+1, nYear);
			strForDdg += strMsg;

			strE.Format("\nTime Elapsed:\t %02d:%02d:%02d:%02d\n", dd_elap, hh_elap, mm_elap, ss_elap);
			strForDdg += strE;

			double lfPerc = lfPart/lfAll;

			if(pWndProgress->GetSafeHwnd() != NULL)
			{
				pWndProgress->SetText(strForDdg);
				pWndProgress->SetPos((int)(lfPerc*100));
				pWndProgress->PeekAndPump();
			}

			if(pWndProgress->Cancelled()) 
				goto L001;
		}
	}
//end of simulation
	
	// evaluation factor calculation
	lfNumOfYears = N/(1/deltd * (365+nLeapYears));//number of year 

	for (i=0; i<NBMP; i++)
	{			          
		// compute evaluation factors
		pos = pBMPData->routeList.FindIndex(i);
		pBMPSite = (CBMPSite*) pBMPData->routeList.GetAt(pos);
		if (pBMPSite->m_factorList.GetCount() > 0)
		{
			//flow duration curve
			if (nRunMode != RUN_PREDEV && pBMPSite->m_nPreDevFlag > 0)
			{
				//process the predeveloped timeseries data
				if (!pBMPSite->ProcessSimRunData(strRunMode,pBMPData->nLANDTimeStep,tStart,tEnd,pBMPData->strOutputDir))
				{
					CString strErr;
					strErr.Format("Check flow duration curve option for BMPSite ID: %s", pBMPSite->m_strID);
					AfxMessageBox(strErr);
					goto L001;
				}
			}

			if (lfNumOfYears > 0)
			{
				bmpAAFlowVol[i] = bmpTotalFlow[i] / lfNumOfYears;	// ft3/yr
				bmpFlowExcFreq[i] = nExceedFlow[i] / lfNumOfYears;	// /yr
				
				for (k=0; k<nNumBins; k++)
				{
					bmpAAFlowVol_bin[i*nNumBins+k] = bmpTotalFlow_bin[i*nNumBins+k] / lfNumOfYears;	// ft3/yr
					bmpFlowExcFreq_bin[i*nNumBins+k] = nExceedFlow_bin[i*nNumBins+k] / lfNumOfYears;	// /yr
				}

				for (j=0; j<NPOL; j++)
				{
					bmpAALoad[i*NPOL+j] = bmpTotalLoad[i*NPOL+j] / lfNumOfYears;// lb/yr
					if (bmpAAFlowVol[i] > 0)
						bmpAAConc[i*NPOL+j] = bmpAALoad[i*NPOL+j] / bmpAAFlowVol[i] * LBpCFT2MGpL;	//mg/l

					bmpConcExcDays[i*NPOL+j] = nExceedConc[i*NPOL+j] / (1/deltd * lfNumOfYears);
					lfWetDaysPerYear = pBMPData->nWetInt/(1/deltd * lfNumOfYears);
					
					//(optional)
					for (k=0; k<nNumBins; k++)
					{
						bmpAALoad_bin[i*NPOL*nNumBins+j*nNumBins+k] = bmpTotalLoad_bin[i*NPOL*nNumBins+j*nNumBins+k] / lfNumOfYears;// lb/yr
						if (bmpAAFlowVol_bin[i*nNumBins+k] > 0)
							bmpAAConc_bin[i*NPOL*nNumBins+j*nNumBins+k] = bmpAALoad_bin[i*NPOL*nNumBins+j*nNumBins+k] / bmpAAFlowVol_bin[i*nNumBins+k] * LBpCFT2MGpL;	//mg/l
					}
				}
			}
		}
	}

	// start loop through each BMP site for assigning and outputting evaluation factors
	for (i=0; i<NBMP; i++)
	{
		pos = pBMPData->routeList.FindIndex(i);
		pBMPSite = (CBMPSite*) pBMPData->routeList.GetAt(pos);
		int nIndex = ::FindObIndexFromList(pBMPData->bmpsiteList, pBMPSite) + 1;
		CString strValue = "";
		if (pBMPSite->m_factorList.GetCount() > 0)
		{
			if(nRunMode == RUN_INIT)	// Existing Scenario (Before Optimization Scenario)
			{
				pos1 = pBMPSite->m_factorList.GetHeadPosition();
				while (pos1 != NULL)
				{
					EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
					
					//optional
					for (k=0; k<nNumBins; k++)
					{
						if (ef->m_nFactorGroup == -1)
						{
							if (ef->m_nFactorType == AAFV)
								ef->m_lfInit = bmpAAFlowVol_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == PDF)
								ef->m_lfInit = bmpPkDisFlow_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == FEF)
								ef->m_lfInit = bmpFlowExcFreq_bin[i*nNumBins+k];
						}
						else if (ef->m_nFactorGroup > 0)
						{
							if (ef->m_nFactorType == AAL)
								ef->m_lfInit = bmpAALoad_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == AAC)
								ef->m_lfInit = bmpAAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == MAC)
								ef->m_lfInit = bmpMAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
						}
						else
						{
							AfxMessageBox("Check card 815!, Factor Group can not have zero value");
							goto L001;
						}
						
						CString strFactor = ef->m_strFactor;
						if (strFactor.Right(2) == "_%")
							strFactor.TrimRight("_%");
						else if (strFactor.Right(2) == "_S")
							strFactor.TrimRight("_S");
						else if (strFactor.Right(2) == "_s")
							strFactor.TrimRight("_s");

						strValue.Format("%s\t%s\t%d\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, pBMPData->nBinIDs[k], ef->m_lfInit, totalCost);
						fputs(LPCSTR(strValue), fp_bin);
					}
					
					if (ef->m_nFactorGroup == -1)
					{
						if (ef->m_nFactorType == AAFV)
							ef->m_lfInit = bmpAAFlowVol[i];
						else if (ef->m_nFactorType == PDF)
							ef->m_lfInit = bmpPkDisFlow[i];
						else if (ef->m_nFactorType == FEF)
							ef->m_lfInit = bmpFlowExcFreq[i];
						else if (ef->m_nFactorType == FDC)
						{
							double lfBinFlowDiff = 0.;
							if(pBMPSite->m_pBinCFS != NULL && pBMPSite->m_pPreDevBinCFS != NULL)
							{
								//sum of bin flow differences 
								for (j=pBMPSite->m_nStartBin-1; j<pBMPSite->m_nEndBin; j++)
									lfBinFlowDiff += fabs(pBMPSite->m_pPreDevBinCFS[j]-pBMPSite->m_pBinCFS[j]);
							}
							ef->m_lfInit = lfBinFlowDiff;
						}
					}
					else if (ef->m_nFactorGroup > 0)
					{
						if (ef->m_nFactorType == AAL)
							ef->m_lfInit = bmpAALoad[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == AAC)
							ef->m_lfInit = bmpAAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == MAC)
							ef->m_lfInit = bmpMAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == CEF)	
							ef->m_lfInit = bmpConcExcDays[i*NPOL + ef->m_nFactorGroup - 1];
					}
					else
					{
						AfxMessageBox("Check card 815!, Factor Group can not have zero value");
						goto L001;
					}
					
					CString strFactor = ef->m_strFactor;
					if (strFactor.Right(2) == "_%")
						strFactor.TrimRight("_%");
					else if (strFactor.Right(2) == "_S")
						strFactor.TrimRight("_S");
					else if (strFactor.Right(2) == "_s")
						strFactor.TrimRight("_s");

					if (ef->m_nFactorType == CEF)	
						strValue.Format("%s\t%s\t%.2lf\t%.2lf\n", pBMPSite->m_strID, strFactor, ef->m_lfInit, lfWetDaysPerYear);
					else
						strValue.Format("%s\t%s\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, ef->m_lfInit, totalCost);
					fputs(LPCSTR(strValue), fp);
				}
			}
			else if(nRunMode == RUN_PREDEV)			// PRE-DEVELOPED Scenario (Before Optimization Scenario)
			{
				pos1 = pBMPSite->m_factorList.GetHeadPosition();
				while (pos1 != NULL)
				{
					EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
					//optional
					for (k=0; k<nNumBins; k++)
					{
						if (ef->m_nFactorGroup == -1)
						{
							if (ef->m_nFactorType == AAFV)
								ef->m_lfPreDev = bmpAAFlowVol_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == PDF)
								ef->m_lfPreDev = bmpPkDisFlow_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == FEF)
								ef->m_lfPreDev = bmpFlowExcFreq_bin[i*nNumBins+k];
						}
						else if (ef->m_nFactorGroup > 0)
						{
							if (ef->m_nFactorType == AAL)
								ef->m_lfPreDev = bmpAALoad_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == AAC)
								ef->m_lfPreDev = bmpAAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == MAC)
								ef->m_lfPreDev = bmpMAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
						}
						else
						{
							AfxMessageBox("Check card 815!, Factor Group can not have zero value");
							goto L001;
						}

						CString strFactor = ef->m_strFactor;
						if (strFactor.Right(2) == "_%")
							strFactor.TrimRight("_%");
						else if (strFactor.Right(2) == "_S")
							strFactor.TrimRight("_S");
						else if (strFactor.Right(2) == "_s")
							strFactor.TrimRight("_s");

						strValue.Format("%s\t%s\t%d\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, pBMPData->nBinIDs[k], ef->m_lfPreDev, 0.0);
						fputs(LPCSTR(strValue), fp_bin);
					}

					if (ef->m_nFactorGroup == -1)
					{
						if (ef->m_nFactorType == AAFV)
							ef->m_lfPreDev = bmpAAFlowVol[i];
						else if (ef->m_nFactorType == PDF)
							ef->m_lfPreDev = bmpPkDisFlow[i];
						else if (ef->m_nFactorType == FEF)
							ef->m_lfPreDev = bmpFlowExcFreq[i];
						else if (ef->m_nFactorType == FDC)
							ef->m_lfPreDev = 0;
					}
					else if (ef->m_nFactorGroup > 0)
					{
						if (ef->m_nFactorType == AAL)
							ef->m_lfPreDev = bmpAALoad[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == AAC)
							ef->m_lfPreDev = bmpAAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == MAC)
							ef->m_lfPreDev = bmpMAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == CEF)	
							ef->m_lfPreDev = bmpConcExcDays[i*NPOL + ef->m_nFactorGroup - 1];
					}
					else
					{
						AfxMessageBox("Check card 815!, Factor Group can not have zero value");
						goto L001;
					}

					CString strFactor = ef->m_strFactor;
					if (strFactor.Right(2) == "_%")
						strFactor.TrimRight("_%");
					else if (strFactor.Right(2) == "_S")
						strFactor.TrimRight("_S");
					else if (strFactor.Right(2) == "_s")
						strFactor.TrimRight("_s");

					if (ef->m_nFactorType == CEF)	
						strValue.Format("%s\t%s\t%.2lf\t%.2lf\n", pBMPSite->m_strID, strFactor, ef->m_lfPreDev, lfWetDaysPerYear);
					else
						strValue.Format("%s\t%s\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, ef->m_lfPreDev, 0.0);
					fputs(LPCSTR(strValue), fp);
				}
			}
			else if(nRunMode == RUN_POSTDEV)// POST-DEVELOPED Scenario (Before Optimization and Without BMPs)
			{
				pos1 = pBMPSite->m_factorList.GetHeadPosition();
				while (pos1 != NULL)
				{
					EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
					//optional
					for (k=0; k<nNumBins; k++)
					{
						if (ef->m_nFactorGroup == -1)
						{
							if (ef->m_nFactorType == AAFV)
								ef->m_lfPostDev = bmpAAFlowVol_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == PDF)
								ef->m_lfPostDev = bmpPkDisFlow_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == FEF)
								ef->m_lfPostDev = bmpFlowExcFreq_bin[i*nNumBins+k];
						}
						else if (ef->m_nFactorGroup > 0)
						{
							if (ef->m_nFactorType == AAL)
								ef->m_lfPostDev = bmpAALoad_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == AAC)
								ef->m_lfPostDev = bmpAAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == MAC)
								ef->m_lfPostDev = bmpMAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
						}
						else
						{
							AfxMessageBox("Check card 815!, Factor Group can not have zero value");
							goto L001;
						}

						CString strFactor = ef->m_strFactor;
						if (strFactor.Right(2) == "_%")
							strFactor.TrimRight("_%");
						else if (strFactor.Right(2) == "_S")
							strFactor.TrimRight("_S");
						else if (strFactor.Right(2) == "_s")
							strFactor.TrimRight("_s");

						strValue.Format("%s\t%s\t%d\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, pBMPData->nBinIDs[k], ef->m_lfPostDev, 0.0);
						fputs(LPCSTR(strValue), fp_bin);
					}

					if (ef->m_nFactorGroup == -1)
					{
						if (ef->m_nFactorType == AAFV)
							ef->m_lfPostDev = bmpAAFlowVol[i];
						else if (ef->m_nFactorType == PDF)
							ef->m_lfPostDev = bmpPkDisFlow[i];
						else if (ef->m_nFactorType == FEF)
							ef->m_lfPostDev = bmpFlowExcFreq[i];
						else if (ef->m_nFactorType == FDC)
						{
							double lfBinFlowDiff = 0.;
							if(pBMPSite->m_pBinCFS != NULL && pBMPSite->m_pPreDevBinCFS != NULL)
							{
								//sum of bin flow differences 
								for (j=pBMPSite->m_nStartBin-1; j<pBMPSite->m_nEndBin; j++)
									lfBinFlowDiff += fabs(pBMPSite->m_pPreDevBinCFS[j]-pBMPSite->m_pBinCFS[j]);
							}
							ef->m_lfPostDev = lfBinFlowDiff;
						}
					}
					else if (ef->m_nFactorGroup > 0)
					{
						if (ef->m_nFactorType == AAL)
							ef->m_lfPostDev = bmpAALoad[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == AAC)
							ef->m_lfPostDev = bmpAAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == MAC)
							ef->m_lfPostDev = bmpMAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == CEF)	
							ef->m_lfPostDev = bmpConcExcDays[i*NPOL + ef->m_nFactorGroup - 1];
					}
					else
					{
						AfxMessageBox("Check card 815!, Factor Group can not have zero value");
						goto L001;
					}

					CString strFactor = ef->m_strFactor;
					if (strFactor.Right(2) == "_%")
						strFactor.TrimRight("_%");
					else if (strFactor.Right(2) == "_S")
						strFactor.TrimRight("_S");
					else if (strFactor.Right(2) == "_s")
						strFactor.TrimRight("_s");

					if (ef->m_nFactorType == CEF)	
						strValue.Format("%s\t%s\t%.2lf\t%.2lf\n", pBMPSite->m_strID, strFactor, ef->m_lfPostDev, lfWetDaysPerYear);
					else
						strValue.Format("%s\t%s\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, ef->m_lfPostDev, 0.0);
					fputs(LPCSTR(strValue), fp);
				}
			}
			else if(nRunMode == RUN_OPTIMIZE)	//optimizing
			{
				pos1 = pBMPSite->m_factorList.GetHeadPosition();
				while (pos1 != NULL)
				{
					EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
					if (ef->m_nFactorGroup == -1)
					{
						if (ef->m_nFactorType == AAFV)
							ef->m_lfCurrent = bmpAAFlowVol[i];
						else if (ef->m_nFactorType == PDF)
							ef->m_lfCurrent = bmpPkDisFlow[i];
						else if (ef->m_nFactorType == FEF)
							ef->m_lfCurrent = bmpFlowExcFreq[i];
						else if (ef->m_nFactorType == FDC)
						{
							double lfBinFlowDiff = 0.;
							if(pBMPSite->m_pBinCFS != NULL && pBMPSite->m_pPreDevBinCFS != NULL)
							{
								//sum of bin flow differences 
								for (j=pBMPSite->m_nStartBin-1; j<pBMPSite->m_nEndBin; j++)
									lfBinFlowDiff += fabs(pBMPSite->m_pPreDevBinCFS[j]-pBMPSite->m_pBinCFS[j]);
							}
							ef->m_lfCurrent = lfBinFlowDiff;
						}
					}
					else if (ef->m_nFactorGroup > 0)
					{
						if (ef->m_nFactorType == AAL)
							ef->m_lfCurrent = bmpAALoad[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == AAC)
							ef->m_lfCurrent = bmpAAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == MAC)
							ef->m_lfCurrent = bmpMAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == CEF)	
							ef->m_lfCurrent = bmpConcExcDays[i*NPOL + ef->m_nFactorGroup - 1];
					}
					else
					{
						AfxMessageBox("Check card 815!, Factor Group can not have zero value");
						goto L001;
					}
				}
			}
			else if(nRunMode == RUN_OUTPUT)	//outputting
			{
				pos1 = pBMPSite->m_factorList.GetHeadPosition();
				while (pos1 != NULL)
				{
					EVALUATION_FACTOR* ef = (EVALUATION_FACTOR*) pBMPSite->m_factorList.GetNext(pos1);
					//optional
					for (k=0; k<nNumBins; k++)
					{
						if (ef->m_nFactorGroup == -1)
						{
							if (ef->m_nFactorType == AAFV)
								ef->m_lfCurrent = bmpAAFlowVol_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == PDF)
								ef->m_lfCurrent = bmpPkDisFlow_bin[i*nNumBins+k];
							else if (ef->m_nFactorType == FEF)
								ef->m_lfCurrent = bmpFlowExcFreq_bin[i*nNumBins+k];
						}
						else if (ef->m_nFactorGroup > 0)
						{
							if (ef->m_nFactorType == AAL)
								ef->m_lfCurrent = bmpAALoad_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == AAC)
								ef->m_lfCurrent = bmpAAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
							else if (ef->m_nFactorType == MAC)
								ef->m_lfCurrent = bmpMAConc_bin[i*NPOL*nNumBins+(ef->m_nFactorGroup-1)*nNumBins+k];
						}
						else
						{
							AfxMessageBox("Check card 815!, Factor Group can not have zero value");
							goto L001;
						}

						CString strFactor = ef->m_strFactor;
						if (strFactor.Right(2) == "_%")
							strFactor.TrimRight("_%");
						else if (strFactor.Right(2) == "_S")
							strFactor.TrimRight("_S");
						else if (strFactor.Right(2) == "_s")
							strFactor.TrimRight("_s");

						strValue.Format("%s\t%s\t%d\t%.5lf\t%.5lf\n", pBMPSite->m_strID, strFactor, pBMPData->nBinIDs[k], ef->m_lfCurrent, totalCost);
						fputs(LPCSTR(strValue), fp_bin);
					}

					if (ef->m_nFactorGroup == -1)
					{
						if (ef->m_nFactorType == AAFV)
							ef->m_lfCurrent = bmpAAFlowVol[i];
						else if (ef->m_nFactorType == PDF)
							ef->m_lfCurrent = bmpPkDisFlow[i];
						else if (ef->m_nFactorType == FEF)
							ef->m_lfCurrent = bmpFlowExcFreq[i];
						else if (ef->m_nFactorType == FDC)
						{
							double lfBinFlowDiff = 0.;
							if(pBMPSite->m_pBinCFS != NULL && pBMPSite->m_pPreDevBinCFS != NULL)
							{
								//sum of bin flow differences 
								for (j=pBMPSite->m_nStartBin-1; j<pBMPSite->m_nEndBin; j++)
									lfBinFlowDiff += fabs(pBMPSite->m_pPreDevBinCFS[j]-pBMPSite->m_pBinCFS[j]);
							}
							ef->m_lfCurrent = lfBinFlowDiff;
						}
					}
					else if (ef->m_nFactorGroup > 0)
					{
						if (ef->m_nFactorType == AAL)
							ef->m_lfCurrent = bmpAALoad[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == AAC)
							ef->m_lfCurrent = bmpAAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == MAC)
							ef->m_lfCurrent = bmpMAConc[i*NPOL + ef->m_nFactorGroup - 1];
						else if (ef->m_nFactorType == CEF)	
							ef->m_lfCurrent = bmpConcExcDays[i*NPOL + ef->m_nFactorGroup - 1];
					}
					else
					{
						AfxMessageBox("Check card 815!, Factor Group can not have zero value");
						goto L001;
					}

					if (pBMPData->nRunOption == OPTION_MIMIMIZE_COST)
					{
						double output1 = 0;
						if (ef->m_nCalcMode == CALC_PERCENT) // if the calculation mode is percentage
						{
							if (ef->m_lfInit > 0)	
								output1 = ef->m_lfCurrent/ef->m_lfInit*100;
//							else
//								output1 = ef->m_lfCurrent;
						}
						else if (ef->m_nCalcMode == CALC_VALUE) // if the calculation mode is value
						{
							output1 = ef->m_lfCurrent;
						}
						else // if the calculation mode is scale
						{
							if (ef->m_lfInit - ef->m_lfPreDev > 0)
								output1 = (ef->m_lfCurrent - ef->m_lfPreDev) / (ef->m_lfInit-ef->m_lfPreDev);
//							else
//								output1 = ef->m_lfCurrent;
						}
						strValue.Format("%s\t%s\t%.5lf\t%.5lf\n", pBMPSite->m_strID, ef->m_strFactor, output1, ef->m_lfTarget);
						fputs(LPCSTR(strValue), fp);
					}
					else if (pBMPData->nRunOption == OPTION_MAXIMIZE_CONTROL)
					{
//						if (ef->m_nFactorType == CEF)
//							strValue.Format("%s\t%s\t%.2lf\t%.2lf\n", pBMPSite->m_strID, ef->m_strFactor, ef->m_lfCurrent, lfWetDaysPerYear);
//						else
							strValue.Format("%s\t%s\t%.5lf\n", pBMPSite->m_strID, ef->m_strFactor, ef->m_lfCurrent);
						fputs(LPCSTR(strValue), fp);
					}
					else if (pBMPData->nRunOption == OPTION_TRADE_OFF_CURVE) 
					{
						//KA (evaluation file)
						strValue.Format("%s\t%s\t%.5lf\n", pBMPSite->m_strID, ef->m_strFactor, ef->m_lfCurrent);
						fputs(LPCSTR(strValue), fp);
					}
				}
			}
		}
	}

	fflush(fp_bin);
	fflush(fp);
L001:
	//release memory
	if (counter_p != NULL)			delete []counter_p;
	if (BmpFlowInput_per != NULL) 	delete []BmpFlowInput_per;
	if (BmpFlowInput_imp != NULL) 	delete []BmpFlowInput_imp;
	if (BmpFlowInput != NULL) 		delete []BmpFlowInput;
	if (LandGWrecharge != NULL) 	delete []LandGWrecharge;
	if (bmpoflow != NULL)			delete []bmpoflow;
	if (vol_p != NULL)				delete []vol_p;
	if (bmpvol_p != NULL)			delete []bmpvol_p;
	if (osa_p != NULL)				delete []osa_p;
	if (ostorage_p != NULL)			delete []ostorage_p;
	if (weir_p != NULL)				delete []weir_p;
	if (orifice_p != NULL)			delete []orifice_p;
	if (infilt_p != NULL)			delete []infilt_p;
	if (undrain_p != NULL)			delete []undrain_p;
	if (seepage_p != NULL)			delete []seepage_p;

	//output variables
	if (bmpvol_s != NULL)			delete []bmpvol_s;
	if (bmpstage_s != NULL)			delete []bmpstage_s;
	if (BmpFlowInput_s != NULL)		delete []BmpFlowInput_s;
	if (weir_s != NULL)				delete []weir_s;
	if (orifice_s != NULL)			delete []orifice_s;
	if (bmpudout_s != NULL)			delete []bmpudout_s;
	if (bmpbypass_s != NULL)		delete []bmpbypass_s;
	if (bmpoutflow_s != NULL)		delete []bmpoutflow_s;
	if (infilt_s != NULL)			delete []infilt_s;
	if (perc_s != NULL)				delete []perc_s;
	if (AET_s != NULL)				delete []AET_s;
	if (seepage_s != NULL)			delete []seepage_s;
//	if (usstorage_s != NULL)		delete []usstorage_s;
//	if (udstorage_s != NULL)		delete []udstorage_s;
	if (bmpoflow_w != NULL)			delete []bmpoflow_w;			
	if (bmpoflow_o != NULL)			delete []bmpoflow_o;			
	if (bmpoflow_ud != NULL)		delete []bmpoflow_ud;			
	if (bmpoflow_ut != NULL)		delete []bmpoflow_ut;
	if (rbsedtot != NULL)			delete []rbsedtot; 

	if (BmpWqInput_per != NULL)		delete []BmpWqInput_per;
	if (BmpWqInput_imp != NULL)		delete []BmpWqInput_imp;
	if (BmpWqInput != NULL)			delete []BmpWqInput;
	if (bmpc != NULL)				delete []bmpc;
	if (bmpc2 != NULL)				delete []bmpc2;
	if (bmpqconc_sand != NULL)		delete []bmpqconc_sand;
	if (bmpqconc_silt != NULL)		delete []bmpqconc_silt;
	if (bmpqconc_clay != NULL)		delete []bmpqconc_clay;
	if (bmpmassout != NULL)			delete []bmpmassout;
	if (bmpmassout_w != NULL)		delete []bmpmassout_w;
	if (bmpmassout_o != NULL)		delete []bmpmassout_o;
	if (bmpmassout_ud != NULL)		delete []bmpmassout_ud;
	if (bmpmassout_ut != NULL)		delete []bmpmassout_ut;
	if (bmudconc != NULL)			delete []bmudconc;
	if (romat_o2 != NULL)			delete []romat_o2; 
	if (romat_ut2 != NULL)			delete []romat_ut2; 
	if (rbsed != NULL)				delete []rbsed; 

	//output variables
	if (BmpWqInput_s != NULL)		delete []BmpWqInput_s;
	if (bmpmass_w_s != NULL)		delete []bmpmass_w_s;
	if (bmpmass_o_s != NULL)		delete []bmpmass_o_s;
	if (bmpmass_ud_s != NULL)		delete []bmpmass_ud_s;
	if (bmpmass_ut_s != NULL)		delete []bmpmass_ut_s;
	if (bmpmassout_s != NULL)		delete []bmpmassout_s;
	if (bmpconcout_s != NULL)		delete []bmpconcout_s;

	// evaluation factor calculation
	if (nExceedFlow != NULL)		delete []nExceedFlow;
	if (nInterEvent != NULL)		delete []nInterEvent;
	if (nExceedFlag != NULL)		delete []nExceedFlag;
	if (bmpTotalFlow != NULL)		delete []bmpTotalFlow;
	if (bmpAAFlowVol != NULL)		delete []bmpAAFlowVol;
	if (bmpPkDisFlow != NULL)		delete []bmpPkDisFlow;
	if (bmpFlowExcFreq != NULL)		delete []bmpFlowExcFreq;
	if (bmpTotalLoad != NULL)		delete []bmpTotalLoad; 
	if (bmpAALoad != NULL)			delete []bmpAALoad; 
	if (bmpAAConc != NULL)			delete []bmpAAConc; 
	if (bmpMAConc != NULL)			delete []bmpMAConc; 
	if (bmpmassout2 != NULL)		delete []bmpmassout2;
	//optional	
	if (nExceedConc != NULL)		delete []nExceedConc;
	if (lfDailyFlow != NULL)		delete []lfDailyFlow; 
	if (lfSumFlow != NULL)			delete []lfSumFlow; 
	if (lfSumMass != NULL)			delete []lfSumMass; 
	if (bmpConcExcDays != NULL)		delete []bmpConcExcDays;

	//optional
	if (nExceedFlow_bin != NULL)	delete []nExceedFlow_bin;
	if (bmpTotalFlow_bin != NULL)	delete []bmpTotalFlow_bin;
	if (bmpAAFlowVol_bin != NULL)	delete []bmpAAFlowVol_bin;
	if (bmpPkDisFlow_bin != NULL)	delete []bmpPkDisFlow_bin;
	if (bmpFlowExcFreq_bin != NULL)	delete []bmpFlowExcFreq_bin;
	if (bmpTotalLoad_bin != NULL)	delete []bmpTotalLoad_bin; 
	if (bmpAALoad_bin != NULL)		delete []bmpAALoad_bin; 
	if (bmpAAConc_bin != NULL)		delete []bmpAAConc_bin; 
	if (bmpMAConc_bin != NULL)		delete []bmpMAConc_bin; 

	//release memory for the tradeoff curve
	pos = pBMPData->routeList.GetHeadPosition();
	while (pos != NULL)
	{
		pBMPSite = (CBMPSite*) pBMPData->routeList.GetNext(pos);

		if (pBMPSite->m_nBreakPoints > 1)
		{
			int nBrPtIndex = int(pBMPSite->m_lfBreakPtID + 2.0);
			
			if (nRunMode == RUN_POSTDEV)
				nBrPtIndex = 0;				//index = -2+2=0
			else if (nRunMode == RUN_PREDEV)
				nBrPtIndex = 1;				//index (-1+2=1)
			else if(nRunMode == RUN_INIT)
				nBrPtIndex = 2;				//index = 0+2=2

			pBMPSite->UnLoadTradeOffCurveData(nBrPtIndex);
		}
	}
}

bool CBMPRunner::OpenEvalOutputFiles(const CString& runID, int nRunOption, int nRunMode)
{
	CString strFilePath;
	strFilePath.Format("%s_Eval.out", runID);
	strFilePath = pBMPData->strOutputDir + strFilePath;
	fp = fopen(LPCSTR(strFilePath), "wt");
	if(fp == NULL)
	{
		AfxMessageBox("Cannot open file " + strFilePath + " for writing.");
		return false;
	}

	WriteEvalFileHeader(nRunOption, nRunMode);

	return true;
}

bool CBMPRunner::CloseEvalOutputFiles()
{
	if (fp != NULL)
	{
		fclose(fp);
		fp = NULL;
	}

	return true;
}

void CBMPRunner::WriteEvalFileHeader(int nRunOption, int nRunMode)
{
	fputs("TT-----------------------------------------------------------------------------------------\n",fp);
	fputs("TT\n",fp);
	fputs("TT SUSTAIN: System for Urban Stormwater Treatment and Analysis INtegration\n",fp);
	fputs("TT\n",fp);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp);
	fputs("TT BMP Site Assessment Results\n",fp);
	fprintf(fp, "TT %s\n",SUSTAIN_VERSION);
	fputs("TT\n",fp);
	fputs("TT Designed and maintained by:\n",fp);
	fputs("TT     Tetra Tech, Inc.\n",fp);
	fputs("TT     10306 Eaton Place, Suite 340\n",fp);
	fputs("TT     Fairfax, VA 22030\n",fp);
	fputs("TT     (703) 385-6000\n",fp);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp);

	fputs("TT	\n", fp);
	SYSTEMTIME tm;
	GetLocalTime(&tm);
	CString str;
	str.Format("TT This output file was created at %02d:%02d:%02d%s on %02d/%02d/%04d\n",(tm.wHour>12)?tm.wHour-12:tm.wHour,tm.wMinute,tm.wSecond,(tm.wHour>=12)?"pm":"am",tm.wMonth,tm.wDay,tm.wYear);
	fputs(LPCSTR(str),fp);

	fputs("TT    \n", fp);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp);
	fputs("TT    \n", fp);

	if (nRunMode == RUN_INIT || nRunMode == RUN_PREDEV || nRunMode == RUN_POSTDEV)
		fprintf(fp, "Assessment Point (ID)     Factor Name     Factor Value     Total Cost\n");
	else if (nRunOption == OPTION_MIMIMIZE_COST)
		fprintf(fp, "Assessment Point (ID)     Factor Name     Factor Value		Target Value\n");
	else if (nRunOption == OPTION_MAXIMIZE_CONTROL)
		fprintf(fp, "Assessment Point (ID)     Factor Name     Factor Value\n");
	else if (nRunOption == OPTION_TRADE_OFF_CURVE) //KA
		fprintf(fp, "Assessment Point (ID)     Factor Name     Factor Value\n");
	fflush(fp);
}				

bool CBMPRunner::OpenBinOutputFiles(const CString& runID, int nRunOption, int nRunMode)
{
	CString strFilePath;
	strFilePath.Format("%s_Bin.out", runID);
	strFilePath = pBMPData->strOutputDir + strFilePath;
	fp_bin = fopen(LPCSTR(strFilePath), "wt");
	if(fp_bin == NULL)
	{
		AfxMessageBox("Cannot open file " + strFilePath + " for writing.");
		return false;
	}

	WriteBinFileHeader(nRunOption, nRunMode);

	return true;
}

bool CBMPRunner::CloseBinOutputFiles()
{
	if (fp_bin != NULL)
	{
		fclose(fp_bin);
		fp_bin = NULL;
	}

	return true;
}

void CBMPRunner::WriteBinFileHeader(int nRunOption, int nRunMode)
{
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_bin);
	fputs("TT\n",fp_bin);
	fputs("TT SUSTAIN: System for Urban Stormwater Treatment and Analysis INtegration\n",fp_bin);
	fputs("TT\n",fp_bin);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_bin);
	fputs("TT BMP Site Assessment Results for Precipitation Bins\n",fp_bin);
	fprintf(fp_bin, "TT %s\n",SUSTAIN_VERSION);
	fputs("TT\n",fp_bin);
	fputs("TT Designed and maintained by:\n",fp_bin);
	fputs("TT     Tetra Tech, Inc.\n",fp_bin);
	fputs("TT     10306 Eaton Place, Suite 340\n",fp_bin);
	fputs("TT     Fairfax, VA 22030\n",fp_bin);
	fputs("TT     (703) 385-6000\n",fp_bin);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_bin);

	fputs("TT	\n", fp_bin);
	SYSTEMTIME tm;
	GetLocalTime(&tm);
	CString str;
	str.Format("TT This output file was created at %02d:%02d:%02d%s on %02d/%02d/%04d\n",(tm.wHour>12)?tm.wHour-12:tm.wHour,tm.wMinute,tm.wSecond,(tm.wHour>=12)?"pm":"am",tm.wMonth,tm.wDay,tm.wYear);
	fputs(LPCSTR(str),fp_bin);

	fputs("TT    \n", fp_bin);
	fputs("TT-----------------------------------------------------------------------------------------\n",fp_bin);
	fputs("TT    \n", fp_bin);

	if (nRunMode == RUN_INIT || nRunMode == RUN_PREDEV || nRunMode == RUN_POSTDEV)
		fprintf(fp_bin, "AssessmentPoint_ID     FactorName     BIN_ID     FactorValue     TotalCost\n");
	else if (nRunOption == OPTION_MIMIMIZE_COST)
		fprintf(fp_bin, "AssessmentPoint_ID     FactorName     BIN_ID     FactorValue     TotalCost\n");
	else if (nRunOption == OPTION_MAXIMIZE_CONTROL)
		fprintf(fp_bin, "AssessmentPoint_ID     FactorName     BIN_ID     FactorValue     TotalCost\n");
	else if (nRunOption == OPTION_TRADE_OFF_CURVE) 
		fprintf(fp_bin, "AssessmentPoint_ID     FactorName     BIN_ID     FactorValue     TotalCost\n");
	fflush(fp_bin);
}				

//SWMM5
void findLinkQual2(int i,float tStep,double wAdded,double kDecay,double& c)
//
//  Input:   i = link index
//			 j = pollutant index
//           tStep = routing time step (sec)
//  Output:  none
//  Purpose: finds new quality in a link after the current time step.
//
{
    int   k;
    double q, vOld, vPlus;
//    float c;

    // --- find flow at inlet of link
    //     (for conduits, use Conduit[k].q1 since for
    //     KW routing, inlet flow can differ from outlet flow)
    q = Link[i].newFlow;
    if ( Link[i].type == CONDUIT )
    {
        k = Link[i].subIndex;
        q = Conduit[k].q1 * (float)Conduit[k].barrels;
    }

    // --- find old volume (vOld) & volume after inflow is added (vPlus)
    vOld = Link[i].oldVolume;
    vPlus = vOld + (fabs(q) * tStep);

    // --- find exponential 1st order decay over time step
//    c = Link[i].oldQual[j];
    if ( kDecay != 0.0 )
        c = c * exp(-kDecay * tStep);

    // --- combine inflow with old volume to compute new concen.
    if ( vPlus <= TINY ) c = 0.0;
    else c = (c*vOld + wAdded) / vPlus;
    c = MAX(c, 0.0);
//    Link[i].newQual[j] = c;

	return;
}

float getHydRad(TXsect* xsect, float y)
//
//  Input:   xsect = ptr. to conduit cross section
//           y     = flow depth (ft)
//  Output:  returns hydraulic radius (ft)
//  Purpose: computes hydraulic radius of flow cross-section in a conduit.
//
{
    float hRadius;                     // hyd. radius (ft)
    y = MIN(y, xsect->yFull);
    hRadius = xsect_getRofY(xsect, y);
    hRadius = MAX(hRadius, FUDGE);
    return hRadius;
}

